@@ -76,6 +76,8 @@ void MyMoneyQifProfile::clear(void)
   m_profileName = "";
   m_profileDescription = "";
   m_profileType = "Bank";
+  
+  m_attemptMatchDuplicates = true;
 }
 
 void MyMoneyQifProfile::loadProfile(const QString& name)
@@ -95,6 +97,8 @@ void MyMoneyQifProfile::loadProfile(const QString& name)
   m_voidMark = config->readEntry("VoidMark", m_voidMark);
   m_filterScriptImport = config->readEntry("FilterScriptImport", m_filterScriptImport);
   m_filterScriptExport = config->readEntry("FilterScriptExport", m_filterScriptExport);
+  
+  m_attemptMatchDuplicates = config->readBoolEntry("AttemptMatchDuplicates", m_attemptMatchDuplicates);
 
   // make sure, we remove any old stuff for now
   config->deleteEntry("FilterScript");
@@ -135,6 +139,7 @@ void MyMoneyQifProfile::saveProfile(void)
     config->writeEntry("VoidMark", m_voidMark);
     config->writeEntry("FilterScriptImport", m_filterScriptImport);
     config->writeEntry("FilterScriptExport", m_filterScriptExport);
+    config->writeEntry("AttemptMatchDuplicates", m_attemptMatchDuplicates);
 
     QString tmp;
 
@@ -334,7 +339,7 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
   QString formatDelim[2];
   int part;
   int delim;
-  unsigned int i, j;
+  unsigned int i,j;
 
   part = -1;
   delim = 0;
@@ -354,9 +359,9 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
         formatParts[part] += m_dateFormat[i];
         break;
       case '/':
-      case '\'':
-      case '.':
       case '-':
+      case '.':
+      case '\'':
         if(delim == 2) {
           qWarning("MyMoneyQifProfile::date(const QString& datein) Too many delimiters in date format");
           return QDate();
@@ -378,8 +383,8 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
    switch(datein[i].latin1()) {
       case '/':
       case '.':
-      case '\'':
       case '-':
+      case '\'':
         if(delim == 2) {
           qWarning("MyMoneyQifProfile::date(const QString& datein) Too many delimiters in date field");
           return QDate();
@@ -421,13 +426,13 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
     switch(formatParts[i][0].latin1()) {
       case 'd':
         day = scannedParts[i].toUInt(&ok);
-        if(!ok)
+        if (!ok)
           msg = "Invalid numeric character in day string";
         break;
       case 'm':
         if(formatParts[i].length() != 3) {
           mon = scannedParts[i].toUInt(&ok);
-          if(!ok)
+          if (!ok)
             msg = "Invalid numeric character in month string";
         } else {
           for(j = 1; j <= 12; ++j) {
@@ -446,7 +451,7 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
         ok = false;
         if(scannedParts[i].length() == formatParts[i].length()) {
           yr = scannedParts[i].toUInt(&ok);
-          if(!ok)
+          if (!ok)
             msg = "Invalid numeric character in month string";
           if(yr < 100) {      // two digit year info
             if(i > 1) {
@@ -495,7 +500,7 @@ const QDate MyMoneyQifProfile::date(const QString& datein) const
         break;
     }
     if(!msg.isEmpty()) {
-      qWarning(QString("MyMoneyQifProfile::date(const QString& datein) ") + msg.latin1());
+      qWarning("MyMoneyQifProfile::date(const QString& datein) %s",msg.latin1());
       return QDate();
     }
   }
@@ -549,7 +554,6 @@ const MyMoneyMoney MyMoneyQifProfile::value(const QChar& def, const QString& val
   unsigned char _decimalSeparator;
   unsigned char _thousandsSeparator;
   MyMoneyMoney res;
-  QString value(valuein);
 
   _decimalSeparator = MyMoneyMoney::decimalSeparator();
   _thousandsSeparator = MyMoneyMoney::thousandSeparator();
@@ -557,12 +561,7 @@ const MyMoneyMoney MyMoneyQifProfile::value(const QChar& def, const QString& val
   MyMoneyMoney::setDecimalSeparator(amountDecimal(def));
   MyMoneyMoney::setThousandSeparator(amountThousands(def));
 
-  if(value.find(amountDecimal(def)) == -1) {
-    value += amountDecimal(def);
-    value += "00";
-  }
-    
-  res = MyMoneyMoney(value);
+  res = MyMoneyMoney(valuein);
 
   MyMoneyMoney::setDecimalSeparator(_decimalSeparator);
   MyMoneyMoney::setThousandSeparator(_thousandsSeparator);
@@ -585,3 +584,13 @@ void MyMoneyQifProfile::setFilterScriptExport(const QString& script)
 
   m_filterScriptExport = script;
 }
+
+void MyMoneyQifProfile::setAttemptMatchDuplicates(bool f)
+{
+  if ( m_attemptMatchDuplicates != f )
+    m_isDirty = true;
+  
+  m_attemptMatchDuplicates = f;    
+}
+
+#include "mymoneyqifprofile.moc"
