@@ -32,10 +32,12 @@
 #include <kmessagebox.h>
 #include <knumvalidator.h>
 #include <klocale.h>
+#include <kapplication.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "../widgets/kmymoneycombo.h"
+#include "../widgets/kmymoneyaccountcombo.h"
 #include "../mymoney/mymoneyinstitution.h"
 #include "../mymoney/mymoneyaccount.h"
 #include "../mymoney/mymoneypayee.h"
@@ -47,6 +49,34 @@
 #include "../widgets/kmymoneydateinput.h"
 #include "ieditscheduledialog.h"
 
+/*
+void dumpSplit (int i, MyMoneySplit s) {
+  qDebug ("Split %d", i);
+  qDebug ("Acct: %s", s.accountId().data());
+  qDebug ("Payee: %s", s.payeeId().data());
+  qDebug ("Action = %s", s.action().data());
+  qDebug ("Value: %s", s.value().toString().latin1());
+} */
+
+// list of schedule occurrences in ascending order of period
+#define END_OCCURS 99999
+MyMoneySchedule::occurenceE KEditScheduleDialog::occurMasks[] = {
+  MyMoneySchedule::OCCUR_ONCE,
+  MyMoneySchedule::OCCUR_DAILY,
+  MyMoneySchedule::OCCUR_WEEKLY,
+  MyMoneySchedule::OCCUR_FORTNIGHTLY,
+  MyMoneySchedule::OCCUR_EVERYOTHERWEEK,
+  MyMoneySchedule::OCCUR_MONTHLY,
+  MyMoneySchedule::OCCUR_EVERYFOURWEEKS,
+  MyMoneySchedule::OCCUR_EVERYOTHERMONTH,
+  MyMoneySchedule::OCCUR_EVERYTHREEMONTHS,
+  MyMoneySchedule::OCCUR_QUARTERLY,
+  MyMoneySchedule::OCCUR_EVERYFOURMONTHS,
+  MyMoneySchedule::OCCUR_TWICEYEARLY,
+  MyMoneySchedule::OCCUR_YEARLY,
+  MyMoneySchedule::OCCUR_EVERYOTHERYEAR,
+  (MyMoneySchedule::occurenceE) END_OCCURS };
+
 KEditScheduleDialog::KEditScheduleDialog(const QCString& action, const MyMoneySchedule& schedule, QWidget *parent, const char *name)
  : kEditScheduledTransferDlgDecl(parent,name, true)
 {
@@ -55,15 +85,17 @@ KEditScheduleDialog::KEditScheduleDialog(const QCString& action, const MyMoneySc
   m_qbuttonCancel->setGuiItem(KStdGuiItem::cancel());
   m_qbuttonSplit->setGuiItem(KMyMoneyUtils::splitGuiItem());
   m_helpButton->setGuiItem(KStdGuiItem::help());
-  
+
   m_actionType = action;
   m_schedule = schedule;
   m_transaction = schedule.transaction();
+  createSecondSplit();
+
   // override the the action type if we have a scheduled transaction
-  if(m_transaction.splitCount() > 0) {
-    m_actionType = m_transaction.splits()[0].action();
-  }
-  
+//  if(m_transaction.splitCount() > 0) {
+//    m_actionType = m_transaction.splits()[0].action();
+//  }
+
   KIntValidator *validator = new KIntValidator(1, 32768, this);
   m_qlineeditRemaining->setValidator(validator);
 
@@ -78,7 +110,8 @@ KEditScheduleDialog::KEditScheduleDialog(const QCString& action, const MyMoneySc
     TextLabel1_3->setEnabled(true);
     m_kcomboTo->setEnabled(true);
     m_category->setEnabled(false);
-    m_qbuttonSplit->setEnabled(false);
+    //  transfers now allow splits
+    // m_qbuttonSplit->setEnabled(false);
     setCaption(i18n("Edit Transfer Schedule"));
   }
   else if (m_actionType == MyMoneySplit::ActionAmortization)
@@ -97,7 +130,7 @@ KEditScheduleDialog::KEditScheduleDialog(const QCString& action, const MyMoneySc
     m_qbuttonSplit->setEnabled(true);
     m_qcheckboxEnd->setEnabled(false);
     setCaption(i18n("Edit Loan Payment Schedule"));
-  }  
+  }
   else if (m_actionType == MyMoneySplit::ActionDeposit)
   {
     m_accountCombo->setEnabled(false);
@@ -119,18 +152,20 @@ KEditScheduleDialog::KEditScheduleDialog(const QCString& action, const MyMoneySc
   connect(m_qbuttonSplit, SIGNAL(clicked()), this, SLOT(slotSplitClicked()));
   connect(m_qcheckboxEnd, SIGNAL(toggled(bool)), this, SLOT(slotWillEndToggled(bool)));
   connect(m_qbuttonOK, SIGNAL(clicked()), this, SLOT(okClicked()));
+  connect(m_helpButton, SIGNAL(clicked()), this, SLOT(slotHelp()));
+
   connect(m_qlineeditRemaining, SIGNAL(textChanged(const QString&)),
     this, SLOT(slotRemainingChanged(const QString&)));
   connect(m_kdateinputFinal, SIGNAL(dateChanged(const QDate&)),
     this, SLOT(slotEndDateChanged(const QDate&)));
   connect(m_kmoneyeditAmount, SIGNAL(valueChanged(const QString&)),
     this, SLOT(slotAmountChanged(const QString&)));
-  connect(m_accountCombo, SIGNAL(activated(const QString&)),
-    this, SLOT(slotAccountChanged(const QString&)));
+  connect(m_accountCombo, SIGNAL(accountSelected(const QCString&)),
+    this, SLOT(slotAccountChanged(const QCString&)));
   connect(m_scheduleName, SIGNAL(textChanged(const QString&)),
     this, SLOT(slotScheduleNameChanged(const QString&)));
-  connect(m_kcomboTo, SIGNAL(activated(const QString&)),
-    this, SLOT(slotToChanged(const QString&)));
+  connect(m_kcomboTo, SIGNAL(accountSelected(const QCString&)),
+    this, SLOT(slotToChanged(const QCString&)));
   connect(m_kcomboMethod, SIGNAL(activated(int)),
     this, SLOT(slotMethodChanged(int)));
   connect(m_kcomboPayTo, SIGNAL(textChanged(const QString&)),
@@ -177,35 +212,40 @@ void KEditScheduleDialog::reloadFromFile(void)
     m_kcomboMethod->insertItem(i18n("Direct Debit"));
     m_kcomboMethod->insertItem(i18n("Direct Deposit"));
     m_kcomboMethod->insertItem(i18n("Manual Deposit"));
-    m_kcomboMethod->insertItem(i18n("Write Cheque"));
+    m_kcomboMethod->insertItem(i18n("Write Check"));
     m_kcomboMethod->insertItem(i18n("Other"));
+
+    if (m_actionType == MyMoneySplit::ActionAmortization)
+    {
+      m_paymentMethod->hide();
+    }
+    else
+    {
+      m_paymentMethod->insertItem("Transfer");
+    }
   }
   else if (m_actionType == MyMoneySplit::ActionDeposit)
   {
     m_kcomboMethod->insertItem(i18n("Direct Deposit"));
     m_kcomboMethod->insertItem(i18n("Manual Deposit"));
     m_kcomboMethod->insertItem(i18n("Other"));
+
+    m_paymentMethod->insertItem(i18n("Check"));
+    m_paymentMethod->insertItem(i18n("Deposit"));
+    m_paymentMethod->insertItem(i18n("Transfer"));
   }
   else // Withdrawal
   {
     m_kcomboMethod->insertItem(i18n("Direct Debit"));
-    m_kcomboMethod->insertItem(i18n("Write Cheque"));
+    m_kcomboMethod->insertItem(i18n("Write Check"));
     m_kcomboMethod->insertItem(i18n("Other"));
+
+    m_paymentMethod->insertItem(i18n("Check"));
+    m_paymentMethod->insertItem(i18n("Transfer"));
+    m_paymentMethod->insertItem(i18n("Withdrawal"));
+    m_paymentMethod->insertItem(i18n("ATM"));
   }
-
-  m_kcomboFreq->insertItem(i18n("Once"));
-  m_kcomboFreq->insertItem(i18n("Daily"));
-  m_kcomboFreq->insertItem(i18n("Weekly"));
-  m_kcomboFreq->insertItem(i18n("Every two weeks"));
-  m_kcomboFreq->insertItem(i18n("Every four weeks"));
-  m_kcomboFreq->insertItem(i18n("Monthly"));
-  m_kcomboFreq->insertItem(i18n("Every two months"));
-  m_kcomboFreq->insertItem(i18n("Every three months"));
-  m_kcomboFreq->insertItem(i18n("Every four months"));
-  m_kcomboFreq->insertItem(i18n("Twice a year"));
-  m_kcomboFreq->insertItem(i18n("Yearly"));
-  m_kcomboFreq->insertItem(i18n("Every other year"));
-
+  for (int i = 0; occurMasks[i] != END_OCCURS; i++) m_kcomboFreq->insertItem (KMyMoneyUtils::occurenceToString (occurMasks[i]));
   m_accountCombo->blockSignals(true);
   m_kcomboTo->blockSignals(true);
   if (m_actionType == MyMoneySplit::ActionTransfer
@@ -213,13 +253,15 @@ void KEditScheduleDialog::reloadFromFile(void)
   ||  m_actionType == MyMoneySplit::ActionAmortization
       )
   {
-    m_accountCombo->loadAccounts(true, false);
+    m_accountCombo->loadList
+        ((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
   }
 
   if (m_actionType == MyMoneySplit::ActionDeposit ||
       m_actionType == MyMoneySplit::ActionTransfer)
   {
-    m_kcomboTo->loadAccounts(true, false);
+    m_kcomboTo->loadList
+        ((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
   }
   m_accountCombo->blockSignals(false);
   m_kcomboTo->blockSignals(false);
@@ -244,6 +286,18 @@ void KEditScheduleDialog::reloadFromFile(void)
     slotDateChanged(QDate::currentDate());
 }
 
+void KEditScheduleDialog::createSecondSplit(void)
+{
+  if(m_transaction.splitCount() == 1) {
+    MyMoneySplit split, osplit;
+    split.setAction(m_transaction.splits()[0].action());
+    split.setPayeeId(m_transaction.splits()[0].payeeId());
+    split.setMemo(m_transaction.splits()[0].memo());
+    split.setValue(-m_transaction.splits()[0].value());
+    m_transaction.addSplit(split);
+  }
+}
+
 /*
  * Cribbed from : KLedgerViewCheckings::slotOpenSplitDialog(void)
  */
@@ -269,7 +323,7 @@ void KEditScheduleDialog::slotSplitClicked()
     checkPayee();
 
     MyMoneyAccount acc  = MyMoneyFile::instance()->account(theAccountId());
-    
+
     // theAccountId() returned the account that this schedule is attached
     // to. In case of a loan payment, we need to find the account that
     // the amortization goes to. This account contains a member that gives
@@ -311,7 +365,7 @@ void KEditScheduleDialog::slotSplitClicked()
 
       MyMoneySplit s;
       QString category;
-      
+
       disconnect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
       switch(m_transaction.splitCount())
       {
@@ -326,7 +380,7 @@ void KEditScheduleDialog::slotSplitClicked()
         case 0:
           break;
         default:
-          category = i18n("Splitted Transaction");
+          category = i18n("Split Transaction");
           connect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
           break;
       }
@@ -335,8 +389,7 @@ void KEditScheduleDialog::slotSplitClicked()
 
       MyMoneySplit split = m_transaction.splits()[0];
       MyMoneyMoney amount(split.value());
-      if (amount < 0)
-        amount = -amount;
+      amount = amount.abs();
       m_kmoneyeditAmount->setText(amount.formatMoney());
     }
 
@@ -361,11 +414,14 @@ void KEditScheduleDialog::slotWillEndToggled(bool on)
     m_kcomboFreq->setFocus();
     return;
   }
-  
+
   m_endLabel1->setEnabled(on);
   m_qlineeditRemaining->setEnabled(on);
   m_endLabel2->setEnabled(on);
   m_kdateinputFinal->setEnabled(on);
+  if(!on) {
+    m_schedule.setEndDate(QDate());
+  }
 }
 
 MyMoneySchedule KEditScheduleDialog::schedule(void)
@@ -412,10 +468,10 @@ void KEditScheduleDialog::okClicked()
     m_kdateinputFinal->setFocus();
     return;
   }
-  
+
   if (m_actionType == MyMoneySplit::ActionTransfer)
   {
-    if (m_accountCombo->currentText() == m_kcomboTo->currentText())
+    if (m_fromAccountId == m_toAccountId)
     {
       KMessageBox::detailedError(this, i18n("Unable to edit schedule"), i18n("Account from and account to are the same"));
       m_kcomboTo->setFocus();
@@ -425,9 +481,9 @@ void KEditScheduleDialog::okClicked()
 
   if (!checkCategory())
     return;
-    
+
   checkPayee();
-  
+
   slotEstimateChanged(); // Fix estimate being set when it shouldnt
 
   switch (m_weekendOption->currentItem())
@@ -442,10 +498,34 @@ void KEditScheduleDialog::okClicked()
       m_schedule.setWeekendOption(MyMoneySchedule::MoveMonday);
       break;
   }
-  
+
+  // Change the transaction action if one is specified
+  if (m_paymentMethod->currentText().length()>0)
+  {
+    for (unsigned i=0; i<m_transaction.splitCount(); i++)
+    {
+      MyMoneySplit split = m_transaction.splits()[i];
+      QString s = m_paymentMethod->currentText();
+      if (s == i18n("Check"))
+        split.setAction(MyMoneySplit::ActionCheck);
+      else if (s == i18n("Deposit"))
+        split.setAction(MyMoneySplit::ActionDeposit);
+      else if (s == i18n("Transfer"))
+        split.setAction(MyMoneySplit::ActionTransfer);
+      else if (s == i18n("Withdrawal"))
+        split.setAction(MyMoneySplit::ActionWithdrawal);
+      else if (s == i18n("ATM"))
+        split.setAction(MyMoneySplit::ActionATM);
+      else
+        split.setAction(QCString(s));
+
+      m_transaction.modifySplit(split);
+    }
+  }
+
   // Just reset the schedules transaction here.
   m_schedule.setTransaction(m_transaction);
-  
+
 /*
   QString message;
   message += "Schedule Name: " + m_schedule.name() + "\n";
@@ -462,7 +542,7 @@ void KEditScheduleDialog::okClicked()
   message += "endDate: " + m_schedule.endDate().toString() + "\n";
   KMessageBox::information(this, message);
 */
-  
+
   accept();
 }
 
@@ -472,14 +552,24 @@ void KEditScheduleDialog::loadWidgetsFromSchedule(void)
   {
     if (m_schedule.account().name().isEmpty())
       return;
-      
+
+    // always disable some fields when editing
+    if(!m_schedule.id().isEmpty()) {
+      m_kdateinputDue->setEnabled(false);
+      m_kdateinputFinal->setEnabled(false);
+      m_qlineeditRemaining->setEnabled(false);
+    }
+    // FIXME we allow to modify the end of a schedule at any time
+    // m_qcheckboxEnd->setEnabled(false);
+
+    // for (int i = 0; i < m_transaction.splitCount(); i++) dumpSplit (i, m_transaction.splits()[i]);
     if (m_actionType == MyMoneySplit::ActionTransfer)
     {
       // Jiggle the splits to how we want them
       MyMoneySplit s1 = m_transaction.splits()[0];
       MyMoneySplit s2 = m_transaction.splits()[1];
 
-      if (s1.value() >= 0)
+      if (!s1.value().isNegative())
       {
         try {
         m_transaction.removeSplits();
@@ -489,25 +579,32 @@ void KEditScheduleDialog::loadWidgetsFromSchedule(void)
         m_transaction.addSplit(s1);
         } catch (MyMoneyException *e)
         {
+          KMessageBox::detailedError(this, i18n("Exception in loadWidgetsFromSchedule(1)"), e->what());
           qDebug("e: %s", e->what().latin1());
           delete e;
         }
       }
     }
 
-    if (m_actionType != MyMoneySplit::ActionDeposit)
-      m_accountCombo->setCurrentText(MyMoneyFile::instance()->account(m_transaction.splits()[0].accountId()).name());
-    else
-      m_kcomboTo->setCurrentText(MyMoneyFile::instance()->account(m_transaction.splits()[0].accountId()).name());
-
+    if (m_actionType != MyMoneySplit::ActionDeposit) {
+      m_fromAccountId = m_transaction.splits()[0].accountId();
+      //m_accountCombo->setCurrentText(MyMoneyFile::instance()->account(m_transaction.splits()[0].accountId()).name());
+      m_accountCombo->setSelected (m_fromAccountId);
+    } else{
+      m_toAccountId = m_transaction.splits()[0].accountId();
+      //m_kcomboTo->setCurrentText(MyMoneyFile::instance()->account(m_transaction.splits()[0].accountId()).name());
+      m_kcomboTo->setSelected (m_toAccountId);
+    }
     if (m_actionType == MyMoneySplit::ActionTransfer
     ||  m_actionType == MyMoneySplit::ActionAmortization)
     {
-      m_kcomboTo->setCurrentText(
-        MyMoneyFile::instance()->account(m_transaction.splits()[1].accountId()).name());
+      m_toAccountId = m_transaction.splits()[1].accountId();
+     // m_kcomboTo->setCurrentText(
+       // MyMoneyFile::instance()->account(m_transaction.splits()[1].accountId()).name());
+      m_kcomboTo->setSelected (m_toAccountId);
     }
-    
-    m_kcomboPayTo->loadText(MyMoneyFile::instance()->payee(m_schedule.transaction().splitByAccount(theAccountId()).payeeId()).name());
+    m_kcomboPayTo->loadText(MyMoneyFile::instance()->payee(m_transaction.splitByAccount(theAccountId()).payeeId()).name());
+
     m_kdateinputDue->setDate(m_schedule.nextPayment(m_schedule.lastPayment())/*m_schedule.startDate()*/);
 
     int method=0;
@@ -582,71 +679,20 @@ void KEditScheduleDialog::loadWidgetsFromSchedule(void)
     }
 
     int frequency=0;
-    switch (m_schedule.occurence())
-    {
-      case MyMoneySchedule::OCCUR_ONCE:
-        frequency=0;
-        m_kcomboFreq->setCurrentItem(0);
-        break;
-      case MyMoneySchedule::OCCUR_DAILY:
-        frequency=1;
-        m_kcomboFreq->setCurrentItem(1);
-        break;
-      case MyMoneySchedule::OCCUR_WEEKLY:
-        frequency=2;
-        m_kcomboFreq->setCurrentItem(2);
-        break;
-      case MyMoneySchedule::OCCUR_FORTNIGHTLY:
-        frequency=3;
-        m_kcomboFreq->setCurrentItem(3);
-        break;
-      case MyMoneySchedule::OCCUR_EVERYFOURWEEKS:
-        frequency=4;
-        m_kcomboFreq->setCurrentItem(4);
-        break;
-      case MyMoneySchedule::OCCUR_MONTHLY:
-        frequency=5;
-        m_kcomboFreq->setCurrentItem(5);
-        break;
-      case MyMoneySchedule::OCCUR_EVERYOTHERMONTH:
-        frequency=6;
-        m_kcomboFreq->setCurrentItem(6);
-        break;
-      case MyMoneySchedule::OCCUR_EVERYTHREEMONTHS:
-        frequency=7;
-        m_kcomboFreq->setCurrentItem(7);
-        break;
-      case MyMoneySchedule::OCCUR_EVERYFOURMONTHS:
-        frequency=8;
-        m_kcomboFreq->setCurrentItem(8);
-        break;
-      case MyMoneySchedule::OCCUR_TWICEYEARLY:
-        frequency=9;
-        m_kcomboFreq->setCurrentItem(9);
-        break;
-      case MyMoneySchedule::OCCUR_YEARLY:
-        frequency=10;
-        m_kcomboFreq->setCurrentItem(10);
-        break;
-      case MyMoneySchedule::OCCUR_EVERYOTHERYEAR:
-        frequency=11;
-        m_kcomboFreq->setCurrentItem(11);
-        break;
-      default:
-        break;
-    }
+    for (frequency = 0; occurMasks[frequency] != END_OCCURS; frequency++) if (occurMasks[frequency] == m_schedule.occurence()) break;
+    if (occurMasks[frequency] == END_OCCURS) frequency = 0;
+    m_kcomboFreq->setCurrentItem(frequency);
 
-    MyMoneyMoney amount = m_schedule.transaction().splitByAccount(theAccountId()).value();
-    if (amount < 0)
-      amount = -amount;
+    MyMoneyMoney amount = m_transaction.splitByAccount(theAccountId()).value();
+    amount = amount.abs();
     m_kmoneyeditAmount->setText(amount.formatMoney());
     m_qcheckboxEstimate->setChecked(!m_schedule.isFixed());
 
     if (m_actionType != MyMoneySplit::ActionTransfer)
     {
-      if (m_schedule.transaction().splitCount() >= 3)
+      if (m_transaction.splitCount() >= 3)
       {
-        m_category->loadText(i18n("Splitted Transaction"));
+        m_category->loadText(i18n("Split Transaction"));
         connect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
       }
       else if(m_actionType == MyMoneySplit::ActionAmortization)
@@ -654,10 +700,10 @@ void KEditScheduleDialog::loadWidgetsFromSchedule(void)
         m_category->loadText(i18n("Loan payment"));
       }
       else
-        m_category->loadText(MyMoneyFile::instance()->accountToCategory(m_schedule.transaction().splitByAccount(theAccountId(), false).accountId()));
+        m_category->loadAccount(m_transaction.splitByAccount(theAccountId(), false).accountId());
     }
-    
-    m_qlineeditMemo->setText(m_schedule.transaction().splitByAccount(theAccountId()).memo());
+
+    m_qlineeditMemo->setText(m_transaction.splitByAccount(theAccountId()).memo());
     m_qcheckboxAuto->setChecked(m_schedule.autoEnter());
     m_qcheckboxEnd->setChecked(m_schedule.willEnd());
     if (m_schedule.willEnd())
@@ -684,12 +730,22 @@ void KEditScheduleDialog::loadWidgetsFromSchedule(void)
         m_weekendOption->setCurrentItem(2);
         break;
     }
-    
+
+    for (int i=0; i < m_paymentMethod->count(); i++)
+    {
+      if (QString(m_paymentMethod->text(i)) == i18n(m_transaction.splits()[0].action()))
+      {
+        m_paymentMethod->setCurrentItem(i);
+        break;
+      }
+    }
+
     // Quick hack
     slotFrequencyChanged(frequency);
     slotMethodChanged(method);
   } catch (MyMoneyException *e)
   {
+    KMessageBox::detailedError(this, i18n("Exception in loadWidgetsFromSchedule(2)"), e->what());
     delete e;
   }
 }
@@ -699,7 +755,7 @@ void KEditScheduleDialog::slotRemainingChanged(const QString& text)
   // Make sure the required fields are set
   m_schedule.setStartDate(m_kdateinputDue->getQDate());
   m_schedule.setOccurence(comboToOccurence());
-  
+
   if (m_schedule.transactionsRemaining() != text.toInt())
   {
     m_kdateinputFinal->setDate(m_schedule.dateAfter(text.toInt()));
@@ -722,52 +778,7 @@ void KEditScheduleDialog::slotEndDateChanged(const QDate& date)
 
 MyMoneySchedule::occurenceE KEditScheduleDialog::comboToOccurence()
 {
-  MyMoneySchedule::occurenceE occur;
-  
-  switch (m_kcomboFreq->currentItem())
-  {
-    case 0:
-      occur = MyMoneySchedule::OCCUR_ONCE;
-      break;
-    case 1:
-      occur = MyMoneySchedule::OCCUR_DAILY;
-      break;
-    case 2:
-      occur = MyMoneySchedule::OCCUR_WEEKLY;
-      break;
-    case 3:
-      occur = MyMoneySchedule::OCCUR_FORTNIGHTLY;
-      break;
-    case 4:
-      occur = MyMoneySchedule::OCCUR_EVERYFOURWEEKS;
-      break;
-    case 5:
-      occur = MyMoneySchedule::OCCUR_MONTHLY;
-      break;
-    case 6:
-      occur = MyMoneySchedule::OCCUR_EVERYOTHERMONTH;
-      break;
-    case 7:
-      occur = MyMoneySchedule::OCCUR_EVERYTHREEMONTHS;
-      break;
-    case 8:
-      occur = MyMoneySchedule::OCCUR_EVERYFOURMONTHS;
-      break;
-    case 9:
-      occur = MyMoneySchedule::OCCUR_TWICEYEARLY;
-      break;
-    case 10:
-      occur = MyMoneySchedule::OCCUR_YEARLY;
-      break;
-    case 11:
-      occur = MyMoneySchedule::OCCUR_EVERYOTHERYEAR;
-      break;
-    default:
-      occur = MyMoneySchedule::OCCUR_ANY;
-      break;
-  }
-
-  return occur;
+  return (occurMasks [m_kcomboFreq->currentItem()]);
 }
 
 void KEditScheduleDialog::slotAmountChanged(const QString&)
@@ -779,19 +790,19 @@ void KEditScheduleDialog::slotAmountChanged(const QString&)
     {
       createSplits();
     }
-    
+
     MyMoneySplit s = m_transaction.splits()[0];
     MyMoneyMoney amount = s.value();
-    if (m_kmoneyeditAmount->getMoneyValue() != amount)
+    if (m_kmoneyeditAmount->value() != amount)
     {
       // Cribbed from KLedgerView::slotAmountChanged
-      
+
       MyMoneyMoney val = MyMoneyMoney(m_kmoneyeditAmount->text());
-      
+
       // if someone enters a negative number, we have to make sure that
       // the action is corrected. For transfers, we don't have to do anything
       // The accounts will be 'exchanged' in reloadEditWidgets() and fillForm()
-      if(val < 0)
+      if(val.isNegative())
       {
         if (m_actionType == MyMoneySplit::ActionDeposit)
         {
@@ -812,13 +823,13 @@ void KEditScheduleDialog::slotAmountChanged(const QString&)
 
       s.setValue(val);
       m_transaction.modifySplit(s);
-      
+
       if(m_transaction.splitCount() == 2)
       {
         MyMoneySplit split = m_transaction.splits()[1];
         split.setValue(-s.value());
         m_transaction.modifySplit(split);
-      }  
+      }
       else if (count >= 3)
       {
         KMessageBox::information(this, i18n("All split data lost.  Please re-enter splits"));
@@ -836,8 +847,9 @@ void KEditScheduleDialog::slotAmountChanged(const QString&)
   }
 }
 
-void KEditScheduleDialog::slotAccountChanged(const QString&)
+void KEditScheduleDialog::slotAccountChanged(const QCString& id)
 {
+  m_fromAccountId = id;
   if (m_actionType != MyMoneySplit::ActionDeposit)
   {
     int count = m_transaction.splitCount();
@@ -847,8 +859,12 @@ void KEditScheduleDialog::slotAccountChanged(const QString&)
     }
 
     MyMoneySplit s = m_transaction.splits()[0];
-    s.setAccountId(m_accountCombo->currentAccountId());
+    s.setAccountId(id);
     m_transaction.modifySplit(s);
+    if(!id.isEmpty()) {
+      MyMoneyAccount acc = MyMoneyFile::instance()->account(id);
+      m_transaction.setCommodity(acc.currencyId());
+    }
   }
 }
 
@@ -857,8 +873,9 @@ void KEditScheduleDialog::slotScheduleNameChanged(const QString& text)
   m_schedule.setName(text);
 }
 
-void KEditScheduleDialog::slotToChanged(const QString&)
+void KEditScheduleDialog::slotToChanged(const QCString& id)
 {
+  m_toAccountId = id;
   if (m_actionType == MyMoneySplit::ActionTransfer
   ||  m_actionType == MyMoneySplit::ActionAmortization)
   {
@@ -868,7 +885,7 @@ void KEditScheduleDialog::slotToChanged(const QString&)
       createSplits();
     }
     MyMoneySplit s = m_transaction.splits()[1];
-    s.setAccountId(m_kcomboTo->currentAccountId());
+    s.setAccountId(id);
     m_transaction.modifySplit(s);
   }
   else if (m_actionType == MyMoneySplit::ActionDeposit)
@@ -879,7 +896,7 @@ void KEditScheduleDialog::slotToChanged(const QString&)
       createSplits();
     }
     MyMoneySplit s = m_transaction.splits()[0];
-    s.setAccountId(m_kcomboTo->currentAccountId());
+    s.setAccountId(id);
     m_transaction.modifySplit(s);
   }
 }
@@ -973,7 +990,7 @@ void KEditScheduleDialog::slotPayeeChanged(const QString& text)
   }
   catch (MyMoneyException *e)
   {
-    // We add it later
+       // KMessageBox::detailedError(this, i18n("Exception in slot PayeeChanged"), e->what());
     delete e;
   }
 }
@@ -1032,7 +1049,7 @@ void KEditScheduleDialog::slotEstimateChanged()
 
 void KEditScheduleDialog::slotCategoryChanged(const QString& text)
 {
-  if (text != i18n("Splitted Transaction"))
+  if (text != i18n("Split Transaction"))
   {
     int count = m_transaction.splitCount();
     if (count == 0)
@@ -1119,7 +1136,7 @@ void KEditScheduleDialog::createSplits()
     if (m_actionType == MyMoneySplit::ActionTransfer)
     {
       MyMoneySplit split2;
-      split2.setAccountId(m_kcomboTo->currentAccountId());
+      split2.setAccountId((QCString)m_kcomboTo->currentText());
       split2.setAction(MyMoneySplit::ActionTransfer);
       m_transaction.addSplit(split2);
     }
@@ -1138,9 +1155,9 @@ void KEditScheduleDialog::createSplits()
 bool KEditScheduleDialog::checkCategory()
 {
   bool exitDialog = true;
-  
+
   // Make sure a category has been set
-  if (m_category->text() != i18n("Splitted Transaction") && !m_category->text().isEmpty())
+  if (m_category->text() != i18n("Split Transaction") && !m_category->text().isEmpty())
   {
     bool invalid=false;
     QCString categoryId = MyMoneyFile::instance()->categoryToAccount(m_category->text());
@@ -1262,12 +1279,20 @@ QCString KEditScheduleDialog::theAccountId()
       m_actionType == MyMoneySplit::ActionAmortization ||
       m_actionType == MyMoneySplit::ActionWithdrawal)
   {
-    return m_accountCombo->currentAccountId();
+    return m_fromAccountId;
   }
   else if (m_actionType == MyMoneySplit::ActionDeposit)
   {
-    return m_kcomboTo->currentAccountId();
+    return m_toAccountId;
   }
 
   return QCString();
 }
+
+void KEditScheduleDialog::slotHelp(void)
+{
+  kapp->invokeHelp("details.schedules.new");
+}
+
+
+#include "ieditscheduledialog.moc"
