@@ -42,12 +42,13 @@
 #include "../mymoney/mymoneyfile.h"
 #include "../mymoney/mymoneyfinancialcalculator.h"
 #include "../kmymoneyutils.h"
-#include "../widgets/kmymoneycombo.h"
+#include "../widgets/kmymoneyaccountcombo.h"
 #include "../widgets/kmymoneypayee.h"
 #include "../widgets/kmymoneydateinput.h"
 #include "../widgets/kmymoneycategory.h"
 #include "../dialogs/ksplittransactiondlg.h"
 #include "../dialogs/kconfirmmanualenterdialog.h"
+#include "../dialogs/kcurrencycalculator.h"
 
 KEnterScheduleDialog::KEnterScheduleDialog(QWidget *parent, const MyMoneySchedule& schedule,
   const QDate& date)
@@ -64,10 +65,10 @@ KEnterScheduleDialog::KEnterScheduleDialog(QWidget *parent, const MyMoneySchedul
 
   connect(m_splitButton, SIGNAL(clicked()), this, SLOT(slotSplitClicked()));
   connect(m_buttonOk, SIGNAL(clicked()), this, SLOT(slotOK()));
-  connect(m_from, SIGNAL(activated(int)),
-    this, SLOT(slotFromActivated(int)));
-  connect(m_to, SIGNAL(activated(int)),
-    this, SLOT(slotToActivated(int)));
+  connect(m_from, SIGNAL(accountSelected(const QCString&)),
+    this, SLOT(slotFromActivated(const QCString&)));
+  connect(m_to, SIGNAL(accountSelected(const QCString&)),
+    this, SLOT(slotToActivated(const QCString&)));
   connect(m_date, SIGNAL(dateChanged(const QDate&)), this, SLOT(checkDateInPeriod(const QDate&)));
 }
 
@@ -77,8 +78,8 @@ KEnterScheduleDialog::~KEnterScheduleDialog()
 
 void KEnterScheduleDialog::initWidgets()
 {
-  kMyMoneyCombo* loanAccount = 0;
-  
+  kMyMoneyAccountCombo* loanAccount = 0;
+
   // Work around backwards transfers
   try
   {
@@ -88,7 +89,7 @@ void KEnterScheduleDialog::initWidgets()
 
     if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
     {
-      if (s1.value() < 0)
+      if (s1.value().isNegative())
       {
         transaction.removeSplits();
         s2.setId(QCString());
@@ -101,7 +102,7 @@ void KEnterScheduleDialog::initWidgets()
     }
     else if(m_schedule.type() != MyMoneySchedule::TYPE_LOANPAYMENT)
     {
-      if (s1.value() > 0)
+      if (s1.value().isPositive())
       {
         transaction.removeSplits();
         s2.setId(QCString());
@@ -125,38 +126,47 @@ void KEnterScheduleDialog::initWidgets()
 
   if (m_schedule.type() != MyMoneySchedule::TYPE_DEPOSIT
   &&  m_schedule.type() != MyMoneySchedule::TYPE_LOANPAYMENT)
-    m_from->loadAccounts(true, false);
+    m_from->loadList((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
 
   if (m_schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT)
   {
+    try
+    {
     MyMoneySplit amortizationSplit;
     QValueList<MyMoneySplit>::ConstIterator it_s;
     for(it_s = m_transaction.splits().begin(); it_s != m_transaction.splits().end(); ++it_s) {
-      if((*it_s).value() == MyMoneyMoney::minValue+1
+      if((*it_s).value() == MyMoneyMoney::autoCalc
       && (*it_s).action() == MyMoneySplit::ActionAmortization) {
         MyMoneyAccount acc = MyMoneyFile::instance()->account((*it_s).accountId());
         switch(acc.accountType()) {
           case MyMoneyAccount::Loan:
             m_from->setEnabled(true);
             m_to->setEnabled(false);
-            m_from->loadAccounts(true, false);
+            m_from->loadList((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
             loanAccount = m_from;
             break;
 
           case MyMoneyAccount::AssetLoan:
             m_from->setEnabled(false);
             m_to->setEnabled(true);
-            m_to->loadAccounts(true, false);
+            m_to->loadList((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
             loanAccount = m_to;
             break;
 
           default:
             qFatal("invalid account type in %s:%d", __FILE__, __LINE__);
             break;
-        } 
+        }
         break;
       }
-    }    
+    }
+    }
+    catch (MyMoneyException* e)
+    {
+      KMessageBox::detailedError(this, i18n("Unable to load schedule details"), e->what());
+      delete e;
+      return;
+    }
   }
   else if (m_schedule.type() == MyMoneySchedule::TYPE_TRANSFER
        ||  m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
@@ -166,11 +176,12 @@ void KEnterScheduleDialog::initWidgets()
     else
     {
       m_category->setEnabled(false);
-      m_splitButton->setEnabled(false);
+      // FIXME splits now allows in transfers
+      // m_splitButton->setEnabled(false);
     }
-      
+
     m_to->setEnabled(true);
-    m_to->loadAccounts(true, false);
+    m_to->loadList((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
   }
 
   try
@@ -183,20 +194,25 @@ void KEnterScheduleDialog::initWidgets()
       m_from->setCurrentText(m_schedule.account().name());
       m_to->setCurrentText(/*m_schedule.transferAccount().name()*/
         MyMoneyFile::instance()->account(m_schedule.transaction().splits()[1].accountId()).name());
+        m_toAccountId = m_schedule.transaction().splits()[1].accountId();
     }
-    else if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
+    else if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT) {
       m_to->setCurrentText(m_schedule.account().name());
-      
+      m_toAccountId = m_schedule.account().id();
+    }
     else if (m_schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT)
     {
       loanAccount->setCurrentText(m_schedule.account().name());
+      loanAccount == m_to ?
+          m_toAccountId = m_schedule.account().id() : m_fromAccountId = m_schedule.account().id();
     }
     else
     {
       m_from->setCurrentText(m_schedule.account().name());
+      m_fromAccountId = m_schedule.account().id();
       m_to->setEnabled(false);
     }
-      
+
     m_payee->setText(MyMoneyFile::instance()->payee(m_transaction.splitByAccount(m_schedule.account().id()).payeeId()).name());
     if (m_schedDate.isValid())
       m_date->setDate(m_schedDate);
@@ -204,18 +220,17 @@ void KEnterScheduleDialog::initWidgets()
       m_date->setDate(m_schedule.nextPayment(m_schedule.lastPayment()));
 
     MyMoneyMoney amount = m_transaction.splitByAccount(m_schedule.account().id()).value();
-    if (amount < 0)
+    if (amount.isNegative())
       amount = -amount;
     m_amount->setText(amount.formatMoney());
 
-    if (m_schedule.type() != MyMoneySchedule::TYPE_TRANSFER)
+    if (m_transaction.splitCount() >= 3)
+    {
+      m_category->setText(i18n("Split Transaction"));
+      connect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
+    }
+    else if (m_schedule.type() != MyMoneySchedule::TYPE_TRANSFER)
     {
-      if (m_transaction.splitCount() >= 3)
-      {
-        m_category->setText(i18n("Splitted Transaction"));
-        connect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
-      }
-      else
         m_category->setText(MyMoneyFile::instance()->accountToCategory(m_transaction.splitByAccount(m_schedule.account().id(), false).accountId()));
     }
 
@@ -280,7 +295,7 @@ void KEnterScheduleDialog::slotSplitClicked()
     m_category->blockSignals(true);
 
     MyMoneyAccount acc = MyMoneyFile::instance()->account(theAccountId());
-    
+
     KSplitTransactionDlg* dlg = new KSplitTransactionDlg(m_transaction,
                                                          acc,
                                                          isValidAmount,
@@ -314,7 +329,7 @@ void KEnterScheduleDialog::slotSplitClicked()
           disconnect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
           break;
         default:
-          category = i18n("Splitted Transaction");
+          category = i18n("Split Transaction");
           connect(m_category, SIGNAL(signalFocusIn()), this, SLOT(slotSplitClicked()));
           break;
       }
@@ -323,15 +338,14 @@ void KEnterScheduleDialog::slotSplitClicked()
 
       MyMoneySplit split = m_transaction.splitByAccount(theAccountId());
       MyMoneyMoney amount(split.value());
-      if (amount < 0)
-        amount = -amount;
+      amount = amount.abs();
       m_amount->setText(amount.formatMoney());
     }
 
     delete dlg;
   } catch (MyMoneyException *e)
   {
-//    KMessageBox::detailedError(this, i18n("Exception in slot split clicked"), e->what());
+    KMessageBox::detailedError(this, i18n("Exception in slot split clicked"), e->what());
     delete e;
   }
 
@@ -344,7 +358,7 @@ bool KEnterScheduleDialog::checkData(void)
   int noItemsChanged=0;
   QString payeeName;
 
-  if (m_from->currentText() == m_to->currentText())
+  if (m_fromAccountId == m_toAccountId)
   {
     KMessageBox::error(this, i18n("Account and transfer account are the same.  Please change one."));
     m_from->setFocus();
@@ -362,7 +376,7 @@ bool KEnterScheduleDialog::checkData(void)
     if (m_payee->text() != payeeName)
     {
       noItemsChanged++;
-      messageDetail += QString(i18n("Payee changed.  Old: \"%1\", New: \"%2\""))
+      messageDetail += i18n("Payee changed.  Old: \"%1\", New: \"%2\"")
         .arg(payeeName).arg(m_payee->text()) + QString("\n");
     }
 
@@ -371,7 +385,7 @@ bool KEnterScheduleDialog::checkData(void)
           m_from->currentText() != m_schedule.account().name())
     {
       noItemsChanged++;
-      messageDetail += QString(i18n("Account changed.  Old: \"%1\", New: \"%2\""))
+      messageDetail += i18n("Account changed.  Old: \"%1\", New: \"%2\"")
         .arg(m_schedule.account().name()).arg(m_from->currentText()) + QString("\n");
     }
 
@@ -379,7 +393,7 @@ bool KEnterScheduleDialog::checkData(void)
           m_to->currentText() != m_schedule.account().name())
     {
       noItemsChanged++;
-      messageDetail += QString(i18n("Account changed.  Old: \"%1\", New: \"%2\""))
+      messageDetail += i18n("Account changed.  Old: \"%1\", New: \"%2\"")
         .arg(m_schedule.account().name()).arg(m_to->currentText()) + QString("\n");
     }
 
@@ -387,7 +401,7 @@ bool KEnterScheduleDialog::checkData(void)
           m_to->currentText() != MyMoneyFile::instance()->account(m_schedule.transaction().splits()[1].accountId()).name())
     {
       noItemsChanged++;
-      messageDetail += QString(i18n("Transfer account changed.  Old: \"%1\", New: \"%2\""))
+      messageDetail += i18n("Transfer account changed.  Old: \"%1\", New: \"%2\"")
         .arg(MyMoneyFile::instance()->account(m_schedule.transaction().splits()[1].accountId()).name())
         .arg(m_to->currentText()) + QString("\n");
     }
@@ -396,7 +410,7 @@ bool KEnterScheduleDialog::checkData(void)
     {
       QString category;
       if (m_schedule.transaction().splitCount() >= 3)
-        category = i18n("Splitted Transaction");
+        category = i18n("Split Transaction");
       else
         category = MyMoneyFile::instance()->accountToCategory(m_schedule.transaction()
           .splitByAccount(m_schedule.account().id(), false).accountId());
@@ -404,7 +418,7 @@ bool KEnterScheduleDialog::checkData(void)
       if (category != m_category->text())
       {
         noItemsChanged++;
-        messageDetail += QString(i18n("Category changed.  Old: \"%1\", New: \"%2\""))
+        messageDetail += i18n("Category changed.  Old: \"%1\", New: \"%2\"")
           .arg(category).arg(m_category->text()) + QString("\n");
       }
     }
@@ -415,19 +429,18 @@ bool KEnterScheduleDialog::checkData(void)
       if(memo != m_memo->text())
       {
         noItemsChanged++;
-        messageDetail += QString(i18n("Memo changed.  Old: \"%1\", New: \"%2\""))
+        messageDetail += i18n("Memo changed.  Old: \"%1\", New: \"%2\"")
           .arg(memo).arg(m_memo->text()) + QString("\n");
       }
     }
-    
+
     MyMoneyMoney amount = m_schedule.transaction().splitByAccount(m_schedule.account().id()).value();
-    if (amount < 0)
-      amount = -amount;
-    if (amount != m_amount->getMoneyValue())
+    amount = amount.abs();
+    if (amount != m_amount->value())
     {
       noItemsChanged++;
-      messageDetail += QString(i18n("Amount changed.  Old: \"%1\", New: \"%2\""))
-        .arg(amount.formatMoney()).arg(m_amount->getMoneyValue().formatMoney()) + QString("\n");
+      messageDetail += i18n("Amount changed.  Old: \"%1\", New: \"%2\"")
+        .arg(amount.formatMoney()).arg(m_amount->value().formatMoney()) + QString("\n");
     }
 
     if (noItemsChanged > 0)
@@ -435,12 +448,12 @@ bool KEnterScheduleDialog::checkData(void)
       QString header = QString("\n") +
         i18n("%1 items of the details for the transaction have changed.")
         .arg(noItemsChanged);
-        
+
       KConfirmManualEnterDialog dlg(this, "kconfirmmanualenterdlg");
       dlg.m_message->setText(header);
       dlg.m_details->setText(messageDetail);
       dlg.m_onceRadio->setChecked(true);
-      
+
       // for loan payments, we do not allow to override the
       // stored transaction as this would destroy the automatic
       // amortization/interest calculation. Also, we cannot use
@@ -451,7 +464,7 @@ bool KEnterScheduleDialog::checkData(void)
         dlg.m_SetRadio->setEnabled(false);
         dlg.m_discardRadio->setEnabled(false);
       }
-      
+
       if (dlg.exec())
       {
         if (dlg.m_onceRadio->isChecked())
@@ -487,7 +500,7 @@ bool KEnterScheduleDialog::checkData(void)
             return false;
           }
         }
-        
+
         return true;
       }
     }
@@ -507,7 +520,7 @@ bool KEnterScheduleDialog::checkData(void)
 
 void KEnterScheduleDialog::checkCategory()
 {
-  if (m_category->text() != i18n("Splitted Transaction") &&
+  if (m_category->text() != i18n("Split Transaction") &&
       m_schedule.type() != MyMoneySchedule::TYPE_TRANSFER)
   {
     QString category = m_category->text();
@@ -544,7 +557,7 @@ void KEnterScheduleDialog::checkCategory()
 void KEnterScheduleDialog::commitTransaction()
 {
   QDate realLastPayment;
-  
+
   try
   {
     m_schedDate = m_date->getQDate();
@@ -567,11 +580,11 @@ void KEnterScheduleDialog::commitTransaction()
     if (m_schedDate > m_schedule.nextPayment(m_schedule.lastPayment()))
     {
       realLastPayment = m_schedDate;
-      
+
       if (m_schedDate < m_schedule.nextPayment(m_schedule.nextPayment(m_schedule.lastPayment())))
         realLastPayment = m_schedule.nextPayment(m_schedule.lastPayment());
 
-      QString message = QString(i18n("Some occurences that are older than '%1' have not been entered yet.\n\nDelete all occurences that have not been entered before this date?")).arg(m_schedDate.toString());
+      QString message = i18n("Some occurences that are older than '%1' have not been entered yet.\n\nDelete all occurences that have not been entered before this date?").arg(m_schedDate.toString());
 
       if (realLastPayment != m_schedule.nextPayment(m_schedule.lastPayment()) &&
           KMessageBox::warningYesNo(this, message) == KMessageBox::No)
@@ -582,11 +595,11 @@ void KEnterScheduleDialog::commitTransaction()
       {
         m_schedule.setLastPayment(realLastPayment);
       }
-    }        
+    }
     else if (m_schedDate > QDate::currentDate())
     {
       // FIXME: we should probably respect the configurable pre-enter period
-      QString message = QString(i18n("Are you sure you want to enter this occurence which is %1 days after today?")).arg(QDate::currentDate().daysTo(m_schedDate));
+      QString message = i18n("Are you sure you want to enter this occurence which is %1 days after today?").arg(QDate::currentDate().daysTo(m_schedDate));
       if (KMessageBox::warningYesNo(this, message) == KMessageBox::No)
         return;
 
@@ -624,15 +637,152 @@ void KEnterScheduleDialog::commitTransaction()
 
     m_transaction.setEntryDate(QDate::currentDate());
     m_transaction.setPostDate(m_schedDate);
-    MyMoneyFile::instance()->addTransaction(m_transaction);
+    m_transaction.setCommodity(m_schedule.account().currencyId());
 
-    try
-    {
+    try {
+      MyMoneyFile* file = MyMoneyFile::instance();
+      QValueList<MyMoneySplit> list = m_transaction.splits();
+      QValueList<MyMoneySplit>::Iterator it;
+
+      // Fix the payeeId. For non-asset and non-liability accounts,
+      // the payeeId will be cleared. If a transfer has one split
+      // with an empty payeeId the other one will be copied.
+      //
+      // Splits not referencing any account will be removed.
+      // Price information for other currencies will be collected
+
+      QMap<QCString, MyMoneyMoney> priceInfo;
+      for(it = list.begin(); it != list.end(); ++it) {
+        if((*it).accountId().isEmpty()) {
+          m_transaction.removeSplit(*it);
+          continue;
+        }
+        MyMoneyAccount acc = file->account((*it).accountId());
+        MyMoneySecurity currency(file->currency(m_schedule.account().currencyId()));
+        int fract = currency.smallestAccountFraction();
+        if(acc.accountType() == MyMoneyAccount::Cash)
+          fract = currency.smallestCashFraction();
+
+        MyMoneyAccount::accountTypeE accType = file->accountGroup(acc.accountType());
+        switch(accType) {
+          case MyMoneyAccount::Asset:
+          case MyMoneyAccount::Liability:
+            break;
+
+          default:
+            (*it).setPayeeId(QCString());
+            m_transaction.modifySplit(*it);
+            break;
+        }
+
+        if(m_transaction.commodity() != acc.currencyId()) {
+          // different currencies, try to find recent price info
+          QMap<QCString, MyMoneyMoney>::Iterator it_p;
+          QCString key = m_transaction.commodity() + "-" + acc.currencyId();
+          it_p = priceInfo.find(key);
+
+          // if it's not found, then collect it from the user first
+          MyMoneyMoney price;
+          if(it_p == priceInfo.end()) {
+            MyMoneySecurity fromCurrency, toCurrency;
+            MyMoneyMoney fromValue, toValue;
+            if(m_transaction.commodity() != m_schedule.account().currencyId()) {
+              toCurrency = file->currency(m_transaction.commodity());
+              fromCurrency = file->currency(acc.currencyId());
+              toValue = (*it).value();
+              fromValue = (*it).shares();
+            } else {
+              fromCurrency = file->currency(m_transaction.commodity());
+              toCurrency = file->currency(acc.currencyId());
+              fromValue = (*it).value();
+              toValue = (*it).shares();
+            }
+
+            KCurrencyCalculator calc(fromCurrency,
+                                    toCurrency,
+                                    fromValue,
+                                    toValue,
+                                    m_transaction.postDate(),
+                                    fract,
+                                    this, "currencyCalculator");
+            if(calc.exec() == QDialog::Rejected) {
+              return;
+            }
+            price = calc.price();
+            priceInfo[key] = price;
+          } else {
+            price = (*it_p);
+          }
+
+          // update shares if the transaction commodity is the currency
+          // of the current selected account
+          if(m_transaction.commodity() == m_schedule.account().currencyId()) {
+            (*it).setShares(((*it).value() * price).convert(fract));
+          }
+
+          // now update the split
+          m_transaction.modifySplit(*it);
+        } else {
+          if((*it).shares() != (*it).value()) {
+            (*it).setShares((*it).value());
+            m_transaction.modifySplit(*it);
+          }
+        }
+      }
+
+#if 0
+      // FIXME: This should be run through, but currently we don't have
+      // access to the transactionType() routine. So we keep the source here
+      // and let things go.
+      if(transactionType(m_transaction) == Transfer && !m_split.payeeId().isEmpty()) {
+        for(it = list.begin(); it != list.end(); ++it) {
+          if((*it).id() == m_split.id())
+            continue;
+
+          if((*it).payeeId().isEmpty()) {
+            (*it).setPayeeId(m_split.payeeId());
+            m_transaction.modifySplit(*it);
+          }
+        }
+      }
+
+      // check if this is a transfer to/from loan account and
+      // mark it as amortization in this case
+      if(m_transaction.splitCount() == 2) {
+        bool isAmortization = false;
+        for(it = list.begin(); !isAmortization && it != list.end(); ++it) {
+          if((*it).action() == MyMoneySplit::ActionTransfer) {
+            MyMoneyAccount acc = file->account((*it).accountId());
+            if(acc.accountType() == MyMoneyAccount::Loan
+            || acc.accountType() == MyMoneyAccount::AssetLoan)
+              isAmortization = true;
+          }
+        }
+
+        if(isAmortization) {
+          for(it = list.begin(); it != list.end(); ++it) {
+            (*it).setAction(MyMoneySplit::ActionAmortization);
+            m_transaction.modifySplit(*it);
+          }
+        }
+      }
+#endif
+
+      // If we are writing a check, we adjust the split's
+      // number to the highest number used in this account plus one
+      if(m_schedule.paymentType() == MyMoneySchedule::STYPE_WRITECHEQUE) {
+        MyMoneySplit s = m_transaction.splitByAccount(m_schedule.account().id(), true);
+        s.setAction(MyMoneySplit::ActionCheck);
+        unsigned64 no = MyMoneyFile::instance()->highestCheckNo(s.accountId()).toULongLong();
+        s.setNumber(QString::number(no+1));
+        m_transaction.modifySplit(s);
+      }
+
+      MyMoneyFile::instance()->addTransaction(m_transaction);
       MyMoneyFile::instance()->modifySchedule(m_schedule);
-    }
-    catch (MyMoneyException *e)
-    {
-      KMessageBox::error(this, i18n("Unable to modify schedule: ") + e->what());
+    } catch(MyMoneyException *e) {
+      KMessageBox::detailedSorry(0, i18n("Unable to add transaction/modify schedule"),
+        (e->what() + " " + i18n("thrown in") + " " + e->file()+ ":%1").arg(e->line()));
       delete e;
     }
   }
@@ -678,7 +828,7 @@ void KEnterScheduleDialog::setTo()
     else
       id = 0;
     MyMoneySplit s = m_transaction.splits()[id];
-    s.setAccountId(m_to->currentAccountId());
+    s.setAccountId(m_toAccountId);
     m_transaction.modifySplit(s);
   }
 }
@@ -694,7 +844,7 @@ void KEnterScheduleDialog::setFrom()
     }
 
     MyMoneySplit s = m_transaction.splits()[0];
-    s.setAccountId(m_from->currentAccountId());
+    s.setAccountId(m_fromAccountId);
     m_transaction.modifySplit(s);
   }
 }
@@ -736,13 +886,13 @@ void KEnterScheduleDialog::setAmount()
 
     MyMoneySplit s = m_transaction.splits()[0];
     MyMoneyMoney amount = s.value();
-    if (m_amount->getMoneyValue() != amount)
+    if (m_amount->value() != amount)
     {
       if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
-        s.setValue(m_amount->getMoneyValue());
+        s.setValue(m_amount->value());
       else if(m_schedule.type() != MyMoneySchedule::TYPE_LOANPAYMENT)
-        s.setValue(-m_amount->getMoneyValue());
-        
+        s.setValue(-m_amount->value());
+
       m_transaction.modifySplit(s);
 
       // if we have exactly two splits, we can update the 'other' side
@@ -782,43 +932,48 @@ void KEnterScheduleDialog::createSplits()
 
     MyMoneySplit split1;
     split1.setAccountId(theAccountId());
-    
+
+#if 0
+    // These actions are not required anymore
     if (m_schedule.type() == MyMoneySchedule::TYPE_BILL)
       split1.setAction(MyMoneySplit::ActionWithdrawal);
     else if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
       split1.setAction(MyMoneySplit::ActionDeposit);
     else
       split1.setAction(MyMoneySplit::ActionTransfer);
-
+#endif
     split1.setPayeeId(payeeId);
     split1.setMemo(m_memo->text());
-    
+
     if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
-      split1.setValue(m_amount->getMoneyValue());
+      split1.setValue(m_amount->value());
     else
-      split1.setValue(-m_amount->getMoneyValue());
-      
+      split1.setValue(-m_amount->value());
+
     m_transaction.addSplit(split1);
 
-    
+
     MyMoneySplit split2;
     if (m_schedule.type() != MyMoneySchedule::TYPE_TRANSFER)
     {
       checkCategory();
     }
     else
-      split2.setAccountId(m_to->currentAccountId());
+      split2.setAccountId(m_toAccountId);
 
     split2.setPayeeId(split1.payeeId());
     split2.setMemo(split1.memo());
     split2.setValue(-split1.value());
-    
+
+#if 0
+    // These actions are not required anymore
     if (split1.action() == MyMoneySplit::ActionDeposit)
       split2.setAction(MyMoneySplit::ActionWithdrawal);
     else if (split1.action() == MyMoneySplit::ActionWithdrawal)
       split2.setAction(MyMoneySplit::ActionDeposit);
     else
       split2.setAction(MyMoneySplit::ActionTransfer);
+#endif
 
     m_transaction.addSplit(split2);
   }
@@ -828,49 +983,51 @@ QCString KEnterScheduleDialog::theAccountId()
 {
   if( m_schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT) {
     if(m_from->isEnabled())
-      return m_from->currentAccountId();
+      return m_fromAccountId;
     if(m_to->isEnabled())
-      return m_to->currentAccountId();
+      return m_toAccountId;
     qFatal("This should never happen %s:%d", __FILE__, __LINE__);
   }
-  
+
   if (m_schedule.type() != MyMoneySchedule::TYPE_DEPOSIT)
-    return m_from->currentAccountId();
+    return m_fromAccountId;
   else
-    return m_to->currentAccountId();
+    return m_toAccountId;
 }
 
-void KEnterScheduleDialog::slotFromActivated(int)
+void KEnterScheduleDialog::slotFromActivated(const QCString& id)
 {
   if (m_schedule.type() != MyMoneySchedule::TYPE_DEPOSIT)
   {
     // Change the splits because otherwise they wont be found
     // if the user clicks on the split button
     MyMoneySplit s = m_transaction.splits()[0];
-    s.setAccountId(m_from->currentAccountId());
+    s.setAccountId(id);
     m_transaction.modifySplit(s);
   }
 }
 
-void KEnterScheduleDialog::slotToActivated(int)
+void KEnterScheduleDialog::slotToActivated(const QCString& id)
 {
   if (m_schedule.type() == MyMoneySchedule::TYPE_DEPOSIT)
   {
     // Change the splits because otherwise they wont be found
     // if the user clicks on the split button
     MyMoneySplit s = m_transaction.splits()[0];
-    s.setAccountId(m_to->currentAccountId());
+    s.setAccountId(id);
     m_transaction.modifySplit(s);
   }
 }
 
 void KEnterScheduleDialog::calculateInterest(void)
 {
+  try
+  {
   if (m_schedule.type() == MyMoneySchedule::TYPE_LOANPAYMENT) {
     MyMoneySplit interestSplit, amortizationSplit;
     QValueList<MyMoneySplit>::ConstIterator it_s;
     for(it_s = m_transaction.splits().begin(); it_s != m_transaction.splits().end(); ++it_s) {
-      if((*it_s).value() == MyMoneyMoney::minValue+1) {
+      if((*it_s).value() == MyMoneyMoney::autoCalc) {
         if((*it_s).action() == MyMoneySplit::ActionAmortization) {
           amortizationSplit = (*it_s);
         } else if((*it_s).action() == MyMoneySplit::ActionInterest) {
@@ -895,8 +1052,9 @@ void KEnterScheduleDialog::calculateInterest(void)
       }
 
       // we need to calculate the balance at the time the payment is due
-      MyMoneyMoney balance = acc.openingBalance();
+      MyMoneyMoney balance = MyMoneyFile::instance()->balance(acc.id(), dueDate.addDays(-1));
 
+/*
       QValueList<MyMoneyTransaction> list;
       QValueList<MyMoneyTransaction>::ConstIterator it;
       MyMoneySplit split;
@@ -915,7 +1073,8 @@ void KEnterScheduleDialog::calculateInterest(void)
           delete e;
         }
       }
-      
+*/
+
       // FIXME: for now, we only support interest calculation at the end of the period
       calc.setBep();
       // FIXME: for now, we only support periodic compounding
@@ -924,15 +1083,15 @@ void KEnterScheduleDialog::calculateInterest(void)
       calc.setPF(KMyMoneyUtils::occurenceToFrequency(m_schedule.occurence()));
       // FIXME: for now we only support compounding frequency == payment frequency
       calc.setCF(KMyMoneyUtils::occurenceToFrequency(m_schedule.occurence()));
-      
-      calc.setPv(balance.value());
-      calc.setIr(static_cast<FCALC_DOUBLE> (acc.interestRate(dueDate).abs().value()) / 100.0);
-      calc.setPmt(acc.periodicPayment().value());
 
-      MyMoneyMoney interest(calc.interestDue()/100.0), amortization;
+      calc.setPv(balance.toDouble());
+      calc.setIr(static_cast<FCALC_DOUBLE> (acc.interestRate(dueDate).abs().toDouble()));
+      calc.setPmt(acc.periodicPayment().toDouble());
+
+      MyMoneyMoney interest(calc.interestDue()), amortization;
       interest = interest.abs();    // make sure it's positive for now
       amortization = acc.periodicPayment() - interest;
-      
+
       if(acc.accountType() == MyMoneyAccount::AssetLoan) {
         interest = -interest;
         amortization = -amortization;
@@ -944,6 +1103,13 @@ void KEnterScheduleDialog::calculateInterest(void)
       m_transaction.modifySplit(interestSplit);
     }
   }
+  }
+  catch (MyMoneyException* e)
+  {
+    KMessageBox::detailedError(this, i18n("Unable to load schedule details"), e->what());
+    delete e;
+    reject();
+  }
 }
 
 bool KEnterScheduleDialog::checkDateInPeriod(const QDate& date)
@@ -994,9 +1160,14 @@ bool KEnterScheduleDialog::checkDateInPeriod(const QDate& date)
 
   if (date < firstDate || date > nextDate)
   {
-    KMessageBox::error(this, i18n("The date must lie in the range %1 to %2").arg(firstDate.addDays(1).toString()).arg(nextDate.toString()));
-    return false;
+    // let the user continue anyway as the calculation is not always correct
+    // proper fix will be available in 0.7/0.9.
+    if (KMessageBox::warningContinueCancel(this, i18n("The date must lie in the range %1 to %2").arg(firstDate.addDays(1).toString()).arg(nextDate.toString())) != KMessageBox::Continue)
+      return false;
   }
 
   return true;
 }
+// vim:cin:si:ai:et:ts=2:sw=2:
+
+#include "kenterscheduledialog.moc"
