@@ -22,6 +22,7 @@
 
 #include <qlistview.h>
 #include <qdatetime.h>
+#include <qmap.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -34,7 +35,7 @@
 #include "../mymoney/mymoneytransactionfilter.h"
 #include "../mymoney/mymoneyobserver.h"
 
-#include "kfindtransactiondlgdecl.h"
+#include "../dialogs/kfindtransactiondlgdecl.h"
 
 class QListView;
 class QListViewItem;
@@ -93,6 +94,9 @@ public:
     * transaction passed as @p transaction.
     *
     * @param transaction pointer to a transaction
+    * @param match identifies the number of the matching split within the transaction
+    *              should be used to extract the account information. 0 will use
+    *              the first matching split.
     *
     * @return const QCString containing the account's id.
     */
@@ -114,7 +118,7 @@ public:
     *       is not shown in the find transaction dialog but the method
     *       must be provided as part of the IMyMoneyRegisterParent interface.
     */
-  const MyMoneyMoney balance(const int /* idx */) const { return 0; };
+  const MyMoneyMoney balance(const int idx) const;
 
   /**
     * This method is called by the engine whenever a required notification
@@ -123,14 +127,17 @@ public:
     * object.
     */
   void update(const QCString& id);
-  
+
+  virtual bool eventFilter( QObject *o, QEvent *e );
+
 public slots:
   void slotUpdateSelections(void);
   void slotReset(void);
 
 protected:
   void resizeEvent(QResizeEvent*);
-  
+  bool focusNextPrevChild(bool next);
+
 protected slots:
   void slotDateRangeChanged(int);
   void slotDateChanged(void);
@@ -140,14 +147,14 @@ protected slots:
 
   void slotSelectAllPayees(void);
   void slotDeselectAllPayees(void);
-  
+
   void slotNrSelected(void);
   void slotNrRangeSelected(void);
 
   void slotSearch(void);
 
   void slotRefreshView(void);
-    
+
   /**
     *
     */
@@ -157,14 +164,19 @@ protected slots:
   void slotPreviousTransaction(void);
   void slotSelectTransaction(void);
   void slotSelectTransaction(const QCString& id);
-  
-private slots:
+
   void slotRightSize(void);
-  
+
+  /**
+    * This slot opens the detailed help page in khelpcenter. The
+    * anchor for the information is taken from m_helpAnchor.
+    */
+  void slotShowHelp(void);
+
 signals:
   void transactionSelected(const QCString& accountId, const QCString& transactionId);
-  
-private:
+
+protected:
   enum opTypeE {
     addAccountToFilter = 0,
     addCategoryToFilter,
@@ -177,10 +189,14 @@ private:
   void setupAmountPage(void);
   void setupPayeesPage(void);
   void setupDetailsPage(void);
-    
-  void selectAllItems(QListView* view, const bool state);  
+
+  void setupFilter(void);
+
+  void selectAllItems(QListView* view, const bool state);
   void selectAllSubItems(QListViewItem* item, const bool state);
-  
+  void selectItems(QListView* view, const QCStringList& list, const bool state);
+  void selectSubItems(QListViewItem* item, const QCStringList& list, const bool state);
+
   void readConfig(void);
   void writeConfig(void);
 
@@ -189,7 +205,7 @@ private:
     * found in the engine.
     */
   void loadPayees(void);
-  
+
   /**
     * This method returns information about the selection state
     * of the items in the m_accountsView.
@@ -204,11 +220,11 @@ private:
   const bool allItemsSelected(const QListView* view) const;
   const bool allItemsSelected(const QListViewItem *item) const;
 
-  void scanCheckListItems(QListView* view, const opTypeE op);
-  void scanCheckListItems(QListViewItem* item, const opTypeE op);
+  void scanCheckListItems(const QListView* view, const opTypeE op);
+  void scanCheckListItems(const QListViewItem* item, const opTypeE op);
   void addItemToFilter(const opTypeE op, const QCString& id);
 
-private:
+protected:
   QDate                m_startDates[dateOptionCount];
   QDate                m_endDates[dateOptionCount];
 
@@ -231,7 +247,8 @@ private:
   KMyMoneyTransaction*            m_transactionPtr;
 
   MyMoneyTransactionFilter        m_filter;
-  
+
+  QMap<QWidget*, QString>         m_helpAnchor;
 };
 
 #endif
