@@ -53,9 +53,6 @@
 KImportDlg::KImportDlg(QWidget *parent, const char * name)
   : KImportDlgDecl(parent, name, TRUE)
 {
-  QString filename = KGlobal::dirs()->findResource("appdata", "pics/dlg_qif_import.png");
-  m_qpixmaplabel->setPixmap(QPixmap(filename));
-
   // Set all the last used options
   readConfig();
 
@@ -131,6 +128,7 @@ void KImportDlg::readConfig(void)
   KConfig *kconfig = KGlobal::config();
   kconfig->setGroup("Last Use Settings");
   m_qlineeditFile->setText(kconfig->readEntry("KImportDlg_LastFile"));
+  m_payeeCreation->setChecked(kconfig->readBoolEntry("KImportDlg_CreatePayees",false));
 }
 
 void KImportDlg::writeConfig(void)
@@ -139,6 +137,7 @@ void KImportDlg::writeConfig(void)
   kconfig->setGroup("Last Use Settings");
   kconfig->writeEntry("KImportDlg_LastFile", m_qlineeditFile->text());
   kconfig->writeEntry("KImportDlg_LastProfile", m_profileComboBox->currentText());
+  kconfig->writeEntry("KImportDlg_CreatePayees",m_payeeCreation->isChecked());
   kconfig->sync();
 }
 
@@ -267,3 +266,5 @@ void KImportDlg::addCategories(QStringList& strList, const QCString& id, const Q
   }
 }
 
+
+#include "kimportdlg.moc"
