@@ -2,7 +2,9 @@
                           knewaccountdlg.cpp
                              -------------------
     copyright            : (C) 2000 by Michael Edwardes
+                               2004 by Thomas Baumgart
     email                : mte@users.sourceforge.net
+                           ipwizard@users.sourceforge.net
  ***************************************************************************/
 
 /***************************************************************************
@@ -24,6 +26,9 @@
 #include <qtooltip.h>
 #include <qcheckbox.h>
 #include <qtimer.h>
+#include <qtabwidget.h>
+#include <qbuttongroup.h>
+#include <qradiobutton.h>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
@@ -34,13 +39,7 @@
 #include <kcombobox.h>
 #include <klistview.h>
 #include <kconfig.h>
-
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-
 #include <kiconloader.h>
 #include <kguiitem.h>
 #include <kpushbutton.h>
@@ -51,6 +50,10 @@
 #include "knewaccountdlg.h"
 #include "../widgets/kmymoneyedit.h"
 #include "../widgets/kmymoneydateinput.h"
+#include "../widgets/kmymoneycurrencyselector.h"
+#include "../widgets/kmymoneyequity.h"
+#include "../widgets/kmymoneyaccountselector.h"
+
 #include "../mymoney/mymoneyexception.h"
 #include "../mymoney/mymoneyfile.h"
 #include "../dialogs/knewbankdlg.h"
@@ -58,6 +61,9 @@
 #include "../views/kmymoneyfile.h"
 #include "../kmymoneyutils.h"
 
+#define TAB_GENERAL      0
+#define TAB_TAX          1
+
 KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bool categoryEditor, QWidget *parent,
     const char *name, const char *title)
   : KNewAccountDlgDecl(parent,name,true),
@@ -66,12 +72,8 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
     m_categoryEditor(categoryEditor),
     m_isEditing(isEditing)
 {
-  QCString pngFile = ( (categoryEditor) ? "pics/dlg_edit_category.png" : "pics/dlg_new_account.png" );
   QString columnName = ( (categoryEditor) ? i18n("Categories") : i18n("Accounts") );
 
-  QString filename = KGlobal::dirs()->findResource("appdata", pngFile);
-  m_qpixmaplabel->setPixmap(QPixmap(filename));
-
   m_qlistviewParentAccounts->setRootIsDecorated(true);
   m_qlistviewParentAccounts->setAllColumnsShowFocus(true);
   m_qlistviewParentAccounts->addColumn(columnName);
@@ -81,64 +83,75 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   m_qlistviewParentAccounts->setEnabled(false);
   // never show the horizontal scroll bar
   m_qlistviewParentAccounts->setHScrollBarMode(QScrollView::AlwaysOff);
-  
+
   m_subAccountLabel->setText(i18n("Is a sub account"));
-  
+
   m_qlistviewParentAccounts->header()->setFont(KMyMoneyUtils::headerFont());
 
   accountNameEdit->setText(account.name());
   descriptionEdit->setText(account.description());
 
   typeCombo->setEnabled(true);
-  
+  MyMoneyFile *file = MyMoneyFile::instance();
+
   if (categoryEditor)
   {
+
     m_qlistviewParentAccounts->setEnabled(true);
     startDateEdit->setEnabled(false);
-    startBalanceEdit->setEnabled(false);
     accountNoEdit->setEnabled(false);
-    
-    // m_qcomboboxInstitutions->setEnabled(false);
-    // m_qbuttonNew->setEnabled(false);
-    m_qcomboboxInstitutions->hide();
-    m_qbuttonNew->hide();
+
+    m_institutionBox->hide();
+    m_qcheckboxNoVat->hide();
 
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Income));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Expense));
-    
+
     // Hardcoded but acceptable
     switch (account.accountType())
     {
       case MyMoneyAccount::Income:
         typeCombo->setCurrentItem(0);
         break;
+
       case MyMoneyAccount::Expense:
+      default:
         typeCombo->setCurrentItem(1);
         break;
-      default:
-        typeCombo->setCurrentItem(0);
     }
     if (m_isEditing)
     {
       typeCombo->setEnabled(false);
     }
     m_qcheckboxPreferred->hide();
+    m_currency->setEnabled(false);
+    m_equityText->hide();
+    m_equity->hide();
+
+    m_qcheckboxTax->setChecked(account.value("Tax") == "Yes");
+    loadVatAccounts();
   }
   else
   {
+    QWidget* tab = m_tab->page(TAB_TAX);
+    if(tab) {
+      m_tab->removePage(tab);
+    }
+
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Checkings));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Savings));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Cash));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::CreditCard));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Loan));
+    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Investment));
+    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Asset));
+    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Liability));
+    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Stock));
 /*
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::CertificateDep));
-    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Investment));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::MoneyMarket));
     typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Currency));
 */
-    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Asset));
-    typeCombo->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Liability));
 
     // Hardcoded but acceptable
     switch (account.accountType())
@@ -159,13 +172,23 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
       case MyMoneyAccount::Loan:
         typeCombo->setCurrentItem(4);
         break;
+      case MyMoneyAccount::Investment:
+        typeCombo->setCurrentItem(5);
+        break;
+      case MyMoneyAccount::Asset:
+        typeCombo->setCurrentItem(6);
+        break;
+      case MyMoneyAccount::Liability:
+        typeCombo->setCurrentItem(7);
+        break;
+      case MyMoneyAccount::Stock:
+        m_institutionBox->hide();
+        typeCombo->setCurrentItem(8);
+        break;
 /*
       case MyMoneyAccount::CertificateDep:
         typeCombo->setCurrentItem(5);
         break;
-      case MyMoneyAccount::Investment:
-        typeCombo->setCurrentItem(6);
-        break;
       case MyMoneyAccount::MoneyMarket:
         typeCombo->setCurrentItem(7);
         break;
@@ -173,36 +196,46 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
         typeCombo->setCurrentItem(8);
         break;
 */
-      case MyMoneyAccount::Asset:
-        typeCombo->setCurrentItem(5);
-        break;
-      case MyMoneyAccount::Liability:
-        typeCombo->setCurrentItem(6);
-        break;
     }
 
     if(!m_account.openingDate().isValid())
       m_account.setOpeningDate(QDate::currentDate());
-      
+
     startDateEdit->setDate(m_account.openingDate());
-    startBalanceEdit->setText(account.openingBalance().formatMoney());
     accountNoEdit->setText(account.number());
     m_qcheckboxPreferred->setChecked(account.value("PreferredAccount") == "Yes");
+    m_qcheckboxNoVat->setChecked(account.value("NoVat") == "Yes");
+
+    if(account.accountType() == MyMoneyAccount::Stock) {
+      m_equity->loadEquity(account.currencyId());
+    } else {
+      m_currency->setSecurity(file->currency(account.currencyId()));
+    }
 
     // we do not allow to change the account type once an account
-    // was created    
+    // was created. Same applies to currency.
     if (m_isEditing)
     {
       typeCombo->setEnabled(false);
+      m_currency->setEnabled(false);
     }
+    if(m_account.accountType() == MyMoneyAccount::Stock) {
+      typeCombo->setEnabled(false);
+      m_qcheckboxPreferred->hide();
+      m_currencyText->hide();
+      m_currency->hide();
+    } else {
+      m_equityText->hide();
+      m_equity->hide();
+    }
+
+    m_qcheckboxTax->hide();
   }
 
   // Load the institutions
   // then the accounts
   QString institutionName;
 
-  MyMoneyFile *file = MyMoneyFile::instance();
-
   try
   {
     if (m_isEditing && !account.institutionId().isEmpty())
@@ -218,6 +251,9 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
 
   initParentWidget(account.parentAccountId(), account.id());
 
+  if(m_account.accountType() == MyMoneyAccount::Stock)
+    m_qlistviewParentAccounts->setEnabled(false);
+
   if (!categoryEditor)
     loadInstitutions(institutionName);
 
@@ -247,7 +283,21 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
   connect(m_qbuttonNew, SIGNAL(clicked()), this, SLOT(slotNewClicked()));
   connect(typeCombo, SIGNAL(activated(const QString&)),
     this, SLOT(slotAccountTypeChanged(const QString&)));
-    
+
+  connect(accountNameEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
+  // connect(m_equity, SIGNAL(equityChanged(const QCString&)), this, SLOT(slotCheckFinished()));
+  connect(m_equity, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
+
+  connect(m_vatCategory, SIGNAL(toggled(bool)), this, SLOT(slotVatChanged(bool)));
+  connect(m_vatAssignment, SIGNAL(toggled(bool)), this, SLOT(slotVatAssignmentChanged(bool)));
+  connect(m_vatCategory, SIGNAL(toggled(bool)), this, SLOT(slotCheckFinished()));
+  connect(m_vatAssignment, SIGNAL(toggled(bool)), this, SLOT(slotCheckFinished()));
+  connect(m_vatRate, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckFinished()));
+  connect(m_vatAccount, SIGNAL(stateChanged()), this, SLOT(slotCheckFinished()));
+
+  m_vatCategory->setChecked(false);
+  m_vatAssignment->setChecked(false);
+
   // make sure our account does not have an id and no parent assigned
   // and certainly no children in case we create a new account
   if(!m_isEditing) {
@@ -265,7 +315,33 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
         type = MyMoneyAccount::Checkings;
       slotAccountTypeChanged(KMyMoneyUtils::accountTypeToString(type));
     }
+  } else {
+    if(categoryEditor) {
+      if(!m_account.value("VatRate").isEmpty()) {
+        m_vatCategory->setChecked(true);
+        m_vatRate->setValue(MyMoneyMoney(m_account.value("VatRate"))*MyMoneyMoney(100,1));
+      } else {
+        if(!m_account.value("VatAccount").isEmpty()) {
+          QCString accId = m_account.value("VatAccount").latin1();
+          try {
+            // make sure account exists
+            MyMoneyFile::instance()->account(accId);
+            m_vatAssignment->setChecked(true);
+            m_vatAccount->setSelected(accId);
+            m_grossAmount->setChecked(true);
+            if(m_account.value("VatAmount") == "Net")
+              m_netAmount->setChecked(true);
+          } catch(MyMoneyException *e) {
+            delete e;
+          }
+        }
+      }
+    }
   }
+  slotVatChanged(m_vatCategory->isChecked());
+  slotVatAssignmentChanged(m_vatAssignment->isChecked());
+  slotCheckFinished();
+
   // using a timeout is the only way, I got the 'ensureItemVisible'
   // working when creating the dialog. I assume, this
   // has something to do with the delayed update of the display somehow.
@@ -288,7 +364,7 @@ KNewAccountDlg::~KNewAccountDlg()
 void KNewAccountDlg::okClicked()
 {
   MyMoneyFile* file = MyMoneyFile::instance();
-  
+
   QString accountNameText = accountNameEdit->text();
   if (accountNameText.isEmpty())
   {
@@ -341,7 +417,7 @@ void KNewAccountDlg::okClicked()
     if(acctype == MyMoneyAccount::Loan
     && file->accountGroup(parent.accountType()) == MyMoneyAccount::Asset)
       acctype = MyMoneyAccount::AssetLoan;
-      
+
     if(!file->nameToAccount(accountNameText).isEmpty()
     && (file->nameToAccount(accountNameText) != m_account.id())) {
       KMessageBox::error(this, i18n("Account with that name already exists."));
@@ -363,12 +439,51 @@ void KNewAccountDlg::okClicked()
 
   if (!m_categoryEditor)
   {
-    m_account.setOpeningBalance(startBalanceEdit->getMoneyValue());
     m_account.setOpeningDate(startDateEdit->getQDate());
+    if(m_account.accountType() == MyMoneyAccount::Stock) {
+      m_account.setCurrencyId(m_equity->id());
+    } else {
+      m_account.setCurrencyId(m_currency->security().id());
+    }
+
     if(m_qcheckboxPreferred->isChecked())
       m_account.setValue("PreferredAccount", "Yes");
     else
       m_account.deletePair("PreferredAccount");
+    if(m_qcheckboxNoVat->isChecked())
+      m_account.setValue("NoVat", "Yes");
+    else
+      m_account.deletePair("NoVat");
+  }
+  else
+  {
+    KConfig *kconfig = KGlobal::config();
+    kconfig->setGroup("List Options");
+    if(kconfig->readBoolEntry("HideUnusedCategory", false) == true) {
+      KMessageBox::information(this, i18n("You have selected to suppress the display of unused categories in the KMyMoney configuration dialog. The category you just created will therefore only be shown if it is used. Otherwise, it will be hidden in the accounts/categories view."), i18n("Hidden categories"), "NewHiddenCategory");
+    }
+  }
+
+  if (m_categoryEditor)
+  {
+    if ( m_qcheckboxTax->isChecked())
+      m_account.setValue("Tax", "Yes");
+    else
+      m_account.deletePair("Tax");
+
+    m_account.deletePair("VatAccount");
+    m_account.deletePair("VatAmount");
+    m_account.deletePair("VatRate");
+
+    if(m_vatCategory->isChecked()) {
+      m_account.setValue("VatRate", (m_vatRate->value().abs() / MyMoneyMoney(100,1)).toString());
+    } else {
+      if(m_vatAssignment->isChecked()) {
+        m_account.setValue("VatAccount", m_vatAccount->selectedAccounts().first());
+        if(m_netAmount->isChecked())
+          m_account.setValue("VatAmount", "Net");
+      }
+    }
   }
 
   accept();
@@ -399,8 +514,11 @@ const MyMoneyAccount& KNewAccountDlg::parentAccount(void)
       case MyMoneyAccount::Expense:
         m_parentAccount = file->expense();
         break;
+      case MyMoneyAccount::Equity:
+        m_parentAccount = file->equity();
+        break;
       default:
-        qDebug("Seems we have an account that hasn't been mapped to the top four");
+        qDebug("Seems we have an account that hasn't been mapped to the top five");
         if(m_categoryEditor)
           m_parentAccount = file->income();
         else
@@ -418,11 +536,12 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
   MyMoneyAccount assetAccount = file->asset();
   MyMoneyAccount expenseAccount = file->expense();
   MyMoneyAccount incomeAccount = file->income();
+  MyMoneyAccount equityAccount = file->equity();
 
   m_parentItem = 0;
   m_accountItem = 0;
 
-  // Determine the parent account  
+  // Determine the parent account
   try
   {
     m_parentAccount = file->account(parentId);
@@ -446,7 +565,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
   groupType = file->accountGroup(type);
 
   m_qlistviewParentAccounts->clear();
-    
+
   // Now scan all 4 account roots to load the list and mark the parent
   try
   {
@@ -466,7 +585,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
           m_parentItem = assetTopLevelAccount;
 
         assetTopLevelAccount->setOpen(true);
-        
+
         for ( QCStringList::ConstIterator it = assetAccount.accountList().begin();
               it != assetAccount.accountList().end();
               ++it )
@@ -491,7 +610,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
         }
       }
 
-      if(groupType == MyMoneyAccount::Liability) {      
+      if(groupType == MyMoneyAccount::Liability) {
         // Liability
         KAccountListItem *liabilityTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
                           liabilityAccount);
@@ -505,7 +624,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
           m_parentItem = liabilityTopLevelAccount;
 
         liabilityTopLevelAccount->setOpen(true);
-        
+
         for ( QCStringList::ConstIterator it = liabilityAccount.accountList().begin();
               it != liabilityAccount.accountList().end();
               ++it )
@@ -546,7 +665,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
           m_parentItem = incomeTopLevelAccount;
 
         incomeTopLevelAccount->setOpen(true);
-        
+
         for ( QCStringList::ConstIterator it = incomeAccount.accountList().begin();
               it != incomeAccount.accountList().end();
               ++it )
@@ -570,7 +689,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
           }
         }
       }
-      
+
       if(groupType == MyMoneyAccount::Expense) {
         // Expense
         KAccountListItem *expenseTopLevelAccount = new KAccountListItem(m_qlistviewParentAccounts,
@@ -585,7 +704,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
           m_parentItem = expenseTopLevelAccount;
 
         expenseTopLevelAccount->setOpen(true);
-        
+
         for ( QCStringList::ConstIterator it = expenseAccount.accountList().begin();
               it != expenseAccount.accountList().end();
               ++it )
@@ -625,7 +744,7 @@ void KNewAccountDlg::initParentWidget(QCString parentId, const QCString& account
     m_qlistviewParentAccounts->setOpen(m_parentItem, true);
     m_qlistviewParentAccounts->setSelected(m_parentItem, true);
   }
-  
+
   m_qlistviewParentAccounts->setEnabled(true);
 }
 
@@ -633,7 +752,7 @@ void KNewAccountDlg::showSubAccounts(QCStringList accounts, KAccountListItem *pa
                                      const QCString& parentId, const QCString& accountId)
 {
   MyMoneyFile *file = MyMoneyFile::instance();
-  
+
   for ( QCStringList::ConstIterator it = accounts.begin(); it != accounts.end(); ++it )
   {
     KAccountListItem *accountItem  = new KAccountListItem(parentItem,
@@ -685,6 +804,24 @@ void KNewAccountDlg::slotSelectionChanged(QListViewItem *item)
   }
 }
 
+void KNewAccountDlg::loadVatAccounts(void)
+{
+  QValueList<MyMoneyAccount> list = MyMoneyFile::instance()->accountList();
+  QValueList<MyMoneyAccount>::Iterator it;
+  QCStringList loadListExpense;
+  QCStringList loadListIncome;
+  for(it = list.begin(); it != list.end(); ++it) {
+    if(!(*it).value("VatRate").isEmpty()) {
+      if((*it).accountType() == MyMoneyAccount::Expense)
+        loadListExpense += (*it).id();
+      else
+        loadListIncome += (*it).id();
+    }
+  }
+  m_vatAccount->loadList(i18n("Income"), loadListIncome, true);
+  m_vatAccount->loadList(i18n("Expense"), loadListExpense, false);
+}
+
 void KNewAccountDlg::loadInstitutions(const QString& name)
 {
   int id=-1, counter=0;
@@ -745,7 +882,7 @@ void KNewAccountDlg::slotAccountTypeChanged(const QString& typeStr)
   MyMoneyAccount::accountTypeE type;
   MyMoneyAccount::accountTypeE oldType;
   MyMoneyFile* file = MyMoneyFile::instance();
-  
+
   type = KMyMoneyUtils::stringToAccountType(typeStr);
   try {
     oldType = m_account.accountType();
@@ -775,3 +912,45 @@ void KNewAccountDlg::slotAccountTypeChanged(const QString& typeStr)
     qWarning("Unexpected exception in KNewAccountDlg::slotAccountTypeChanged()");
   }
 }
+
+void KNewAccountDlg::slotCheckFinished(void)
+{
+  bool showButton = true;
+
+  if(accountNameEdit->text().length() == 0) {
+    showButton = false;
+  }
+
+  if(m_account.accountType() == MyMoneyAccount::Stock) {
+    if(m_equity->text().length() == 0) {
+      showButton = false;
+    }
+  }
+
+  if(m_vatCategory->isChecked() && m_vatRate->value() <= MyMoneyMoney(0)) {
+    showButton = false;
+  } else {
+    if(m_vatAssignment->isChecked() && m_vatAccount->selectedAccounts().isEmpty())
+      showButton = false;
+  }
+  createButton->setEnabled(showButton);
+}
+
+void KNewAccountDlg::slotVatChanged(bool state)
+{
+  if(state) {
+    m_vatCategoryFrame->show();
+    m_vatAssignmentFrame->hide();
+  } else {
+    m_vatCategoryFrame->hide();
+    m_vatAssignmentFrame->show();
+  }
+}
+
+void KNewAccountDlg::slotVatAssignmentChanged(bool state)
+{
+  m_vatAccount->setEnabled(state);
+  m_amountGroup->setEnabled(state);
+}
+
+#include "knewaccountdlg.moc"
