@@ -26,10 +26,12 @@
 #include <qcheckbox.h>
 #include <qheader.h>
 #include <qlabel.h>
+#include <qradiobutton.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include <kapplication.h>
 #include <klocale.h>
 #include <klistbox.h>
 #include <kcombobox.h>
@@ -38,6 +40,7 @@
 #include <klineedit.h>
 #include <kconfig.h>
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -47,12 +50,17 @@
 #include "../widgets/kmymoneycombo.h"
 #include "../widgets/kmymoneycategory.h"
 #include "../widgets/kmymoneypayee.h"
+#include "../widgets/kmymoneycurrencyselector.h"
+
 #include "../mymoney/mymoneyinstitution.h"
 #include "../mymoney/mymoneyfile.h"
+
 #include "../kmymoneyutils.h"
+
 #include "ieditscheduledialog.h"
 #include "knewaccountwizard.h"
 #include "knewloanwizard.h"
+#include "kcurrencyeditdlg.h"
 
 KNewAccountWizard::KNewAccountWizard(QWidget *parent, const char *name )
   : KNewAccountWizardDecl(parent,name,true),
@@ -61,6 +69,16 @@ KNewAccountWizard::KNewAccountWizard(QWidget *parent, const char *name )
   // keep title of payment page
   m_accountPaymentPageTitle = title(accountPaymentPage);
 
+  // We don't need to add the default into the list (see ::help() why)
+  m_helpAnchor[institutionPage] = QString("firsttime-accwiz1");
+  m_helpAnchor[accountTypePage] = QString("firsttime-accwiz2");
+  m_helpAnchor[accountNamePage] = QString("firsttime-accwiz3");
+  m_helpAnchor[accountNumberPage] = QString("firsttime-accwiz4");
+  //m_helpAnchor[brokerageAccountPage] = QString("");
+  m_helpAnchor[accountDetailsPage] = QString("firsttime-accwiz5");
+  m_helpAnchor[accountPaymentPage] = QString("firsttime-accwiz5.1");
+  //m_helpAnchor[summaryPage] = QString("");
+
   accountListView->setRootIsDecorated(true);
   accountListView->setAllColumnsShowFocus(true);
   accountListView->addColumn(i18n("Account"));
@@ -78,19 +96,20 @@ KNewAccountWizard::KNewAccountWizard(QWidget *parent, const char *name )
   connect(m_name, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
   connect(m_payee, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
   connect(m_amount, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
-  
+
   connect(m_payee, SIGNAL(newPayee(const QString&)), this, SLOT(slotNewPayee(const QString&)));
-  
+  connect(m_currencyComboBox, SIGNAL(activated(int)), this, SLOT(slotCurrencyChanged(int)));
+  connect(m_priceButton, SIGNAL(clicked()), this, SLOT(slotPriceUpdate()));
+
   // always select the first item and show the appropriate note
   loadAccountTypes();
+  loadPaymentMethods();
+
   accountTypeListBox->setCurrentItem(0);
 
   m_name->setFocus();
 
-  // FIXME: we don't have currency support, so we hide the two widgets
-  // that have been prepared for this support
-  m_currencyLabel->hide();
-  m_currencyComboBox->hide();
+  slotCurrencyChanged(0);
 }
 
 KNewAccountWizard::~KNewAccountWizard()
@@ -99,42 +118,40 @@ KNewAccountWizard::~KNewAccountWizard()
 
 void KNewAccountWizard::next()
 {
+  openingBalanceLabel->setText(i18n("What is the opening balance and date of this account?"));
+
+  setAppropriate(accountDetailsPage, true);
+  setAppropriate(accountPaymentPage, false);
+  setAppropriate(brokerageAccountPage, false);
+
   switch(m_accountType) {
     case MyMoneyAccount::Cash:
     case MyMoneyAccount::Asset:
-      if(indexOf(accountPaymentPage) != -1) {
-        removePage(accountPaymentPage);
-      }
       setAppropriate(accountNumberPage, false);
-      setFinishEnabled(accountDetailsPage, true);
+      break;
+
+    case MyMoneyAccount::Investment:
+      openingBalanceLabel->setText(i18n("What is the opening balance and date of this brokerage account?"));
+      setAppropriate(brokerageAccountPage, true);
+      setAppropriate(accountNumberPage, false);
       break;
 
     case MyMoneyAccount::CreditCard:
-      if(indexOf(accountPaymentPage) == -1) {
-        loadPaymentMethods();
-        addPage(accountPaymentPage, m_accountPaymentPageTitle);
-      }
       setAppropriate(accountPaymentPage, true);
-      setFinishEnabled(accountPaymentPage, true);
-      setFinishEnabled(accountDetailsPage, false);
       break;
 
     default:
       setAppropriate(accountNumberPage, institutionComboBox->currentText() != "");
-      if(indexOf(accountPaymentPage) != -1) {
-        removePage(accountPaymentPage);
-      }
-      setFinishEnabled(accountDetailsPage, true);
       break;
   }
 
   if(currentPage() == accountTypePage && m_accountType == MyMoneyAccount::Loan) {
     int rc;
-    
+
     // we logically never come back to this wizard, so we hide it.
     hide();
 
-    KNewLoanWizard* loanWizard = new KNewLoanWizard(this);
+    KNewLoanWizard* loanWizard = new KNewLoanWizard(0);
     if((rc = loanWizard->exec()) == QDialog::Accepted) {
       m_account = loanWizard->account();
       // copy relevant data into my widgets so that
@@ -147,6 +164,11 @@ void KNewAccountWizard::next()
       preferredAccount->setChecked(false);
 
       m_schedule = loanWizard->schedule();
+      if(loanWizard->initialPaymentDate().isValid()
+      && !loanWizard->initialPaymentAccount().isEmpty()) {
+        m_account.setValue("kmm-loan-payment-date", loanWizard->initialPaymentDate().toString(Qt::ISODate));
+        m_account.setValue("kmm-loan-payment-acc", loanWizard->initialPaymentAccount());
+      }
     }
     delete loanWizard;
 
@@ -154,10 +176,13 @@ void KNewAccountWizard::next()
       accept();
     else
       reject();
-      
+
   } else {
+    if(currentPage() == brokerageAccountPage) {
+      setAppropriate(accountDetailsPage, brokerageYesButton->isChecked());
+    }
     KNewAccountWizardDecl::next();
-    
+
     // setup the availability of widgets on the selected page
     slotCheckPageFinished();
   }
@@ -172,18 +197,35 @@ void KNewAccountWizard::accept()
   m_account.setAccountType(m_accountType);
   m_account.setName(accountName->text());
   m_account.setNumber(accountNumber->text());
+  m_account.setCurrencyId(m_currencyComboBox->security().id());
+
+  m_brokerage.setAccountType(MyMoneyAccount::Checkings);
+  m_brokerage.setCurrencyId(m_currencyComboBox->security().id());
+  m_brokerage.setNumber(accountNumber->text());
 
   if(!institutionComboBox->currentText().isEmpty()) {
     QValueList<MyMoneyInstitution> list;
     QValueList<MyMoneyInstitution>::ConstIterator it;
 
     list = MyMoneyFile::instance()->institutionList();
-    for(it = list.begin(); it != list.end(); ++it)
-      if((*it).name() == institutionComboBox->currentText())
+    for(it = list.begin(); it != list.end(); ++it) {
+      if((*it).name() == institutionComboBox->currentText()) {
         m_account.setInstitutionId((*it).id());
+        m_brokerage.setInstitutionId((*it).id());
+      }
+    }
   }
-  m_account.setOpeningBalance(MyMoneyMoney(openingBalance->text()));
-  m_account.setOpeningDate(openingDate->getQDate());
+  if(m_accountType == MyMoneyAccount::Investment) {
+    if(brokerageYesButton->isChecked()) {
+      m_brokerage.setName(m_account.name()+i18n(" (Brokerage)"));
+    }
+    m_brokerage.setOpeningBalance(MyMoneyMoney(openingBalance->text()));
+    m_brokerage.setOpeningDate(openingDate->getQDate());
+  } else {
+    m_account.setOpeningBalance(MyMoneyMoney(openingBalance->text()));
+    m_account.setOpeningDate(openingDate->getQDate());
+  }
+
   if(preferredAccount->isChecked())
     m_account.setValue("PreferredAccount", "Yes");
 
@@ -222,7 +264,7 @@ void KNewAccountWizard::accept()
       m_payee->setFocus();
       return;
     }
-    
+
     KAccountListItem *item = (KAccountListItem*)accountListView->selectedItem();
     if (!item)
     {
@@ -233,7 +275,7 @@ void KNewAccountWizard::accept()
 
     // Create the schedule transaction
     QCString payeeId;
-  
+
     try
     {
       payeeId = MyMoneyFile::instance()->payeeByName(m_payee->text()).id();
@@ -245,9 +287,9 @@ void KNewAccountWizard::accept()
       payeeId = payee.id();
       delete e;
     }
-    
+
     MyMoneySplit s1, s2;
-    s1.setValue(m_amount->getMoneyValue());
+    s1.setValue(m_amount->value());
     s2.setValue(-s1.value());
     s1.setAction(MyMoneySplit::ActionTransfer);
     s2.setAction(MyMoneySplit::ActionTransfer);
@@ -261,7 +303,7 @@ void KNewAccountWizard::accept()
     t.addSplit(s2);
 
     MyMoneySchedule::paymentTypeE paymentType;
-    
+
     switch (m_method->currentItem())
     {
       case 0:
@@ -302,23 +344,21 @@ void KNewAccountWizard::accept()
 int KNewAccountWizard::exec()
 {
   int rc;
-  
+
   accountListView->header()->setFont(KMyMoneyUtils::headerFont());
 
-  // currently, we don't have help :-(
-  setHelpEnabled(institutionPage, false);
-  setHelpEnabled(accountTypePage, false);
-  setHelpEnabled(accountNamePage, false);
-  setHelpEnabled(accountNumberPage, false);
-  setHelpEnabled(accountDetailsPage, false);
-  setHelpEnabled(accountPaymentPage, false);
+  // currently, we don't have help for these pages :-(
+  setHelpEnabled(brokerageAccountPage, false);
+  setHelpEnabled(summaryPage, false);
 
   setFinishEnabled(institutionPage, false);
   setFinishEnabled(accountTypePage, false);
   setFinishEnabled(accountNamePage, false);
   setFinishEnabled(accountNumberPage, false);
   setFinishEnabled(accountDetailsPage, false);
-  setFinishEnabled(accountPaymentPage, true);
+  setFinishEnabled(accountPaymentPage, false);
+  setFinishEnabled(brokerageAccountPage, false);
+  setFinishEnabled(summaryPage, true);
 
   // always start on first page
   showPage(institutionPage);
@@ -334,6 +374,7 @@ int KNewAccountWizard::exec()
 
   // reset everything else if not preset
   accountNumber->setText(QString());
+  m_brokerage = MyMoneyAccount();
 
   rc = KNewAccountWizardDecl::exec();
 
@@ -421,13 +462,13 @@ void KNewAccountWizard::loadAccountTypes(void)
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::CreditCard));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Cash));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Loan));
-  
+
 /*
   // accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::AssetLoan));
-  accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Investment));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::MoneyMarket));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Currency));
 */
+  accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Investment));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Asset));
   accountTypeListBox->insertItem(KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Liability));
 }
@@ -444,8 +485,8 @@ void KNewAccountWizard::slotAccountType(const QString& sel)
 
   if(sel == KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Checkings)) {
     txt += i18n(
-      "Use the checkings account type to manage "
-      "activities on your checkings account e.g. payments, cheques and cash card "
+      "Use the checking account type to manage "
+      "activities on your checking account e.g. payments, checks and cash card "
       "purchases."
     );
     m_accountType = MyMoneyAccount::Checkings;
@@ -486,7 +527,7 @@ void KNewAccountWizard::slotAccountType(const QString& sel)
       "like mortgages you should create a loan account."
     );
     m_accountType = MyMoneyAccount::Liability;
-    
+
   } else if(sel == KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Loan)) {
     txt += i18n(
       "Use the loan account type to manage amortization loans "
@@ -501,6 +542,11 @@ void KNewAccountWizard::slotAccountType(const QString& sel)
     );
     m_accountType = MyMoneyAccount::Loan;
 */
+  } else if(sel == KMyMoneyUtils::accountTypeToString(MyMoneyAccount::Investment)) {
+    txt += i18n(
+      "Use the investment account to manage your stock, mutual fund and other investments."
+    );
+    m_accountType = MyMoneyAccount::Investment;
   } else {
     txt += i18n("Explanation is not yet available! UnknownAccountType will be set");
     m_accountType = MyMoneyAccount::UnknownAccountType;
@@ -529,7 +575,7 @@ void KNewAccountWizard::setOpeningDate(const QDate& date)
 void KNewAccountWizard::setAccountType(const MyMoneyAccount::accountTypeE type)
 {
   int i;
-  
+
   for(i = accountTypeListBox->count()-1; i > 0; --i) {
     if(accountTypeListBox->text(i) == KMyMoneyUtils::accountTypeToString(type))
       break;
@@ -543,7 +589,7 @@ void KNewAccountWizard::loadPaymentMethods()
   m_method->insertItem(i18n("Direct Debit"));
   m_method->insertItem(i18n("Direct Deposit"));
   m_method->insertItem(i18n("Manual Deposit"));
-  m_method->insertItem(i18n("Write Cheque"));
+  m_method->insertItem(i18n("Write Check"));
   m_method->insertItem(i18n("Other"));
 }
 
@@ -559,11 +605,11 @@ void KNewAccountWizard::slotCheckPageFinished(void)
 {
   nextButton()->setEnabled(true);
   finishButton()->setEnabled(true);
-  
+
   if(currentPage() == accountNamePage) {
     if(accountName->text().isEmpty())
       nextButton()->setEnabled(false);
-      
+
   } else if(currentPage() == accountPaymentPage) {
     if(reminderCheckBox->isChecked()) {
       if(m_amount->text().isEmpty()
@@ -574,6 +620,9 @@ void KNewAccountWizard::slotCheckPageFinished(void)
       }
     }
   }
+  else if(currentPage() == summaryPage) {
+    finishButton()->setDefault(true);
+  }
 }
 
 void KNewAccountWizard::slotNewPayee(const QString& payeeName)
@@ -581,3 +630,25 @@ void KNewAccountWizard::slotNewPayee(const QString& payeeName)
   KMyMoneyUtils::newPayee(this, m_payee, payeeName);
 }
 
+void KNewAccountWizard::slotCurrencyChanged(int)
+{
+  m_priceButton->setEnabled(m_currencyComboBox->security().id() != MyMoneyFile::instance()->baseCurrency().id());
+}
+
+void KNewAccountWizard::slotPriceUpdate(void)
+{
+  KCurrencyEditDlg dlg(this, "KCurrencyEditDlg");
+  dlg.slotSelectCurrency(m_currencyComboBox->security().id());
+  dlg.exec();
+}
+
+void KNewAccountWizard::help(void)
+{
+  QString anchor = m_helpAnchor[currentPage()];
+  if(anchor.isEmpty())
+    anchor = QString("firsttime-4");
+
+  kapp->invokeHelp(anchor);
+}
+
+#include "knewaccountwizard.moc"
