@@ -13,40 +13,40 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#include <kglobal.h>
-#include <klocale.h>
-#if QT_VERSION > 300
-#include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-#include <kstdguiitem.h>
-#include <kpushbutton.h>
+
+#include "kdecompat.h"
+
+// ----------------------------------------------------------------------------
+// QT Includes
 
 #include <qpixmap.h>
-#include <kmessagebox.h>
 #include <qlineedit.h>
 #include <qlabel.h>
 
+// ----------------------------------------------------------------------------
+// KDE Headers
+
+#include <kglobal.h>
+#include <klocale.h>
+#include <kstandarddirs.h>
+#include <kstdguiitem.h>
+#include <kpushbutton.h>
+#include <kmessagebox.h>
+
+#if KDE_IS_VERSION(3,1,90)
+#include <kabc/addressee.h>
+#include <kabc/stdaddressbook.h>
+#endif
+
+// ----------------------------------------------------------------------------
+// Project Includes
+
 #include "knewfiledlg.h"
 
 KNewFileDlg::KNewFileDlg(QWidget *parent, const char *name, const char *title)
   : KNewFileDlgDecl(parent,name,true)
 {
-  okBtn->setGuiItem(KStdGuiItem::ok());
-  cancelBtn->setGuiItem(KStdGuiItem::cancel());
-
-  QString filename = KGlobal::dirs()->findResource("appdata", "pics/dlg_edit_file.png");
-  QPixmap pm(filename);
-  m_qpixmaplabel->setPixmap(pm);
-
-	if (title)
-	  setCaption(title);
-
-	userNameEdit->setFocus();
-
-	connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()));
-	connect(okBtn, SIGNAL(clicked()), this, SLOT(okClicked()));
+  init(title);
 }
 
 KNewFileDlg::KNewFileDlg(QString userName, QString userStreet,
@@ -54,12 +54,6 @@ KNewFileDlg::KNewFileDlg(QString userName, QString userStreet,
   QString userEmail, QWidget *parent, const char *name, const char *title)
   : KNewFileDlgDecl(parent,name,true)
 {
-  okBtn->setGuiItem(KStdGuiItem::ok());
-  cancelBtn->setGuiItem(KStdGuiItem::cancel());
-
-  QString filename = KGlobal::dirs()->findResource("appdata", "pics/view_info.png");
-  m_qpixmaplabel->setPixmap(QPixmap(filename));
-
   userNameEdit->setText(userName);
   streetEdit->setText(userStreet);
   townEdit->setText(userTown);
@@ -68,13 +62,33 @@ KNewFileDlg::KNewFileDlg(QString userName, QString userStreet,
   telephoneEdit->setText(userTelephone);
   emailEdit->setText(userEmail);
 
-	if (title)
-	  setCaption(title);
+  init(title);
+}
 
-	userNameEdit->setFocus();
+void KNewFileDlg::init(const char* title)
+{
+  bool showLoadButton = false;
+  okBtn->setGuiItem(KStdGuiItem::ok());
+  cancelBtn->setGuiItem(KStdGuiItem::cancel());
 
-	connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()));
-	connect(okBtn, SIGNAL(clicked()), this, SLOT(okClicked()));
+  if (title)
+    setCaption(title);
+
+#if KDE_IS_VERSION( 3, 1, 90 )
+  KABC::StdAddressBook *ab = static_cast<KABC::StdAddressBook*>
+    ( KABC::StdAddressBook::self() );
+  if ( ab && !ab->whoAmI().isEmpty() )
+      showLoadButton = true;
+#endif
+
+  if(!showLoadButton)
+    kabcBtn->hide();
+
+  userNameEdit->setFocus();
+
+  connect(cancelBtn, SIGNAL(clicked()), this, SLOT(reject()));
+  connect(okBtn, SIGNAL(clicked()), this, SLOT(okClicked()));
+  connect(kabcBtn, SIGNAL(clicked()), this, SLOT(loadFromKABC()));
 }
 
 KNewFileDlg::~KNewFileDlg(){
@@ -92,3 +106,33 @@ void KNewFileDlg::okClicked()
 
   accept();
 }
+
+void KNewFileDlg::loadFromKABC(void)
+{
+#if KDE_IS_VERSION( 3, 1, 90 )
+  KABC::StdAddressBook *ab = static_cast<KABC::StdAddressBook*>
+    ( KABC::StdAddressBook::self() );
+  if ( !ab )
+    return;
+
+  KABC::Addressee addr = ab->whoAmI();
+  if ( addr.isEmpty() ) {
+    KMessageBox::sorry(this, i18n("Unable to load data, because no contact has been associated with the onwer of the standard addressbook."), i18n("Addressbook import"));
+    return;
+  }
+
+  userNameEdit->setText( addr.formattedName() );
+  emailEdit->setText( addr.preferredEmail() );
+
+  KABC::PhoneNumber phone = addr.phoneNumber( KABC::PhoneNumber::Home );
+  telephoneEdit->setText( phone.number() );
+
+  KABC::Address a = addr.address( KABC::Address::Home );
+  countyEdit->setText( a.country() + " / " + a.region() );
+  postcodeEdit->setText( a.postalCode() );
+  townEdit->setText( a.locality() );
+  streetEdit->setText( a.street() );
+#endif
+}
+
+#include "knewfiledlg.moc"
