@@ -26,6 +26,7 @@
 // QT Includes
 
 #include <qradiobutton.h>
+#include <qcheckbox.h>
 #include <qlabel.h>
 
 // ----------------------------------------------------------------------------
@@ -57,6 +58,8 @@
 #include "../mymoney/mymoneyfinancialcalculator.h"
 #include "../mymoney/mymoneyfile.h"
 
+#include "../kmymoney2.h"
+
 KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   KNewLoanWizardDecl(parent, name, true)
 {
@@ -65,7 +68,7 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
 
   connect(m_nameEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
   connect(m_payeeEdit, SIGNAL(newPayee(const QString&)), this, SLOT(slotNewPayee(const QString&)));
-  
+
   connect(m_previousPaymentButton, SIGNAL(clicked()), this, SLOT(slotPaymentsMade()));
   connect(m_noPreviousPaymentButton, SIGNAL(clicked()), this, SLOT(slotNoPaymentsMade()));
 
@@ -73,30 +76,34 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   connect(m_thisYearPaymentButton, SIGNAL(clicked()), this, SLOT(slotRecordThisYearsPayments()));
 
   connect(m_firstDueDateEdit, SIGNAL(dateChanged(const QDate&)), this, SLOT(slotCheckPageFinished()));
-  
+
   connect(m_interestOnPaymentButton, SIGNAL(clicked()), this, SLOT(slotInterestOnPayment()));
   connect(m_interestOnReceptionButton, SIGNAL(clicked()), this, SLOT(slotInterestOnReception()));
 
   connect(m_loanAmountEdit, SIGNAL(textChanged(const QString&)), this, SLOT(slotCheckPageFinished()));
-  
+
   connect(m_interestAccountEdit, SIGNAL(stateChanged()), this, SLOT(slotCheckPageFinished()));
 
   connect(m_nextDueDateEdit, SIGNAL(dateChanged(const QDate&)), this, SLOT(slotCheckPageFinished()));
   connect(m_paymentAccountEdit,  SIGNAL(stateChanged()), this, SLOT(slotCheckPageFinished()));
-  
+
+  connect(m_assetAccountEdit,  SIGNAL(stateChanged()), this, SLOT(slotCheckPageFinished()));
+  connect(m_dontCreatePayoutCheckBox,  SIGNAL(clicked()), this, SLOT(slotCheckPageFinished()));
+
   loadComboBoxes();
 
   resetCalculator();
-    
+
   // As default we assume a liability loan, with fixed interest rate,
   // with a first payment due on the 30th of this month. All payments
   // should be recorded and none have been made so far.
+  m_dontCreatePayoutCheckBox->setChecked(false);
   m_borrowButton->animateClick();
   m_fixedInterestButton->animateClick();
   m_noPreviousPaymentButton->animateClick();
   m_allPaymentsButton->animateClick();
   m_interestOnReceptionButton->animateClick();
-  
+
   m_interestFrequencyAmountEdit->setValue(1);
   m_interestFrequencyUnitEdit->setCurrentItem(i18n("Years"));
   m_paymentFrequencyUnitEdit->setCurrentItem(KMyMoneyUtils::occurenceToString(MyMoneySchedule::OCCUR_MONTHLY));
@@ -118,12 +125,19 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   m_additionalFeeButton->setGuiItem(additionalFeeButtenItem);
   connect(m_additionalFeeButton, SIGNAL(clicked()), this, SLOT(slotAdditionalFees()));
 
+  KGuiItem createAssetButtenItem( i18n( "&Create..." ),
+                      QIconSet(il->loadIcon("filenew", KIcon::Small, KIcon::SizeSmall)),
+                      i18n("Create a new asset account"),
+                      i18n("Use this to create a new account to which the initial payment should be made"));
+  m_createNewAssetButton->setGuiItem(createAssetButtenItem);
+  connect(m_createNewAssetButton, SIGNAL(clicked()), kmymoney2, SLOT(slotAccountNew()));
+
   // enable the finish button on the last page
   setFinishEnabled(m_summaryPage, true);
 
   // FIXME: we currently only support interest calculation on reception
   setAppropriate(m_interestCalculationPage, false);
-  
+
   // turn off all pages that are contained here for derived classes
   setAppropriate(m_editIntroPage, false);
   setAppropriate(m_editSelectionPage, false);
@@ -131,7 +145,7 @@ KNewLoanWizard::KNewLoanWizard(QWidget *parent, const char *name ) :
   setAppropriate(m_paymentEditPage, false);
   setAppropriate(m_interestEditPage, false);
   setAppropriate(m_summaryEditPage, false);
-    
+
   // for now, we don't have online help :-(
   helpButton()->hide();
 
@@ -154,7 +168,7 @@ void KNewLoanWizard::resetCalculator(void)
   m_duration1->setText(QString());
   m_payment1->setText(QString());
   m_balloon1->setText(QString());
-  
+
   m_loanAmount2->setText(QString());
   m_interestRate2->setText(QString());
   m_duration2->setText(QString());
@@ -242,7 +256,7 @@ void KNewLoanWizard::slotRecordThisYearsPayments(void)
 void KNewLoanWizard::slotCheckPageFinished(void)
 {
   nextButton()->setEnabled(false);
-  
+
   if(currentPage() == m_namePage) {
     if(!m_nameEdit->text().isEmpty()) {
       nextButton()->setEnabled(true);
@@ -254,7 +268,7 @@ void KNewLoanWizard::slotCheckPageFinished(void)
     && !m_loanAmountEdit->isValid()) {
       nextButton()->setEnabled(false);
     }
-        
+
   } else if(currentPage() == m_interestCategoryPage) {
     if(m_interestAccountEdit->selectedAccounts().count() > 0) {
       nextButton()->setEnabled(true);
@@ -269,7 +283,21 @@ void KNewLoanWizard::slotCheckPageFinished(void)
     && m_nextDueDateEdit->getQDate() >= m_firstDueDateEdit->getQDate()
     && m_paymentAccountEdit->selectedAccounts().count() > 0)
       nextButton()->setEnabled(true);
-      
+
+  } else if(currentPage() == m_assetAccountPage) {
+    if(m_dontCreatePayoutCheckBox->isChecked()) {
+      m_assetAccountEdit->setEnabled(false);
+      m_paymentDate->setEnabled(false);
+      m_createNewAssetButton->setEnabled(false);
+      nextButton()->setEnabled(true);
+    } else {
+      m_assetAccountEdit->setEnabled(true);
+      m_paymentDate->setEnabled(true);
+      m_createNewAssetButton->setEnabled(true);
+      if(!m_assetAccountEdit->selectedAccounts().isEmpty()
+      && m_paymentDate->getQDate().isValid())
+        nextButton()->setEnabled(true);
+    }
   } else
     nextButton()->setEnabled(true);
 }
@@ -280,7 +308,7 @@ void KNewLoanWizard::updateLoanAmount(void)
   if(m_loanAmountEdit->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_loanAmountEdit->getMoneyValue().formatMoney();
+    txt = m_loanAmountEdit->value().formatMoney();
   }
   m_loanAmount1->setText(txt);
   m_loanAmount2->setText(txt);
@@ -295,7 +323,7 @@ void KNewLoanWizard::updateInterestRate(void)
   if(m_interestRateEdit->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_interestRateEdit->getMoneyValue().formatMoney() + QString("%");
+    txt = m_interestRateEdit->value().formatMoney("", 3) + QString("%");
   }
   m_interestRate1->setText(txt);
   m_interestRate2->setText(txt);
@@ -326,7 +354,7 @@ void KNewLoanWizard::updatePayment(void)
   if(m_paymentEdit->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_paymentEdit->getMoneyValue().formatMoney();
+    txt = m_paymentEdit->value().formatMoney();
   }
   m_payment1->setText(txt);
   m_payment2->setText(txt);
@@ -342,7 +370,7 @@ void KNewLoanWizard::updateFinalPayment(void)
   if(m_finalPaymentEdit->text().isEmpty()) {
     txt = QString("<") + i18n("calculate") + QString(">");
   } else {
-    txt = m_finalPaymentEdit->getMoneyValue().formatMoney();
+    txt = m_finalPaymentEdit->value().formatMoney();
   }
   m_balloon1->setText(txt);
   m_balloon2->setText(txt);
@@ -359,16 +387,16 @@ void KNewLoanWizard::updateLoanInfo(void)
   updatePayment();
   updateFinalPayment();
   updatePeriodicPayment();
-  
+
   QString txt;
-  
-  m_loanAmount6->setText(m_loanAmountEdit->getMoneyValue().formatMoney());
-  m_interestRate6->setText(m_interestRateEdit->getMoneyValue().formatMoney() + QString("%"));
+
+  m_loanAmount6->setText(m_loanAmountEdit->value().formatMoney());
+  m_interestRate6->setText(m_interestRateEdit->value().formatMoney("", 3) + QString("%"));
   txt = QString().sprintf("%d ", m_durationValueEdit->value())
         + m_durationUnitEdit->currentText();
   m_duration6->setText(txt);
-  m_payment6->setText(m_paymentEdit->getMoneyValue().formatMoney());
-  m_balloon6->setText(m_finalPaymentEdit->getMoneyValue().formatMoney());
+  m_payment6->setText(m_paymentEdit->value().formatMoney());
+  m_balloon6->setText(m_finalPaymentEdit->value().formatMoney());
 }
 
 void KNewLoanWizard::updatePeriodicPayment(void)
@@ -393,7 +421,7 @@ void KNewLoanWizard::updateSummary(void)
   } else {
     m_summaryPayee->setText(m_payeeEdit->text());
   }
-  
+
   // Calculation
   if(m_interestOnReceptionButton->isChecked())
     m_summaryInterestDue->setText(i18n("on reception"));
@@ -405,7 +433,7 @@ void KNewLoanWizard::updateSummary(void)
   m_summaryTerm->setText(m_duration6->text());
   m_summaryPeriodicPayment->setText(m_payment6->text());
   m_summaryBalloonPayment->setText(m_balloon6->text());
-  
+
   // Payment
   try {
     QCStringList sel = m_interestAccountEdit->selectedAccounts();
@@ -420,7 +448,7 @@ void KNewLoanWizard::updateSummary(void)
   m_summaryAdditionalFees->setText(m_additionalCost->text());
   m_summaryTotalPeriodicPayment->setText(m_periodicPayment->text());
   m_summaryNextPayment->setText(KGlobal::locale()->formatDate(m_nextDueDateEdit->getQDate(), true));
-  
+
   try {
     QCStringList sel = m_paymentAccountEdit->selectedAccounts();
     if(sel.count() != 1)
@@ -441,12 +469,12 @@ void KNewLoanWizard::next()
               "at the same time. "
               "Please enter a value for the %1 on this page or backup to the page where the "
               " current value to be calculated is defined and fill in a value.");
-              
+
   if(currentPage() == m_lendBorrowPage) {
     // load the appropriate categories into the list
     loadAccountList();
     m_nameEdit->setFocus();
-    
+
   } else if(currentPage() == m_interestTypePage) {
     if(m_fixedInterestButton->isChecked()) {
       setAppropriate(m_previousPaymentsPage, true);
@@ -455,13 +483,13 @@ void KNewLoanWizard::next()
       else
         setAppropriate(m_recordPaymentPage, false);
       setAppropriate(m_variableInterestDatePage, false);
-      
+
     } else {
       setAppropriate(m_previousPaymentsPage, false);
       setAppropriate(m_recordPaymentPage, false);
       setAppropriate(m_variableInterestDatePage, true);
     }
-    
+
   } else if(currentPage() == m_loanAmountPage) {
     m_interestRateEdit->setFocus();
     if(m_thisYearPaymentButton->isChecked()
@@ -473,16 +501,16 @@ void KNewLoanWizard::next()
       KMessageBox::error(0, errMsg, i18n("Calculation error"));
     } else
       updateLoanAmount();
-    
+
   } else if(currentPage() == m_interestPage) {
-    
+
     if(m_loanAmountEdit->text().isEmpty()
     && m_interestRateEdit->text().isEmpty()) {
       dontLeavePage = true;
       KMessageBox::error(0, errMsg.arg(i18n("interest rate")), i18n("Calculation error"));
     } else
       updateInterestRate();
-      
+
   } else if(currentPage() == m_durationPage) {
     if((m_loanAmountEdit->text().isEmpty()
     || m_interestRateEdit->text().isEmpty())
@@ -491,7 +519,7 @@ void KNewLoanWizard::next()
       KMessageBox::error(0, errMsg.arg(i18n("term")), i18n("Calculation error"));
     } else
       updateDuration();
-      
+
   } else if(currentPage() == m_paymentPage) {
     if((m_loanAmountEdit->text().isEmpty()
     || m_interestRateEdit->text().isEmpty()
@@ -501,32 +529,43 @@ void KNewLoanWizard::next()
       KMessageBox::error(0, errMsg.arg(i18n("interest and amortization")), i18n("Calculation error"));
     } else
       updatePayment();
-    
+
   } else if(currentPage() == m_finalPaymentPage) {
     if((m_loanAmountEdit->text().isEmpty()
     || m_interestRateEdit->text().isEmpty()
     || m_durationValueEdit->value() == 0
     || m_paymentEdit->text().isEmpty())
     && m_finalPaymentEdit->text().isEmpty()) {
-      dontLeavePage = true;
-      KMessageBox::error(0, errMsg.arg(i18n("final amortization payment")), i18n("Calculation error"));
-    } else {
-      updateFinalPayment();
-      if(!calculateLoan()) {
-        dontLeavePage = true;
-      } else
-        updateLoanInfo();
+      // if two fields are empty and one of them is the final payment
+      // we assume the final payment to be 0 instead of presenting a
+      m_finalPaymentEdit->setValue(MyMoneyMoney(0, 1));
     }
-    
+    updateFinalPayment();
+    if(!calculateLoan()) {
+      dontLeavePage = true;
+    } else
+      updateLoanInfo();
+
   } else if(currentPage() == m_additionalFeesPage) {
+    m_nextDueDateEdit->setEnabled(true);
+    if(m_allPaymentsButton->isChecked() || m_noPreviousPaymentButton->isChecked()) {
+      m_nextDueDateEdit->setDate(m_firstDueDateEdit->getQDate());
+      m_nextDueDateEdit->setEnabled(false);
+      setAppropriate(m_assetAccountPage, true);
+    } else {
+      QDate nextPayment(QDate::currentDate().year(), 1, m_firstDueDateEdit->getQDate().day());
+      m_nextDueDateEdit->setDate(nextPayment);
+      setAppropriate(m_assetAccountPage, false);
+      m_assetAccountEdit->slotDeselectAllAccounts();
+    }
     if(m_nextDueDateEdit->getQDate() < m_firstDueDateEdit->getQDate()) {
       m_nextDueDateEdit->setDate(m_firstDueDateEdit->getQDate());
     }
-    
+
   } else if(currentPage() == m_schedulePage) {
     updateSummary();
   }
-  
+
 /*
   switch(m_accountType) {
     case MyMoneyAccount::Cash:
@@ -649,7 +688,7 @@ int KNewLoanWizard::occurenceToPeriod(const MyMoneySchedule::occurenceE occurenc
 int KNewLoanWizard::occurenceToFrequency(const MyMoneySchedule::occurenceE occurence) const
 {
   int rc = 0;
-  
+
   switch(occurence) {
     case MyMoneySchedule::OCCUR_DAILY:
       rc = 365;
@@ -687,7 +726,7 @@ int KNewLoanWizard::occurenceToFrequency(const MyMoneySchedule::occurenceE occur
     default:
       qWarning("Occurence not supported by financial calculator");
   }
-  
+
   return rc;
 }
 */
@@ -697,48 +736,48 @@ int KNewLoanWizard::calculateLoan(void)
   long double val;
   int PF;
   QString result;
-  
+
   // FIXME: for now, we only support interest calculation at the end of the period
   calc.setBep();
   // FIXME: for now, we only support periodic compounding
   calc.setDisc();
-  
+
   PF = KMyMoneyUtils::occurenceToFrequency(KMyMoneyUtils::stringToOccurence(
                             m_paymentFrequencyUnitEdit->currentText()));
   if(PF == 0)
     return 0;
   calc.setPF(PF);
-  
+
   // FIXME: for now we only support compounding frequency == payment frequency
   calc.setCF(PF);
 
 
   if(!m_loanAmountEdit->text().isEmpty()) {
-    val = static_cast<long double> (m_loanAmountEdit->getMoneyValue().abs().value());
+    val = static_cast<long double> (m_loanAmountEdit->value().abs().toDouble());
     if(m_borrowButton->isChecked())
       val = -val;
     calc.setPv(val);
   }
-  
+
   if(!m_interestRateEdit->text().isEmpty()) {
-    val = static_cast<long double> (m_interestRateEdit->getMoneyValue().abs().value()) / 100.0;
+    val = static_cast<long double> (m_interestRateEdit->value().abs().toDouble());
     calc.setIr(val);
   }
-  
+
   if(!m_paymentEdit->text().isEmpty()) {
-    val = static_cast<long double> (m_paymentEdit->getMoneyValue().abs().value());
+    val = static_cast<long double> (m_paymentEdit->value().abs().toDouble());
     if(m_lendButton->isChecked())
       val = -val;
     calc.setPmt(val);
   }
-  
+
   if(!m_finalPaymentEdit->text().isEmpty()) {
-    val = static_cast<long double> (m_finalPaymentEdit->getMoneyValue().abs().value());
+    val = static_cast<long double> (m_finalPaymentEdit->value().abs().toDouble());
     if(m_lendButton->isChecked())
       val = -val;
     calc.setFv(val);
   }
-  
+
   if(m_durationValueEdit->value() != 0) {
     calc.setNpp(static_cast<long double>(term()));
   }
@@ -748,38 +787,51 @@ int KNewLoanWizard::calculateLoan(void)
     if(m_loanAmountEdit->text().isEmpty()) {
       // calculate the amount of the loan out of the other information
       val = calc.presentValue();
-      m_loanAmountEdit->loadText(MyMoneyMoney(static_cast<double>(val/100.0)).abs().formatMoney());
+      m_loanAmountEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney());
       result = i18n("KMyMoney has calculated the amount of the loan as %1.")
                         .arg(m_loanAmountEdit->text());
-      
+
     } else if(m_interestRateEdit->text().isEmpty()) {
       // calculate the interest rate out of the other information
       val = calc.interestRate();
-      m_interestRateEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney());
+      m_interestRateEdit->loadText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney("", 3));
       result = i18n("KMyMoney has calculated the interest rate to %1%.")
                         .arg(m_interestRateEdit->text());
-      
+
     } else if(m_paymentEdit->text().isEmpty()) {
       // calculate the periodical amount of the payment out of the other information
       val = calc.payment();
-      m_paymentEdit->setText(MyMoneyMoney(static_cast<double>(val/100.0)).abs().formatMoney());
+      m_paymentEdit->setText(MyMoneyMoney(static_cast<double>(val)).abs().formatMoney());
+      // reset payment as it might have changed due to rounding
+      val = static_cast<long double> (m_paymentEdit->value().abs().toDouble());
+      if(m_lendButton->isChecked())
+        val = -val;
+      calc.setPmt(val);
 
       result = i18n("KMyMoney has calculated a periodic payment of %1 to cover amortization and interest.")
                       .arg(m_paymentEdit->text());
-                        
+
       val = calc.futureValue();
-      if((m_borrowButton->isChecked() && val < 0)
-      || (m_lendButton->isChecked() && val > 0)) {
+      if((m_borrowButton->isChecked() && val < 0 && fabsl(val) >= fabsl(calc.payment()))
+      || (m_lendButton->isChecked() && val > 0 && fabs(val) >= fabs(calc.payment()))) {
         calc.setNpp(calc.npp()-1);
         updateTermWidgets(calc.npp());
         val = calc.futureValue();
-        MyMoneyMoney refVal(static_cast<double>(val/100.0));
+        MyMoneyMoney refVal(static_cast<double>(val));
         m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
         result += QString(" ");
         result += i18n("The number of payments has been decremented and the final payment has been modified to %1.")
                       .arg(m_finalPaymentEdit->text());
+      } else if((m_borrowButton->isChecked() && val < 0 && fabsl(val) < fabsl(calc.payment()))
+             || (m_lendButton->isChecked() && val > 0 && fabs(val) < fabs(calc.payment()))) {
+        m_finalPaymentEdit->loadText(MyMoneyMoney(0,1).formatMoney());
+      } else {
+        MyMoneyMoney refVal(static_cast<double>(val));
+        m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
+        result += i18n("The final payment has been modified to %1.")
+                      .arg(m_finalPaymentEdit->text());
       }
-     
+
     } else if(m_durationValueEdit->value() == 0) {
       // calculate the number of payments out of the other information
       val = calc.numPayments();
@@ -790,11 +842,11 @@ int KNewLoanWizard::calculateLoan(void)
       // round it to the smallest integer and calculate the balloon payment
       result = i18n("KMyMoney has calculated the term of your loan as %1. ")
                         .arg(updateTermWidgets(floorl(val)));
-                        
+
       if(val != floorl(val)) {
         calc.setNpp(floorl(val));
         val = calc.futureValue();
-        MyMoneyMoney refVal(static_cast<double>(val/100.0));
+        MyMoneyMoney refVal(static_cast<double>(val));
         m_finalPaymentEdit->loadText(refVal.abs().formatMoney());
         result += i18n("The final payment has been modified to %1.")
                       .arg(m_finalPaymentEdit->text());
@@ -803,18 +855,35 @@ int KNewLoanWizard::calculateLoan(void)
     } else {
       // calculate the future value of the loan out of the other information
       val = calc.futureValue();
-      MyMoneyMoney refVal(static_cast<double>(val/100.0));
 
-      if((m_borrowButton->isChecked() && val < 0)
-      || (m_lendButton->isChecked() && val > 0)) {
-        qDebug("Future Value is %lld", refVal.value());
+      // we differentiate between the following cases:
+      // a) the future value is greater than a payment
+      // b) the future value is less than a payment or the loan is overpaid
+      // c) all other cases
+      //
+      // a) means, we have paid more than we owed. This can't be
+      // b) means, we paid more than we owed but the last payment is
+      //    less in value than regular payments. That means, that the
+      //    future value is to be treated as  (fully payed back)
+      // c) the loan is not payed back yet
+      if((m_borrowButton->isChecked() && val < 0 && fabsl(val) > fabsl(calc.payment()))
+      || (m_lendButton->isChecked() && val > 0 && fabs(val) > fabs(calc.payment()))) {
+        // case a)
+        qDebug("Future Value is %Lf", val);
         throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
+
+      } else if((m_borrowButton->isChecked() && val < 0 && fabsl(val) <= fabsl(calc.payment()))
+             || (m_lendButton->isChecked() && val > 0 && fabs(val) <= fabs(calc.payment()))) {
+        // case b)
+        val = 0;
       }
+
+      MyMoneyMoney refVal(static_cast<double>(val));
       result = i18n("KMyMoney has calculated a final payment of %1 for this loan.")
                         .arg(refVal.abs().formatMoney());
-                        
+
       if(!m_finalPaymentEdit->text().isEmpty()) {
-        if((m_finalPaymentEdit->getMoneyValue().abs() - refVal.abs()).abs().value() > 100) {
+        if((m_finalPaymentEdit->value().abs() - refVal.abs()).abs().toDouble() > 1) {
           throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
         }
         result = i18n("KMyMoney has successfully verified your loan information.");
@@ -851,7 +920,7 @@ const QString KNewLoanWizard::updateTermWidgets(const long double val)
     vl /= 12;
     unit = MyMoneySchedule::OCCUR_YEARLY;
   }
-    
+
   switch(unit) {
     case MyMoneySchedule::OCCUR_MONTHLY:
       valString = i18n("one month", "%n months", vl);
@@ -874,7 +943,7 @@ void KNewLoanWizard::slotCreateCategory(void)
 {
   MyMoneyAccount acc, base;
   MyMoneyFile* file = MyMoneyFile::instance();
-  
+
   if(m_borrowButton->isChecked()) {
     base = file->expense();
     acc.setAccountType(MyMoneyAccount::Expense);
@@ -883,20 +952,20 @@ void KNewLoanWizard::slotCreateCategory(void)
     acc.setAccountType(MyMoneyAccount::Income);
   }
   acc.setParentAccountId(base.id());
-  
+
   KNewAccountDlg* dlg = new KNewAccountDlg(acc, true, true);
   if(dlg->exec() == QDialog::Accepted) {
     acc = dlg->account();
-    
+
     try {
       QCString id;
       id = file->createCategory(base, acc.name());
       if(id.isEmpty())
         throw new MYMONEYEXCEPTION("failure while creating the account hierarchy");
-        
+
       loadAccountList();
       m_interestAccountEdit->setSelected(id);
-      
+
     } catch (MyMoneyException *e) {
       QString message("Unable to add account: ");
       message += e->what();
@@ -909,13 +978,15 @@ void KNewLoanWizard::slotCreateCategory(void)
 
 void KNewLoanWizard::loadAccountList(void)
 {
-  if(m_borrowButton->isChecked())
+  if(m_borrowButton->isChecked()) {
     m_interestAccountEdit->loadList(KMyMoneyUtils::expense);
-  else
+  } else {
     m_interestAccountEdit->loadList(KMyMoneyUtils::income);
+  }
 
   m_paymentAccountEdit->loadList(static_cast<KMyMoneyUtils::categoryTypeE>
                                  (KMyMoneyUtils::asset | KMyMoneyUtils::liability));
+  m_assetAccountEdit->loadList(KMyMoneyUtils::asset);
 }
 
 void KNewLoanWizard::slotAdditionalFees(void)
@@ -923,8 +994,7 @@ void KNewLoanWizard::slotAdditionalFees(void)
   // KMessageBox::information(0, QString("Not yet implemented ... if you want to help, contact kmymoney2-developer@lists.sourceforge.net"), QString("Development notice"));
   MyMoneyAccount account(QCString("Phony-ID"), MyMoneyAccount());
 
-  KSplitTransactionDlg* dlg = new KSplitTransactionDlg(m_transaction, account,
-false, !m_borrowButton->isChecked(), MyMoneyMoney(0));
+  KSplitTransactionDlg* dlg = new KSplitTransactionDlg(m_transaction, account, false, !m_borrowButton->isChecked(), MyMoneyMoney(0));
 
   if(dlg->exec() == QDialog::Accepted) {
     m_transaction = dlg->transaction();
@@ -932,8 +1002,7 @@ false, !m_borrowButton->isChecked(), MyMoneyMoney(0));
     QValueList<MyMoneySplit>::ConstIterator it;
 
     MyMoneyMoney fees;
-    for(it = m_transaction.splits().begin(); it != m_transaction.splits().end()
-; ++it) {
+    for(it = m_transaction.splits().begin(); it != m_transaction.splits().end(); ++it) {
       if((*it).accountId() != account.id()) {
         fees += (*it).value();
       }
@@ -945,27 +1014,27 @@ false, !m_borrowButton->isChecked(), MyMoneyMoney(0));
   updatePeriodicPayment();
 }
 
-const MyMoneyTransaction KNewLoanWizard::transaction(void) const
+const MyMoneyTransaction KNewLoanWizard::transaction() const
 {
   MyMoneyTransaction t;
-  
+
   MyMoneySplit sPayment, sInterest, sAmortization;
   // setup accounts. at this point, we cannot fill in the id of the
   // account that the amortization will be performed on, because we
   // create the is account. So the id is yet unknown.
   sPayment.setAccountId(m_paymentAccountEdit->selectedAccounts().first());
   sInterest.setAccountId(m_interestAccountEdit->selectedAccounts().first());
-  
+
   // values
   if(m_borrowButton->isChecked()) {
-    sPayment.setValue(-m_paymentEdit->getMoneyValue());
+    sPayment.setValue(-m_paymentEdit->value());
   } else {
-    sPayment.setValue(m_paymentEdit->getMoneyValue());
+    sPayment.setValue(m_paymentEdit->value());
   }
-  sInterest.setValue(MyMoneyMoney::minValue+1);
-  sAmortization.setValue(MyMoneyMoney::minValue+1);
+  sInterest.setValue(MyMoneyMoney::autoCalc);
+  sAmortization.setValue(MyMoneyMoney::autoCalc);
 
-  // actions  
+  // actions
   sPayment.setAction(MyMoneySplit::ActionAmortization);
   sAmortization.setAction(MyMoneySplit::ActionAmortization);
   sInterest.setAction(MyMoneySplit::ActionInterest);
@@ -981,7 +1050,7 @@ const MyMoneyTransaction KNewLoanWizard::transaction(void) const
       delete e;
     }
   }
-  
+
   // IMPORTANT: Payment split must be the first one, because
   //            the schedule view expects it this way during display
   t.addSplit(sPayment);
@@ -1015,20 +1084,20 @@ const MyMoneySchedule KNewLoanWizard::schedule() const
                         false);
 
   sched.setTransaction(transaction());
-  
+
   return sched;
 }
 
 const MyMoneyAccount KNewLoanWizard::account(void) const
 {
   MyMoneyAccountLoan acc;
-  
+
   acc.setName(m_nameEdit->text());
   acc.setOpeningDate(m_firstDueDateEdit->getQDate());
   if(m_allPaymentsButton->isChecked()) {
     acc.setOpeningBalance(0);
   } else {
-    acc.setOpeningBalance(m_loanAmountEdit->getMoneyValue());
+    acc.setOpeningBalance(m_loanAmountEdit->value());
   }
   if(m_borrowButton->isChecked()) {
     acc.setAccountType(MyMoneyAccount::Loan);
@@ -1036,8 +1105,8 @@ const MyMoneyAccount KNewLoanWizard::account(void) const
   } else {
     acc.setAccountType(MyMoneyAccount::AssetLoan);
   }
-  acc.setLoanAmount(m_loanAmountEdit->getMoneyValue());
-  acc.setInterestRate(m_firstDueDateEdit->getQDate(), m_interestRateEdit->getMoneyValue());
+  acc.setLoanAmount(m_loanAmountEdit->value());
+  acc.setInterestRate(m_firstDueDateEdit->getQDate(), m_interestRateEdit->value());
   if(m_interestOnReceptionButton->isChecked())
     acc.setInterestCalculation(MyMoneyAccountLoan::paymentReceived);
   else
@@ -1046,7 +1115,7 @@ const MyMoneyAccount KNewLoanWizard::account(void) const
   acc.setFixedInterestRate(m_fixedInterestButton->isChecked());
   acc.setFinalPayment(MyMoneyMoney(m_finalPaymentEdit->text()));
   acc.setTerm(term());
-  acc.setPeriodicPayment(m_paymentEdit->getMoneyValue());
+  acc.setPeriodicPayment(m_paymentEdit->value());
 
   if(!m_payeeEdit->text().isEmpty()) {
     try {
@@ -1058,7 +1127,7 @@ const MyMoneyAccount KNewLoanWizard::account(void) const
     }
   } else
     acc.setPayee(QCString());
-  
+
   if(m_variableInterestButton->isChecked()) {
     acc.setNextInterestChange(m_interestChangeDateEdit->getQDate());
     acc.setInterestChangeFrequency(m_interestFrequencyAmountEdit->value(),
@@ -1071,7 +1140,7 @@ void KNewLoanWizard::loadWidgets(const MyMoneyAccount& account)
 {
   bool borrowing = false;
   MyMoneyAccountLoan acc(account);
-  
+
   m_nameEdit->loadText(acc.name());
   if(acc.accountType() == MyMoneyAccount::Loan) {
     m_borrowButton->animateClick();
@@ -1079,7 +1148,7 @@ void KNewLoanWizard::loadWidgets(const MyMoneyAccount& account)
   } else
     m_lendButton->animateClick();
 
-  if(acc.openingBalance() != 0) {
+  if(!acc.openingBalance().isZero()) {
     if(borrowing)
       m_loanAmountEdit->loadText((-account.openingBalance()).formatMoney());
     else
@@ -1094,7 +1163,7 @@ void KNewLoanWizard::loadWidgets(const MyMoneyAccount& account)
 int KNewLoanWizard::term(void) const
 {
   int factor = 0;
-  
+
   if(m_durationValueEdit->value() != 0) {
     factor = 1;
     switch(m_durationUnitEdit->currentItem()) {
@@ -1118,3 +1187,21 @@ int KNewLoanWizard::term(void) const
   }
   return factor;
 }
+
+const QCString KNewLoanWizard::initialPaymentAccount(void) const
+{
+  if(m_dontCreatePayoutCheckBox->isChecked()) {
+    return QCString();
+  }
+  return m_assetAccountEdit->selectedAccounts().first();
+}
+
+const QDate KNewLoanWizard::initialPaymentDate(void) const
+{
+  if(m_dontCreatePayoutCheckBox->isChecked()) {
+    return QDate();
+  }
+  return m_paymentDate->getQDate();
+}
+
+#include "knewloanwizard.moc"
