@@ -16,6 +16,7 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qapplication.h>
 #include <qlayout.h>
 #include <qvbox.h>
@@ -29,20 +30,10 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 #include <klocale.h>
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-
 #include <kiconloader.h>
 #include <kconfig.h>
-#if QT_VERSION > 300
 #include <kcolorbutton.h>
-#else
-#include <kcolorbtn.h>
-#endif
-
 #include <kmessagebox.h>
 #include <klistview.h>
 #include <kpushbutton.h>
@@ -52,32 +43,39 @@
 
 // ----------------------------------------------------------------------------
 // Project Includes
+
 #include "ksettingsdlg.h"
 #include "../kmymoneyutils.h"
+#include "../widgets/kmymoneyonlinequoteconfig.h"
+#include "../widgets/kmymoneygpgconfig.h"
+
+// #include <libkgpgfile/kgpgfile.h>
 
 /** Standard constructor for the class.
   * The constructor passes some additional parameters to the base class KDialogBase
   * to set the buttons to be showed and the type of dialog to be shown.
-**/
+  **/
 KSettingsDlg::KSettingsDlg(QWidget *parent, const char *name, bool modal)
- : KDialogBase(IconList, i18n("Configure"), Ok|Cancel|Apply|User1, Ok, parent,
+ : KDialogBase(IconList, i18n("Configure"), Ok|Cancel|Apply|User1|Help, Ok, parent,
     name, modal, true, i18n("&Reset"))
 {
   // Setup the pages and then read the configuration object.
   setPageGeneral();
-  setPageAccountsView();
+  // setPageAccountsView();
   setPageList();
   setHomePage();
   setPageSchedule();
+  setPageSecurity();
   setPageColour();
   setPageFont();
-  
+  setPageOnlineQuotes();
+
   configRead();
-  
+
   m_bDoneApply=false;
-  
+
   KPushButton *reset = static_cast<KPushButton *>(actionButton(User1));
-  
+
   KIconLoader* il = KGlobal::iconLoader();
   KGuiItem resetButtonItem( i18n( "&Reset" ),
                     QIconSet(il->loadIcon("undo", KIcon::Small, KIcon::SizeSmall)),
@@ -104,7 +102,7 @@ void KSettingsDlg::setPageGeneral()
   // Startup page options
   // --------------------
 
-/* These settings are deprecated  
+/* These settings are deprecated
   // Create a group box to hold the available options
   QButtonGroup *qfilebuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox1");
   qfilebuttongroup->setTitle( i18n( "Startup file options" ) );
@@ -132,7 +130,7 @@ void KSettingsDlg::setPageGeneral()
   // --------------------
 
   // Create a group box to hold the available options
-  QButtonGroup *qpagebuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox2");
+  QButtonGroup *qpagebuttongroup = new QButtonGroup(qvboxMainFrame, "ButtonGroup1");
   qpagebuttongroup->setTitle( i18n( "Startup page options" ) );
   qpagebuttongroup->setColumnLayout(0, Qt::Vertical );
   qpagebuttongroup->layout()->setSpacing( 0 );
@@ -153,8 +151,51 @@ void KSettingsDlg::setPageGeneral()
   m_qradiobuttonStartLast = new QRadioButton("start_last", qpagebuttongroup);
   m_qradiobuttonStartLast->setText( i18n( "Start with last selected page" ) );
   qvboxpagelayout->addWidget(m_qradiobuttonStartLast);
-}
 
+
+  // Create a group to hold the price precision
+  QButtonGroup *qbuttongroupPrice = new QButtonGroup(qvboxMainFrame, "ButtonGroup2");
+  qbuttongroupPrice->setTitle(i18n("Equity/Currency/Value options"));
+  qbuttongroupPrice->setColumnLayout(0, Qt::Vertical );
+  qbuttongroupPrice->layout()->setSpacing( 0 );
+  qbuttongroupPrice->layout()->setMargin( 0 );
+
+  QHBoxLayout *qhboxlayout2 = new QHBoxLayout(qbuttongroupPrice->layout());
+  qhboxlayout2->setAlignment( Qt::AlignTop );
+  qhboxlayout2->setSpacing( 6 );
+  qhboxlayout2->setMargin( 11 );
+
+  m_qIntPricePrecision = new KLineEdit(qbuttongroupPrice, "priceprecisioninput");
+  QIntValidator *qintvalidator = new QIntValidator(4, 10, m_qIntPricePrecision);
+  m_qIntPricePrecision->setValidator(qintvalidator);
+  qhboxlayout2->addWidget(new QLabel(i18n("Price Precision: "), qbuttongroupPrice));
+  qhboxlayout2->addWidget(m_qIntPricePrecision);
+  qhboxlayout2->addWidget(new QLabel(i18n("digits"), qbuttongroupPrice));
+  qhboxlayout2->addStretch(20);
+
+
+  // Create a check box for the hide category feature
+  QButtonGroup *qbuttongroupAccount = new QButtonGroup(qvboxMainFrame, "ButtonGroup3");
+  qbuttongroupAccount->setTitle(i18n("Account options"));
+
+  qbuttongroupAccount->setColumnLayout(0, Qt::Vertical );
+  qbuttongroupAccount->layout()->setSpacing( 0 );
+  qbuttongroupAccount->layout()->setMargin( 0 );
+
+  QHBoxLayout *qhboxlayout3 = new QHBoxLayout(qbuttongroupAccount->layout());
+  qhboxlayout3->setAlignment( Qt::AlignTop );
+  qhboxlayout3->setSpacing( 6 );
+  qhboxlayout3->setMargin( 11 );
+
+  m_qcheckboxHideCategory = new QCheckBox("hide_categories", qbuttongroupAccount);
+  m_qcheckboxHideCategory->setText( i18n( "Don't show unused categories" ) );
+  qhboxlayout3->addWidget(m_qcheckboxHideCategory);
+
+  // set online help reference
+  // not needed here, as it is the default anyway
+  // m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.general";
+}
+#if 0
 void KSettingsDlg::setPageAccountsView()
 {
   // Create the main frame to hold the widgets
@@ -208,10 +249,8 @@ void KSettingsDlg::setPageAccountsView()
   m_qradiobuttonAccountWizard->setText( i18n( "Use the new account wizard" ) );
   qwizardvboxlayout->addWidget(m_qradiobuttonAccountWizard);
 */
-  // Create a check box for the hide category feature
-  m_qcheckboxHideCategory = new QCheckBox("hide_categories", qvboxMainFrame);
-  m_qcheckboxHideCategory->setText( i18n( "Don't show unused categories" ) );
 }
+#endif
 
 void KSettingsDlg::setHomePage()
 {
@@ -220,7 +259,7 @@ void KSettingsDlg::setHomePage()
     DesktopIcon("home"));
 
   QFrame* frame = new QFrame(qvboxMainFrame, "frame");
-  
+
   QHBoxLayout* Form1Layout = new QHBoxLayout( frame, 11, 6, "Form1Layout");
 
   m_homePageList = new KListView( frame, "KListView1" );
@@ -230,7 +269,7 @@ void KSettingsDlg::setHomePage()
   m_homePageList->header()->hide();
   m_homePageList->setAllColumnsShowFocus(true);
   m_homePageList->setColumnWidth(0, 250);
-  
+
   Form1Layout->addWidget( m_homePageList );
 
   QVBoxLayout*  Layout5 = new QVBoxLayout( 0, 0, 6, "Layout5");
@@ -241,7 +280,7 @@ void KSettingsDlg::setHomePage()
 
   QVBoxLayout* Layout1 = new QVBoxLayout( 0, 0, 6, "Layout1");
 
-  
+
   KIconLoader* il = KGlobal::iconLoader();
   KGuiItem upButtonItem( i18n( "&Up" ),
                     QIconSet(il->loadIcon("up", KIcon::Small, KIcon::SizeSmall)),
@@ -257,7 +296,7 @@ void KSettingsDlg::setHomePage()
   m_downButton = new KPushButton(downButtonItem, frame, "KPushButtonDown" );
   Layout1->addWidget( m_downButton );
   Layout2->addLayout( Layout1 );
-  
+
   QSpacerItem* spacer_2 = new QSpacerItem( 177, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
   Layout2->addItem( spacer_2 );
   Layout5->addLayout( Layout2 );
@@ -283,8 +322,11 @@ void KSettingsDlg::setHomePage()
   connect(m_homePageList, SIGNAL(selectionChanged(QListViewItem*)),
           this, SLOT(slotSelectHomePageItem(QListViewItem *)));
 
-  connect(m_upButton, SIGNAL(clicked()), this, SLOT(slotMoveUp()));          
+  connect(m_upButton, SIGNAL(clicked()), this, SLOT(slotMoveUp()));
   connect(m_downButton, SIGNAL(clicked()), this, SLOT(slotMoveDown()));
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.home";
 }
 
 /** Called to create the Main List page shown in the dialog.
@@ -331,12 +373,6 @@ void KSettingsDlg::setPageList()
   m_qcheckboxShowGrid = new QCheckBox(i18n("Show a grid in the register."), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxShowGrid);
 
-/*
-  // and another widget
-  m_qcheckboxTextPrompt = new QCheckBox(i18n("Show a textual prompt in the register."), qwidgetPage);
-  qvboxlayoutPage->addWidget(m_qcheckboxTextPrompt);
-*/
-
   // and one more
   m_qcheckboxLedgerLens = new QCheckBox(i18n("Use the ledger lens"), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxLedgerLens);
@@ -345,44 +381,30 @@ void KSettingsDlg::setPageList()
   m_qcheckboxTransactionForm = new QCheckBox(i18n("Show transaction form"), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxTransactionForm);
 
-  // In some transaction forms, no Nr. field is shown. This
+  // In some transaction forms, no Nr.(No.) field is shown. This
   // switch allows to override this feature.
-  m_qcheckboxShowNrField = new QCheckBox(i18n("Always show a Nr. field in transaction form"), qwidgetPage);
+  m_qcheckboxShowNrField = new QCheckBox(i18n("Always show a No. field in transaction form"), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxShowNrField);
   connect(m_qcheckboxShowNrField, SIGNAL(toggled(bool)),
     this, SLOT(slotNrFieldToggled(bool)));
 
-  // Setting for switch to copy transaction type into Nr. field upon
+  // Setting for switch to copy transaction type into Nr.(No.) field upon
   // new transactions
-  m_qcheckboxTypeToNr = new QCheckBox(i18n("Insert transaction type into Nr. field for new transactions"), qwidgetPage);
+  m_qcheckboxTypeToNr = new QCheckBox(i18n("Insert transaction type into No. field for new transactions"), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxTypeToNr);
+  connect(m_qcheckboxTypeToNr, SIGNAL(toggled(bool)), this, SLOT(slotTypeToNrToggled(bool)));
 
-#if 0
-  // Create a group to hold two radio buttons
-  QButtonGroup *qbuttongroup = new QButtonGroup(qwidgetPage, "ButtonGroup1");
-  qbuttongroup->setTitle(i18n("Row Colour options"));
-  qbuttongroup->setColumnLayout(0, Qt::Vertical );
-  qbuttongroup->layout()->setSpacing( 0 );
-  qbuttongroup->layout()->setMargin( 0 );
+  // Setting for switch to always increment the number for checks
+  m_qcheckboxIncCheckNumber = new QCheckBox(i18n("Auto increment check number"), qwidgetPage);
+  qvboxlayoutPage->addWidget(m_qcheckboxIncCheckNumber);
 
-  // Create a layout
-  QVBoxLayout *qvboxlayout = new QVBoxLayout(qbuttongroup->layout());
-  qvboxlayout->setAlignment( Qt::AlignTop );
-  qvboxlayout->setSpacing( 6 );
-  qvboxlayout->setMargin( 11 );
+  // Setting for the behaviour how changes should be treated when editing
+  // is left by selecting a different transaction/split
+  m_qcheckboxFocusChangeEnter = new QCheckBox(i18n("Keep changes when selecting a different transaction/split"), qwidgetPage);
+  qvboxlayoutPage->addWidget(m_qcheckboxFocusChangeEnter);
 
-  // Add the first radio button
-  m_qradiobuttonPerTransaction = new QRadioButton(qbuttongroup, "m_per_trans");
-  m_qradiobuttonPerTransaction->setText( i18n("Use one colour per transaction") );
-  qvboxlayout->addWidget(m_qradiobuttonPerTransaction);
-
-  // Add the second radio button
-  m_qradiobuttonOtherRow = new QRadioButton(qbuttongroup, "m_every_other");
-  m_qradiobuttonOtherRow->setText( i18n( "Change colour every other row" ) );
-  qvboxlayout->addWidget(m_qradiobuttonOtherRow);
-  
-  qvboxlayoutPage->addWidget(qbuttongroup);
-#endif
+  m_qcheckboxAutoFillTransaction = new QCheckBox(i18n("Auto fill with previous transaction data"), qwidgetPage);
+  qvboxlayoutPage->addWidget(m_qcheckboxAutoFillTransaction);
 
   // Add a vertical spacer to take up the remaining available space
   QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
@@ -393,9 +415,6 @@ void KSettingsDlg::setPageList()
 
 
 
-
-
-
   // Create a new tab for the filter options
   QWidget *qwidgetFilter = new QWidget(qtabwidget, "filterTab");
 
@@ -428,6 +447,9 @@ void KSettingsDlg::setPageList()
 
   // Add the page to the tab
   qtabwidget->insertTab(qwidgetFilter, i18n("Filter"));
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.register";
 }
 
 /** Read all the settings in from the global KConfig object and set all the
@@ -441,18 +463,10 @@ void KSettingsDlg::configRead()
   this->resize(kconfig->readSizeEntry("Geometry", &qsizeDefaultSize));
 
   kconfig->setGroup("General Options");
-  // FIXME: to be removed
-  // m_bTempStartPrompt = kconfig->readBoolEntry("StartDialog", true);
-  // m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
-  // m_qradiobuttonStartFile->setChecked(!m_bTempStartPrompt);
 
   m_bTempStartPage = kconfig->readBoolEntry("StartLastViewSelected", false);
   m_qradiobuttonStartHome->setChecked(!m_bTempStartPage);
   m_qradiobuttonStartLast->setChecked(m_bTempStartPage);
-  
-  // m_bTempAccountWizard = kconfig->readBoolEntry("NewAccountWizard", true);
-  // m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
-  // m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
 
   m_bTempLedgerLens = kconfig->readBoolEntry("LedgerLens", true);
   m_qcheckboxLedgerLens->setChecked(m_bTempLedgerLens);
@@ -463,17 +477,30 @@ void KSettingsDlg::configRead()
   m_bTempShowNrField = kconfig->readBoolEntry("AlwaysShowNrField", false);
   m_qcheckboxShowNrField->setChecked(m_bTempShowNrField);
 
+  m_bTempAutoIncCheckNumber = kconfig->readBoolEntry("AutoIncCheckNumber", false);
+  m_qcheckboxIncCheckNumber->setChecked(m_bTempAutoIncCheckNumber);
+
   if(m_bTempShowNrField == true) {
     m_bTempTypeToNr = kconfig->readBoolEntry("CopyTypeToNr", false);
     m_qcheckboxTypeToNr->setChecked(m_bTempTypeToNr);
     m_qcheckboxTypeToNr->setEnabled(true);
+    m_qcheckboxIncCheckNumber->setEnabled(!m_bTempTypeToNr);
   } else {
     m_bTempTypeToNr = false;
     m_qcheckboxTypeToNr->setChecked(m_bTempTypeToNr);
     m_qcheckboxTypeToNr->setEnabled(false);
+    m_qcheckboxIncCheckNumber->setEnabled(true);
   }
 
- 
+  m_iTempPricePrecision = kconfig->readNumEntry("PricePrecision", 4);
+  m_qIntPricePrecision->setText(QString("%1").arg(m_iTempPricePrecision));
+
+  m_bTempFocusChangeEnter = kconfig->readBoolEntry("FocusChangeIsEnter", false);
+  m_qcheckboxFocusChangeEnter->setChecked(m_bTempFocusChangeEnter);
+
+  m_bTempAutoFillTransaction = kconfig->readBoolEntry("AutoFillTransaction", false);
+  m_qcheckboxAutoFillTransaction->setChecked(m_bTempAutoFillTransaction);
+
   kconfig->setGroup("List Options");
 
   m_qcolorTempList = KMyMoneyUtils::defaultListColour();
@@ -485,6 +512,10 @@ void KSettingsDlg::configRead()
   m_qcolorTempListGrid = KMyMoneyUtils::defaultGridColour();
   m_kcolorbuttonGrid->setColor(m_qcolorTempListGrid);
 
+  m_qcolorTempRate = QColor(0, 0, 255); // blue is the application default
+  m_qcolorTempRate = kconfig->readColorEntry("listMissingConversionRate", &m_qcolorTempRate);
+  m_kcolorbuttonRate->setColor(m_qcolorTempRate);
+
   QFont qfontDefault = KMyMoneyUtils::headerFont();
   m_qfontTempHeader = kconfig->readFontEntry("listHeaderFont", &qfontDefault);
   m_kfontchooserHeader->setFont(m_qfontTempHeader);
@@ -502,21 +533,17 @@ void KSettingsDlg::configRead()
   m_bTempHideCategory = kconfig->readBoolEntry("HideUnusedCategory", false);
   m_qcheckboxHideCategory->setChecked(m_bTempHideCategory);
 
-#if 0  
-  m_bTempColourPerTransaction = kconfig->readBoolEntry("ColourPerTransaction", true);
-  m_qradiobuttonPerTransaction->setChecked(m_bTempColourPerTransaction);
-  m_qradiobuttonOtherRow->setChecked(!m_bTempColourPerTransaction);
-#endif
-
   QDateTime defaultDate;
   defaultDate.setTime_t(0);
 
   m_qdateTempStart = kconfig->readDateTimeEntry("StartDate", &defaultDate).date();
   m_dateinputStart->setDate(m_qdateTempStart);
 
+#if 0
   m_bTempNormalView = kconfig->readBoolEntry("NormalAccountsView", false);
   m_qradiobuttonNormalView->setChecked(m_bTempNormalView);
   m_qradiobuttonAccountView->setChecked(!m_bTempNormalView);
+#endif
 
   kconfig->setGroup("Homepage Options");
   m_tempHomePageItems = kconfig->readListEntry("Itemlist");
@@ -530,6 +557,9 @@ void KSettingsDlg::configRead()
   m_intSchedulePreview->setEnabled(m_bTempCheckSchedule);
   m_iTempSchedulePreview = kconfig->readNumEntry("CheckSchedulePreview", 0);
   m_intSchedulePreview->setValue(m_iTempSchedulePreview);
+
+  m_onlineQuotesWidget->readConfig();
+  m_encryptionWidget->readConfig();
 }
 
 /** Write out all the settings to the global KConfig object.
@@ -544,6 +574,7 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("listColor", m_kcolorbuttonList->color());
   kconfig->writeEntry("listBGColor", m_kcolorbuttonBack->color());
   kconfig->writeEntry("listGridColor", m_kcolorbuttonGrid->color());
+  kconfig->writeEntry("listMissingConversionRate", m_kcolorbuttonRate->color());
   kconfig->writeEntry("listHeaderFont", m_kfontchooserHeader->font());
   kconfig->writeEntry("listCellFont", m_kfontchooserCell->font());
   kconfig->writeEntry("useSystemFont", m_systemFont->isChecked());
@@ -552,13 +583,9 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("ShowGrid", m_qcheckboxShowGrid->isChecked());
   // kconfig->writeEntry("ColourPerTransaction", m_qradiobuttonPerTransaction->isChecked());
   kconfig->writeEntry("HideUnusedCategory", m_qcheckboxHideCategory->isChecked());
-#if QT_VERSION > 300
   kconfig->writeEntry("StartDate", QDateTime(m_dateinputStart->getQDate()));
-#else
-  kconfig->writeEntry("StartDate", m_dateinputStart->getQDate());
-#endif
 
-  kconfig->writeEntry("NormalAccountsView", m_qradiobuttonNormalView->isChecked());
+  // kconfig->writeEntry("NormalAccountsView", m_qradiobuttonNormalView->isChecked());
 
   kconfig->setGroup("General Options");
   // kconfig->writeEntry("StartDialog", m_qradiobuttonStartPrompt->isChecked());
@@ -567,7 +594,11 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("LedgerLens", m_qcheckboxLedgerLens->isChecked());
   kconfig->writeEntry("TransactionForm", m_qcheckboxTransactionForm->isChecked());
   kconfig->writeEntry("CopyTypeToNr", m_qcheckboxTypeToNr->isChecked());
+  kconfig->writeEntry("AutoIncCheckNumber", m_qcheckboxIncCheckNumber->isChecked());
   kconfig->writeEntry("AlwaysShowNrField", m_qcheckboxShowNrField->isChecked());
+  kconfig->writeEntry("PricePrecision", m_qIntPricePrecision->text());
+  kconfig->writeEntry("FocusChangeIsEnter", m_qcheckboxFocusChangeEnter->isChecked());
+  kconfig->writeEntry("AutoFillTransaction", m_qcheckboxAutoFillTransaction->isChecked());
 
   kconfig->setGroup("Homepage Options");
   kconfig->writeEntry("Itemlist", homePageItems());
@@ -576,6 +607,9 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("CheckSchedules", m_qradiobuttonCheckSchedules->isChecked());
   kconfig->writeEntry("CheckSchedulePreview", m_intSchedulePreview->value());
 
+  m_onlineQuotesWidget->writeConfig();
+  m_encryptionWidget->writeConfig();
+
   kconfig->sync();
 }
 
@@ -597,14 +631,6 @@ void KSettingsDlg::slotOk()
 /** Called on Apply being pressed */
 void KSettingsDlg::slotApply()
 {
-/*
-  int nCount = m_klineeditRowCount->text().toInt();
-  if (nCount <= 0 || nCount >= 4) {
-    KMessageBox::information(this, i18n("The row count has to be between 1 and 3"));
-    m_klineeditRowCount->setFocus();
-    return;
-  }
-*/
   m_bDoneApply = true;
   configWrite();
   emit signalApply();
@@ -620,33 +646,31 @@ void KSettingsDlg::slotCancel()
   kconfig->setGroup("List Options");
   kconfig->writeEntry("listColor", m_qcolorTempList);
   kconfig->writeEntry("listBGColor", m_qcolorTempListBG);
+  kconfig->writeEntry("listGridColor", m_qcolorTempListGrid);
+  kconfig->writeEntry("listMissingConversionRate", m_qcolorTempRate);
   kconfig->writeEntry("listHeaderFont", m_qfontTempHeader);
   kconfig->writeEntry("listCellFont", m_qfontTempCell);
-  // kconfig->writeEntry("RowCount", m_qstringTempRowCount);
   kconfig->writeEntry("ShowGrid", m_bTempShowGrid);
-  // kconfig->writeEntry("ColourPerTransaction", m_bTempColourPerTransaction);
   kconfig->writeEntry("HideUnusedCategory", m_bTempHideCategory);
-#if QT_VERSION > 300
   kconfig->writeEntry("StartDate", QDateTime(m_qdateTempStart));
-#else
-  kconfig->writeEntry("StartDate", m_qdateTempStart);
-#endif
-
-  kconfig->writeEntry("NormalAccountsView", m_bTempNormalView);
 
   kconfig->setGroup("General Options");
-  // kconfig->writeEntry("StartDialog", m_bTempStartPrompt);
   kconfig->writeEntry("StartLastViewSelected", m_bTempStartPage);
-  // kconfig->writeEntry("NewAccountWizard", m_bTempAccountWizard);
   kconfig->writeEntry("LedgerLens", m_bTempLedgerLens);
   kconfig->writeEntry("TransactionForm", m_bTempTransactionForm);
   kconfig->writeEntry("CopyTypeToNr", m_bTempTypeToNr);
+  kconfig->writeEntry("AutoIncCheckNumber", m_bTempAutoIncCheckNumber);
   kconfig->writeEntry("AlwaysShowNrField", m_bTempShowNrField);
   kconfig->writeEntry("CheckSchedule", m_bTempCheckSchedule);
   kconfig->writeEntry("CheckSchedulePreview", m_iTempSchedulePreview);
-  
+  kconfig->writeEntry("PricePrecision", m_iTempPricePrecision);
+  kconfig->writeEntry("FocusChangeIsEnter", m_bTempFocusChangeEnter);
+
   kconfig->setGroup("Homepage Options");
   kconfig->writeEntry("Itemlist", m_tempHomePageItems);
+  m_onlineQuotesWidget->resetConfig();
+  m_encryptionWidget->resetConfig();
+
   kconfig->sync();
 
   if (m_bDoneApply)
@@ -662,38 +686,39 @@ void KSettingsDlg::slotCancel()
 **/
 void KSettingsDlg::slotUser1()
 {
-  // m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
-  // m_qradiobuttonStartFile->setChecked(!m_bTempStartPrompt);
   m_kcolorbuttonList->setColor(m_qcolorTempList);
   m_kcolorbuttonBack->setColor(m_qcolorTempListBG);
+  m_kcolorbuttonGrid->setColor(m_qcolorTempListGrid);
+  m_kcolorbuttonRate->setColor(m_qcolorTempRate);
   m_kfontchooserHeader->setFont(m_qfontTempHeader);
   m_kfontchooserCell->setFont(m_qfontTempCell);
   m_systemFont->setChecked(m_bTempUseSystemFont);
-  // m_klineeditRowCount->setText(m_qstringTempRowCount);
   m_qcheckboxShowGrid->setChecked(m_bTempShowGrid);
-#if 0  
-  m_qradiobuttonPerTransaction->setChecked(m_bTempColourPerTransaction);
-  m_qradiobuttonOtherRow->setChecked(!m_bTempColourPerTransaction);
-#endif  
   m_qcheckboxHideCategory->setChecked(m_bTempHideCategory);
   m_dateinputStart->setDate(m_qdateTempStart);
+#if 0
   m_qradiobuttonNormalView->setChecked(m_bTempNormalView);
   m_qradiobuttonAccountView->setChecked(!m_bTempNormalView);
-  // m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
-  // m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
+#endif
   m_qcheckboxLedgerLens->setChecked(m_bTempLedgerLens);
   m_qcheckboxTransactionForm->setChecked(m_bTempTransactionForm);
   m_qcheckboxTypeToNr->setChecked(m_bTempTypeToNr);
+  m_qcheckboxIncCheckNumber->setChecked(m_bTempAutoIncCheckNumber);
   m_qcheckboxShowNrField->setChecked(m_bTempShowNrField);
+  m_qcheckboxFocusChangeEnter->setChecked(m_bTempFocusChangeEnter);
+  m_qcheckboxAutoFillTransaction->setChecked(m_bTempAutoFillTransaction);
   m_qradiobuttonStartLast->setChecked(m_bTempStartPage);
   m_qradiobuttonStartHome->setChecked(!m_bTempStartPage);
   m_qradiobuttonCheckSchedules->setChecked(m_bTempCheckSchedule);
   m_intSchedulePreview->setEnabled(m_bTempCheckSchedule);
   m_intSchedulePreview->setValue(m_iTempSchedulePreview);
-  
+  m_qIntPricePrecision->setText(QString("%1").arg(m_iTempPricePrecision));
+
   QStringList list = m_tempHomePageItems;
   KMyMoneyUtils::addDefaultHomePageItems(list);
   fillHomePageItems(list);
+  m_onlineQuotesWidget->resetConfig();
+  m_encryptionWidget->resetConfig();
 }
 
 void KSettingsDlg::slotNrFieldToggled(bool state)
@@ -706,6 +731,15 @@ void KSettingsDlg::slotNrFieldToggled(bool state)
   }
 }
 
+void KSettingsDlg::slotTypeToNrToggled(bool state)
+{
+  if(state == true) {
+    m_qcheckboxIncCheckNumber->setEnabled(false);
+  } else {
+    m_qcheckboxIncCheckNumber->setEnabled(true);
+  }
+}
+
 const QStringList KSettingsDlg::homePageItems(void) const
 {
   QStringList list;
@@ -726,7 +760,7 @@ void KSettingsDlg::fillHomePageItems(QStringList& list)
   int w = 0;
   m_homePageList->clear();
   QCheckListItem *sel = 0;
-    
+
   m_upButton->setEnabled(false);
   m_downButton->setEnabled(false);
 
@@ -745,7 +779,7 @@ void KSettingsDlg::fillHomePageItems(QStringList& list)
     item->setOn(enabled);
     if(item->width(fm, m_homePageList, 0) > w)
       w = item->width(fm, m_homePageList, 0);
-      
+
     if(sel == 0)
       sel = item;
     last = item;
@@ -769,7 +803,7 @@ void KSettingsDlg::slotSystemFontToggled(bool state)
 void KSettingsDlg::slotSelectHomePageItem(QListViewItem *item)
 {
   m_upButton->setEnabled(m_homePageList->firstChild() != item);
-  m_downButton->setEnabled(item->nextSibling());  
+  m_downButton->setEnabled(item->nextSibling());
 }
 
 void KSettingsDlg::slotMoveUp(void)
@@ -792,6 +826,28 @@ void KSettingsDlg::slotMoveDown(void)
   }
 }
 
+void KSettingsDlg::setPageSecurity(void)
+{
+#if 0
+  // Create the main frame to hold the widgets
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Security"), i18n("Security settings"),
+    DesktopIcon("kgpg"));
+
+  QGroupBox* groupBox1 = new QGroupBox( qvboxMainFrame, "groupBox1" );
+  groupBox1->setTitle( i18n( "Encryption options" ) );
+
+  groupBox1->setEnabled(KGPGFile::GPGAvailable());
+#endif
+  // Create the main frame to hold the widgets
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Security"), i18n("Data Encryption Settings"),
+    DesktopIcon("kgpg"));
+
+  m_encryptionWidget = new kMyMoneyGPGConfig(qvboxMainFrame,"encryption settings");
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.security";
+}
+
 void KSettingsDlg::setPageSchedule()
 {
   // Create the main frame to hold the widgets
@@ -828,6 +884,9 @@ void KSettingsDlg::setPageSchedule()
   groupBox1Layout->addItem( spacer );
 
   connect(m_qradiobuttonCheckSchedules, SIGNAL(toggled(bool)), m_intSchedulePreview, SLOT(setEnabled(bool)));
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.schedules";
 }
 
 void KSettingsDlg::setPageColour()
@@ -876,8 +935,21 @@ void KSettingsDlg::setPageColour()
   layout4->addWidget( m_kcolorbuttonGrid );
   groupBox2Layout->addLayout( layout4 );
 
+  QHBoxLayout* layout5 = new QHBoxLayout( 0, 0, 6, "layout5");
+
+  QLabel* textLabel2_4 = new QLabel( groupBox2, "textLabel2_4" );
+  textLabel2_4->setText( i18n( "Value with missing conversion rate :" ) );
+  layout5->addWidget( textLabel2_4 );
+
+  m_kcolorbuttonRate = new KColorButton( groupBox2, "m_kcolorbuttonRate" );
+  layout5->addWidget( m_kcolorbuttonRate );
+  groupBox2Layout->addLayout( layout5 );
+
   QSpacerItem* spacer = new QSpacerItem( 20, 31, QSizePolicy::Minimum, QSizePolicy::Expanding );
   groupBox2Layout->addItem( spacer );
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.colours";
 }
 
 void KSettingsDlg::setPageFont()
@@ -908,4 +980,31 @@ void KSettingsDlg::setPageFont()
 
   m_fontTabWidget->changeTab( tab, i18n( "Header Font" ) );
   m_fontTabWidget->changeTab( tab_2, i18n( "Cell Font" ) );
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.fonts";
 }
+
+void KSettingsDlg::setPageOnlineQuotes(void)
+{
+  // Create the main frame to hold the widgets
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Online Quotes"), i18n("Online Quote Settings"),
+    DesktopIcon("network_local"));
+
+  m_onlineQuotesWidget = new kMyMoneyOnlineQuoteConfig(qvboxMainFrame,"online_quotes");
+
+  // set online help reference
+  m_helpAnchor[pageIndex(qvboxMainFrame)] = "details.settings.onlinequote";
+}
+
+void KSettingsDlg::slotHelp(void)
+{
+  QString anchor = m_helpAnchor[activePageIndex()];
+  if(anchor.isEmpty())
+    anchor = "details.settings.general";
+  setHelp(anchor);
+  KDialogBase::slotHelp();
+}
+
+
+#include "ksettingsdlg.moc"
