@@ -18,33 +18,35 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qcheckbox.h>
 #include <qradiobutton.h>
 #include <qbuttongroup.h>
 #include <qcolor.h>
 #include <qfont.h>
 #include <qstringlist.h>
+#include <qmap.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
+
 #include <kdialogbase.h>
 #include <kfontdialog.h>
-#if QT_VERSION > 300
 #include <kcolorbutton.h>
-#else
-#include <kcolorbtn.h>
-#endif
-
 #include <klineedit.h>
 
 class KIntNumInput;
 class KListView;
 class KPushButton;
+class QTabWidget;
+class kMyMoneyOnlineQuoteConfig;
+class kMyMoneyGPGConfig;
 
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "../widgets/kmymoneydateinput.h"
 
+
 /**
   * This class is used to manipulate all the settings available for KMyMoney2.
   * It currently stores the values for the list settings and whether to show
@@ -58,14 +60,16 @@ class KPushButton;
   * @see KDialogBase
   *
   * @author Michael Edwardes 2000-2001
-  * $Id: ksettingsdlg.h,v 1.15.2.2 2004/10/10 12:28:26 ipwizard Exp $
   *
   * @short A class to manipulate the settings needed for running KMyMoney2
-**/
-class KSettingsDlg : public KDialogBase  {
-   Q_OBJECT
+  */
+class KSettingsDlg : public KDialogBase
+{
+  Q_OBJECT
 
 private:
+  QMap<int, QString>         m_helpAnchor;
+
   /** Start prompt dialog */
   QRadioButton *m_qradiobuttonStartPrompt;
   /** Start file */
@@ -74,13 +78,15 @@ private:
   QRadioButton *m_qradiobuttonStartHome;
   /** Start last page */
   QRadioButton *m_qradiobuttonStartLast;
-  
+
   /** Color list */
   KColorButton *m_kcolorbuttonList;
   /** Color background */
   KColorButton *m_kcolorbuttonBack;
   /** Color grid */
   KColorButton *m_kcolorbuttonGrid;
+  /** Missing conversion rate */
+  KColorButton *m_kcolorbuttonRate;
   /** Select font header */
   KFontChooser *m_kfontchooserHeader;
   /** Font cell setting */
@@ -107,21 +113,38 @@ private:
   /* copy type to nr field */
   QCheckBox* m_qcheckboxTypeToNr;
 
+  /* increment check number */
+  QCheckBox* m_qcheckboxIncCheckNumber;
+
   /* hide unused categories */
   QCheckBox* m_qcheckboxHideCategory;
 
   /* always show a nr field in the transaction form */
   QCheckBox* m_qcheckboxShowNrField;
 
+  /* keep changes when selecting different transaction / split */
+  QCheckBox* m_qcheckboxFocusChangeEnter;
+
+  /* auto prefill a transaction with previous data of the same payee */
+  QCheckBox* m_qcheckboxAutoFillTransaction;
+
   /** colour options */
   QRadioButton *m_qradiobuttonPerTransaction;
   QRadioButton *m_qradiobuttonOtherRow;
 
+  /* Online quotes configuration widget */
+  kMyMoneyOnlineQuoteConfig* m_onlineQuotesWidget;
+
+  /* GPG encryption configuration widget */
+  kMyMoneyGPGConfig* m_encryptionWidget;
+
   /** Restrict options */
   kMyMoneyDateInput *m_dateinputStart;
 
+#if 0
   QRadioButton *m_qradiobuttonNormalView;
   QRadioButton *m_qradiobuttonAccountView;
+#endif
 
   QRadioButton *m_qradiobuttonAccountDialog;
   QRadioButton *m_qradiobuttonAccountWizard;
@@ -133,7 +156,10 @@ private:
   KPushButton*  m_upButton;
   KPushButton*  m_downButton;
   int           m_currentItem;
-  
+
+  KLineEdit*    m_qIntPricePrecision;
+  int           m_iTempPricePrecision;
+
   /** Set page general */
   void setPageGeneral();
   /** Set page list settings */
@@ -149,12 +175,17 @@ private:
   /** Set schedule options */
   void setPageSchedule();
 
+  /** Set security options */
+  void setPageSecurity(void);
+
   /** Set colour options */
   void setPageColour();
 
   /** Set font options */
   void setPageFont();
 
+  /** Set online quote options */
+  void setPageOnlineQuotes(void);
 
   /** Write settings */
   void configWrite();
@@ -167,6 +198,7 @@ private:
   QColor m_qcolorTempList;
   QColor m_qcolorTempListBG;
   QColor m_qcolorTempListGrid;
+  QColor m_qcolorTempRate;
   QFont m_qfontTempHeader;
   QFont m_qfontTempCell;
 //  QString m_qstringTempRowCount;
@@ -176,18 +208,24 @@ private:
   bool m_bDoneApply;
   bool m_bTempHideCategory;
   QDate m_qdateTempStart;
-  bool m_bTempNormalView;
+//  bool m_bTempNormalView;
   bool m_bTempAccountWizard;
   bool m_bTempLedgerLens;
   bool m_bTempTransactionForm;
   bool m_bTempTypeToNr;
+  bool m_bTempAutoIncCheckNumber;
   bool m_bTempShowNrField;
   bool m_bTempStartPage;
   bool m_bTempCheckSchedule;
   bool m_bTempUseSystemFont;
+  bool m_bTempFocusChangeEnter;
+  bool m_bTempAutoFillTransaction;
   int  m_iTempSchedulePreview;
   QStringList m_tempHomePageItems;
 
+protected slots:
+  void slotHelp(void);
+
 private slots:
   /** Called when OK pressed */
   void slotOk();
@@ -204,6 +242,9 @@ private slots:
   /** Called when 'Always show nr field' is toggled */
   void slotNrFieldToggled(bool state);
 
+  /** Called when 'Copy Type to Nr field' is toggled */
+  void slotTypeToNrToggled(bool state);
+
   /** Called when 'Use system font' is toggled */
   void slotSystemFontToggled(bool state);
 
@@ -212,7 +253,7 @@ private slots:
   void slotMoveUp(void);
   void slotMoveDown(void);
 
- 
+
 public:
   /**
     * Standard constructor.
