@@ -15,7 +15,7 @@
  ***************************************************************************/
 #ifndef KMYMONEY2_H
 #define KMYMONEY2_H
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -28,17 +28,13 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#if QT_VERSION > 300
 #include <kapplication.h>
-#else
-#include <kapp.h>
-#endif
-
 #include <kmainwindow.h>
 #include <kaccel.h>
 #include <kaction.h>
 #include <kprocess.h>
 #include <kurl.h>
+#include <dcopobject.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -46,12 +42,17 @@
 #include "mymoney/mymoneyobserver.h"
 #include "mymoney/mymoneyscheduled.h"
 
+class QSignalMapper;
 class KProgress;
-class KStartupLogo;
 class KMyMoneyView;
 class MyMoneyQifReader;
+class MyMoneyStatementReader;
+class MyMoneyStatement;
+class MyMoneyOfxStatement;
 class IMyMoneyStorage;
 
+namespace KMyMoneyPlugin { class ImporterPlugin; }
+
 /*! \mainpage KMyMoney Main Page for API documentation.
  *
  * \section intro Introduction
@@ -76,24 +77,31 @@ class IMyMoneyStorage;
   * @author Michael Edwardes 2000-2001
   *
   * @short Main application class.
-**/
-class KMyMoney2App : public KMainWindow, MyMoneyObserver
+  */
+class KMyMoney2App : public KMainWindow, MyMoneyObserver, public DCOPObject
 {
   Q_OBJECT
-
+  K_DCOP
 
 protected slots:
-  void slotKeySettings();
+  /**
+    * This slot re-enables all message for which the "Don't show again"
+    * option had been selected.
+    */
+  void slotEnableMessages(void);
 
+  void slotKeySettings();
+  void slotEditToolbars();
+  void slotNewToolBarConfig();
   /**
     * Called when the user asks for file information.
-  **/
+    */
   void slotFileFileInfo();
 
 
   /**
     * Called when the user asks for the personal information.
-  **/
+    */
   void slotFileViewPersonal();
 
   /**
@@ -102,46 +110,79 @@ protected slots:
     * work.  Calls KMyMoneyView::slotAccountImportAscii.
     *
     * @see MyMoneyAccount
-    **/
+    */
   void slotQifImport();
 
   /**
     * Called when a QIF import is finished.
     */
   void slotQifImportFinished(void);
-  
+
+  /**
+    * Opens a file selector dialog for the user to choose an existing OFX
+    * file from the file system to be imported.  This slot is expected to
+    * be called from the UI.
+    */
+  void slotGncImport(void);
+
+  void slotPluginImport(const QString&);
+
+  void slotPluginImport(const QString& format, const QString& url);
+ 
+  /**
+    * Opens a file selector dialog for the user to choose an existing KMM
+    * statement file from the file system to be imported.  This is for testing
+    * only.  KMM statement files are not designed to be exposed to the user.
+    */
+  void slotStatementImport(void);
+
+  void slotStatementImportFinished(void);
+
+  void slotLoadAccountTemplates(void);
+  void slotSaveAccountTemplates(void);
+
+  void loadAccountTemplates(const QStringList& filelist);
+
   /**
     * Called when the user wishes to export some transaction to a
     * QIF formatted file. An account must be open for this to work.
     * Uses MyMoneyQifWriter() for the actual output.
-    **/
+    */
   void slotQifExport();
 
   /**
     * Open up the application wide settings dialog.
     *
     * @see KSettingsDlg
-  **/
+    */
   void slotSettings();
 
   /** No descriptions */
   void slotFileBackup();
-  void slotHomeView();
-  void slotAccountsView();
-  void slotScheduledView();
-  void slotCategoryView();
-  void slotPayeeView();
+
+  /**
+    * This slot modifies the actions according to the specific view passed
+    * as argument @p view.
+    */
+  void slotSetViewSpecificActions(int view);
 
   void slotShowTipOfTheDay(void);
+
   void slotQifProfileEditor(void);
 
+  void slotShowPreviousView(void);
+
+  void slotShowNextView(void);
+
+  void slotSecurityEditor(void);
+
 public:
   /**
     * This method checks if there is at least one asset or liability account
     * in the current storage object. If not, it starts the new account wizard.
     */
   void createInitialAccount(void);
-    
+
   /**
     * This method returns the last URL used or an empty URL
     * depending on the option setting if the last file should
@@ -160,7 +201,7 @@ public:
 
   /**
     * Destructor
-    **/
+    */
   ~KMyMoney2App();
 
   /** Init wizard dialog */
@@ -169,17 +210,21 @@ public:
   static void progressCallback(int current, int total, const QString&);
 
   void writeLastUsedDir(const QString& directory);
-  QString readLastUsedDir();
+  QString readLastUsedDir() const;
   void writeLastUsedFile(const QString& fileName);
-  QString readLastUsedFile();
- 
+  QString readLastUsedFile() const;
+
+  /**
+    * Returns whether there is an importer available that can handle this file
+    */
+  bool isImportableFile( const KURL& url );
 
   /**
     * This function will be called by the engine when the engine data changed
     * and the application object needs to update it's state.
     */
   virtual void update(const QCString& id);
-  
+
   /**
     * This method is used to update the caption of the application window.
     * It set's the caption to "filename [modified] - KMyMoney".
@@ -190,12 +235,37 @@ public:
     */
   void updateCaption(const bool skipActions = false);
 
+  /**
+    * This method returns a list of all 'other' dcop registered kmymoney processes.
+    * It's a subset of the return of DCOPclient()->registeredApplications().
+    *
+    * @retval QCStringList of process ids
+    */
+  const QCStringList instanceList(void) const;
+
+k_dcop:
+  const QString filename() const;
+
+  void webConnect(const QString&, const QCString& asn_id);
+
 protected:
   /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
    * file
    */
   void saveOptions();
 
+  /**
+    * Creates the interfaces necessary for the plugins to work. Therefore,
+    * this method must be called prior to loadPlugins().
+    */
+  void createInterfaces(void);
+
+  /**
+    * load all available plugins. Make sure you have called createInterfaces()
+    * before you call this one.
+    */
+  void loadPlugins(void);
+
   /** read general Options again and initialize all variables like the recent file list
    */
   void readOptions();
@@ -224,11 +294,13 @@ protected:
   virtual bool queryExit();
 
   void slotCheckSchedules(void);
-  
+
+  virtual void resizeEvent(QResizeEvent*);
+
 public slots:
   /** */
   void slotFileNew();
-  
+
   /** Open a new window */
   void slotFileNewWindow();
 
@@ -262,28 +334,45 @@ public slots:
   /** asks for saving if the file is modified, then closes the actual file */
   void slotFileClose();
 
-  /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
-   * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
-   */
+  /**
+    * closes all open windows by calling close() on each memberList item
+    * until the list is empty, then quits the application.
+    * If queryClose() returns false because the user canceled the
+    * saveModified() dialog, the closing breaks.
+    */
   void slotFileQuit();
 
+  void slotAccountNew(void);
+
   void slotFileConsitencyCheck(void);
-  
-  /** toggles the toolbar
-   */
+
+  /**
+    * fires up the price table editor
+    */
+  void slotPriceDialog(void);
+
+  /**
+    * fires up the currency table editor
+    */
+  void slotCurrencyDialog(void);
+
+  /**
+    * toggles the toolbar
+    */
   void slotViewToolBar();
 
-  /** toggles the statusbar
-   */
+  /**
+    * toggles the statusbar
+    */
   void slotViewStatusBar();
 
   /**
     * changes the statusbar contents for the standard label permanently,
     * used to indicate current actions. Returns the previous value for
     * 'stacked' usage.
-
-   * @param text the text that is displayed in the statusbar
-   */
+    *
+    * @param text the text that is displayed in the statusbar
+    */
   const QString slotStatusMsg(const QString &text);
 
   /**
@@ -298,15 +387,39 @@ public slots:
     * @param current the current value with respect to the initialised
     *                 100% mark
     * @param total the total value (100%)
-  */
+    */
   void slotStatusProgressBar(const int current, const int total = 0);
 
   /** No descriptions */
   void slotProcessExited();
 
+  /**
+    * Called to update stock and currency prices from the user menu
+    */
+  void slotEquityPriceUpdate();
+
+  /**
+    * Imports a KMM statement into the engine, triggering the appropriate
+    * UI to handle account matching, payee creation, and someday
+    * payee and transaction matching.
+    */
+  bool slotStatementImport(const MyMoneyStatement& s);
+
+  /**
+    * Essentially similiar to the above slot, except this will load the file
+    * from disk first, given the URL.
+    */
+  bool slotStatementImport(const QString& url);
+
+  /**
+    * Essentially similiar to the above slot, except this imports the whole
+    * list of statements
+    */
+  bool slotStatementImport(const QValueList<MyMoneyStatement>& list);
+
 private:
-  bool verifyImportedData(void);
-  void slotCommitTransaction(const MyMoneySchedule& schedule, const QDate&);
+  bool verifyImportedData(const MyMoneyAccount& account);
+  bool slotCommitTransaction(const MyMoneySchedule& schedule, const QDate&);
 
 signals:
   /**
@@ -314,7 +427,10 @@ signals:
     * is closed, this signal is also emitted with an empty url.
     */
   void fileLoaded(const KURL& url);
-      
+
+public:
+  KAction* newTransaction;
+
 private:
   /** the configuration object of the application */
 
@@ -331,6 +447,7 @@ private:
   KAction* fileClose;
   KAction* fileCloseWindow;
   KAction* fileQuit;
+  KAction* filePrint;
   KToggleAction* viewToolBar;
   KToggleAction* viewStatusBar;
   KToggleAction* viewTransactionForm;
@@ -343,7 +460,20 @@ private:
   KAction *actionQifImport;
   KAction *actionQifExport;
   KAction *actionFindTransaction;
+  KAction *actionOfxImport;
+  KAction *actionGncImport;
+  KAction *actionStatementImport;
+  KAction *actionAqbImport;
+  KAction* actionLoadTemplate;
+  KAction* actionSaveTemplate;
+  KAction* actionSecurities;
+  KAction* actionCurrencies;
+  KAction* actionPrices;
+  KAction* actionPriceUpdate;
+  KAction* accountReconcile;
 
+  QSignalMapper *m_pluginSignalMapper;
+  QMap<QString,KMyMoneyPlugin::ImporterPlugin*> m_importerPlugins;
 
   enum backupStateE {
     BACKUP_IDLE = 0,
@@ -373,13 +503,13 @@ private:
     * the backup volume.
     */
   bool    m_backupMount;
-  
+
   KProcess proc;
-  
-  // A pointer to the view holding the tabs.
+
+  /// A pointer to the view holding the tabs.
   KMyMoneyView *myMoneyView;
-  
-  // The URL of the file currently being edited when open.
+
+  /// The URL of the file currently being edited when open.
   KURL  fileName;
 
   bool m_startDialog;
@@ -389,15 +519,19 @@ private:
 
   QString m_statusMsg;
 
-  KStartupLogo* m_startLogo;
-
   int m_progressUpdate;
   int m_nextUpdate;
 
   IMyMoneyStorage*  m_engineBackup;
   MyMoneyQifReader* m_reader;
+  MyMoneyStatementReader* m_smtReader;
 
   bool m_bCheckSchedules;
+
+  KToolBarPopupAction*  m_previousViewButton;
+  KToolBarPopupAction*  m_nextViewButton;
+
+  QObject*              m_pluginInterface;
 };
 
 extern  KMyMoney2App *kmymoney2;
