@@ -31,7 +31,9 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "storage/mymoneyseqaccessmgr.h"
+// #include "mymoneycurrency.h"
 #include "mymoneyfile.h"
+#include "mymoneyreport.h"
 #ifndef HAVE_CONFIG_H
 #define VERSION "UNKNOWN"
 #else
@@ -42,10 +44,17 @@
 
 const QCString MyMoneyFile::NotifyClassAccount = "MyMoneyFile::NotifyAccount";
 const QCString MyMoneyFile::NotifyClassPayee = "MyMoneyFile::NotifyPayee";
+const QCString MyMoneyFile::NotifyClassPayeeSet = "MyMoneyFile::NotifyPayeeSet";
 const QCString MyMoneyFile::NotifyClassInstitution = "MyMoneyFile::NotifyInstitution";
 const QCString MyMoneyFile::NotifyClassAccountHierarchy = "MyMoneyFile::NotifyAccountHierarchy";
 const QCString MyMoneyFile::NotifyClassSchedule = "MyMoneyFile::NotifySchedule";
 const QCString MyMoneyFile::NotifyClassAnyChange = "MyMoneyFile::NotifyAnyChange";
+const QCString MyMoneyFile::NotifyClassCurrency = "MyMoneyFile::NotifyCurrency";
+const QCString MyMoneyFile::NotifyClassSecurity = "MyMoneyFile::NotifySecurity";
+const QCString MyMoneyFile::NotifyClassReport = "MyMoneyFile::NotifyReport";
+const QCString MyMoneyFile::NotifyClassPrice = "MyMoneyFile::NotifyPrice";
+
+const QString MyMoneyFile::OpeningBalancesPrefix = "Opening Balances";
 
 // include the following line to get a 'cout' for debug purposes
 // #include <iostream>
@@ -106,7 +115,7 @@ void MyMoneyFile::addInstitution(MyMoneyInstitution& institution)
 
   // automatically notify all observers once this routine is done
   MyMoneyNotifier notifier(this);
-  
+
   m_storage->addInstitution(institution);
 
   addNotification(NotifyClassInstitution);
@@ -131,7 +140,7 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
 
   const MyMoneyTransaction* t = &transaction;
   MyMoneyTransaction tCopy;
-  
+
   // now check the splits
   bool loanAccountAffected = false;
   QValueList<MyMoneySplit>::ConstIterator it_s;
@@ -173,6 +182,10 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
   // and mark all accounts that are referenced
   for(it_s = tr.splits().begin(); it_s != tr.splits().end(); ++it_s) {
     notifyAccountTree((*it_s).accountId());
+    if(!(*it_s).payeeId().isEmpty()) {
+      addNotification((*it_s).payeeId());
+      addNotification(NotifyClassPayee);
+    }
   }
 
   // perform modification
@@ -181,6 +194,10 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
   // and mark all accounts that are referenced
   for(it_s = t->splits().begin(); it_s != t->splits().end(); ++it_s) {
     notifyAccountTree((*it_s).accountId());
+    if(!(*it_s).payeeId().isEmpty()) {
+      addNotification((*it_s).payeeId());
+      addNotification(NotifyClassPayee);
+    }
   }
   addNotification(NotifyClassAccount);
 }
@@ -205,7 +222,7 @@ void MyMoneyFile::modifyAccount(const MyMoneyAccount& account)
   if(acc.institutionId() != account.institutionId()) {
     addNotification(acc.institutionId());
     addNotification(account.institutionId());
-    addNotification(NotifyClassInstitution);      
+    addNotification(NotifyClassInstitution);
   }
 
   m_storage->modifyAccount(account);
@@ -225,6 +242,7 @@ const MyMoneyAccount::accountTypeE MyMoneyFile::accountGroup(MyMoneyAccount::acc
     case MyMoneyAccount::MoneyMarket:
     case MyMoneyAccount::CertificateDep:
     case MyMoneyAccount::AssetLoan:
+    case MyMoneyAccount::Stock:
       return MyMoneyAccount::Asset;
 
     case MyMoneyAccount::CreditCard:
@@ -247,6 +265,13 @@ void MyMoneyFile::reparentAccount(MyMoneyAccount &account, MyMoneyAccount& paren
   if(accountGroup(account.accountType()) == accountGroup(parent.accountType())
   || (account.accountType() == MyMoneyAccount::Income && parent.accountType() == MyMoneyAccount::Expense)
   || (account.accountType() == MyMoneyAccount::Expense && parent.accountType() == MyMoneyAccount::Income)) {
+
+    if(account.accountType() == MyMoneyAccount::Stock && parent.accountType() != MyMoneyAccount::Investment)
+      throw new MYMONEYEXCEPTION("Unable to reparent Stock to non-investment account");
+
+    if(parent.accountType() == MyMoneyAccount::Investment && account.accountType() != MyMoneyAccount::Stock)
+      throw new MYMONEYEXCEPTION("Unable to reparent non-stock to investment account");
+
     // automatically notify all observers once this routine is done
     MyMoneyNotifier notifier(this);
 
@@ -271,27 +296,14 @@ const MyMoneyInstitution& MyMoneyFile::institution(const QCString& id) const
   return m_storage->institution(id);
 }
 
-const MyMoneyAccount& MyMoneyFile::account(const QCString& id) const
+const MyMoneyAccount MyMoneyFile::account(const QCString& id) const
 {
-/*  It would be nice if this works, but it causes a seg fault.
-    because we're returning a reference to a temporary.
-	MyMoneyAccount account;
-  if (m_storage->isStandardAccount(id))
-	{
-	  if (id == STD_ACC_LIABILITY)
-		  account = liability();
-		else if (id == STD_ACC_ASSET)
-			account = asset();
-		else if (id == STD_ACC_EXPENSE)
-			account = expense();
-	  else if (id == STD_ACC_INCOME)
-			account = income();
-		return account;
-	}
-*/	
   checkStorage();
 
-  return m_storage->account(id);
+  MyMoneyAccount acc;
+  acc = m_storage->account(id);
+  ensureDefaultCurrency(acc);
+  return acc;
 }
 
 void MyMoneyFile::removeTransaction(const MyMoneyTransaction& transaction)
@@ -308,6 +320,10 @@ void MyMoneyFile::removeTransaction(const MyMoneyTransaction& transaction)
   // and mark all accounts that are referenced
   for(it_s = tr.splits().begin(); it_s != tr.splits().end(); ++it_s) {
     notifyAccountTree((*it_s).accountId());
+    if(!(*it_s).payeeId().isEmpty()) {
+      addNotification((*it_s).payeeId());
+      addNotification(NotifyClassPayee);
+    }
   }
   addNotification(NotifyClassAccount);
 
@@ -380,9 +396,9 @@ void MyMoneyFile::removeInstitution(const MyMoneyInstitution& institution)
   MyMoneyNotifier notifier(this);
 
   addNotification(institution.id());
-  
+
   m_storage->removeInstitution(institution);
-  
+
   addNotification(NotifyClassInstitution);
 }
 
@@ -414,14 +430,12 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
   if(account.accountType() == MyMoneyAccount::UnknownAccountType)
     throw new MYMONEYEXCEPTION("Account has invalid type");
 
-  if(account.file() != 0)
-    throw new MYMONEYEXCEPTION("File pointer must be 0 for new account");
-
   // make sure, that the parent account exists
   // if not, an exception is thrown. If it exists,
   // get a copy of the current data
   MyMoneyFile::account(parent.id());
 
+#if 0
   // make sure that no account with the same name and type exist
   // asset andliability as well as income and expense are considered
   // the same type in this aspect.
@@ -430,6 +444,8 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
     case MyMoneyAccount::Expense:
       if(!categoryToAccount(account.name()).isEmpty())
         throw new MYMONEYEXCEPTION("Account with that name already exists");
+      // for now, we only support income/expense accounts in the base currency
+      account.setCurrencyId(baseCurrency().id());
       break;
 
     default:
@@ -437,6 +453,7 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
         throw new MYMONEYEXCEPTION("Account with that name already exists");
       break;
   }
+#endif
 
   // FIXME: make sure, that the parent has the same type
   // I left it out here because I don't know, if there is
@@ -444,6 +461,20 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
   // class asset. It certainly does not make sense to create an
   // expense account under an income account. Maybe it does, I don't know.
 
+  // We enforce, that a stock account can never be a parent and
+  // that the parent for a stock account must be an investment. Also,
+  // an investment cannot have another investment account as it's parent
+  if(parent.accountType() == MyMoneyAccount::Stock)
+    throw new MYMONEYEXCEPTION("Stock account cannot be parent account");
+
+  if(account.accountType() == MyMoneyAccount::Stock
+  && parent.accountType() != MyMoneyAccount::Investment)
+    throw new MYMONEYEXCEPTION("Stock account must have investment account as parent ");
+
+  if(account.accountType() != MyMoneyAccount::Stock
+  && parent.accountType() == MyMoneyAccount::Investment)
+    throw new MYMONEYEXCEPTION("Investment account can only have stock accounts as children");
+
   // automatically notify all observers once this routine is done
   MyMoneyNotifier notifier(this);
 
@@ -461,12 +492,17 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
 
   account.setParentAccountId(parent.id());
 
-  m_storage->newAccount(account);
+  MyMoneyMoney openingBalance(account.openingBalance());
+  account.setOpeningBalance(MyMoneyMoney(0,1));
+
+  m_storage->addAccount(account);
   m_storage->addAccount(parent, account);
 
   if(account.institutionId().length() != 0)
     m_storage->addAccount(institution, account);
 
+  createOpeningBalanceTransaction(account, openingBalance);
+
   // parse the complete account tree and collect all
   // account and institution ids and also the pseudo account class
   notifyAccountTree(account.id());
@@ -474,6 +510,151 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
   addNotification(NotifyClassAccountHierarchy);
 }
 
+void MyMoneyFile::createOpeningBalanceTransaction(const MyMoneyAccount& acc, const MyMoneyMoney& balance)
+{
+  // if the opening balance is not zero, we need
+  // to create the respective transaction
+  if(!balance.isZero()) {
+    MyMoneySecurity currency = security(acc.currencyId());
+    MyMoneyAccount openAcc = openingBalanceAccount(currency);
+
+    if(openAcc.openingDate() > acc.openingDate()) {
+      openAcc.setOpeningDate(acc.openingDate());
+      modifyAccount(openAcc);
+    }
+
+    MyMoneyTransaction t;
+    MyMoneySplit s;
+
+    t.setPostDate(acc.openingDate());
+    t.setCommodity(acc.currencyId());
+
+    s.setAccountId(acc.id());
+    s.setShares(balance);
+    s.setValue(balance);
+    t.addSplit(s);
+
+    s.setId(QCString());
+    s.setAccountId(openAcc.id());
+    s.setShares(-balance);
+    s.setValue(-balance);
+    t.addSplit(s);
+
+    addTransaction(t);
+  }
+}
+
+QCString MyMoneyFile::openingBalanceTransaction(const MyMoneyAccount& acc) const
+{
+  QCString result;
+
+  MyMoneySecurity currency = security(acc.currencyId());
+  MyMoneyAccount openAcc;
+
+  try
+  {
+    openAcc = openingBalanceAccount(currency);
+  }
+  catch(MyMoneyException *e)
+  {
+    delete e;
+    return result;
+  }
+
+  // Iterate over all the opening balance transactions for this currency
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(openAcc.id());
+  QValueList<MyMoneyTransaction> transactions = transactionList(filter);
+  QValueList<MyMoneyTransaction>::const_iterator it_t = transactions.begin();
+  while ( it_t != transactions.end() )
+  {
+    try
+    {
+      // Test whether the transaction also includes a split into
+      // this account
+      (*it_t).splitByAccount(acc.id(), true /*match*/);
+
+      // If so, we have a winner!
+      result = (*it_t).id();
+      break;
+    }
+    catch(MyMoneyException *e)
+    {
+      // If not, keep searching
+      ++it_t;
+      delete e;
+    }
+  }
+
+  return result;
+}
+
+const MyMoneyAccount MyMoneyFile::openingBalanceAccount(const MyMoneySecurity& security)
+{
+  if(!security.isCurrency())
+    throw new MYMONEYEXCEPTION("Opening balance for non currencies not supported");
+
+  try
+  {
+    return openingBalanceAccount_internal(security);
+  }
+  catch(MyMoneyException *e)
+  {
+    delete e;
+    return createOpeningBalanceAccount(security);
+  }
+}
+
+const MyMoneyAccount MyMoneyFile::openingBalanceAccount(const MyMoneySecurity& security) const
+{
+  return openingBalanceAccount_internal(security);
+}
+
+const MyMoneyAccount MyMoneyFile::openingBalanceAccount_internal(const MyMoneySecurity& security) const
+{
+  if(!security.isCurrency())
+    throw new MYMONEYEXCEPTION("Opening balance for non currencies not supported");
+
+  MyMoneyAccount acc;
+  QRegExp match(QString("^%1").arg(MyMoneyFile::OpeningBalancesPrefix));
+
+  QValueList<MyMoneyAccount> accounts;
+  QValueList<MyMoneyAccount>::Iterator it;
+
+  accounts = accountList(equity().accountList(), true);
+
+  for(it = accounts.begin(); it != accounts.end(); ++it) {
+    if(match.search((*it).name()) != -1) {
+      if((*it).currencyId() == security.id()) {
+        acc = *it;
+        break;
+      }
+    }
+  }
+
+  if(acc.id().isEmpty()) {
+    throw new MYMONEYEXCEPTION(QString("No opening balance account for %1").arg(security.tradingSymbol()));
+  }
+
+  return acc;
+}
+
+const MyMoneyAccount MyMoneyFile::createOpeningBalanceAccount(const MyMoneySecurity& security)
+{
+  MyMoneyAccount acc;
+  QString name = MyMoneyFile::OpeningBalancesPrefix;
+  if(security.id() != baseCurrency().id()) {
+    name += QString(" (%1)").arg(security.id());
+  }
+  acc.setName(name);
+  acc.setAccountType(MyMoneyAccount::Equity);
+  acc.setCurrencyId(security.id());
+
+  MyMoneyAccount parent = equity();
+  this->addAccount(acc, parent);
+  return acc;
+}
+
 void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
 {
   checkStorage();
@@ -511,7 +692,7 @@ void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
     for(it_s = transaction.splits().begin(); it_s != transaction.splits().end(); ++it_s) {
       if((*it_s).action() == MyMoneySplit::ActionTransfer) {
         MyMoneyAccount acc = MyMoneyFile::account((*it_s).accountId());
-        
+
         if(acc.accountGroup() == MyMoneyAccount::Asset
         || acc.accountGroup() == MyMoneyAccount::Liability) {
           MyMoneySplit s = (*it_s);
@@ -521,13 +702,22 @@ void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
       }
     }
   }
-  
+
+  // check that we have a commodity
+  if(transaction.commodity().isEmpty()) {
+    transaction.setCommodity(baseCurrency().id());
+  }
+
   // then add the transaction to the file global pool
   m_storage->addTransaction(transaction);
 
   // scan the splits again to update notification list
   for(it_s = transaction.splits().begin(); it_s != transaction.splits().end(); ++it_s) {
     notifyAccountTree((*it_s).accountId());
+    if(!(*it_s).payeeId().isEmpty()) {
+      addNotification((*it_s).payeeId());
+      addNotification(NotifyClassPayee);
+    }
   }
   addNotification(NotifyClassAccount);
 }
@@ -556,6 +746,7 @@ void MyMoneyFile::addPayee(MyMoneyPayee& payee)
   m_storage->addPayee(payee);
 
   addNotification(NotifyClassPayee);
+  addNotification(NotifyClassPayeeSet);
 }
 
 const MyMoneyPayee MyMoneyFile::payee(const QCString& id) const
@@ -594,14 +785,14 @@ void MyMoneyFile::removePayee(const MyMoneyPayee& payee)
   m_storage->removePayee(payee);
 
   addNotification(NotifyClassPayee);
+  addNotification(NotifyClassPayeeSet);
 }
 
-
-const QValueList<MyMoneyAccount> MyMoneyFile::accountList(void) const
+const QValueList<MyMoneyAccount> MyMoneyFile::accountList(const QCStringList& idlist, const bool recursive) const
 {
   checkStorage();
 
-  return m_storage->accountList();
+  return m_storage->accountList(idlist, recursive);
 }
 
 const QValueList<MyMoneyInstitution> MyMoneyFile::institutionList(void) const
@@ -650,32 +841,58 @@ const unsigned int MyMoneyFile::accountCount(void) const
   return m_storage->accountCount();
 }
 
+void MyMoneyFile::ensureDefaultCurrency(MyMoneyAccount& acc) const
+{
+  if(acc.currencyId().isEmpty()) {
+    if(!baseCurrency().id().isEmpty())
+      acc.setCurrencyId(baseCurrency().id());
+    // m_storage->modifyAccount(acc);
+  }
+}
+
 const MyMoneyAccount MyMoneyFile::liability(void) const
 {
   checkStorage();
 
-  return m_storage->liability();
+  MyMoneyAccount acc = m_storage->liability();
+  ensureDefaultCurrency(acc);
+  return acc;
 }
 
 const MyMoneyAccount MyMoneyFile::asset(void) const
 {
   checkStorage();
 
-  return m_storage->asset();
+  MyMoneyAccount acc = m_storage->asset();
+  ensureDefaultCurrency(acc);
+  return acc;
 }
 
 const MyMoneyAccount MyMoneyFile::expense(void) const
 {
   checkStorage();
 
-  return m_storage->expense();
+  MyMoneyAccount acc = m_storage->expense();
+  ensureDefaultCurrency(acc);
+  return acc;
 }
 
 const MyMoneyAccount MyMoneyFile::income(void) const
 {
   checkStorage();
 
-  return m_storage->income();
+  MyMoneyAccount acc = m_storage->income();
+  ensureDefaultCurrency(acc);
+  return acc;
+}
+
+const MyMoneyAccount MyMoneyFile::equity(void) const
+{
+  checkStorage();
+
+  MyMoneyAccount acc = m_storage->equity();
+  ensureDefaultCurrency(acc);
+  return acc;
 }
 
 const unsigned int MyMoneyFile::transactionCount(const QCString& account) const
@@ -699,18 +916,168 @@ const unsigned int MyMoneyFile::institutionCount(void) const
   return m_storage->institutionCount();
 }
 
-const MyMoneyMoney MyMoneyFile::balance(const QCString& id) const
+const MyMoneyMoney MyMoneyFile::balance(const QCString& id, const QDate& date) const
 {
   checkStorage();
 
-  return m_storage->balance(id);
+  return m_storage->balance(id, date);
 }
 
-const MyMoneyMoney MyMoneyFile::totalBalance(const QCString& id) const
+const MyMoneyMoney MyMoneyFile::totalBalance(const QCString& id, const QDate& date) const
 {
   checkStorage();
 
-  return m_storage->totalBalance(id);
+  return m_storage->totalBalance(id, date);
+}
+
+void MyMoneyFile::warningMissingRate(const QCString& fromId, const QCString& toId) const
+{
+  MyMoneySecurity from, to;
+  try {
+    from = security(fromId);
+    to = security(toId);
+    qWarning("Missing price info for conversion from %s to %s", from.name().latin1(), to.name().latin1());
+
+  } catch(MyMoneyException *e) {
+    qFatal("Missing security caught in MyMoneyFile::warningMissingRate(): %s(%ld) %s", e->file().data(), e->line(), e->what().data());
+    delete e;
+  }
+}
+
+const bool MyMoneyFile::accountValueValid(const QCString& id) const
+{
+  bool result = true;
+  try {
+    // if the balance is zero, we don't care about conversion. It must be 0
+    // and thus the currency / security doesn't matter. It will remain 0
+    // and thus the value is valid.
+    if(!balance(id).isZero()) {
+      MyMoneyAccount acc;
+
+      acc = account(id);
+
+      if(acc.currencyId() != baseCurrency().id()) {
+        // if the currency of the account differs from the base currency,
+        // we have to distinguish between 'monetary' and 'stock' accounts
+        // here. For monetary accounts, we assume that there is a direct price
+        // information available (currency exchange rate). For stock this is
+        // different, as stocks might be traded in a different currency than
+        // the base currency. So we convert the price twice in this case.
+
+        if(acc.accountType() == MyMoneyAccount::Stock) {
+          MyMoneySecurity security = this->security(acc.currencyId());
+          result = price(acc.currencyId(), security.tradingCurrency()).isValid();
+          if(!result) {
+            warningMissingRate(acc.currencyId(), security.tradingCurrency());
+          }
+          // if the trading currency differs from the base currency, we
+          // calculate the conversion between these two currencies now.
+          if(result == true && security.tradingCurrency() != baseCurrency().id()) {
+            result = price(security.tradingCurrency(), baseCurrency().id()).isValid();
+            if(!result) {
+              warningMissingRate(security.tradingCurrency(), baseCurrency().id());
+            }
+          }
+        } else {
+          result = price(acc.currencyId(), baseCurrency().id()).isValid();
+          if(!result) {
+            warningMissingRate(acc.currencyId(), baseCurrency().id());
+          }
+        }
+      }
+    }
+  } catch(MyMoneyException *e) {
+    qDebug("MyMoneyFile::totalValueValid: %s thrown in %s line %ld",
+      e->what().data(), e->file().data(), e->line());
+    delete e;
+  }
+  return result;
+}
+
+const bool MyMoneyFile::totalValueValid(const QCString& id) const
+{
+  QCStringList accounts;
+  QCStringList::ConstIterator it_a;
+
+  bool result = accountValueValid(id);
+  try {
+    MyMoneyAccount acc;
+
+    acc = this->account(id);
+    accounts = acc.accountList();
+
+    for(it_a = accounts.begin(); result == true && it_a != accounts.end(); ++it_a) {
+      result = totalValueValid(*it_a);
+    }
+  } catch(MyMoneyException *e) {
+    qDebug("MyMoneyFile::totalValueValid: %s thrown in %s line %ld",
+      e->what().data(), e->file().data(), e->line());
+    delete e;
+  }
+  return result;
+}
+
+const MyMoneyMoney MyMoneyFile::accountValue(const QCString& id) const
+{
+  MyMoneyMoney result(balance(id));
+  try {
+    MyMoneyAccount acc;
+
+    acc = this->account(id);
+
+    // we don't have to convert anything, if we're already on base currency
+    if(acc.currencyId() != baseCurrency().id()) {
+      // if the currency of the account differs from the base currency,
+      // we have to distinguish between 'monetary' and 'stock' accounts
+      // here. For monetary accounts, we assume that there is a direct price
+      // information available (currency exchange rate). For stock this is
+      // different, as stocks might be traded in a different currency than
+      // the base currency. So we convert the price twice in this case.
+
+      if(acc.accountType() == MyMoneyAccount::Stock) {
+        MyMoneySecurity security = this->security(acc.currencyId());
+        MyMoneyPrice price = this->price(acc.currencyId(), security.tradingCurrency());
+        result = result * price.rate();
+        // if the trading currency differs from the base currency, we
+        // calculate the conversion between these two currencies now.
+        if(security.tradingCurrency() != baseCurrency().id()) {
+          price = this->price(security.tradingCurrency(), baseCurrency().id());
+          result = result * price.rate();
+        }
+      } else {
+        MyMoneyPrice price = this->price(acc.currencyId(), baseCurrency().id());
+        result = result * price.rate();
+      }
+    }
+  } catch(MyMoneyException *e) {
+    qDebug("MyMoneyFile::accountValue: %s thrown in %s line %ld",
+      e->what().data(), e->file().data(), e->line());
+    delete e;
+  }
+  return result;
+}
+
+const MyMoneyMoney MyMoneyFile::totalValue(const QCString& id) const
+{
+  QCStringList accounts;
+  QCStringList::ConstIterator it_a;
+
+  MyMoneyMoney result(accountValue(id));
+  try {
+    MyMoneyAccount acc;
+
+    acc = account(id);
+    accounts = acc.accountList();
+
+    for(it_a = accounts.begin(); it_a != accounts.end(); ++it_a) {
+      result += totalValue(*it_a);
+    }
+  } catch(MyMoneyException *e) {
+    qDebug("MyMoneyFile::totalValue: %s thrown in %s line %ld",
+      e->what().data(), e->file().data(), e->line());
+    delete e;
+  }
+  return result;
 }
 
 void MyMoneyFile::attach(const QCString& id, MyMoneyObserver* observer)
@@ -749,15 +1116,18 @@ void MyMoneyFile::notify(void)
 {
   if(!m_suspendNotify) {
     QMap<QCString, bool>::ConstIterator it;
-    for(it = m_notificationList.begin(); it != m_notificationList.end(); ++it) {
+    // keep a local copy so that the called update()
+    // members can modify the original list
+    QMap<QCString, bool> list = m_notificationList;
+    for(it = list.begin(); it != list.end(); ++it) {
       notify(it.key());
     }
 
-    if(m_notificationList.count() > 0)
+    if(list.count() > 0)
       notify(NotifyClassAnyChange);
-      
+
     clearNotification();
-  }  
+  }
 }
 
 void MyMoneyFile::notifyAccountTree(const QCString& id)
@@ -883,14 +1253,24 @@ void MyMoneyFile::setValue(const QCString& key, const QString& val)
 {
   checkStorage();
 
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
   m_storage->setValue(key, val);
+
+  addNotification(NotifyClassAnyChange);
 }
 
 void MyMoneyFile::deletePair(const QCString& key)
 {
   checkStorage();
 
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
   m_storage->deletePair(key);
+
+  addNotification(NotifyClassAnyChange);
 }
 
 void MyMoneyFile::addSchedule(MyMoneySchedule& sched)
@@ -957,21 +1337,21 @@ const QStringList MyMoneyFile::consistencyCheck(void)
   QValueList<MyMoneyAccount>::Iterator it_a;
   QCStringList accountRebuild;
   QCStringList::ConstIterator it_c;
-  
+
   MyMoneyAccount parent;
   MyMoneyAccount child;
   MyMoneyAccount toplevel;
   MyMoneyAccount::accountTypeE group;
-  
+
   QCString parentId;
   QStringList rc;
 
   int problemCount = 0;
   QString problemAccount;
-        
+
   // check that we have a storage object
   checkStorage();
-  
+
   // automatically notify all observers once this routine is done
   MyMoneyNotifier notifier(this);
 
@@ -982,13 +1362,13 @@ const QStringList MyMoneyFile::consistencyCheck(void)
   list << MyMoneyFile::instance()->liability();
   list << MyMoneyFile::instance()->income();
   list << MyMoneyFile::instance()->expense();
-    
+
   for(it_a = list.begin(); it_a != list.end(); ++it_a) {
     // no more checks for standard accounts
     if(isStandardAccount((*it_a).id())) {
       continue;
     }
-    
+
     group = accountGroup((*it_a).accountType());
     switch(group) {
       case MyMoneyAccount::Asset:
@@ -1007,7 +1387,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
         qWarning("%s:%d This should never happen!", __FILE__ , __LINE__);
         break;
     }
-      
+
     // check that the parent exists
     parentId = (*it_a).parentAccountId();
     try {
@@ -1045,9 +1425,9 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       rc << QString("  * The parent with id %1 does not exist anymore.").arg(parentId);
       rc << QString("    New parent account is the top level account '%1'.").arg(toplevel.name());
       (*it_a).setParentAccountId(toplevel.id());
-      
+
       addNotification((*it_a).id());
-            
+
       // make sure to rebuild the sub-accounts of the top account
       if(accountRebuild.contains(toplevel.id()) == 0)
         accountRebuild << toplevel.id();
@@ -1098,8 +1478,8 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       }
     }
   }
-  
-  // reconstruct the lists  
+
+  // reconstruct the lists
   for(it_a = list.begin(); it_a != list.end(); ++it_a) {
     QValueList<MyMoneyAccount>::Iterator it;
     parentId = (*it_a).parentAccountId();
@@ -1128,16 +1508,15 @@ const QStringList MyMoneyFile::consistencyCheck(void)
 
   addNotification(NotifyClassAccount);
   addNotification(NotifyClassAccountHierarchy);
-  addNotification(NotifyClassAnyChange);
-  
+
   // add more checks here
-  
+
   if(problemCount == 0)
     rc << "Finish! Data is consistent.";
   else
     rc << QString("Finish! %1 problems corrected. Data is consistent.")
             .arg(QString::number(problemCount));
-    
+
   return rc;
 }
 
@@ -1145,10 +1524,10 @@ QCString MyMoneyFile::createCategory(const MyMoneyAccount& base, const QString&
 {
   MyMoneyAccount parent = base;
   QString categoryText;
-  
+
   if(base.id() != expense().id() && base.id() != income().id())
     throw MYMONEYEXCEPTION("Invalid base category");
-    
+
   QStringList subAccounts = QStringList::split(CATEGORY_SEPERATOR, name);
   QStringList::Iterator it;
   for (it = subAccounts.begin(); it != subAccounts.end(); ++it)
@@ -1205,8 +1584,346 @@ void MyMoneyFile::suspendNotify(const bool state)
 {
   bool prevState = m_suspendNotify;
   m_suspendNotify = state;
-  
+
   if(state == false && prevState == true)
     notify();
   // qDebug("Notification turned %s", state ? "off" : "on");
 }
+
+
+void MyMoneyFile::addSecurity(MyMoneySecurity& security)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->addSecurity(security);
+  addNotification(NotifyClassSecurity);
+}
+
+void MyMoneyFile::modifySecurity(const MyMoneySecurity& security)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->modifySecurity(security);
+  addNotification(security.id());
+  addNotification(NotifyClassSecurity);
+}
+
+void MyMoneyFile::removeSecurity(const MyMoneySecurity& security)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->removeSecurity(security);
+  addNotification(NotifyClassSecurity);
+}
+
+const MyMoneySecurity MyMoneyFile::security(const QCString& id) const
+{
+  if(id.isEmpty())
+    return baseCurrency();
+
+  checkStorage();
+
+  MyMoneySecurity e = m_storage->security(id);
+  if(e.id().isEmpty())
+    e = m_storage->currency(id);
+  return e;
+}
+
+const QValueList<MyMoneySecurity> MyMoneyFile::securityList(void) const
+{
+  checkStorage();
+
+  return m_storage->securityList();
+}
+
+void MyMoneyFile::addCurrency(const MyMoneySecurity& currency)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->addCurrency(currency);
+  addNotification(NotifyClassCurrency);
+}
+
+void MyMoneyFile::modifyCurrency(const MyMoneySecurity& currency)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->modifyCurrency(currency);
+  addNotification(NotifyClassCurrency);
+}
+
+void MyMoneyFile::removeCurrency(const MyMoneySecurity& currency)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->removeCurrency(currency);
+  addNotification(NotifyClassCurrency);
+}
+
+const MyMoneySecurity MyMoneyFile::currency(const QCString& id) const
+{
+  if(id.isEmpty())
+    return baseCurrency();
+
+  checkStorage();
+  return m_storage->currency(id);
+}
+
+const QValueList<MyMoneySecurity> MyMoneyFile::currencyList(void) const
+{
+  checkStorage();
+
+  return m_storage->currencyList();
+}
+
+const MyMoneySecurity MyMoneyFile::baseCurrency(void) const
+{
+  QCString id = QCString(value("kmm-baseCurrency"));
+  if(id.isEmpty())
+    return MyMoneySecurity();
+
+  return currency(id);
+}
+
+void MyMoneyFile::setBaseCurrency(const MyMoneySecurity& curr)
+{
+  // make sure the currency exists
+  MyMoneySecurity c = currency(curr.id());
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  setValue("kmm-baseCurrency", curr.id());
+
+  addNotification(NotifyClassCurrency);
+}
+
+void MyMoneyFile::addPrice(const MyMoneyPrice& price)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->addPrice(price);
+
+  addNotification(NotifyClassPrice);
+  addNotification(price.from());
+  addNotification(price.to());
+}
+
+void MyMoneyFile::removePrice(const MyMoneyPrice& price)
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  addNotification(NotifyClassPrice);
+  addNotification(price.from());
+  addNotification(price.to());
+
+  m_storage->removePrice(price);
+}
+
+const MyMoneyPrice MyMoneyFile::price(const QCString& fromId, const QCString& toId, const QDate& date, const bool exactDate) const
+{
+  checkStorage();
+
+  QCString to(toId);
+  if(to.isEmpty())
+    to = value("kmm-baseCurrency");
+  // if some id is missing, we can return an empty price object
+  if(fromId.isEmpty() || to.isEmpty())
+    return MyMoneyPrice();
+
+  MyMoneyPrice rc = m_storage->price(fromId, to, date, exactDate);
+  if(!rc.isValid()) {
+    rc = m_storage->price(to, fromId, date, exactDate);
+  }
+  return rc;
+}
+
+const MyMoneyPriceList MyMoneyFile::priceList(void) const
+{
+  checkStorage();
+
+  return m_storage->priceList();
+}
+
+const bool MyMoneyFile::hasAccount(const QCString& id, const QString& name) const
+{
+  checkStorage();
+
+  MyMoneyAccount acc = m_storage->account(id);
+  QCStringList list = acc.accountList();
+  QCStringList::ConstIterator it;
+  bool rc = false;
+  for(it = list.begin(); rc == false && it != list.end(); ++it) {
+    MyMoneyAccount a = m_storage->account(*it);
+    if(a.name() == name)
+      rc = true;
+  }
+  return rc;
+}
+
+const QValueList<MyMoneyReport> MyMoneyFile::reportList( void ) const
+{
+  checkStorage();
+
+  return m_storage->reportList();
+}
+
+void MyMoneyFile::addReport( MyMoneyReport& report )
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->addReport( report );
+
+  addNotification(NotifyClassReport);
+}
+
+void MyMoneyFile::modifyReport( const MyMoneyReport& report )
+{
+  checkStorage();
+
+  // automatically notify all observers once this routine is done
+  MyMoneyNotifier notifier(this);
+
+  m_storage->modifyReport( report );
+
+  addNotification(NotifyClassReport);
+}
+
+unsigned MyMoneyFile::countReports(void) const
+{
+  checkStorage();
+
+  return m_storage->countReports();
+}
+
+MyMoneyReport MyMoneyFile::report( const QCString& id ) const
+{
+  checkStorage();
+
+  return m_storage->report(id);
+}
+
+void MyMoneyFile::removeReport(const MyMoneyReport& report)
+{
+  checkStorage();
+  MyMoneyNotifier notifier(this);
+
+  m_storage->removeReport(report);
+
+  addNotification(NotifyClassReport);
+}
+
+const bool MyMoneyFile::isReferenced(const MyMoneySecurity& security) const
+{
+  // check the account list
+  QValueList<MyMoneyAccount> list_a = accountList();
+  QValueList<MyMoneyAccount>::ConstIterator it_a;
+  for(it_a = list_a.begin(); it_a != list_a.end(); ++it_a) {
+    if((*it_a).currencyId() == security.id())
+      return true;
+  }
+
+  // check the security list
+  QValueList<MyMoneySecurity> list_s = securityList();
+  list_s += currencyList();
+  QValueList<MyMoneySecurity>::ConstIterator it_s;
+  for(it_s = list_s.begin(); it_s != list_s.end(); ++it_s) {
+    // don't check myself
+    if((*it_s).id() == security.id())
+      continue;
+    if((*it_s).tradingCurrency() == security.id())
+      return true;
+  }
+
+  // check the price list
+  MyMoneyPriceList list_p = priceList();
+  MyMoneyPriceList::ConstIterator it_p;
+  for(it_p = list_p.begin(); it_p != list_p.end(); ++it_p) {
+    if(it_p.key().first == security.id()
+    || it_p.key().second == security.id())
+      return true;
+  }
+  return false;
+}
+
+const bool MyMoneyFile::checkNoUsed(const QCString& accId, const QString& no) const
+{
+  // by definition, an empty number is not used
+  if(no.isEmpty())
+    return false;
+
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(accId);
+  QValueList<MyMoneyTransaction> transactions = transactionList(filter);
+  QValueList<MyMoneyTransaction>::const_iterator it_t = transactions.begin();
+  while ( it_t != transactions.end() ) {
+    try {
+      MyMoneySplit split;
+      // Test whether the transaction also includes a split into
+      // this account
+      split = (*it_t).splitByAccount(accId, true /*match*/);
+      if(!split.number().isEmpty() && split.number() == no)
+        return true;
+    } catch(MyMoneyException *e) {
+      delete e;
+    }
+    ++it_t;
+  }
+  return false;
+}
+
+QString MyMoneyFile::highestCheckNo(const QCString& accId) const
+{
+  unsigned64 lno = 0, cno;
+  QString no;
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(accId);
+  QValueList<MyMoneyTransaction> transactions = transactionList(filter);
+  QValueList<MyMoneyTransaction>::const_iterator it_t = transactions.begin();
+  while ( it_t != transactions.end() ) {
+    try {
+      // Test whether the transaction also includes a split into
+      // this account
+      MyMoneySplit split = (*it_t).splitByAccount(accId, true /*match*/);
+      if(!split.number().isEmpty()) {
+        // non-numerical values stored in number will return 0 in the next line
+        cno = split.number().toULongLong();
+        if(cno > lno) {
+          lno = cno;
+          no = split.number();
+        }
+      }
+    } catch(MyMoneyException *e) {
+      delete e;
+    }
+    ++it_t;
+  }
+  return no;
+}
