@@ -17,6 +17,10 @@
 #ifndef MYMONEYFILE_H
 #define MYMONEYFILE_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -27,25 +31,30 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "storage/imymoneystorage.h"
-#include "mymoneyexception.h"
-#include "mymoneyutils.h"
-#include "mymoneyinstitution.h"
-#include "mymoneyaccount.h"
-#include "mymoneytransaction.h"
-#include "mymoneypayee.h"
-#include "mymoneyobserver.h"
-#include "mymoneysubject.h"
-#include "mymoneykeyvaluecontainer.h"
+#include <kmymoney/imymoneystorage.h>
+#include <kmymoney/mymoneyexception.h>
+#include <kmymoney/mymoneyutils.h>
+#include <kmymoney/mymoneyinstitution.h>
+#include <kmymoney/mymoneyaccount.h>
+#include <kmymoney/mymoneytransaction.h>
+#include <kmymoney/mymoneypayee.h>
+#include <kmymoney/mymoneyobserver.h>
+#include <kmymoney/mymoneysubject.h>
+#include <kmymoney/mymoneykeyvaluecontainer.h>
+#include <kmymoney/mymoneysecurity.h>
+#include <kmymoney/mymoneyprice.h>
+#include <kmymoney/mymoneyreport.h>
+#include <kmymoney/mymoneyscheduled.h>
+#include <kmymoney/export.h>
 
 /**
-  * @author Thomas Baumgart, Michael Edwardes
+  * @author Thomas Baumgart, Michael Edwardes, Kevin Tambascio
   */
 
 class IMyMoneyStorage;
 class MyMoneyTransactionFilter;
 
-/**.
+/**
   * This class represents the interface to the MyMoney engine.
   * For historical reasons it is still called MyMoneyFile.
   * It is implemented using the singleton pattern and thus only
@@ -98,6 +107,15 @@ class MyMoneyTransactionFilter;
   * transaction() and transactionList() are used to retrieve
   * a single instance or a QValueList of MyMoneyTransaction objects.
   *
+  * The methods addSecurity(), modifySecurity() and removeSecurity()
+  * implement the general access to equities held in the engine.
+  *
+  * The methods addCurrency(), modifyCurrency() and removeCurrency()
+  * implement the general access to multiple currencies held in the engine.
+  * The methods baseCurrency() and setBaseCurrency() allow to retrieve/set
+  * the currency selected by the user as base currency. If a currency
+  * reference is emtpy, it will usually be interpreted as baseCurrency().
+  *
   * The methods liability(), asset(), expense() and income() are used to
   * retrieve the four standard accounts. isStandardAccount() checks if a
   * given accountId references one of the or not. setAccountName() is used
@@ -112,11 +130,11 @@ class MyMoneyTransactionFilter;
   * for MyMoneyKeyValueContainer which provides a container to store
   * these values indexed by an alphanumeric key.
   *
-  * @exceptions MyMoneyException is thrown whenever an error occurs
+  * @exception MyMoneyException is thrown whenever an error occurs
   * while the engine code is running. The MyMoneyException:: object
   * describes the problem.
   */
-class MyMoneyFile
+class KMYMONEY_EXPORT MyMoneyFile
 {
 public:
 
@@ -191,7 +209,7 @@ public:
     *              if @p false, notifications will be send out.
     */
   void suspendNotify(const bool state);
-  
+
   /**
     * This method is used to attach a storage to the MyMoneyFile object
     * Without an attached storage object, the MyMoneyFile object is
@@ -252,6 +270,65 @@ public:
     */
   const MyMoneyAccount income(void) const;
 
+  /**
+    * This method is used to return the standard equity account
+    * @return MyMoneyAccount equity account(group)
+    */
+  const MyMoneyAccount equity(void) const;
+
+  /**
+    * This method returns the account information for the opening
+    * balances account for the given @p security. If the respective
+    * account does not exist, it will be created. The name is constructed
+    * using MyMoneyFile::OpeningBalancesPrefix and appending " (xxx)" in
+    * case the @p security is not the baseCurrency(). The account created
+    * will be a sub-account of the standard equity account provided by equity().
+    *
+    * @param security Security for which the account is searched
+    *
+    * @return The opening balance account
+    *
+    * @note No notifications will be sent!
+    */
+  const MyMoneyAccount openingBalanceAccount(const MyMoneySecurity& security);
+
+  /**
+    * This method is essentially the same as the above, except it works on
+    * const objects.  If there is no opening balance account, this method
+    * WILL NOT create one.  Instead it will thrown an exception.
+    *
+    * @param security Security for which the account is searched
+    *
+    * @return The opening balance account
+    *
+    * @note No notifications will be sent!
+    */
+  const MyMoneyAccount openingBalanceAccount(const MyMoneySecurity& security) const;
+
+  /**
+    * Create an opening balance transaction for the account @p acc
+    * with a value of @p balance. If the corresponding opening balance account
+    * for the account's currency does not exist it will be created. If it exists
+    * and it's opening date is later than the opening date of @p acc,
+    * the opening date of the opening balances account will be adjusted to the
+    * one of @p acc.
+    *
+    * @param acc reference to account for which the opening balance transaction
+    *            should be created
+    * @param balance reference to the value of the opening balance transaction
+    */
+  void createOpeningBalanceTransaction(const MyMoneyAccount& acc, const MyMoneyMoney& balance);
+
+  /**
+    * Retrieve the opening balance transaction for the account @p acc.
+    * If there is no opening balance transaction, QCString() will be returned.
+    *
+    * @param acc reference to account for which the opening balance transaction
+    *            should be retrieved
+    * @return QCString id for the transaction, or QCString() if no transaction exists
+    */
+  QCString openingBalanceTransaction(const MyMoneyAccount& acc) const;
+
   /**
     * This method returns an indicator if the MyMoneyFile object has been
     * changed after it has last been saved to permanent storage.
@@ -267,7 +344,7 @@ public:
     * to a local temp file was OK.
     */
   void setDirty(void) const;
-  
+
   /**
     * Adds an institution to the file-global institution pool. A
     * respective institution-ID will be generated for this object.
@@ -455,6 +532,9 @@ public:
     * global transaction pool. It returns all those transactions
     * that match the filter passed as argument. If the filter is empty,
     * the whole journal will be returned.
+    * The list returned is sorted according to the transactions posting date.
+    * If more than one transaction exists for the same date, the order among
+    * them is undefined.
     *
     * @param filter MyMoneyTransactionFilter object with the match criteria
     *
@@ -462,21 +542,6 @@ public:
     */
   const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
 
-  /**
-    * This method is used to pull a list of transactions from the file
-    * global transaction pool. It returns either the whole journal or
-    * the set of transaction referenced by a specific account depending
-    * on the argument given.
-    *
-    * @param account QCString reference to account id. If account is empty
-    *                all transactions (the journal) is returned. If account
-    *                is not empty, it returns the set of transactions
-    *                that have splits in this account.
-    *
-    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
-    */
-  // const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
-
   /**
     * This method is used to remove a transaction from the transaction
     * pool (journal).
@@ -487,21 +552,93 @@ public:
 
   /**
     * This method is used to return the actual balance of an account
-    * without it's sub-ordinate accounts
+    * without it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date (default = QDate())
     * @return balance of the account as MyMoneyMoney object
     */
-  const MyMoneyMoney balance(const QCString& id) const;
+  const MyMoneyMoney balance(const QCString& id, const QDate& date = QDate()) const;
 
   /**
     * This method is used to return the actual balance of an account
-    * including it's sub-ordinate accounts
+    * including it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date (default = QDate())
     * @return balance of the account as MyMoneyMoney object
     */
-  const MyMoneyMoney totalBalance(const QCString& id) const;
+  const MyMoneyMoney totalBalance(const QCString& id, const QDate& date = QDate()) const;
+
+  /**
+    * This method is used to return the actual value of an account
+    * including it's sub-ordinate accounts. A value of an account
+    * is calculated by multiplying the balance with the currencies
+    * price for today. The price for accounts held in the baseCurrency()
+    * is 1. The same applies for prices that are not available (no entry
+    * in price list for the current date).
+    *
+    * See totalValueValid(const QCString&) for information about the
+    * validity of the returned value.
+    *
+    * @param id account id of the account in question
+    * @return value of the account as MyMoneyMoney object
+    */
+  const MyMoneyMoney totalValue(const QCString& id) const;
+
+  /**
+    * This method is used to return the actual value of an account
+    * excluding it's sub-ordinate accounts. A value of an account
+    * is calculated by multiplying the balance with the security's
+    * price for today. If the security's trading currency differs
+    * from baseCurrency() then the result of this multiplication
+    * is multiplied again by the trading currency's price for today.
+    *
+    * The price for accounts held in the baseCurrency()
+    * is 1. The same applies for prices that are not available (no entry
+    * in price list for the current date).
+    *
+    * See accountValueValid(const QCString&) for information about the
+    * validity of the returned value.
+    *
+    * @param id account id of the account in question
+    * @return value of the account as MyMoneyMoney object
+    */
+  const MyMoneyMoney accountValue(const QCString& id) const;
+
+  /**
+    * This method returns if the value returned by totalValue(const QCString&)
+    * is valid or not. The value is considered valid, if all conversion rates (prices)
+    * are available to convert the account's balance to the base currency.
+    * The same applies for any of the subaccounts of @p id.
+    *
+    * If for any of the accounts a conversion rate for the current date cannot
+    * be found, false is returned. If for all such currencies a conversion rate
+    * is available, true is returned.
+    *
+    * @param id account id of the account in question
+    * @retval false value returned by totalValue(const QCString&) is not
+    *               completely correct (missing conversion rates are assumed to be 1)
+    * @retval true value returned by totalValue(const QCString&) is correct
+    */
+  const bool totalValueValid(const QCString& id) const;
+
+  /**
+    * This method returns if the value returned by accountValue(const QCString&)
+    * is valid or not. The value is considered valid, if a conversion rate (price)
+    * is available for the currency used for the account referenced by @p id.
+    *
+    * @param id account id of the account in question
+    * @retval false value returned by accountValue(const QCString&) is not correct
+    * @retval true value returned by accountValue(const QCString&) is correct
+    */
+  const bool accountValueValid(const QCString& id) const;
 
   /**
     * This method returns the number of transactions currently known to file
@@ -564,18 +701,26 @@ public:
     * Returns the account addressed by it's id.
     *
     * @param id id of the account to locate.
-    * @return reference to MyMoneyAccount object. An exception is thrown
+    * @return MyMoneyAccount object carrying the @p id. An exception is thrown
     *         if the id is unknown
     */
-  const MyMoneyAccount& account(const QCString& id) const;
+  const MyMoneyAccount account(const QCString& id) const;
 
   /**
-    * This method returns a list of the accounts
-    * inside a MyMoneyFile object
+    * This method returns a list of accounts inside a MyMoneyFile object.
+    * An optional parameter is a list of id's. If this list is emtpy (the default)
+    * the returned list contains all accounts, otherwise only those referenced
+    * in the id-list.
+    *
+    * @param idlist QCStringList of account ids of those accounts that
+    *        should be returned. If this list is empty, all accounts
+    *        currently known will be returned.
+    *
+    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
     *
     * @return QValueList containing the account objects
     */
-  const QValueList<MyMoneyAccount> accountList(void) const;
+  const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const;
 
   /**
     * This method is used to convert an account id to a string representation
@@ -583,7 +728,7 @@ public:
     * is part of a hierarchy, the category name will be the concatenation of
     * the single account names seperated by MyMoneyAccount::AccountSeperator.
     *
-    * @param id const QCString reference of the account's id
+    * @param accountId const QCString reference of the account's id
     *
     * @return QString of the constructed name.
     */
@@ -623,7 +768,7 @@ public:
     * @return parent name (full account name excluding the last part)
     */
   const QString parentName(const QString& name) const;
-  
+
   /**
     * This method is used to attach an observer to a subject
     * represented by it's id. Whenever the object represented
@@ -730,6 +875,9 @@ public:
     *
     * @param key const reference to QCString containing the key
     * @param val const reference to QString containing the value
+    *
+    * @note Keys starting with the leadin @p kmm- are reserved for internal use
+    *       by the MyMoneyFile object.
     */
   void setValue(const QCString& key, const QString& val);
 
@@ -795,8 +943,8 @@ public:
     * @param type      only schedules of type @p type are searched for.
     *                  See MyMoneySchedule::typeE for details.
     *                  Default is MyMoneySchedule::TYPE_ANY
-    * @param occurance only schedules of occurance type @p occurance are searched for.
-    *                  See MyMoneySchedule::occuranceE for details.
+    * @param occurence only schedules of occurence type @p occurence are searched for.
+    *                  See MyMoneySchedule::occurenceE for details.
     *                  Default is MyMoneySchedule::OCCUR_ANY
     * @param paymentType only schedules of payment method @p paymentType
     *                  are searched for.
@@ -811,16 +959,16 @@ public:
     *
     * @return const QValueList<MyMoneySchedule> list of schedule objects.
     */
-  const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
-                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
-                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
-                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
-                                     const QDate& = QDate(),
-                                     const QDate& = QDate(),
-                                     const bool = false) const;
+  const QValueList<MyMoneySchedule> scheduleList(const QCString& accountId = QCString(),
+                                     const MyMoneySchedule::typeE type = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE occurence = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE paymentType = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& startDate = QDate(),
+                                     const QDate& endDate = QDate(),
+                                     const bool overdue = false) const;
 
   const QStringList consistencyCheck(void);
-    
+
   /**
     * MyMoneyFile::NotifyClassAccount
     * is a special id that will be notified whenever any account is changed.
@@ -852,10 +1000,18 @@ public:
 
   /**
     * MyMoneyFile::NotifyClassPayee
-    * is a special id that will be notified whenever any account is changed
+    * is a special id that will be notified whenever any account is add,
+    * changed or removed from the engine
     */
   static const QCString NotifyClassPayee;
 
+  /**
+    * MyMoneyFile::NotifyClassPayeeSet
+    * is a special id that will be notified whenever any account is add
+    * or removed from the engine
+    */
+  static const QCString NotifyClassPayeeSet;
+
   /**
     * MyMoneyFile::NotifyClassPayee
     * is a special id that will be notified whenever any account is changed
@@ -874,6 +1030,37 @@ public:
     */
   static const QCString NotifyClassAnyChange;
 
+  /**
+    * MyMoneyFile::NotifyClassCurrency
+    * is a special id that will be notified whenever any currency is changed
+    */
+  static const QCString NotifyClassCurrency;
+
+  /**
+    * MyMoneyFile::NotifyClassSecurity
+    * is a special id that will be notified whenever any security is changed
+    */
+  static const QCString NotifyClassSecurity;
+
+  /**
+    * MyMoneyFile::NotifyClassPrice
+    * is a special id that will be notified whenever a price is changed
+    */
+  static const QCString NotifyClassPrice;
+
+  /**
+    * MyMoneyFile::NotifyClassReport
+    * is a special id that will be notified whenever any report is changed
+    */
+  static const QCString NotifyClassReport;
+
+  /**
+    * MyMoneyFile::OpeningBalancesPrefix is a special string used
+    * to generate the name for opening balances accounts. See openingBalanceAccount()
+    * for details.
+    */
+  static const QString OpeningBalancesPrefix;
+
   /**
     * createCategory creates a category from a text name.
     *
@@ -899,7 +1086,305 @@ public:
                                               int schedulePaymentTypes,
                                               QDate startDate,
                                               const QCStringList& accounts=QCStringList()) const;
-  
+
+  /**
+    * This method is used to add a new security object to the engine.
+    * The ID of the object is the trading symbol, so there is no need for an additional
+    * ID since the symbol is guaranteed to be unique.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object
+    */
+  void addSecurity(MyMoneySecurity& security);
+
+  /**
+    * This method is used to modify an existing MyMoneySchedule
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be updated
+    */
+  void modifySecurity(const MyMoneySecurity& security);
+
+  /**
+    * This method is used to remove an existing MyMoneySecurity object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be removed
+    */
+  void removeSecurity(const MyMoneySecurity& security);
+
+  /**
+    * This method is used to retrieve a single MyMoneySecurity object.
+    * The id of the object must be supplied in the parameter @p id.
+    * If no security with the given id is found, then a corresponding
+    * currency is searched. If @p id is empty, the baseCurrency() is returned.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySecurity object
+    * @return MyMoneySecurity object
+    */
+  const MyMoneySecurity security(const QCString& id) const;
+
+  /**
+    * This method is used to retrieve a list of all MyMoneySecurity objects.
+    */
+  const QValueList<MyMoneySecurity> securityList(void) const;
+
+  /**
+    * This method is used to add a new currency object to the engine.
+    * The ID of the object is the trading symbol, so there is no need for an additional
+    * ID since the symbol is guaranteed to be unique.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneySecurity object
+    */
+  void addCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to modify an existing MyMoneySecurity
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneySecurity object
+    */
+  void modifyCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to remove an existing MyMoneySecurity object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneySecurity object
+    */
+  void removeCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    * If @p id is empty, this method returns baseCurrency().
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  const MyMoneySecurity currency(const QCString& id) const;
+
+  /**
+    * This method is used to retrieve the list of all currencies
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneySecurity objects.
+    */
+  const QValueList<MyMoneySecurity> currencyList(void) const;
+
+  /**
+    * This method retrieves a MyMoneySecurity object representing
+    * the selected base currency. If the base currency is not
+    * selected (e.g. due to a previous call to setBaseCurrency())
+    * a standard MyMoneySecurity object will be returned. See
+    * MyMoneySecurity() for details.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return MyMoneySecurity describing base currency
+    */
+  const MyMoneySecurity baseCurrency(void) const;
+
+  /**
+    * This method allows to select the base currency. It does
+    * not perform any changes to the data in the engine. It merely
+    * stores a reference to the base currency. The currency
+    * passed as argument must exist in the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency
+    */
+  void setBaseCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method adds/replaces a price to/from the price list
+    */
+  void addPrice(const MyMoneyPrice& price);
+
+  /**
+    * This method removes a price from the price list
+    */
+  void removePrice(const MyMoneyPrice& price);
+
+  /**
+    * This method is used to retrieve a price for a specific security
+    * on a specific date. If there is no price for this date, the last
+    * known price for this currency is used. If no price information
+    * is available, 1.0 will be returned as price.
+    *
+    * @param fromId the id of the currency in question
+    * @param toId the id of the currency to convert to (if emtpy, baseCurrency)
+    * @param date the date for which the price should be returned (default = today)
+    * @param exactDate if true, entry for date must exist, if false any price information
+    *                  with a date less or equal to @p date will be returned
+    *
+    * @return price found as MyMoneyPrice object
+    * @note This throws an exception when the base currency is not set and toId is empty
+    */
+  const MyMoneyPrice price(const QCString& fromId, const QCString& toId = QCString(), const QDate& date = QDate::currentDate(), const bool exactDate = false) const;
+
+  /**
+    * This method returns a list of all prices.
+    *
+    * @return MyMoneyPriceList of all MyMoneyPrice objects.
+    */
+  const MyMoneyPriceList priceList(void) const;
+
+  /**
+    * This method allows to interrogate the engine, if a known account
+    * with id @p id has a subaccount with the name @p name.
+    *
+    * @param id id of the account to look at
+    * @param name account name that needs to be searched force
+    * @retval true account with name @p name found as subaccounts
+    * @retval false no subaccount present with that name
+    */
+  const bool hasAccount(const QCString& id, const QString& name) const;
+
+  /**
+    * This method is used to retrieve the list of all reports
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyReport objects.
+    */
+  const QValueList<MyMoneyReport> reportList( void ) const;
+
+  /**
+    * Adds a report to the file-global institution pool. A
+    * respective report-ID will be generated for this object.
+    * The ID is stored as QString in the object passed as argument.
+    *
+    * An exception will be thrown upon error conditions.
+    *
+    * @param report The complete report information in a
+    *        MyMoneyReport object
+    */
+  void addReport( MyMoneyReport& report );
+
+  /**
+    * Modifies an already existing report in the file global
+    * report pool.
+    *
+    * An exception will be thrown upon error conditions.
+    *
+    * @param report The complete new report information
+    */
+  void modifyReport( const MyMoneyReport& report );
+
+  /**
+    * This method returns the number of reports currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of reports known to file
+    */
+  unsigned countReports( void ) const;
+
+  /**
+    * This method is used to retrieve a single MyMoneyReport object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyReport object
+    * @return MyMoneyReport object
+    */
+  MyMoneyReport report( const QCString& id ) const;
+
+  /**
+    * This method is used to remove an existing MyMoneyReport object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report const reference to the MyMoneyReport object to be updated
+    */
+  void removeReport(const MyMoneyReport& report);
+
+  /**
+    * This method checks, if the given @p security is referenced
+    * by another engine object.
+    *
+    * @param security const reference to object to be checked
+    *
+    * @retval false @p security is not used
+    * @retval true @p security is used
+    */
+  const bool isReferenced(const MyMoneySecurity& security) const;
+
+  /**
+    * This method checks, if the given @p account is referenced
+    * by another engine object.
+    *
+    * @param account const reference to object to be checked
+    *
+    * @retval false @p account is not used
+    * @retval true @p account is used
+    */
+  const bool isReferenced(const MyMoneyAccount& account) const;
+
+  /**
+    * This method checks, if the given @p institution is referenced
+    * by another engine object.
+    *
+    * @param institution const reference to object to be checked
+    *
+    * @retval false @p institution is not used
+    * @retval true @p institution is used
+    */
+  const bool isReferenced(const MyMoneyInstitution& institution) const;
+
+  /**
+    * This method checks, if the given @p payee is referenced
+    * by another engine object.
+    *
+    * @param payee const reference to object to be checked
+    *
+    * @retval false @p payee is not used
+    * @retval true @p payee is used
+    */
+  const bool isReferenced(const MyMoneyPayee& payee) const;
+
+  /**
+    * This method checks if the given check no &p no is used in
+    * a transaction referencing account &p accId. If  @p accId is empty,
+    * @p false is returned.
+    *
+    * @param accId id of account to checked
+    * @param no check number to be verified if used or not
+    * @retval false @p no is not in use
+    * @retval true @p no is already assigned
+    */
+  const bool checkNoUsed(const QCString& accId, const QString& no) const;
+
+  /**
+    * This method returns the highest assigned check no for
+    * account @p accId.
+    *
+    * @param accId id of account to be scanned
+    * @return highest check no. used
+    */
+  QString highestCheckNo(const QCString& accId) const;
+
 protected:
   /**
     * This is the constructor for a new empty file description
@@ -909,6 +1394,21 @@ protected:
 private:
   const QCString locateSubAccount(const MyMoneyAccount& base, const QString& category) const;
 
+  void ensureDefaultCurrency(MyMoneyAccount& acc) const;
+
+  void warningMissingRate(const QCString& fromId, const QCString& toId) const;
+
+  /**
+    * This method creates an opening balances account. The name is constructed
+    * using MyMoneyFile::OpeningBalancesPrefix and appending " (xxx)" in
+    * case the @p security is not the baseCurrency(). The account created
+    * will be a sub-account of the standard equity account provided by equity().
+    *
+    * @param security Security for which the account is searched
+    */
+  const MyMoneyAccount createOpeningBalanceAccount(const MyMoneySecurity& security);
+
+  const MyMoneyAccount openingBalanceAccount_internal(const MyMoneySecurity& security) const;
 
 private:
   /**
@@ -975,7 +1475,7 @@ private:
   /**
     */
   bool m_suspendNotify;
-  
+
   static MyMoneyFile* _instance;
 };
 #endif
