@@ -32,13 +32,16 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "../mymoneyutils.h"
-#include "../mymoneyinstitution.h"
-#include "../mymoneyaccount.h"
-#include "../mymoneytransaction.h"
-#include "../mymoneypayee.h"
-#include "../mymoneyscheduled.h"
-#include "../mymoneytransactionfilter.h"
+#include <kmymoney/mymoneyutils.h>
+#include <kmymoney/mymoneyinstitution.h>
+#include <kmymoney/mymoneyaccount.h>
+#include <kmymoney/mymoneytransaction.h>
+#include <kmymoney/mymoneypayee.h>
+#include <kmymoney/mymoneyscheduled.h>
+#include <kmymoney/mymoneytransactionfilter.h>
+#include <kmymoney/mymoneysecurity.h>
+#include <kmymoney/mymoneyprice.h>
+#include <kmymoney/mymoneyreport.h>
 
 /**
   * @author Thomas Baumgart
@@ -48,7 +51,7 @@
   * This class represents the interface to serialize a MyMoneyStorage object
   */
 class IMyMoneySerialize {
-public: 
+public:
   IMyMoneySerialize();
   virtual ~IMyMoneySerialize();
 
@@ -84,12 +87,20 @@ public:
   virtual void setLastModificationDate(const QDate& val) = 0;
 
   /**
-    * This method returns a list of the accounts
-    * inside a MyMoneyStorage object
+    * This method returns a list of accounts inside a MyMoneyFile object.
+    * An optional parameter is a list of id's. If this list is emtpy (the default)
+    * the returned list contains all accounts, otherwise only those referenced
+    * in the id-list.
     *
-    * @return QMap<QString,MyMoneyAccount> with accounts
+    * @param idlist QCStringList of account ids of those accounts that
+    *        should be returned. If this list is empty, all accounts
+    *        currently known will be returned.
+    *
+    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
+    *
+    * @return QValueList containing the account objects
     */
-  virtual const QValueList<MyMoneyAccount> accountList(void) const = 0;
+  virtual const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const = 0;
 
   /**
     * This method returns a list of the institutions
@@ -147,6 +158,13 @@ public:
                                      const QDate& = QDate(),
                                      const QDate& = QDate(),
                                      const bool = false) const = 0;
+
+   /**
+    * This method returns a list of security objects that the engine has
+    * knowledge of.
+    */
+  virtual const QValueList<MyMoneySecurity> securityList(void) const = 0;
+
   /**
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
@@ -171,6 +189,12 @@ public:
     */
   virtual const MyMoneyAccount income(void) const = 0;
 
+  /**
+    * This method is used to return the standard equity account
+    * @return MyMoneyAccount equity account(group)
+    */
+  virtual const MyMoneyAccount equity(void) const = 0;
+
   /**
     * This method is used to create a new account
     *
@@ -178,7 +202,7 @@ public:
     *
     * @param account MyMoneyAccount filled with data
     */
-  virtual void newAccount(MyMoneyAccount& account) = 0;
+  virtual void addAccount(MyMoneyAccount& account) = 0;
 
   /**
     * This method is used to add one account as sub-ordinate to another
@@ -274,18 +298,25 @@ public:
   virtual void loadInstitution(const MyMoneyInstitution& inst) = 0;
   virtual void loadPayee(const MyMoneyPayee& payee) = 0;
   virtual void loadSchedule(const MyMoneySchedule& sched) = 0;
+  virtual void loadSecurity(const MyMoneySecurity& security) = 0;
+  virtual void loadCurrency(const MyMoneySecurity& currency) = 0;
+  virtual void loadReport( const MyMoneyReport& report ) = 0;
 
   virtual const unsigned long accountId(void) = 0;
   virtual const unsigned long transactionId(void) = 0;
   virtual const unsigned long payeeId(void) = 0;
   virtual const unsigned long institutionId(void) = 0;
   virtual const unsigned long scheduleId(void) = 0;
+  virtual const unsigned long securityId(void) = 0;
+  virtual const unsigned long reportId(void) = 0;
 
   virtual void loadAccountId(const unsigned long id) = 0;
   virtual void loadTransactionId(const unsigned long id) = 0;
   virtual void loadPayeeId(const unsigned long id) = 0;
   virtual void loadInstitutionId(const unsigned long id) = 0;
   virtual void loadScheduleId(const unsigned long id) = 0;
+  virtual void loadSecurityId(const unsigned long id) = 0;
+  virtual void loadReportId(const unsigned long id) = 0;
 
   /**
     * This method is used to retrieve the whole set of key/value pairs
@@ -315,6 +346,39 @@ public:
                                               int schedulePaymentTypes,
                                               QDate startDate,
                                               const QCStringList& accounts=QCStringList()) const = 0;
+
+  /**
+    * This method is used to retrieve the list of all currencies
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneySecurity objects representing a currency.
+    */
+  virtual const QValueList<MyMoneySecurity> currencyList(void) const = 0;
+
+  /**
+    * This method is used to retrieve the list of all reports
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyReport objects.
+    */
+  virtual const QValueList<MyMoneyReport> reportList( void ) const = 0;
+
+  /**
+    * This method adds a price entry to the price list.
+    */
+  virtual void addPrice(const MyMoneyPrice& price) = 0;
+
+  /**
+    * This method returns a list of all prices.
+    *
+    * @return MyMoneyPriceList of all MyMoneyPrice objects.
+    */
+  virtual const MyMoneyPriceList priceList(void) const = 0;
+
 };
 
 #endif
