@@ -31,14 +31,17 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "../mymoneyutils.h"
-#include "../mymoneyinstitution.h"
-#include "../mymoneyaccount.h"
-#include "../mymoneytransaction.h"
-#include "../mymoneypayee.h"
-#include "../mymoneyscheduled.h"
-#include "../mymoneyobserver.h"
-#include "../mymoneytransactionfilter.h"
+#include <kmymoney/mymoneyutils.h>
+#include <kmymoney/mymoneyinstitution.h>
+#include <kmymoney/mymoneyaccount.h>
+#include <kmymoney/mymoneytransaction.h>
+#include <kmymoney/mymoneypayee.h>
+#include <kmymoney/mymoneyscheduled.h>
+#include <kmymoney/mymoneyobserver.h>
+#include <kmymoney/mymoneytransactionfilter.h>
+#include <kmymoney/mymoneysecurity.h>
+#include <kmymoney/mymoneyprice.h>
+#include <kmymoney/mymoneyreport.h>
 
 /**
   * @author Thomas Baumgart
@@ -51,7 +54,7 @@
   * @see MyMoneySeqAccessMgr
   */
 class IMyMoneyStorage {
-public: 
+public:
   IMyMoneyStorage();
   virtual ~IMyMoneyStorage();
 
@@ -94,7 +97,7 @@ public:
     *
     * @param account MyMoneyAccount filled with data
     */
-  virtual void newAccount(MyMoneyAccount& account) = 0;
+  virtual void addAccount(MyMoneyAccount& account) = 0;
 
   /**
     * This method is used to add one account as sub-ordinate to another
@@ -187,7 +190,7 @@ public:
     * @return reference to MyMoneyAccount object. An exception is thrown
     *         if the id is unknown
     */
-  virtual const MyMoneyAccount& account(const QCString id) const = 0;
+  virtual const MyMoneyAccount account(const QCString id) const = 0;
 
   /**
     * This method is used to check whether a given
@@ -250,21 +253,29 @@ public:
 
   /**
     * This method is used to return the actual balance of an account
-    * without it's sub-ordinate accounts
+    * without it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  virtual const MyMoneyMoney balance(const QCString& id) = 0;
+  virtual const MyMoneyMoney balance(const QCString& id, const QDate& date)= 0;
 
   /**
     * This method is used to return the actual balance of an account
-    * including it's sub-ordinate accounts
+    * including it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  virtual const MyMoneyMoney totalBalance(const QCString& id) = 0;
+  virtual const MyMoneyMoney totalBalance(const QCString& id, const QDate& date) = 0;
 
   /**
     * Returns the institution of a given ID
@@ -290,7 +301,7 @@ public:
     * to a local disk was ok.
     */
   virtual void setDirty(void) = 0;
-  
+
   /**
     * This method returns the number of accounts currently known to this storage
     * in the range 0..MAXUINT
@@ -387,6 +398,9 @@ public:
     * global transaction pool. It returns all those transactions
     * that match the filter passed as argument. If the filter is empty,
     * the whole journal will be returned.
+    * The list returned is sorted according to the transactions posting date.
+    * If more than one transaction exists for the same date, the order among
+    * them is undefined.
     *
     * @param filter MyMoneyTransactionFilter object with the match criteria
     *
@@ -460,12 +474,20 @@ public:
   virtual const unsigned int institutionCount(void) const = 0;
 
   /**
-    * This method returns a list of the accounts
-    * inside a MyMoneyStorage object
+    * This method returns a list of accounts inside a MyMoneyFile object.
+    * An optional parameter is a list of id's. If this list is emtpy (the default)
+    * the returned list contains all accounts, otherwise only those referenced
+    * in the id-list.
     *
-    * @return QValueList<MyMoneyAccount> with accounts
+    * @param idlist QCStringList of account ids of those accounts that
+    *        should be returned. If this list is empty, all accounts
+    *        currently known will be returned.
+    *
+    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
+    *
+    * @return QValueList containing the account objects
     */
-  virtual const QValueList<MyMoneyAccount> accountList(void) const = 0;
+  virtual const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const = 0;
 
   /**
     * This method is used to return the standard liability account
@@ -491,6 +513,73 @@ public:
     */
   virtual const MyMoneyAccount income(void) const = 0;
 
+  /**
+    * This method is used to create a new security object.  The ID will be created
+    * automatically. The object passed with the parameter @p security is modified
+    * to contain the assigned id.
+    *
+    * An exception will be thrown upon error conditions.
+    *
+    * @param security MyMoneySecurity filled with data
+    */
+  virtual void addSecurity(MyMoneySecurity& security) = 0;
+
+  /**
+    * This method is used to modify an existing MyMoneySecurity
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be updated
+    */
+  virtual void modifySecurity(const MyMoneySecurity& security) = 0;
+
+  /**
+    * This method is used to remove an existing MyMoneySecurity object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be removed
+    */
+  virtual void removeSecurity(const MyMoneySecurity& security) = 0;
+
+  /**
+    * This method is used to retrieve a single MyMoneySecurity object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySecurity object
+    * @return MyMoneySecurity object
+    */
+  virtual const MyMoneySecurity security(const QCString& id) const = 0;
+
+  /**
+    * This method returns a list of the security objects
+    * inside a MyMoneyStorage object
+    *
+    * @return QValueList<MyMoneySecurity> containing objects
+    */
+  virtual const QValueList<MyMoneySecurity> securityList(void) const = 0;
+
+  virtual void addPrice(const MyMoneyPrice& price) = 0;
+  virtual void removePrice(const MyMoneyPrice& price) = 0;
+  virtual const MyMoneyPrice price(const QCString& fromId, const QCString& toId, const QDate& date, const bool exactDate) const = 0;
+
+  /**
+    * This method returns a list of all prices.
+    *
+    * @return MyMoneyPriceList of all MyMoneyPrice objects.
+    */
+  virtual const MyMoneyPriceList priceList(void) const = 0;
+
+  /**
+    * This method is used to return the standard equity account
+    * @return MyMoneyAccount equity account(group)
+    */
+  virtual const MyMoneyAccount equity(void) const = 0;
+
   /**
     * This method is used to add a scheduled transaction to the engine.
     * It must be sure, that the id of the object is not filled. When the
@@ -544,8 +633,8 @@ public:
     * @param type      only schedules of type @p type are searched for.
     *                  See MyMoneySchedule::typeE for details.
     *                  Default is MyMoneySchedule::TYPE_ANY
-    * @param occurance only schedules of occurance type @p occurance are searched for.
-    *                  See MyMoneySchedule::occuranceE for details.
+    * @param occurence only schedules of occurence type @p occurance are searched for.
+    *                  See MyMoneySchedule::occurenceE for details.
     *                  Default is MyMoneySchedule::OCCUR_ANY
     * @param paymentType only schedules of payment method @p paymentType
     *                  are searched for.
@@ -560,13 +649,13 @@ public:
     *
     * @return const QValueList<MyMoneySchedule> list of schedule objects.
     */
-  virtual const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
-                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
-                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
-                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
-                                     const QDate& = QDate(),
-                                     const QDate& = QDate(),
-                                     const bool = false) const = 0;
+  virtual const QValueList<MyMoneySchedule> scheduleList(const QCString& accountId = QCString(),
+                                     const MyMoneySchedule::typeE type = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE occurence = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE paymentType = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& startDate = QDate(),
+                                     const QDate& endDate = QDate(),
+                                     const bool overdue = false) const = 0;
 
   virtual QValueList<MyMoneySchedule> scheduleListEx( int scheduleTypes,
                                               int scheduleOcurrences,
@@ -574,6 +663,119 @@ public:
                                               QDate startDate,
                                               const QCStringList& accounts=QCStringList()) const = 0;
 
+  /**
+    * This method is used to add a new currency object to the engine.
+    * The ID of the object is the trading symbol, so there is no need for an additional
+    * ID since the symbol is guaranteed to be unique.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneySecurity object
+    */
+  virtual void addCurrency(const MyMoneySecurity& currency) = 0;
+
+  /**
+    * This method is used to modify an existing MyMoneySecurity
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneyCurrency object
+    */
+  virtual void modifyCurrency(const MyMoneySecurity& currency) = 0;
+
+  /**
+    * This method is used to remove an existing MyMoneySecurity object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneySecurity object
+    */
+  virtual void removeCurrency(const MyMoneySecurity& currency) = 0;
+
+  /**
+    * This method is used to retrieve a single MyMoneySecurity object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySecurity object
+    * @return MyMoneyCurrency object
+    */
+  virtual const MyMoneySecurity currency(const QCString& id) const = 0;
+
+  /**
+    * This method is used to retrieve the list of all currencies
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneySecurity objects representing a currency.
+    */
+  virtual const QValueList<MyMoneySecurity> currencyList(void) const = 0;
+
+  /**
+    * This method is used to retrieve the list of all reports
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyReport objects.
+    */
+  virtual const QValueList<MyMoneyReport> reportList( void ) const = 0;
+
+  /**
+    * This method is used to add a new report to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report reference to the MyMoneyReport object
+    */
+  virtual void addReport( MyMoneyReport& report ) = 0;
+
+  /**
+    * This method is used to modify an existing MyMoneyReport
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report const reference to the MyMoneyReport object to be updated
+    */
+  virtual void modifyReport( const MyMoneyReport& report ) = 0;
+
+  /**
+    * This method returns the number of reports currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of reports known to file
+    */
+  virtual unsigned countReports( void ) const = 0;
+
+  /**
+    * This method is used to retrieve a single MyMoneyReport object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyReport object
+    * @return MyMoneyReport object
+    */
+  virtual MyMoneyReport report( const QCString& id ) const = 0;
+
+  /**
+    * This method is used to remove an existing MyMoneyReport object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report const reference to the MyMoneyReport object to be updated
+    */
+  virtual void removeReport(const MyMoneyReport& report) = 0;
+
 };
 
 #endif
