@@ -36,6 +36,8 @@ MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
   m_nextTransactionID = 0;
   m_nextPayeeID = 0;
   m_nextScheduleID = 0;
+  m_nextSecurityID = 0;
+  m_nextReportID = 0;
   m_userName =
   m_userStreet =
   m_userTown =
@@ -61,6 +63,9 @@ MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
   MyMoneyAccount acc_i;
   acc_i.setAccountType(MyMoneyAccount::Income);
   acc_i.setName("Income");
+  MyMoneyAccount acc_q;
+  acc_q.setAccountType(MyMoneyAccount::Equity);
+  acc_q.setName("Equity");
 
   MyMoneyAccount* a;
   a = new MyMoneyAccount(STD_ACC_LIABILITY, acc_l);
@@ -79,12 +84,18 @@ MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
   m_accountList[STD_ACC_INCOME] = *a;
   delete a;
 
+  a = new MyMoneyAccount(STD_ACC_EQUITY, acc_q);
+  m_accountList[STD_ACC_EQUITY] = *a;
+  delete a;
+
   MyMoneyBalanceCacheItem balance;
 
+  m_balanceCache.clear();
   m_balanceCache[STD_ACC_LIABILITY] = balance;
   m_balanceCache[STD_ACC_ASSET] = balance;
   m_balanceCache[STD_ACC_EXPENSE] = balance;
   m_balanceCache[STD_ACC_INCOME] = balance;
+  m_balanceCache[STD_ACC_EQUITY] = balance;
 }
 
 MyMoneySeqAccessMgr::~MyMoneySeqAccessMgr()
@@ -103,7 +114,8 @@ const bool MyMoneySeqAccessMgr::isStandardAccount(const QCString& id) const
   return id == STD_ACC_LIABILITY
       || id == STD_ACC_ASSET
       || id == STD_ACC_EXPENSE
-      || id == STD_ACC_INCOME;
+      || id == STD_ACC_INCOME
+      || id == STD_ACC_EQUITY;
 }
 
 void MyMoneySeqAccessMgr::setAccountName(const QCString& id, const QString& name)
@@ -114,7 +126,7 @@ void MyMoneySeqAccessMgr::setAccountName(const QCString& id, const QString& name
   m_accountList[id].setName(name);
 }
 
-const MyMoneyAccount& MyMoneySeqAccessMgr::account(const QCString id) const
+const MyMoneyAccount MyMoneySeqAccessMgr::account(const QCString id) const
 {
   QMap<QCString, MyMoneyAccount>::ConstIterator pos;
 
@@ -128,21 +140,38 @@ const MyMoneyAccount& MyMoneySeqAccessMgr::account(const QCString id) const
   throw new MYMONEYEXCEPTION(msg);
 }
 
-const QValueList<MyMoneyAccount> MyMoneySeqAccessMgr::accountList(void) const
+const QValueList<MyMoneyAccount> MyMoneySeqAccessMgr::accountList(const QCStringList& idlist, const bool recursive) const
 {
   QValueList<MyMoneyAccount> list;
   QMap<QCString, MyMoneyAccount>::ConstIterator it;
 
   for(it = m_accountList.begin(); it != m_accountList.end(); ++it) {
-    if(!isStandardAccount((*it).id()))
-      list.append(*it);
+    if(!isStandardAccount((*it).id())) {
+      if(idlist.isEmpty() || idlist.findIndex((*it).id()) != -1) {
+        list.append(*it);
+        if(!idlist.isEmpty() && recursive == true) {
+          list += accountList((*it).accountList());
+        }
+      }
+    }
   }
   return list;
 }
 
-void MyMoneySeqAccessMgr::newAccount(MyMoneyAccount& account)
+void MyMoneySeqAccessMgr::addAccount(MyMoneyAccount& account)
 {
-  // create the account
+  // we do not allow an opening balance different from 0 anymore
+  // This used to work in older releases
+  if(!account.openingBalance().isZero()) {
+    // FIXME we should really generate an exception here. For now,
+    // we only print a warning and hope that only old binary files
+    // still have this feature. Once we get rid of the binary
+    // reader we can safely throw the exception
+    // throw new MYMONEYEXCEPTION("Cannot create account with opening balance != 0");
+    qWarning("Createing accounts with opening balance != 0 in the engine is deprecated!");
+  }
+
+  // create the account.
   MyMoneyAccount newAccount(nextAccountID(), account);
   m_accountList[newAccount.id()] = newAccount;
   touch();
@@ -164,7 +193,7 @@ const MyMoneyPayee MyMoneySeqAccessMgr::payee(const QCString& id) const
 
   it = m_payeeList.find(id);
   if(it == m_payeeList.end())
-    throw new MYMONEYEXCEPTION("Unknown payee");
+    throw new MYMONEYEXCEPTION("Unknown payee '" + id + "'");
 
   return *it;
 }
@@ -173,7 +202,7 @@ const MyMoneyPayee MyMoneySeqAccessMgr::payeeByName(const QString& payee) const
 {
   if(payee.isEmpty())
     return MyMoneyPayee();
-    
+
   QMap<QCString, MyMoneyPayee>::ConstIterator it_p;
 
   for(it_p = m_payeeList.begin(); it_p != m_payeeList.end(); ++it_p) {
@@ -218,6 +247,7 @@ void MyMoneySeqAccessMgr::removePayee(const MyMoneyPayee& payee)
         throw new MYMONEYEXCEPTION("Cannot remove payee that is referenced");
     }
   }
+  // FIXME: check referential integrity in schedules
 
   m_payeeList.remove(it_p);
   touch();
@@ -225,13 +255,7 @@ void MyMoneySeqAccessMgr::removePayee(const MyMoneyPayee& payee)
 
 const QValueList<MyMoneyPayee> MyMoneySeqAccessMgr::payeeList(void) const
 {
-  QValueList<MyMoneyPayee> list;
-  QMap<QCString, MyMoneyPayee>::ConstIterator it;
-
-  for(it = m_payeeList.begin(); it != m_payeeList.end(); ++it) {
-    list.append(*it);
-  }
-  return list;
+  return m_payeeList.values();
 }
 
 
@@ -403,6 +427,15 @@ const QCString MyMoneySeqAccessMgr::nextScheduleID(void)
   return id;
 }
 
+const QCString MyMoneySeqAccessMgr::nextSecurityID(void)
+{
+  QCString id;
+  id.setNum(++m_nextSecurityID);
+  id = "E" + id.rightJustify(SECURITY_ID_SIZE, '0');
+  return id;
+}
+
+
 void MyMoneySeqAccessMgr::addTransaction(MyMoneyTransaction& transaction, const bool skipAccountUpdate)
 {
   // perform some checks to see that the transaction stuff is OK. For
@@ -507,6 +540,9 @@ void MyMoneySeqAccessMgr::modifyAccount(const MyMoneyAccount& account, const boo
 {
   QMap<QCString, MyMoneyAccount>::Iterator pos;
 
+  if(!account.openingBalance().isZero())
+    throw new MYMONEYEXCEPTION("Cannot modify account with opening balance != 0");
+
   // locate the account in the file global pool
   pos = m_accountList.find(account.id());
   if(pos != m_accountList.end()) {
@@ -533,6 +569,7 @@ void MyMoneySeqAccessMgr::modifyAccount(const MyMoneyAccount& account, const boo
           (*newInst).addAccountId(account.id());
         }
       }
+
       // update information in account list
       m_accountList[account.id()] = account;
 
@@ -659,6 +696,9 @@ void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccoun
     oldParent = m_accountList.find(account.parentAccountId());
   }
 
+  if(account.accountType() == MyMoneyAccount::Stock && parent.accountType() != MyMoneyAccount::Investment)
+    throw new MYMONEYEXCEPTION("Cannot move a stock acocunt into a non-investment account");
+
   newParent = m_accountList.find(parent.id());
   childAccount = m_accountList.find(account.id());
 
@@ -668,9 +708,10 @@ void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccoun
   (*newParent).addAccountId(account.id());
   (*childAccount).setParentAccountId(parent.id());
 
-  // make sure the type is the same as the new parent
-  (*childAccount).setAccountType((*newParent).accountType());
-  
+  // make sure the type is the same as the new parent. This does not work for stock and investment
+  if(account.accountType() != MyMoneyAccount::Stock && account.accountType() != MyMoneyAccount::Investment)
+    (*childAccount).setAccountType((*newParent).accountType());
+
   parent = *newParent;
   account = *childAccount;
 
@@ -705,6 +746,8 @@ void MyMoneySeqAccessMgr::removeTransaction(const MyMoneyTransaction& transactio
     modifiedAccounts[(*it_s).accountId()] = true;
   }
 
+  // FIXME: check if any split is frozen and throw exception
+
   // remove the transaction from the two lists
   m_transactionList.remove(it_t);
   m_transactionKeys.remove(it_k);
@@ -765,11 +808,12 @@ void MyMoneySeqAccessMgr::removeAccount(const MyMoneyAccount& account)
   if(it_p == m_accountList.end())
     throw new MYMONEYEXCEPTION("Internal error: parent account not found in list");
 
+  // FIXME: check referential integrity for the account to be removed
+
   // check if the new info is based on the old one.
   // this is the case, when the file and the id
   // as well as the type are equal.
-  if((*it_a).file() == account.file()
-  && (*it_a).id() == account.id()
+  if((*it_a).id() == account.id()
   && (*it_a).institutionId() == account.institutionId()
   && (*it_a).accountType() == account.accountType()) {
 
@@ -820,7 +864,7 @@ void MyMoneySeqAccessMgr::removeInstitution(const MyMoneyInstitution& institutio
         try {
           modifyAccount(acc);
         } catch(MyMoneyException *e) {
-          qDebug(e->what());
+          qDebug("%s", e->what().data());
           delete e;
         }
       }
@@ -903,7 +947,7 @@ const MyMoneyTransaction& MyMoneySeqAccessMgr::transaction(const QCString& accou
     filter.addCategory(account);
   else
     filter.addAccount(account);
-    
+
   list = transactionList(filter);
   if(idx < 0 || idx >= static_cast<int> (list.count()))
     throw new MYMONEYEXCEPTION("Unknown idx for transaction");
@@ -911,12 +955,12 @@ const MyMoneyTransaction& MyMoneySeqAccessMgr::transaction(const QCString& accou
   return transaction(list[idx].id());
 }
 
-const MyMoneyMoney MyMoneySeqAccessMgr::balance(const QCString& id)
+const MyMoneyMoney MyMoneySeqAccessMgr::balance(const QCString& id, const QDate& date)
 {
   MyMoneyMoney result(0);
   MyMoneyAccount acc;
 
-  if(m_balanceCache[id].valid == true)
+  if(m_balanceCache[id].valid == true && !date.isValid())
     result = m_balanceCache[id].balance;
 
   else {
@@ -933,12 +977,13 @@ const MyMoneyMoney MyMoneySeqAccessMgr::balance(const QCString& id)
     QValueList<MyMoneyTransaction>::ConstIterator it;
     MyMoneySplit split;
     MyMoneyTransactionFilter filter(id);
+    filter.setDateFilter(QDate(), date);
     list = transactionList(filter);
 
     for(it = list.begin(); it != list.end(); ++it) {
       try {
         split = (*it).splitByAccount(id);
-        result += split.value();
+        result += split.value((*it).commodity(), acc.currencyId());
 
       } catch(MyMoneyException *e) {
         // account is not referenced within this transaction
@@ -946,18 +991,20 @@ const MyMoneyMoney MyMoneySeqAccessMgr::balance(const QCString& id)
       }
     }
 
-    MyMoneyBalanceCacheItem balance(result);
-    m_balanceCache[id] = balance;
+    if(!date.isValid()) {
+      MyMoneyBalanceCacheItem balance(result);
+      m_balanceCache[id] = balance;
+    }
   }
   return result;
 }
 
-const MyMoneyMoney MyMoneySeqAccessMgr::totalBalance(const QCString& id)
+const MyMoneyMoney MyMoneySeqAccessMgr::totalBalance(const QCString& id, const QDate& date)
 {
   QCStringList accounts;
   QCStringList::ConstIterator it_a;
 
-  MyMoneyMoney result(balance(id));
+  MyMoneyMoney result(balance(id, date));
 
   MyMoneyAccount acc;
 
@@ -965,7 +1012,7 @@ const MyMoneyMoney MyMoneySeqAccessMgr::totalBalance(const QCString& id)
   accounts = acc.accountList();
 
   for(it_a = accounts.begin(); it_a != accounts.end(); ++it_a) {
-    result += totalBalance(*it_a);
+    result += totalBalance(*it_a, date);
   }
 
   return result;
@@ -1040,7 +1087,8 @@ void MyMoneySeqAccessMgr::loadAccount(const MyMoneyAccount& acc)
   if(acc.id() == asset().id()
   || acc.id() == liability().id()
   || acc.id() == expense().id()
-  || acc.id() == income().id()) {
+  || acc.id() == income().id()
+  || acc.id() == equity().id()) {
     m_accountList[acc.id()] = acc;
     return;
   }
@@ -1077,7 +1125,8 @@ void MyMoneySeqAccessMgr::loadTransaction(const MyMoneyTransaction& tr)
 
   for(it_s = list.begin(); it_s != list.end(); ++it_s) {
     QCString id = (*it_s).accountId();
-    m_balanceCache[id] = MyMoneyBalanceCacheItem(balance(id) + (*it_s).value());
+    MyMoneyAccount acc = account(id);
+    m_balanceCache[id] = MyMoneyBalanceCacheItem(balance(id, QDate()) + (*it_s).value(tr.commodity(), acc.currencyId()));
   }
 }
 
@@ -1107,6 +1156,33 @@ void MyMoneySeqAccessMgr::loadPayee(const MyMoneyPayee& payee)
   m_payeeList[payee.id()] = payee;
 }
 
+void MyMoneySeqAccessMgr::loadSecurity(const MyMoneySecurity& security)
+{
+  QMap<QCString, MyMoneySecurity>::ConstIterator it;
+
+  it = m_securitiesList.find(security.id());
+  if(it != m_securitiesList.end())
+  {
+    QString msg = "Duplicate security  '";
+    msg += security.id() + "' during loadSecurity()";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+  m_securitiesList[security.id()] = security;
+  //qDebug("loadSecurity: Security list size is %d, this=%8p", m_equitiesList.size(), (void*)this);
+}
+
+void MyMoneySeqAccessMgr::loadCurrency(const MyMoneySecurity& currency)
+{
+  QMap<QCString, MyMoneySecurity>::ConstIterator it;
+
+  it = m_currencyList.find(currency.id());
+  if(it != m_currencyList.end())
+  {
+    throw new MYMONEYEXCEPTION(QString("Duplicate currency '%1' during loadCurrency()").arg(currency.id().data()));
+  }
+  m_currencyList[currency.id()] = currency;
+}
+
 void MyMoneySeqAccessMgr::loadAccountId(const unsigned long id)
 {
   m_nextAccountID = id;
@@ -1127,6 +1203,16 @@ void MyMoneySeqAccessMgr::loadInstitutionId(const unsigned long id)
   m_nextInstitutionID = id;
 }
 
+void MyMoneySeqAccessMgr::loadSecurityId(const unsigned long id)
+{
+  m_nextSecurityID = id;
+}
+
+void MyMoneySeqAccessMgr::loadReportId(const unsigned long id)
+{
+  m_nextReportID = id;
+}
+
 const QString MyMoneySeqAccessMgr::value(const QCString& key) const
 {
   return MyMoneyKeyValueContainer::value(key);
@@ -1135,11 +1221,13 @@ const QString MyMoneySeqAccessMgr::value(const QCString& key) const
 void MyMoneySeqAccessMgr::setValue(const QCString& key, const QString& val)
 {
   MyMoneyKeyValueContainer::setValue(key, val);
+  touch();
 }
 
 void MyMoneySeqAccessMgr::deletePair(const QCString& key)
 {
   MyMoneyKeyValueContainer::deletePair(key);
+  touch();
 }
 
 QMap<QCString, QString> MyMoneySeqAccessMgr::pairs(void) const
@@ -1150,6 +1238,7 @@ QMap<QCString, QString> MyMoneySeqAccessMgr::pairs(void) const
 void MyMoneySeqAccessMgr::setPairs(const QMap<QCString, QString>& list)
 {
   MyMoneyKeyValueContainer::setPairs(list);
+  touch();
 }
 
 void MyMoneySeqAccessMgr::addSchedule(MyMoneySchedule& sched)
@@ -1194,6 +1283,8 @@ void MyMoneySeqAccessMgr::removeSchedule(const MyMoneySchedule& sched)
     throw new MYMONEYEXCEPTION(msg);
   }
 
+  // FIXME: check referential integrity for loan accounts
+
   m_scheduleList.remove(it);
   touch();
 }
@@ -1225,28 +1316,28 @@ const QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleList(
   QValueList<MyMoneySchedule> list;
 
   // qDebug("scheduleList()");
-  
+
   for(pos = m_scheduleList.begin(); pos != m_scheduleList.end(); ++pos) {
     // qDebug("  '%s'", (*pos).id().data());
-    
+
     if(type != MyMoneySchedule::TYPE_ANY) {
       if(type != (*pos).type()) {
         continue;
       }
     }
-    
+
     if(occurence != MyMoneySchedule::OCCUR_ANY) {
       if(occurence != (*pos).occurence()) {
         continue;
       }
     }
-    
+
     if(paymentType != MyMoneySchedule::STYPE_ANY) {
       if(paymentType != (*pos).paymentType()) {
         continue;
       }
     }
-    
+
     if(!accountId.isEmpty()) {
       MyMoneyTransaction t = (*pos).transaction();
       QValueList<MyMoneySplit>::ConstIterator it;
@@ -1266,13 +1357,13 @@ const QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleList(
         continue;
       }
     }
-    
+
     if(startDate.isValid() && !endDate.isValid()) {
       if(!(*pos).nextPayment(startDate.addDays(-1)).isValid()) {
         continue;
       }
     }
-    
+
     if(!startDate.isValid() && endDate.isValid()) {
       if((*pos).startDate() > endDate) {
         continue;
@@ -1322,7 +1413,7 @@ QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleListEx(int scheduleType
                                                                 const QCStringList& accounts) const
 {
 //  qDebug("scheduleListEx");
-  
+
   QMap<QCString, MyMoneySchedule>::ConstIterator pos;
   QValueList<MyMoneySchedule> list;
 
@@ -1332,11 +1423,11 @@ QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleListEx(int scheduleType
   for(pos = m_scheduleList.begin(); pos != m_scheduleList.end(); ++pos)
   {
     if (scheduleTypes && !(scheduleTypes & (*pos).type()))
-      continue; 
+      continue;
 
     if (scheduleOcurrences && !(scheduleOcurrences & (*pos).occurence()))
       continue;
-      
+
     if (schedulePaymentTypes && !(schedulePaymentTypes & (*pos).paymentType()))
       continue;
 
@@ -1354,10 +1445,265 @@ QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleListEx(int scheduleType
       if (accounts.contains((*pos).account().id()))
         continue;
     }
-    
+
 //    qDebug("\tAdding '%s'", (*pos).name().latin1());
     list << *pos;
   }
-  
-  return list;  
+
+  return list;
 }
+
+void MyMoneySeqAccessMgr::addSecurity(MyMoneySecurity& security)
+{
+  // create the account
+  MyMoneySecurity newSecurity(nextSecurityID(), security);
+
+  m_securitiesList[newSecurity.id()] = newSecurity;
+
+  touch();
+  security = newSecurity;
+}
+
+void MyMoneySeqAccessMgr::modifySecurity(const MyMoneySecurity& security)
+{
+  QMap<QCString, MyMoneySecurity>::ConstIterator it;
+
+  it = m_securitiesList.find(security.id());
+  if(it == m_securitiesList.end())
+  {
+    QString msg = "Unknown security  '";
+    msg += security.id() + "' during modifySecurity()";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+
+  m_securitiesList[security.id()] = security;
+  touch();
+}
+
+void MyMoneySeqAccessMgr::removeSecurity(const MyMoneySecurity& security)
+{
+  QMap<QCString, MyMoneySecurity>::ConstIterator it;
+
+  // FIXME: check referential integrity
+
+  it = m_securitiesList.find(security.id());
+  if(it == m_securitiesList.end())
+  {
+    QString msg = "Unknown security  '";
+    msg += security.id() + "' during removeSecurity()";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+
+  m_securitiesList.erase(security.id());
+  touch();
+}
+
+const MyMoneySecurity MyMoneySeqAccessMgr::security(const QCString& id) const
+{
+  QMap<QCString, MyMoneySecurity>::ConstIterator it = m_securitiesList.find(id);
+  if(it != m_securitiesList.end())
+  {
+    return it.data();
+  }
+
+  return MyMoneySecurity();
+}
+
+const QValueList<MyMoneySecurity> MyMoneySeqAccessMgr::securityList(void) const
+{
+  //qDebug("securityList: Security list size is %d, this=%8p", m_equitiesList.size(), (void*)this);
+  return m_securitiesList.values();
+}
+
+void MyMoneySeqAccessMgr::addCurrency(const MyMoneySecurity& currency)
+{
+  QMap<QCString, MyMoneySecurity>::Iterator it;
+
+  it = m_currencyList.find(currency.id());
+  if(it != m_currencyList.end()) {
+    throw new MYMONEYEXCEPTION(QString("Cannot add currency with existing id %1").arg(currency.id().data()));
+  }
+
+  m_currencyList[currency.id()] = currency;
+  touch();
+}
+
+void MyMoneySeqAccessMgr::modifyCurrency(const MyMoneySecurity& currency)
+{
+  QMap<QCString, MyMoneySecurity>::Iterator it;
+
+  it = m_currencyList.find(currency.id());
+  if(it == m_currencyList.end()) {
+    throw new MYMONEYEXCEPTION(QString("Cannot modify currency with unknown id %1").arg(currency.id().data()));
+  }
+
+  *it = currency;
+  touch();
+}
+
+void MyMoneySeqAccessMgr::removeCurrency(const MyMoneySecurity& currency)
+{
+  QMap<QCString, MyMoneySecurity>::Iterator it;
+
+  // FIXME: check referential integrity
+
+  it = m_currencyList.find(currency.id());
+  if(it == m_currencyList.end()) {
+    throw new MYMONEYEXCEPTION(QString("Cannot remove currency with unknown id %1").arg(currency.id().data()));
+  }
+
+  m_currencyList.remove(it);
+  touch();
+}
+
+const MyMoneySecurity MyMoneySeqAccessMgr::currency(const QCString& id) const
+{
+  if(id.isEmpty()) {
+
+  }
+  QMap<QCString, MyMoneySecurity>::ConstIterator it;
+
+  it = m_currencyList.find(id);
+  if(it == m_currencyList.end()) {
+    throw new MYMONEYEXCEPTION(QString("Cannot retrieve currency with unknown id %1").arg(id.data()));
+  }
+
+  return *it;
+}
+
+const QValueList<MyMoneySecurity> MyMoneySeqAccessMgr::currencyList(void) const
+{
+  return m_currencyList.values();
+}
+
+const QValueList<MyMoneyReport> MyMoneySeqAccessMgr::reportList(void) const
+{
+  QValueList<MyMoneyReport> result;
+  QMap<QCString, MyMoneyReport>::ConstIterator it_report = m_reportList.begin();
+  while ( it_report != m_reportList.end() )
+  {
+    result += *it_report;
+    ++it_report;
+  }
+  return result;
+}
+
+void MyMoneySeqAccessMgr::addReport( MyMoneyReport& report )
+{
+  if(!report.id().isEmpty())
+    throw new MYMONEYEXCEPTION("transaction already contains an id");
+
+  report.setId( nextReportID() );
+
+  m_reportList[report.id()] = report;
+
+  touch();
+
+}
+
+void MyMoneySeqAccessMgr::loadReport( const MyMoneyReport& report )
+{
+  // in order to be 'loaded', this report must have an ID, and it must not already
+  // be in the list.
+  if ( report.id().isEmpty() )
+    throw new MYMONEYEXCEPTION(QString("Unable to load report %1 with unassigned ID during loadReport()").arg(report.name()));
+  if ( m_reportList.contains(report.id()) )
+    throw new MYMONEYEXCEPTION(QString("Duplicate report %1 during loadReport()").arg(report.id()));
+
+  m_reportList[report.id()] = report;
+}
+
+void MyMoneySeqAccessMgr::modifyReport( const MyMoneyReport& report )
+{
+  QMap<QCString, MyMoneyReport>::Iterator it;
+
+  it = m_reportList.find(report.id());
+  if(it == m_reportList.end()) {
+    QString msg = "Unknown report '" + report.id() + "'";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+  touch();
+
+  *it = report;
+}
+
+const QCString MyMoneySeqAccessMgr::nextReportID(void)
+{
+  QCString id;
+  id.setNum(++m_nextReportID);
+  id = "R" + id.rightJustify(REPORT_ID_SIZE, '0');
+  return id;
+}
+
+unsigned MyMoneySeqAccessMgr::countReports(void) const
+{
+  return m_reportList.count();
+}
+
+MyMoneyReport MyMoneySeqAccessMgr::report( const QCString& _id ) const
+{
+  return m_reportList[_id];
+}
+
+void MyMoneySeqAccessMgr::removeReport( const MyMoneyReport& report )
+{
+  QMap<QCString, MyMoneyReport>::Iterator it;
+
+  it = m_reportList.find(report.id());
+  if(it == m_reportList.end()) {
+    QString msg = "Unknown report '" + report.id() + "'";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+
+  m_reportList.remove(it);
+  touch();
+}
+
+void MyMoneySeqAccessMgr::addPrice(const MyMoneyPrice& price)
+{
+  MyMoneyPriceEntries::ConstIterator it;
+  it = m_priceList[MyMoneySecurityPair(price.from(), price.to())].find(price.date());
+
+  // do not replace, if the information did not change.
+  if(it != m_priceList[MyMoneySecurityPair(price.from(), price.to())].end()) {
+    if((*it).rate() == price.rate()
+    && (*it).source() == price.source())
+      return;
+  }
+
+  m_priceList[MyMoneySecurityPair(price.from(), price.to())][price.date()] = price;
+  touch();
+}
+
+void MyMoneySeqAccessMgr::removePrice(const MyMoneyPrice& price)
+{
+  m_priceList[MyMoneySecurityPair(price.from(), price.to())].remove(price.date());
+  touch();
+}
+
+const MyMoneyPriceList MyMoneySeqAccessMgr::priceList(void) const
+{
+  return m_priceList;
+}
+
+const MyMoneyPrice MyMoneySeqAccessMgr::price(const QCString& fromId, const QCString& toId, const QDate& date, const bool exactDate) const
+{
+  MyMoneyPrice rc;
+  MyMoneyPriceEntries::ConstIterator it;
+
+  for(it = m_priceList[MyMoneySecurityPair(fromId, toId)].begin(); it != m_priceList[MyMoneySecurityPair(fromId, toId)].end(); ++it) {
+    if(date < it.key())
+      break;
+
+    if(exactDate) {
+      if(date == it.key())
+        rc = *it;
+
+    } else {
+      if(date >= it.key())
+        rc = *it;
+    }
+  }
+  return rc;
+}
+
