@@ -1,5 +1,5 @@
 /***************************************************************************
-                          imymoneystoragestream.h  -  description
+                          mymoneyseqaccessmgr.h  -  description
                              -------------------
     begin                : Sun May 5 2002
     copyright            : (C) 2000-2002 by Michael Edwardes
@@ -33,7 +33,7 @@
 #include "imymoneyserialize.h"
 
 /**
-  *@author Thomas Baumgart
+  * @author Thomas Baumgart
   */
 
 /**
@@ -84,6 +84,7 @@ public:
 #define STD_ACC_ASSET     "AStd::Asset"
 #define STD_ACC_EXPENSE   "AStd::Expense"
 #define STD_ACC_INCOME    "AStd::Income"
+#define STD_ACC_EQUITY    "AStd::Equity"
 
   MyMoneySeqAccessMgr();
   ~MyMoneySeqAccessMgr();
@@ -116,7 +117,7 @@ public:
     * new owner of the object and must destroy it.
     */
   MyMoneySeqAccessMgr * const duplicate(void);
-  
+
   /**
     * Returns the account addressed by it's id.
     *
@@ -124,7 +125,7 @@ public:
     * @return reference to MyMoneyAccount object. An exception is thrown
     *         if the id is unknown
     */
-  const MyMoneyAccount& account(const QCString id) const;
+  const MyMoneyAccount account(const QCString id) const;
 
   /**
     * This method is used to check whether a given
@@ -147,6 +148,7 @@ public:
     *           @li STD_ACC_ASSET
     *           @li STD_ACC_EXPENSE
     *           @li STD_ACC_INCOME
+    *           @li STD_ACC_EQUITY
     *
     * @param name QString reference to the name to be set
     *
@@ -160,7 +162,7 @@ public:
     *
     * @param account MyMoneyAccount filled with data
     */
-  void newAccount(MyMoneyAccount& account);
+  void addAccount(MyMoneyAccount& account);
 
   /**
     * This method is used to create a new payee
@@ -369,21 +371,29 @@ public:
 
   /**
     * This method is used to return the actual balance of an account
-    * without it's sub-ordinate accounts
+    * without it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  const MyMoneyMoney balance(const QCString& id);
+  const MyMoneyMoney balance(const QCString& id, const QDate& date = QDate());
 
   /**
     * This method is used to return the actual balance of an account
-    * including it's sub-ordinate accounts
+    * including it's sub-ordinate accounts. If a @p date is presented,
+    * the balance at the beginning of this date (not including any
+    * transaction on this date) is returned. Otherwise all recorded
+    * transactions are included in the balance.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
+    * @param date return balance for specific date
     * @return balance of the account as MyMoneyMoney object
     */
-  const MyMoneyMoney totalBalance(const QCString& id);
+  const MyMoneyMoney totalBalance(const QCString& id, const QDate& date = QDate());
 
   /**
     * Returns the institution of a given ID
@@ -419,25 +429,36 @@ public:
   const QValueList<MyMoneyInstitution> institutionList(void) const;
 
   /**
-    * This method returns a list of the accounts
-    * inside a MyMoneyFile object
+    * This method returns a list of accounts inside a MyMoneyFile object.
+    * An optional parameter is a list of id's. If this list is emtpy (the default)
+    * the returned list contains all accounts, otherwise only those referenced
+    * in the id-list.
     *
-    * @return QMap<QString,MyMoneyAccount> with accounts
+    * @param idlist QCStringList of account ids of those accounts that
+    *        should be returned. If this list is empty, all accounts
+    *        currently known will be returned.
+    *
+    * @param recursive if @p true, then recurse in all found accounts. The default is @p false
+    *
+    * @return QValueList containing the account objects
     */
-  const QValueList<MyMoneyAccount> accountList(void) const;
+  const QValueList<MyMoneyAccount> accountList(const QCStringList& idlist = QCStringList(), const bool recursive = false) const;
 
   /**
     * This method is used to pull a list of transactions from the file
     * global transaction pool. It returns all those transactions
     * that match the filter passed as argument. If the filter is empty,
     * the whole journal will be returned.
+    * The list returned is sorted according to the transactions posting date.
+    * If more than one transaction exists for the same date, the order among
+    * them is undefined.
     *
     * @param filter MyMoneyTransactionFilter object with the match criteria
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
   const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
-    
+
   /**
     * This method is used to pull a list of transactions from the file
     * global transaction pool. It returns either the whole journal or
@@ -511,26 +532,39 @@ public:
     */
   const MyMoneyAccount income(void) const { return account(STD_ACC_INCOME); };
 
+  /**
+    * This method is used to return the standard equity account
+    * @return MyMoneyAccount equity account(group)
+    */
+  const MyMoneyAccount equity(void) const { return account(STD_ACC_EQUITY); };
+
   virtual void loadAccount(const MyMoneyAccount& acc);
   virtual void loadTransaction(const MyMoneyTransaction& tr);
   virtual void loadInstitution(const MyMoneyInstitution& inst);
   virtual void loadPayee(const MyMoneyPayee& payee);
   virtual void loadSchedule(const MyMoneySchedule& sched);
-  
+  virtual void loadSecurity(const MyMoneySecurity& security);
+  virtual void loadCurrency(const MyMoneySecurity& currency);
+
   virtual void loadAccountId(const unsigned long id);
   virtual void loadTransactionId(const unsigned long id);
   virtual void loadPayeeId(const unsigned long id);
   virtual void loadInstitutionId(const unsigned long id);
   virtual void loadScheduleId(const unsigned long id);
-  
+  virtual void loadSecurityId(const unsigned long id);
+  virtual void loadReportId(const unsigned long id);
+
   virtual const unsigned long accountId(void) { return m_nextAccountID; };
   virtual const unsigned long transactionId(void) { return m_nextTransactionID; };
   virtual const unsigned long payeeId(void) { return m_nextPayeeID; };
   virtual const unsigned long institutionId(void) { return m_nextInstitutionID; };
   virtual const unsigned long scheduleId(void) { return m_nextScheduleID; };
-  
+  virtual const unsigned long securityId(void) { return m_nextSecurityID; };
+  virtual const unsigned long reportId(void) { return m_nextReportID; };
+
+
   /**
-    * This method is used to extract a value from 
+    * This method is used to extract a value from
     * KeyValueContainer. For details see MyMoneyKeyValueContainer::value().
     *
     * @param key const reference to QCString containing the key
@@ -539,7 +573,7 @@ public:
   const QString value(const QCString& key) const;
 
   /**
-    * This method is used to set a value in the 
+    * This method is used to set a value in the
     * KeyValueContainer. For details see MyMoneyKeyValueContainer::setValue().
     *
     * @param key const reference to QCString containing the key
@@ -605,6 +639,120 @@ public:
     */
   const MyMoneySchedule schedule(const QCString& id) const;
 
+  /**
+    * This method is used to create a new security object.  The ID will be created
+    * automatically. The object passed with the parameter @p security is modified
+    * to contain the assigned id.
+    *
+    * An exception will be thrown upon error conditions.
+    *
+    * @param security MyMoneySecurity filled with data
+    */
+  virtual void addSecurity(MyMoneySecurity& security);
+
+#if 0
+  /**
+    * This method is used to add a new security object to the engine.
+    * The ID of the object is the trading symbol, so there is no need for an additional
+    * ID since the symbol is guaranteed to be unique.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object
+    */
+  void addSecurity(MyMoneySecurity& security);
+#endif
+
+  /**
+    * This method is used to modify an existing MyMoneySchedule
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be updated
+    */
+  void modifySecurity(const MyMoneySecurity& security);
+
+  /**
+    * This method is used to remove an existing MyMoneySecurity object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param security reference to the MyMoneySecurity object to be removed
+    */
+  void removeSecurity(const MyMoneySecurity& security);
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  const MyMoneySecurity security(const QCString& id) const;
+
+
+  /**
+    * This method returns a list of security objects that the engine has
+    * knowledge of.
+    */
+  const QValueList<MyMoneySecurity> securityList(void) const;
+
+  /**
+    * This method is used to add a new currency object to the engine.
+    * The ID of the object is the trading symbol, so there is no need for an additional
+    * ID since the symbol is guaranteed to be unique.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneyCurrency object
+    */
+  void addCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to modify an existing MyMoneyCurrency
+    * object.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneyCurrency object
+    */
+  void modifyCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to remove an existing MyMoneyCurrency object
+    * from the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param currency reference to the MyMoneyCurrency object
+    */
+  void removeCurrency(const MyMoneySecurity& currency);
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  const MyMoneySecurity currency(const QCString& id) const;
+
+  /**
+    * This method is used to retrieve the list of all currencies
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyCurrency objects.
+    */
+  const QValueList<MyMoneySecurity> currencyList(void) const;
+
   /**
     * This method is used to extract a list of scheduled transactions
     * according to the filter criteria passed as arguments.
@@ -615,8 +763,8 @@ public:
     * @param type      only schedules of type @p type are searched for.
     *                  See MyMoneySchedule::typeE for details.
     *                  Default is MyMoneySchedule::TYPE_ANY
-    * @param occurance only schedules of occurance type @p occurance are searched for.
-    *                  See MyMoneySchedule::occuranceE for details.
+    * @param occurence only schedules of occurence type @p occurence are searched for.
+    *                  See MyMoneySchedule::occurenceE for details.
     *                  Default is MyMoneySchedule::OCCUR_ANY
     * @param paymentType only schedules of payment method @p paymentType
     *                  are searched for.
@@ -631,20 +779,113 @@ public:
     *
     * @return const QValueList<MyMoneySchedule> list of schedule objects.
     */
-  const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
-                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
-                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
-                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
-                                     const QDate& = QDate(),
-                                     const QDate& = QDate(),
-                                     const bool = false) const;
+  const QValueList<MyMoneySchedule> scheduleList(const QCString& accountId = QCString(),
+                                     const MyMoneySchedule::typeE type = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE occurence = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE paymentType = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& startDate = QDate(),
+                                     const QDate& endDate = QDate(),
+                                     const bool overdue = false) const;
 
   QValueList<MyMoneySchedule> scheduleListEx( int scheduleTypes,
                                               int scheduleOcurrences,
                                               int schedulePaymentTypes,
                                               QDate startDate,
                                               const QCStringList& accounts=QCStringList()) const;
-    
+
+  /**
+    * This method is used to retrieve the list of all reports
+    * known to the engine.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @return QValueList of all MyMoneyReport objects.
+    */
+  const QValueList<MyMoneyReport> reportList( void ) const;
+
+  /**
+    * This method is used to add a new report to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report reference to the MyMoneyReport object
+    */
+  void addReport( MyMoneyReport& report );
+
+  /**
+    * This method is used to load a report into the engine.  This is
+    * used when loading from storage, and an ID is already known.  It
+    * is similiar to addReport, however loadReport will not assign
+    * a new ID.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report reference to the MyMoneyReport object
+    */
+  void loadReport( const MyMoneyReport& report );
+
+  /**
+    * This method is used to modify an existing MyMoneyReport
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report const reference to the MyMoneyReport object to be updated
+    */
+  void modifyReport( const MyMoneyReport& report );
+
+  /**
+    * This method returns the number of reports currently known to file
+    * in the range 0..MAXUINT
+    *
+    * @return number of reports known to file
+    */
+  unsigned countReports(void) const;
+
+  /**
+    * This method is used to retrieve a single MyMoneyReport object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneyReport object
+    * @return MyMoneyReport object
+    */
+  MyMoneyReport report( const QCString& id ) const;
+
+  /**
+    * This method is used to remove an existing MyMoneyReport object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param report const reference to the MyMoneyReport object to be updated
+    */
+  void removeReport(const MyMoneyReport& report);
+
+  /**
+    * This method adds/replaces a price to/from the price list
+    */
+  void addPrice(const MyMoneyPrice& price);
+
+  /**
+    * This method removes a price from the price list
+    */
+  void removePrice(const MyMoneyPrice& price);
+
+  /**
+    * This method retrieves a price from the price list
+    */
+  const MyMoneyPrice price(const QCString& fromId, const QCString& toId, const QDate& date, const bool exactDate) const;
+
+  /**
+    * This method returns a list of all price entries.
+    */
+  const MyMoneyPriceList priceList(void) const;
+
 private:
 
   static const int INSTITUTION_ID_SIZE = 6;
@@ -652,6 +893,8 @@ private:
   static const int TRANSACTION_ID_SIZE = 18;
   static const int PAYEE_ID_SIZE = 6;
   static const int SCHEDULE_ID_SIZE = 6;
+  static const int SECURITY_ID_SIZE = 6;
+  static const int REPORT_ID_SIZE = 6;
 
   static const int YEAR_SIZE = 4;
   static const int MONTH_SIZE = 2;
@@ -675,7 +918,7 @@ private:
     * This method is used to invalidate the cached balance for
     * the selected account and all it's parents.
     *
-    * @param account id of the account in question
+    * @param id id of the account in question
     */
   void invalidateBalanceCache(const QCString& id);
 
@@ -756,6 +999,15 @@ private:
     */
   unsigned long m_nextScheduleID;
 
+  /**
+    * The member variable m_nextSecurityID keeps the number that will be
+    * assigned to the next security object created.  It is maintained by
+    * nextSecurityID()
+    */
+  unsigned long m_nextSecurityID;
+
+  unsigned long m_nextReportID;
+
   /**
     * The member variable m_institutionList is the container for the
     * institutions known within this file.
@@ -797,7 +1049,25 @@ private:
     * A list containing all the scheduled transactions
     */
   QMap<QCString, MyMoneySchedule> m_scheduleList;
-  
+
+  /**
+    * A list containing all the security information objects.  Each object
+    * can represent a stock, bond, or mutual fund.  It contains a price
+    * history that a user can add entries to.  The price history will be used
+    * to determine the cost basis for sales, as well as the source of
+    * information for reports in a security account.
+    */
+  QMap<QCString, MyMoneySecurity> m_securitiesList;
+
+  /**
+    * A list containing all the currency information objects.
+    */
+  QMap<QCString, MyMoneySecurity> m_currencyList;
+
+  QMap<QCString, MyMoneyReport> m_reportList;
+
+  MyMoneyPriceList              m_priceList;
+
   /**
     * This member signals if the file has been modified or not
     */
@@ -846,6 +1116,14 @@ private:
     */
   const QCString nextScheduleID(void);
 
+  /**
+    * This method is used to get the next valid ID for an security object.
+    * @return id for an security object
+    */
+  const QCString nextSecurityID(void);
+
+  const QCString nextReportID(void);
+
   /**
     * This method re-parents an existing account
     *
