@@ -30,13 +30,17 @@
 #include <qmap.h>
 
 // ----------------------------------------------------------------------------
-// Third party Includes
+// KDE Includes
+
+#include "kdecompat.h"
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "mymoneystoragexml.h"
 #include "../../kmymoneyutils.h"
+#include "../mymoneyreport.h"
+#include "../mymoneyinstitution.h"
 
 unsigned int MyMoneyStorageXML::fileVersionRead = 0;
 unsigned int MyMoneyStorageXML::fileVersionWrite = 0;
@@ -49,7 +53,7 @@ MyMoneyStorageXML::MyMoneyStorageXML()
 
 MyMoneyStorageXML::~MyMoneyStorageXML()
 {
-  
+
 }
 
 //Function to read in the file, send to XML parser.
@@ -62,7 +66,7 @@ void MyMoneyStorageXML::readFile(QIODevice* pDevice, IMyMoneySerialize* storage)
     return;
   }
   m_storage = storage;
-  
+
   m_doc = new QDomDocument;
   Q_CHECK_PTR(m_doc);
   if(m_doc->setContent(pDevice, FALSE))
@@ -101,18 +105,36 @@ void MyMoneyStorageXML::readFile(QIODevice* pDevice, IMyMoneySerialize* storage)
         {
           readTransactions(childElement);
         }
-        else if(QString("KEYVALPAIRS") == childElement.tagName())
+        else if(QString("KEYVALUEPAIRS") == childElement.tagName())
         {
           m_storage->setPairs(readKeyValuePairs(childElement));
+          m_baseCurrencyId = m_storage->pairs()["kmm-baseCurrency"];
         }
         else if(QString("SCHEDULES") == childElement.tagName())
         {
           readSchedules(childElement);
         }
+        else if(QString("EQUITIES") == childElement.tagName()
+             || QString("SECURITIES") == childElement.tagName())
+        {
+          readSecurities(childElement);
+        }
+        else if(QString("CURRENCIES") == childElement.tagName())
+        {
+          readCurrencies(childElement);
+        }
+        else if(QString("PRICES") == childElement.tagName())
+        {
+          readPrices(childElement);
+        }
+        else if(QString("REPORTS") == childElement.tagName())
+        {
+          readReports(childElement);
+        }
         child = child.nextSibling();
       }
     }
-    
+
     delete m_doc;
     m_doc = NULL;
 
@@ -126,6 +148,8 @@ void MyMoneyStorageXML::readFile(QIODevice* pDevice, IMyMoneySerialize* storage)
   }
   else
   {
+    delete m_doc;
+    m_doc = NULL;
     throw new MYMONEYEXCEPTION("File was not parsable!");
   }
 }
@@ -152,7 +176,7 @@ void MyMoneyStorageXML::writeFile(QIODevice* qf, IMyMoneySerialize* storage)
   QDomElement fileInfo = m_doc->createElement("FILEINFO");
   writeFileInformation(fileInfo);
   mainElement.appendChild(fileInfo);
-  
+
   QDomElement userInfo = m_doc->createElement("USER");
   writeUserInformation(userInfo);
   mainElement.appendChild(userInfo);
@@ -180,9 +204,31 @@ void MyMoneyStorageXML::writeFile(QIODevice* qf, IMyMoneySerialize* storage)
   writeSchedules(schedules);
   mainElement.appendChild(schedules);
 
+  QDomElement equities = m_doc->createElement("SECURITIES");
+  writeSecurities(equities);
+  mainElement.appendChild(equities);
+
+  QDomElement currencies = m_doc->createElement("CURRENCIES");
+  writeCurrencies(currencies);
+  mainElement.appendChild(currencies);
+
+  QDomElement prices = m_doc->createElement("PRICES");
+  writePrices(prices);
+  mainElement.appendChild(prices);
+
+  QDomElement reports = m_doc->createElement("REPORTS");
+  writeReports(reports);
+  mainElement.appendChild(reports);
+
   QTextStream stream(qf);
+#if KDE_IS_VERSION(3,2,0)
   stream.setEncoding(QTextStream::UnicodeUTF8);
   stream << m_doc->toString();
+#else
+  //stream.setEncoding(QTextStream::Locale);
+  QCString temp = m_doc->toCString();
+  stream << temp.data();
+#endif
 
   // qDebug("File contains %s", temp.data());
 
@@ -199,17 +245,17 @@ void MyMoneyStorageXML::writeFile(QIODevice* qf, IMyMoneySerialize* storage)
 void MyMoneyStorageXML::readFileInformation(QDomElement fileInfo)
 {
   signalProgress(0, 3, QObject::tr("Loading file information..."));
-  
+
   QDomElement temp = findChildElement(QString("CREATION_DATE"), fileInfo);
   QString strDate = QStringEmpty(temp.attribute("date"));
   m_storage->setCreationDate(getDate(strDate));
   signalProgress(1, 0);
-  
+
   temp = findChildElement(QString("LAST_MODIFIED_DATE"), fileInfo);
   strDate = QStringEmpty(temp.attribute("date"));
   m_storage->setLastModificationDate(getDate(strDate));
   signalProgress(2, 0);
-  
+
   temp = findChildElement(QString("VERSION"), fileInfo);
   QString strVersion = QStringEmpty(temp.attribute("id"));
   fileVersionRead = strVersion.toUInt(NULL, 16);
@@ -222,11 +268,11 @@ void MyMoneyStorageXML::writeFileInformation(QDomElement& fileInfo)
   QDomElement creationDate = m_doc->createElement("CREATION_DATE");
   creationDate.setAttribute(QString("date"), getString(m_storage->creationDate()));
   fileInfo.appendChild(creationDate);
-  
+
   QDomElement lastModifiedDate = m_doc->createElement("LAST_MODIFIED_DATE");
   lastModifiedDate.setAttribute(QString("date"), getString(m_storage->lastModificationDate()));
   fileInfo.appendChild(lastModifiedDate);
-  
+
   QDomElement version = m_doc->createElement("VERSION");
 
   //if we haven't written a file yet, write using the default version.
@@ -300,7 +346,7 @@ void MyMoneyStorageXML::readInstitutions(QDomElement& institutions)
         {
           m_storage->loadInstitutionId(id);
         }
-        
+
         //return childElement;
       }
 
@@ -324,6 +370,8 @@ void MyMoneyStorageXML::writeInstitutions(QDomElement& institutions)
   }
 }
 
+//#include "../mymoneykeyvaluecontainer.h"
+
 MyMoneyInstitution MyMoneyStorageXML::readInstitution(const QDomElement& institution)
 {
   MyMoneyInstitution i;
@@ -368,7 +416,22 @@ MyMoneyInstitution MyMoneyStorageXML::readInstitution(const QDomElement& institu
   {
     qWarning("XMLREADER: Institution %s does not have an accountids section.", i.name().data());
   }
-    
+
+  QDomElement ofxsettings = findChildElement(QString("OFXSETTINGS"), institution);
+  if(!ofxsettings.isNull() && ofxsettings.isElement())
+  {
+    MyMoneyKeyValueContainer values;
+
+    QDomNamedNodeMap attributes = ofxsettings.attributes();
+    int index = attributes.count();
+    while ( index-- )
+    {
+      QDomAttr it_attr = attributes.item(index).toAttr();
+      values.setValue(it_attr.name().utf8(),it_attr.value());
+    }
+    i.setOfxConnectionSettings(values);
+  }
+
   return MyMoneyInstitution(id, i);
 }
 
@@ -383,7 +446,6 @@ void MyMoneyStorageXML::writeInstitution(QDomElement& institution, const MyMoney
   address.setAttribute(QString("street"), i.street());
   address.setAttribute(QString("city"), i.city());
   address.setAttribute(QString("zip"), i.postcode());
-  address.setAttribute(QString("state"), i.town());
   address.setAttribute(QString("telephone"), i.telephone());
 
   institution.appendChild(address);
@@ -398,6 +460,20 @@ void MyMoneyStorageXML::writeInstitution(QDomElement& institution, const MyMoney
   }
 
   institution.appendChild(accounts);
+
+  QMap<QCString,QString> s = i.ofxConnectionSettings().pairs();
+  if ( s.count() )
+  {
+    QDomElement ofxsettings = m_doc->createElement("OFXSETTINGS");
+    QMap<QCString,QString>::const_iterator it_key = s.begin();
+    while ( it_key != s.end() )
+    {
+      ofxsettings.setAttribute(it_key.key(), it_key.data());
+      ++it_key;
+    }
+
+    institution.appendChild(ofxsettings);
+  }
 }
 
 void MyMoneyStorageXML::readPayees(QDomElement& payees)
@@ -406,7 +482,7 @@ void MyMoneyStorageXML::readPayees(QDomElement& payees)
   QDomNode child = payees.firstChild();
   int x = 0;
   signalProgress(0, getChildCount(payees), QObject::tr("Loading payees..."));
-  
+
   while(!child.isNull())
   {
     if(child.isElement())
@@ -437,7 +513,7 @@ void MyMoneyStorageXML::writePayees(QDomElement& payees)
   QValueList<MyMoneyPayee>::ConstIterator it;
 
   list = m_storage->payeeList();
-  
+
   for(it = list.begin(); it != list.end(); ++it)
   {
     QDomElement payee = m_doc->createElement("PAYEE");
@@ -498,7 +574,7 @@ void MyMoneyStorageXML::readAccounts(QDomElement& accounts)
   QDomNode child = accounts.firstChild();
   int x = 0;
   signalProgress(0, getChildCount(accounts), QObject::tr("Loading accounts..."));
-  
+
   while(!child.isNull() && child.isElement())
   {
     QDomElement childElement = child.toElement();
@@ -529,7 +605,7 @@ void MyMoneyStorageXML::writeAccounts(QDomElement& accounts)
 
   list = m_storage->accountList();
 
-  QDomElement asset, liability, expense, income;
+  QDomElement asset, liability, expense, income, equity;
 
   asset = m_doc->createElement("ACCOUNT");
   writeAccount(asset, m_storage->asset());
@@ -547,6 +623,10 @@ void MyMoneyStorageXML::writeAccounts(QDomElement& accounts)
   writeAccount(income, m_storage->income());
   accounts.appendChild(income);
 
+  equity = m_doc->createElement("ACCOUNT");
+  writeAccount(equity, m_storage->equity());
+  accounts.appendChild(equity);
+
   //signalProgress(0, list.count(), QObject::tr("Saving accounts..."));
   int i = 0;
   for(it = list.begin(); it != list.end(); ++it, ++i)
@@ -564,17 +644,18 @@ MyMoneyAccount MyMoneyStorageXML::readAccount(const QDomElement& account)
   MyMoneyAccount acc;
   QCString id;
   QString tmp;
-  
+
   acc.setName(account.attribute(QString("name")));
 
   // qDebug("Reading information for account %s", acc.name().data());
-  
+
   acc.setParentAccountId(QCStringEmpty(account.attribute(QString("parentaccount"))));
   acc.setLastModified(getDate(QStringEmpty(account.attribute(QString("lastmodified")))));
   acc.setLastReconciliationDate(getDate(QStringEmpty(account.attribute(QString("lastreconciled")))));
   acc.setInstitutionId(QCStringEmpty(account.attribute(QString("institution"))));
   acc.setNumber(QStringEmpty(account.attribute(QString("number"))));
   acc.setOpeningDate(getDate(QStringEmpty(account.attribute(QString("opened")))));
+  acc.setCurrencyId(QCStringEmpty(account.attribute(QString("currency"))));
 
   tmp = QStringEmpty(account.attribute(QString("type")));
   bool bOK = false;
@@ -587,10 +668,10 @@ MyMoneyAccount MyMoneyStorageXML::readAccount(const QDomElement& account)
   {
     qWarning("XMLREADER: Account %s had invalid or no account type information.", acc.name().data());
   }
-      
+
   acc.setOpeningBalance(MyMoneyMoney(account.attribute(QString("openingbalance"))));
   acc.setDescription(account.attribute(QString("description")));
-  
+
   id = QCStringEmpty(account.attribute(QString("id")));
   Q_ASSERT(id.size());
   // qDebug("Account %s has id of %s, type of %d, parent is %s.", acc.name().data(), id.data(), type, acc.parentAccountId().data());
@@ -608,11 +689,11 @@ MyMoneyAccount MyMoneyStorageXML::readAccount(const QDomElement& account)
         QDomElement childElement = child.toElement();
         if(QString("SUBACCOUNT") == childElement.tagName())
         {
-          acc.addAccountId(QCString(childElement.attribute(QString("id")))); 
+          acc.addAccountId(QCString(childElement.attribute(QString("id"))));
         }
       }
       child = child.nextSibling();
-    }  
+    }
   }
 
   /////////////////////////////////////////////////////////////////////////////////////////
@@ -622,12 +703,12 @@ MyMoneyAccount MyMoneyStorageXML::readAccount(const QDomElement& account)
   {
     acc.setPairs(readKeyValuePairs(keyValPairs));
   }
-  
+
   return MyMoneyAccount(id, acc);
 }
 
 void MyMoneyStorageXML::writeAccount(QDomElement& account, const MyMoneyAccount& p)
-{   
+{
   account.setAttribute(QString("parentaccount"), p.parentAccountId());
   account.setAttribute(QString("lastreconciled"), getString(p.lastReconciliationDate()));
   account.setAttribute(QString("lastmodified"), getString(p.lastModified()));
@@ -635,10 +716,12 @@ void MyMoneyStorageXML::writeAccount(QDomElement& account, const MyMoneyAccount&
   account.setAttribute(QString("opened"), getString(p.openingDate()));
   account.setAttribute(QString("number"), p.number());
   account.setAttribute(QString("openingbalance"), p.openingBalance().toString());
-  account.setAttribute(QString("type"), p.accountType());  
+  account.setAttribute(QString("type"), p.accountType());
   account.setAttribute(QString("id"), p.id());
   account.setAttribute(QString("name"), p.name());
   account.setAttribute(QString("description"), p.description());
+  if(!p.currencyId().isEmpty())
+    account.setAttribute(QString("currency"), p.currencyId());
 
   //Add in subaccount information, if this account has subaccounts.
   if(p.accountCount())
@@ -657,7 +740,7 @@ void MyMoneyStorageXML::writeAccount(QDomElement& account, const MyMoneyAccount&
   }
 
   //Add in Key-Value Pairs for accounts.
-  QDomElement keyValPairs = writeKeyValuePairs(p.pairs()); 
+  QDomElement keyValPairs = writeKeyValuePairs(p.pairs());
   account.appendChild(keyValPairs);
 }
 
@@ -696,7 +779,7 @@ void MyMoneyStorageXML::writeTransactions(QDomElement& transactions)
   MyMoneyTransactionFilter filter;
 
   list = m_storage->transactionList(filter);
-  
+
   //signalProgress(0, list.count(), QObject::tr("Saving transactions..."));
 
   int i = 0;
@@ -717,6 +800,8 @@ MyMoneyTransaction MyMoneyStorageXML::readTransaction(QDomElement& transaction,
   t.setEntryDate(getDate(QStringEmpty(transaction.attribute(QString("entrydate")))));
   t.setPostDate(getDate(QStringEmpty(transaction.attribute(QString("postdate")))));
   t.setMemo(QStringEmpty(transaction.attribute(QString("memo"))));
+  t.setCommodity(QCStringEmpty(transaction.attribute(QString("commodity"))));
+  t.setBankID(QStringEmpty(transaction.attribute(QString("bankid"))));
 
   id = QCStringEmpty(transaction.attribute(QString("id")));
   if(!withinSchedule)
@@ -748,6 +833,8 @@ void MyMoneyStorageXML::writeTransaction(QDomElement& transaction, const MyMoney
   transaction.setAttribute(QString("postdate"), getString(tx.postDate()));
   transaction.setAttribute(QString("memo"), tx.memo());
   transaction.setAttribute(QString("entrydate"), getString(tx.entryDate()));
+  transaction.setAttribute(QString("commodity"), tx.commodity());
+  transaction.setAttribute(QString("bankid"), tx.bankID());
 
   QDomElement splits = m_doc->createElement("SPLITS");
   QValueList<MyMoneySplit> splitList = tx.splits();
@@ -766,7 +853,7 @@ void MyMoneyStorageXML::readSchedules(QDomElement& schedules)
   QDomNode child = schedules.firstChild();
   int x = 0;
   signalProgress(0, getChildCount(schedules), QObject::tr("Loading schedules..."));
-  
+
   while(!child.isNull() && child.isElement())
   {
     QDomElement childElement = child.toElement();
@@ -802,7 +889,7 @@ void MyMoneyStorageXML::writeSchedules(QDomElement& scheduled)
     scheduled.appendChild(schedTx);
   }
 }
- 
+
 MyMoneySchedule MyMoneyStorageXML::readSchedule(QDomElement& schedule)
 {
   MyMoneySchedule sc;
@@ -845,7 +932,7 @@ MyMoneySchedule MyMoneyStorageXML::readSchedule(QDomElement& schedule)
     sc.setTransaction(t);
   }
 
-  return sc;  
+  return sc;
 }
 
 void MyMoneyStorageXML::writeSchedule(QDomElement& scheduledTx, const MyMoneySchedule& tx)
@@ -911,16 +998,17 @@ MyMoneySplit MyMoneyStorageXML::readSplit(QDomElement& splitElement)
 {
   MyMoneySplit split;
   QString strTmp;
-  
+
   split.setPayeeId(QCStringEmpty(splitElement.attribute(QString("payee"))));
   split.setReconcileDate(getDate(QStringEmpty(splitElement.attribute(QString("reconciledate")))));
   split.setAction(QCStringEmpty(splitElement.attribute(QString("action"))));
   split.setReconcileFlag(static_cast<MyMoneySplit::reconcileFlagE>(splitElement.attribute(QString("reconcileflag")).toInt()));
   split.setMemo(QStringEmpty(splitElement.attribute(QString("memo"))));
   split.setValue(MyMoneyMoney(QStringEmpty(splitElement.attribute(QString("value")))));
+  split.setShares(MyMoneyMoney(QStringEmpty(splitElement.attribute(QString("shares")))));
   split.setAccountId(QCStringEmpty(splitElement.attribute(QString("account"))));
   split.setNumber(QStringEmpty(splitElement.attribute(QString("number"))));
- 
+
   return split;
 }
 
@@ -931,12 +1019,230 @@ void MyMoneyStorageXML::writeSplit(QDomElement& splitElement, const MyMoneySplit
   splitElement.setAttribute(QString("action"), split.action());
   splitElement.setAttribute(QString("reconcileflag"), split.reconcileFlag());
   splitElement.setAttribute(QString("value"), split.value().toString());
+  splitElement.setAttribute(QString("shares"), split.shares().toString());
   splitElement.setAttribute(QString("memo"), split.memo());
-  splitElement.setAttribute(QString("id"), split.id());
+  // No need to write the split id as it will be re-assigned when the file is read
+  // splitElement.setAttribute(QString("id"), split.id());
   splitElement.setAttribute(QString("account"), split.accountId());
   splitElement.setAttribute(QString("number"), split.number());
 }
 
+void MyMoneyStorageXML::writeSecurities(QDomElement& equities)
+{
+  QValueList<MyMoneySecurity> securityList = m_storage->securityList();
+  if(securityList.size())
+  {
+    for(QValueList<MyMoneySecurity>::Iterator it = securityList.begin(); it != securityList.end(); ++it)
+    {
+      QDomElement security = m_doc->createElement("SECURITY");
+      writeSecurity(security, (*it));
+      equities.appendChild(security);
+    }
+  }
+
+}
+
+void MyMoneyStorageXML::writeSecurity(QDomElement& securityElement, const MyMoneySecurity& security)
+{
+  securityElement.setAttribute(QString("name"), security.name());
+  securityElement.setAttribute(QString("symbol"), security.tradingSymbol());
+  securityElement.setAttribute(QString("type"), static_cast<int>(security.securityType()));
+  securityElement.setAttribute(QString("id"), security.id());
+  securityElement.setAttribute(QString("saf"), security.smallestAccountFraction());
+  if(!security.isCurrency())
+    securityElement.setAttribute(QString("trading-currency"), security.tradingCurrency());
+
+  //Add in Key-Value Pairs for security
+  QDomElement keyValPairs = writeKeyValuePairs(security.pairs());
+  securityElement.appendChild(keyValPairs);
+#if 0
+  QDomElement history = m_doc->createElement("HISTORY");
+
+  equity_price_history priceHistory = equity.priceHistory();
+  if(priceHistory.size())
+  {
+    for(equity_price_history::ConstIterator it = priceHistory.begin(); it != priceHistory.end(); ++it)
+    {
+      QDomElement entry = m_doc->createElement("ENTRY");
+      entry.setAttribute(QString("date"), getString(it.key()));
+      entry.setAttribute(QString("price"), it.data().toString());
+      history.appendChild(entry);
+    }
+  }
+
+  equityElement.appendChild(history);
+#endif
+}
+
+void MyMoneyStorageXML::readSecurities(QDomElement& securities)
+{
+  unsigned long id = 0;
+  QDomNode child = securities.firstChild();
+  while(!child.isNull() && child.isElement())
+  {
+    QDomElement childElement = child.toElement();
+    if(QString("EQUITY") == childElement.tagName()
+    || QString("SECURITY") == childElement.tagName())
+    {
+      MyMoneySecurity security = readSecurity(childElement);
+
+      //tell the storage objects we have a new security object.
+      m_storage->loadSecurity(security);
+
+      id = extractId(security.id().data());
+      if(id > m_storage->securityId())
+      {
+        m_storage->loadSecurityId(id);
+      }
+    }
+    child = child.nextSibling();
+  }
+}
+
+void MyMoneyStorageXML::writeCurrencies(QDomElement& currencies)
+{
+  QValueList<MyMoneySecurity> currencyList = m_storage->currencyList();
+  if(currencyList.size())
+  {
+    for(QValueList<MyMoneySecurity>::Iterator it = currencyList.begin(); it != currencyList.end(); ++it)
+    {
+      QDomElement currency = m_doc->createElement("CURRENCY");
+      writeCurrency(currency, (*it));
+      currencies.appendChild(currency);
+    }
+  }
+}
+
+void MyMoneyStorageXML::writeCurrency(QDomElement& currencyElement, const MyMoneySecurity& currency)
+{
+  writeSecurity(currencyElement, currency);
+  currencyElement.setAttribute(QString("ppu"), currency.partsPerUnit());
+  currencyElement.setAttribute(QString("scf"), currency.smallestCashFraction());
+  currencyElement.setAttribute(QString("saf"), currency.smallestAccountFraction());
+}
+
+void MyMoneyStorageXML::readCurrencies(QDomElement& currencies)
+{
+  QDomNode child = currencies.firstChild();
+  while(!child.isNull() && child.isElement())
+  {
+    QDomElement childElement = child.toElement();
+    if(QString("CURRENCY") == childElement.tagName())
+    {
+      MyMoneySecurity currency = readCurrency(childElement);
+
+      //tell the storage objects we have a new currency object.
+      m_storage->loadCurrency(currency);
+    }
+    child = child.nextSibling();
+  }
+}
+
+const MyMoneySecurity MyMoneyStorageXML::readCurrency(QDomElement& currencyElement)
+{
+  MyMoneySecurity c(readSecurity(currencyElement));
+
+  c.setPartsPerUnit(currencyElement.attribute(QString("ppu")).toInt());
+  c.setSmallestCashFraction(currencyElement.attribute(QString("scf")).toInt());
+  c.setSmallestAccountFraction(currencyElement.attribute(QString("saf")).toInt());
+
+  return c;
+}
+
+MyMoneySecurity MyMoneyStorageXML::readSecurity(QDomElement& securityElement)
+{
+  QCString id;
+  MyMoneySecurity e;
+
+  e.setName(QStringEmpty(securityElement.attribute(QString("name"))));
+  e.setTradingSymbol(QStringEmpty(securityElement.attribute(QString("symbol"))));
+  e.setSecurityType(static_cast<MyMoneySecurity::eSECURITYTYPE>(securityElement.attribute(QString("type")).toInt()));
+  // some old versions used to store type NONE
+  // let's default this to stock
+  if(e.securityType() == MyMoneySecurity::SECURITY_NONE)
+    e.setSecurityType(MyMoneySecurity::SECURITY_STOCK);
+
+  e.setTradingCurrency(QCStringEmpty(securityElement.attribute(QString("trading-currency"))));
+  if(e.tradingCurrency().isEmpty()) {
+    e.setTradingCurrency(m_baseCurrencyId);
+  }
+
+  int saf = securityElement.attribute(QString("saf")).toInt();
+  if(saf == 0)
+    saf = 100;
+  e.setSmallestAccountFraction(saf);
+
+  id = QStringEmpty(securityElement.attribute(QString("id")));
+
+  // FIXME: keep the history part for some time. After 0.8 this can be wiped out
+  QDomElement history = findChildElement(QString("HISTORY"), securityElement);
+  if(!history.isNull() && history.isElement())
+  {
+    QDomNode child = history.firstChild();
+    while(!child.isNull())
+    {
+      if(child.isElement())
+      {
+        QDomElement childElement = child.toElement();
+        if(QString("ENTRY") == childElement.tagName())
+        {
+          QDate date = getDate(QStringEmpty(childElement.attribute(QString("date"))));
+          MyMoneyMoney money(QStringEmpty(childElement.attribute(QString("price"))));
+          // e.addPriceHistory(date, money);
+          MyMoneyPrice price(id, m_baseCurrencyId, date, money);
+          m_storage->addPrice(price);
+        }
+      }
+      child = child.nextSibling();
+    }
+  }
+
+  /////////////////////////////////////////////////////////////////////////////////////////
+  //  Process any KeyValue pairs information found inside the security entry.
+  QDomElement keyValPairs = findChildElement(QString("KEYVALUEPAIRS"), securityElement);
+  if(!keyValPairs.isNull() && keyValPairs.isElement())
+  {
+    e.setPairs(readKeyValuePairs(keyValPairs));
+  }
+
+  return MyMoneySecurity(id, e);
+}
+
+void MyMoneyStorageXML::readReports(QDomElement& reports)
+{
+  QDomNode child = reports.firstChild();
+
+  while(!child.isNull())
+  {
+    if(child.isElement())
+    {
+      MyMoneyReport report;
+      if ( report.read(child.toElement() ) )
+      {
+        m_storage->loadReport(report);
+
+        unsigned long id = extractId(report.id());
+        if(id > m_storage->reportId())
+          m_storage->loadReportId(id);
+      }
+    }
+    child = child.nextSibling();
+  }
+}
+
+void MyMoneyStorageXML::writeReports(QDomElement& parent) const
+{
+  QValueList<MyMoneyReport> list = m_storage->reportList();
+  QValueList<MyMoneyReport>::ConstIterator it;
+
+  for(it = list.begin(); it != list.end(); ++it)
+  {
+    QDomElement child = m_doc->createElement("REPORT");
+    (*it).write(child, m_doc);
+    parent.appendChild(child);
+  }
+}
+
 const unsigned long MyMoneyStorageXML::extractId(const QCString& txt) const
 {
   int pos;
@@ -1014,7 +1320,7 @@ QDomElement MyMoneyStorageXML::writeKeyValuePairs(const QMap<QCString, QString>
     }
     return keyValPairs;
   }
-  return QDomElement();  
+  return QDomElement();
 }
 
 QMap<QCString, QString> MyMoneyStorageXML::readKeyValuePairs(QDomElement& element)
@@ -1038,6 +1344,98 @@ QMap<QCString, QString> MyMoneyStorageXML::readKeyValuePairs(QDomElement& elemen
   return pairs;
 }
 
+void MyMoneyStorageXML::readPrices(QDomElement& prices)
+{
+  QDomNode child = prices.firstChild();
+  int x = 0;
+  signalProgress(0, getChildCount(prices), QObject::tr("Loading pricetable ..."));
+
+  while(!child.isNull())
+  {
+    if(child.isElement())
+    {
+      QDomElement childElement = child.toElement();
+      if(QString("PRICEPAIR") == childElement.tagName())
+      {
+        readPricePair(childElement);
+      }
+    }
+    child = child.nextSibling();
+    signalProgress(x++, 0);
+  }
+}
+
+void MyMoneyStorageXML::readPricePair(const QDomElement& pricePair)
+{
+  QCString from = QCString(pricePair.attribute(QString("from")));
+  QCString to = QCString(pricePair.attribute(QString("to")));
+  QDomNode child = pricePair.firstChild();
+
+  while(!child.isNull())
+  {
+    if(child.isElement())
+    {
+      QDomElement childElement = child.toElement();
+      if(QString("PRICE") == childElement.tagName())
+      {
+        MyMoneyPrice p = readPrice(from, to, childElement);
+
+        //tell the storage objects we have a new price.
+        m_storage->addPrice(p);
+      }
+    }
+    child = child.nextSibling();
+  }
+}
+
+const MyMoneyPrice MyMoneyStorageXML::readPrice(const QCString& from, const QCString& to, const QDomElement& price)
+{
+  QDate date;
+  MyMoneyMoney rate;
+  QString source;
+
+  date = QDate::fromString(price.attribute(QString("date")), Qt::ISODate);
+  rate = MyMoneyMoney(price.attribute(QString("price")));
+  source = price.attribute(QString("source"));
+
+  //create actual object to return to add into the engine's list of objects.
+  return MyMoneyPrice(from, to, date, rate, source);
+}
+
+void MyMoneyStorageXML::writePrices(QDomElement& prices)
+{
+  MyMoneyPriceList list;
+  MyMoneyPriceList::ConstIterator it;
+
+  list = m_storage->priceList();
+
+  for(it = list.begin(); it != list.end(); ++it)
+  {
+    QDomElement price = m_doc->createElement("PRICEPAIR");
+    price.setAttribute(QString("from"), it.key().first);
+    price.setAttribute(QString("to"), it.key().second);
+    writePricePair(price, *it);
+    prices.appendChild(price);
+  }
+}
+
+void MyMoneyStorageXML::writePricePair(QDomElement& price, const MyMoneyPriceEntries& p)
+{
+  MyMoneyPriceEntries::ConstIterator it;
+  for(it = p.begin(); it != p.end(); ++it) {
+    QDomElement entry = m_doc->createElement("PRICE");
+    writePrice(entry, *it);
+    price.appendChild(entry);
+  }
+}
+
+void MyMoneyStorageXML::writePrice(QDomElement& price, const MyMoneyPrice& p)
+{
+  price.setAttribute(QString("date"), p.date().toString(Qt::ISODate));
+  price.setAttribute(QString("price"), p.rate().toString());
+  price.setAttribute(QString("source"), p.source());
+}
+
 const QCString MyMoneyStorageXML::QCStringEmpty(const QString& val) const
 {
   QCString rc;
@@ -1075,3 +1473,53 @@ const uint MyMoneyStorageXML::getChildCount(const QDomElement& element) const
   return tempList.count();
 }
 
+/*!
+    This convenience function returns all of the remaining data in the
+    device.
+
+    @note It's copied from the original Qt sources and modified to
+          fix a problem with KFilterDev that does not correctly return
+          atEnd() status in certain circumstances which caused our
+          application to lock at startup.
+*/
+QByteArray QIODevice::readAll()
+{
+  if ( isDirectAccess() ) {
+    // we know the size
+    int n = size()-at(); // ### fix for 64-bit or large files?
+    int totalRead = 0;
+    QByteArray ba( n );
+    char* c = ba.data();
+    while ( n ) {
+      int r = readBlock( c, n );
+      if ( r < 0 )
+        return QByteArray();
+      n -= r;
+      c += r;
+      totalRead += r;
+      // If we have a translated file, then it is possible that
+      // we read less bytes than size() reports
+      if ( atEnd() ) {
+        ba.resize( totalRead );
+        break;
+      }
+    }
+    return ba;
+  } else {
+    // read until we reach the end
+    const int blocksize = 512;
+    int nread = 0;
+    QByteArray ba;
+    int r = 1;
+    while ( !atEnd() && r != 0) {
+      ba.resize( nread + blocksize );
+      r = readBlock( ba.data()+nread, blocksize );
+      if ( r < 0 )
+        return QByteArray();
+      nread += r;
+    }
+    ba.resize( nread );
+    return ba;
+  }
+}
+
