@@ -21,10 +21,7 @@
 #include <qcolor.h>
 #include <qstyle.h>
 #include <qglobal.h>
-
-#if QT_VERSION > 300
 #include <qpainter.h>
-#endif
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -32,11 +29,7 @@
 #include <kglobal.h>
 #include <kconfig.h>
 #include <klocale.h>
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
 #include <kmessagebox.h>
 
 // ----------------------------------------------------------------------------
@@ -46,7 +39,85 @@
 #include "../mymoney/mymoneyfile.h"
 #include "../kmymoneyutils.h"
 
-QPixmap* KAccountListItem::accountPixmap = 0;
+void KCategoryListItem::update(const QCString& accountId)
+{
+  if(m_suspendUpdate == true)
+    return;
+
+  MyMoneyFile*  file = MyMoneyFile::instance();
+  MyMoneySecurity baseCurrency = file->baseCurrency();
+
+  try {
+    MyMoneyAccount acc = file->account(accountId);
+    MyMoneySecurity currency = file->currency(acc.currencyId());
+    int prec = MyMoneyMoney::denomToPrec((acc.accountType() == MyMoneyAccount::Cash)
+                 ? currency.smallestCashFraction()
+                 : currency.smallestAccountFraction());
+
+    try {
+      MyMoneyMoney balance = file->balance(accountId);
+      MyMoneyMoney value = file->totalValue(accountId);
+      m_valueValid = file->totalValueValid(accountId);
+
+      // make sure, we translate the standard account names
+      if(file->isStandardAccount(acc.id())) {
+        if(acc.id() == file->asset().id()) {
+          setText(0, i18n("Asset"));
+        } else if(acc.id() == file->liability().id()) {
+          setText(0, i18n("Liability"));
+        } else if(acc.id() == file->income().id()) {
+          setText(0, i18n("Income"));
+        } else if(acc.id() == file->expense().id()) {
+          setText(0, i18n("Expense"));
+        } else if(acc.id() == file->equity().id()) {
+          setText(0, i18n("Equity"));
+        }
+      } else {
+        // make sure, that 'Opening Balances' is converted to
+        // the i18n version
+        QString name = acc.name();
+        name.replace(MyMoneyFile::OpeningBalancesPrefix, i18n("Opening Balances"));
+        setText(0, name);
+      }
+
+      if(!acc.parentAccountId().isEmpty())
+        setText(1, QString::number(file->transactionCount(accountId)));
+      else
+        setText(1, QString(" "));
+
+      // since income and liabilities are usually negative,
+      // we reverse the sign for display purposes
+      switch(acc.accountGroup()) {
+        case MyMoneyAccount::Income:
+        case MyMoneyAccount::Liability:
+          balance = -balance;
+          value = -value;
+          break;
+
+        default:
+          break;
+      }
+      setText(2, value.formatMoney(baseCurrency.tradingSymbol(), prec));
+
+    } catch(MyMoneyException *e) {
+      KMessageBox::detailedSorry(0, i18n("Unable to retrieve account information"),
+          (e->what() + " " + i18n("thrown in") + " " + e->file()+ ":%1").arg(e->line()));
+      delete e;
+    }
+
+  } catch(MyMoneyException *e) {
+    qDebug("Trying to get account info that does not exist anymore");
+    // try to get account info that does not exist anymore
+    delete e;
+  }
+}
+
+void KAccountListItem::setValue(const MyMoneyMoney& value, const bool valid)
+{
+  int prec = MyMoneyMoney::denomToPrec(MyMoneyFile::instance()->baseCurrency().smallestAccountFraction());
+  setText(3, value.formatMoney(MyMoneyFile::instance()->baseCurrency().tradingSymbol(), prec));
+  m_valueValid = valid;
+}
 
 KAccountListItem::KAccountListItem(KListView *parent, const MyMoneyAccount& account)
   : KListViewItem(parent)
@@ -60,74 +131,54 @@ KAccountListItem::KAccountListItem(KAccountListItem *parent, const MyMoneyAccoun
   newAccount(account);
 }
 
-void KAccountListItem::loadCache(void)
-{
-  if(accountPixmap == 0) {
-    accountPixmap = new QPixmap(KGlobal::dirs()->findResource("appdata", "icons/hicolor/22x22/actions/account.png"));
-  }
-}
-
-void KAccountListItem::cleanCache(void)
-{
-  if(accountPixmap != 0) {
-    delete accountPixmap;
-    accountPixmap = 0;
-  }
-}
-
 void KAccountListItem::newAccount(const MyMoneyAccount& account)
 {
   m_suspendUpdate = false;
-  
-  loadCache();
+  m_valueValid = true;
+
   MyMoneyFile*  file = MyMoneyFile::instance();
 
   setAccountID(account.id());
+  setAccountType(account.accountType());
 
   file->attach(account.id(), this);
 
-  setPixmap(0, *accountPixmap);
-  
-  // make sure, we translate the standard account names
-  if(file->isStandardAccount(account.id())) {
-    if(account.id() == file->asset().id()) {
-      setText(0, i18n("Asset"));
-    } else if(account.id() == file->liability().id()) {
-      setText(0, i18n("Liability"));
-    } else if(account.id() == file->income().id()) {
-      setText(0, i18n("Income"));
-    } else if(account.id() == file->expense().id()) {
-      setText(0, i18n("Expense"));
-    }
-  } else
-    setText(0, account.name());
-
-  MyMoneyMoney balance = file->totalBalance(account.id());
-  
-  // since income and liabilities are usually negative,
-  // we reverse the sign for display purposes
-  switch(account.accountGroup()) {
+  QString icon;
+  switch (MyMoneyAccount::accountGroup(accountType()))
+  {
     case MyMoneyAccount::Income:
+      icon = "account-types_income";
+      break;
+    case MyMoneyAccount::Expense:
+      icon = "account-types_expense";
+      break;
     case MyMoneyAccount::Liability:
-      balance = -balance;
+      icon = "account-types_liability";
+      break;
+    case MyMoneyAccount::Asset:
+      icon = "account-types_asset";
       break;
     default:
-      break;
+      icon = "account";
   }
-  setText(2, balance.formatMoney());
+  setPixmap(0, QPixmap(KGlobal::dirs()->findResource("appdata",QString( "icons/hicolor/22x22/actions/%1.png").arg(icon))));
+
+  // fill the columns with inital data
+  update(account.id());
 }
 
 KAccountListItem::KAccountListItem(KListView *parent, const QString& txt)
-  : KListViewItem(parent), m_bViewNormal(true), m_suspendUpdate(false)
+  : KListViewItem(parent), m_suspendUpdate(false), m_bViewNormal(true)
 {
-  setText(0, txt);  
+  setText(0, txt);
 }
 
 KAccountListItem::KAccountListItem(KListView *parent, const MyMoneyInstitution& institution)
-  : KListViewItem(parent), m_bViewNormal(true), m_suspendUpdate(false)
+  : KListViewItem(parent), m_suspendUpdate(false), m_bViewNormal(true)
 {
   setAccountID(institution.id());
   setText(0, institution.name());
+  setPixmap(0, QPixmap(KGlobal::dirs()->findResource("appdata",QString( "icons/hicolor/22x22/actions/%1.png").arg("bank"))));
 }
 
 KAccountListItem::~KAccountListItem()
@@ -135,36 +186,98 @@ KAccountListItem::~KAccountListItem()
   MyMoneyFile::instance()->detach(accountID(), this);
 }
 
+int KAccountListItem::compare(QListViewItem* i, int col, bool ascending) const
+{
+  KAccountListItem* item = static_cast<KAccountListItem*>(i);
+  if(col != 0 || MyMoneyAccount::accountGroup(accountType()) == MyMoneyAccount::accountGroup(item->accountType()))
+    return KListViewItem::compare(i, col, ascending);
+
+  return MyMoneyAccount::accountGroup(accountType()) - MyMoneyAccount::accountGroup(item->accountType());
+}
+
 void KAccountListItem::update(const QCString& accountId)
 {
   if(m_suspendUpdate == true)
     return;
-    
+
   MyMoneyFile*  file = MyMoneyFile::instance();
-    
+  MyMoneySecurity baseCurrency = file->baseCurrency();
+  MyMoneySecurity security;
+  int prec;
+
   try {
     MyMoneyAccount acc = file->account(accountId);
+    if(acc.accountType() == MyMoneyAccount::Stock) {
+      security = file->security(acc.currencyId());
+      prec = MyMoneyMoney::denomToPrec(security.smallestAccountFraction());
+
+    } else {
+      security = file->currency(acc.currencyId());
+      prec = MyMoneyMoney::denomToPrec((acc.accountType() == MyMoneyAccount::Cash)
+                ? security.smallestCashFraction()
+                : security.smallestAccountFraction());
+    }
 
     try {
-      MyMoneyMoney balance = file->totalBalance(accountId);
+      MyMoneyMoney balance = file->balance(accountId);
+      MyMoneyMoney value;
+
+      if(acc.accountType() == MyMoneyAccount::Investment) {
+        value = file->totalValue(accountId);
+        m_valueValid = file->totalValueValid(accountId);
+      } else {
+        value = file->accountValue(accountId);
+        m_valueValid = file->accountValueValid(accountId);
+      }
+
+      // make sure, we translate the standard account names
+      if(file->isStandardAccount(acc.id())) {
+        if(acc.id() == file->asset().id()) {
+          setText(0, i18n("Asset"));
+        } else if(acc.id() == file->liability().id()) {
+          setText(0, i18n("Liability"));
+        } else if(acc.id() == file->income().id()) {
+          setText(0, i18n("Income"));
+        } else if(acc.id() == file->expense().id()) {
+          setText(0, i18n("Expense"));
+        } else if(acc.id() == file->equity().id()) {
+          setText(0, i18n("Personal Equity"));
+        }
+        value = file->totalValue(accountId);
+        m_valueValid = file->totalValueValid(accountId);
+      } else
+        setText(0, acc.name());
 
-      setText(0, acc.name());
       if(!acc.parentAccountId().isEmpty())
         setText(1, QString::number(file->transactionCount(accountId)));
       else
         setText(1, QString(" "));
-      
+
+      setText(2, " ");
       // since income and liabilities are usually negative,
       // we reverse the sign for display purposes
       switch(acc.accountGroup()) {
         case MyMoneyAccount::Income:
+          setText(2, (-value).formatMoney(baseCurrency.tradingSymbol(), prec));
+          // tricky fall through here
+
         case MyMoneyAccount::Liability:
+        case MyMoneyAccount::Equity:
           balance = -balance;
+          value = -value;
           break;
+
+        case MyMoneyAccount::Expense:
+          setText(2, value.formatMoney(baseCurrency.tradingSymbol(), prec));
+          break;
+
         default:
           break;
       }
-      setText(2, balance.formatMoney());
+      if(security.id() != baseCurrency.id())
+        setText(2, balance.formatMoney(security.tradingSymbol(), prec));
+
+      setText(3, value.formatMoney(baseCurrency.tradingSymbol(), prec));
 
     } catch(MyMoneyException *e) {
       KMessageBox::detailedSorry(0, i18n("Unable to retrieve account information"),
@@ -185,7 +298,7 @@ void KAccountListItem::paintCell(QPainter *p, const QColorGroup & cg, int column
 
   QColor colour = KMyMoneyUtils::listColour();
   QColor bgColour = KMyMoneyUtils::backgroundColour();
-  
+
   QColorGroup cg2(cg);
 
   if (isAlternate())
@@ -193,8 +306,17 @@ void KAccountListItem::paintCell(QPainter *p, const QColorGroup & cg, int column
   else
     cg2.setColor(QColorGroup::Base, bgColour);
 
+  // show values with missing currency conversion rates in specific color
+  if(column == 3 && !m_valueValid) {
+    KConfig *kconfig = KGlobal::config();
+    kconfig->setGroup("List Options");
+    QColor color(0, 0, 255); // blue is the application default
+    color = kconfig->readColorEntry("listMissingConversionRate", &color);
+    cg2.setColor(QColorGroup::Text, color);
+  }
+
   QListViewItem::paintCell(p, cg2, column, width, align);
-  
+
   int indent = 0;
   if (column == 0)
   {
@@ -210,7 +332,7 @@ void KAccountListItem::paintCell(QPainter *p, const QColorGroup & cg, int column
         p->setPen( cg2.highlightedText() );
       else if ( !isEnabled() && listView())
         p->setPen( listView()->palette().disabled().highlightedText() );
-        
+
     } else
       p->fillRect( 0, 0, indent, height(), cg2.base() );
 
@@ -240,15 +362,15 @@ void KAccountListItem::paintCell(QPainter *p, const QColorGroup & cg, int column
         p->drawPoint(i, height()/2);
 
     } else {
-      // draw upper part of vertical line      
+      // draw upper part of vertical line
       ofs = depth()*ts + ts/2 - 1;
       for(int i = 0; i < height()/2-(ts-2)/4; i += 2)
         p->drawPoint(ofs, i);
 
-      // draw horizontal part        
+      // draw horizontal part
       for(int i = ofs + ts/4 ; i < (depth()+1)*ts; i += 2)
         p->drawPoint(i, height()/2);
-        
+
       // need to draw box with +/- in it
       ofs = depth() * ts;
       p->drawRect( ofs + ts/4, height() / 2 - (ts-2)/4, (ts-2)/2, (ts-2)/2 );
@@ -277,6 +399,7 @@ KAccountIconItem::KAccountIconItem(QIconView* parent, const MyMoneyAccount& acco
   MyMoneyFile*  file = MyMoneyFile::instance();
 
   setAccountID(account.id());
+  setAccountType(account.accountType());
 
   file->attach(account.id(), this);
 }
@@ -289,7 +412,7 @@ KAccountIconItem::~KAccountIconItem()
 void KAccountIconItem::update(const QCString& id)
 {
   QString name;
-  
+
   try {
     MyMoneyAccount acc = MyMoneyFile::instance()->account(id);
     name = acc.name();
@@ -334,7 +457,7 @@ void KAccountListItem::paintFocus(QPainter* p, const QColorGroup& cg, const QRec
 
   QRect r2(r);
   r2.setLeft(r2.left() + -indent);
-  
+
   if (isSelected())
     p->fillRect(  r2.left(),
                   r2.top(),
