@@ -31,293 +31,47 @@
 #include <klistview.h>
 #include <kconfig.h>
 #include <kmessagebox.h>
+#include <kiconloader.h>
+#include <kapplication.h>
+#include <kstandarddirs.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "kbanksview.h"
 #include "kbanklistitem.h"
+#include "../widgets/kmymoneyaccounttree.h"
+#include "../widgets/kmymoneytitlelabel.h"
 #include "../mymoney/mymoneyfile.h"
 #include "../dialogs/knewaccountdlg.h"
+#include "../dialogs/knewbankdlg.h"
 #include "../kmymoneyutils.h"
 
-static const char* const assetIconImage[] = {
-"32 32 9 1",
-". c None",
-"# c #000000",
-"a c #0000ff",
-"d c #008183",
-"c c #838183",
-"f c #c5c2c5",
-"g c #ff0000",
-"e c #ffff00",
-"b c #ffffff",
-"................................",
-"............#...................",
-"...........#a#..................",
-"..........#a#a#.................",
-".........#a#b#a#................",
-"........#a#bbb#a#...............",
-".......#a#bbbbb#a#..............",
-"......#a#bbbbbbb#a#.............",
-".....#a#bbbbbbbbb#a#............",
-"....#a#bbbbbbbbbbb#a#...........",
-"...#a#bbbbbbbbbbbbb#a#..........",
-"..#a#bbbbbbbbbbbbbbb#a#.........",
-".#a#bbbbbbbbbbbbbbbbb#a#........",
-"#a##bb####bbbbb####bb##a#.......",
-".#c#bb##d#bbbbb##d#bb#c#c.......",
-".cc#bb#dd#bbbbb#dd#bb#cccc......",
-"...#bb#dd#bbbbb#dd#bb#c.c.......",
-"...#bb####bbbbb####bb#c.........",
-"...#bbbbbb#####bbbbbb#c.........",
-"...#bbbbbb#efe#bbbbbb#c.........",
-"...#bbbbbb#fef#bbb#######.......",
-"...#bbbbbb#efe#bb#ggggggg#......",
-"...#bbbbbb#fef#b#ggggggggg#.....",
-"...#bbbbbb#######ggggggggg#####.",
-"...#bbbbbb##gggggggggggggggggg#.",
-"...#########gg###ggggggg###ggg#c",
-"....ccccccc#g##c##ggggg##c##gg#c",
-"...........###ccc#######ccc####c",
-"............c##f##ccccc##f##cccc",
-".............c###cc.....###cc...",
-"...............ccc.......ccc....",
-"................................"};
-
-static const char* const cashIconImage[] = {
-"32 32 6 1",
-". c None",
-"# c #000000",
-"a c #838100",
-"c c #838183",
-"d c #c5c2c5",
-"b c #ffff00",
-"................................",
-".......................#####....",
-".....................###aba##...",
-"....................##bababab##.",
-"....................#babababab#c",
-"...........#........#aba###aba#c",
-"..........#c#.......##abababa##c",
-"........##dbd##.....#b##bab##b#c",
-"......##.d.#.d.##...#ab#####ba#c",
-"...#####db#c#bdbd##.##abbbbba##c",
-".###aba###ddd##d.d.##b##dad##b#c",
-"##bababab##bdbd##ddbd#######ba#c",
-"#babababab#d.d.dd##d.d.##bbba##c",
-"#aba###aba#bdbdbdbd##ddbd####b#c",
-"##abababa##d.d.d.d.dd##d.d.##a#c",
-"#b##bab##b##dbdbdbdbdbd##ddbd##c",
-"#ab#####ba#d##dd.d.d.d.dd###.d.#",
-"##abbbbba##bdd##dbdbdbdd##ddd##c",
-"#b##dad##b##.d.d##dddd##dd.##dd#",
-"#ab#####ba#d##dbdd####dbd##dd##c",
-"##abbbbba###dd##.d.#dd.##dd##dd#",
-"#b##dad##b#d##dd##dbd##dd##dd##c",
-"#ab#####ba##dd##dd###dd##dd##dd#",
-"##abbbbba##d##dd##d#d##dd##dd##c",
-"#b##dad##b##dd##dd###dd##dd##ccc",
-"#ab#####ba#c##dd##d#d##dd##ccc..",
-"##abbbbba##ccc##dd###dd##ccc....",
-".c##dad##ccc..cc##d#d##ccc......",
-"..c#####cc......cc###ccc........",
-"....ccccc.........c#cc..........",
-"...................cc...........",
-"................................"};
-
-static const char* const creditCardIconImage[] = {
-"32 32 7 1",
-". c None",
-"a c #000000",
-"b c #000083",
-"e c #0000ff",
-"d c #008183",
-"c c #838183",
-"# c #ffffff",
-"................................",
-"................................",
-"................................",
-"................................",
-"................................",
-"...............................#",
-".aaaaaaaaaaaaaaaaaaaaaaaaaaaaa..",
-"a#############################a.",
-"a#bbbbbbbbbbbbbbbbbbbbbbbbbbb#ac",
-"a#bbbbbbbbbbbbbbbbbbbbbbbbbbb#ac",
-"a#bbbbbbbbbbbbbbbbbbbbbbbbbbb#ac",
-"a#bbbbbbbbbbbbbbbbbbbbbbbbbbb#ac",
-"a#bbbbbbbbbbbbbbbbbbbbbbbbbbb#ac",
-"a#dededededededededededededed#ac",
-"a#ededededededededededededede#ac",
-"a#dededededededededededededed#ac",
-"a#ededededededededededededede#ac",
-"a#ded##e##d##e##d##e##d##eded#ac",
-"a#ede##d##e##d##e##d##e##dede#ac",
-"a#dededededededededededededed#ac",
-"a#ededededededededededededede#ac",
-"a#ded##################ededed#ac",
-"a#ededededededededededededede#ac",
-"a#dededededededededededededed#ac",
-"a#############################ac",
-".aaaaaaaaaaaaaaaaaaaaaaaaaaaaac.",
-"..cccccccccccccccccccccccccccc..",
-"................................",
-"................................",
-"................................",
-"................................",
-"................................"};
-
-static const char* const checkingsIconImage[] = {
-"32 32 6 1",
-". c None",
-"# c #000000",
-"b c #0000ff",
-"a c #008183",
-"c c #838183",
-"d c #ffffff",
-"........###.....................",
-"........#ab##...................",
-"........#baba##.................",
-"........#ababab##...............",
-"........#babababa##.............",
-"........#ababababab##...........",
-".........#abababababa##.........",
-".........#babababababab##.......",
-".........#abababababababa##.....",
-"..........#abababababababab##...",
-"..........#babababababababab#...",
-"..........#ababababababababa#...",
-"...........##bababababababab#...",
-"..........##c##ababababababa#...",
-"........##d##cc##bababababab#...",
-"......##ddddd##cc##ababababab#..",
-"....##ddddddddd##cc##babababa#..",
-"..##ddd##dddddddd##cc##ababab#..",
-".#ddd##dd##dddddddd##cc##baba##.",
-".##dddd##dd##dddddddd##cc###ba#.",
-".#c##dddd##dd##dddd#ddd##ccc###.",
-"###cc##dddd##dd##ddd##ddd##cc#b#",
-"#ba##cc##dddd##dd##ddd#dddd###a#",
-"###ab##cc##dddd##dd##dddd##cc#b#",
-"ccc##aa##cc##dddd##dddd##cc##aa#",
-"...cc##ab##cc##dddddd##cc##ab##c",
-".....cc##aa##cc##dd##cc##aa##cc.",
-".......cc##ab##cc##cc##ab##cc...",
-".........cc##aa##cc##aa##cc.....",
-"...........cc##ab##ab##cc.......",
-".............cc##aa##cc.........",
-"...............cc##cc..........."};
-
-static const char* const liabilityIconImage[] = {
-"32 32 5 1",
-". c None",
-"# c #000000",
-"a c #838183",
-"b c #c5c2c5",
-"c c #ffff00",
-"................................",
-"#a..............................",
-"#a..............................",
-"#a..####........................",
-"#a..#bc#b.......................",
-"#a..#cb#b.......................",
-"#a..#bc#b.......................",
-"#a..#cb#b####...................",
-"#a..#bc#b#bc#...................",
-"#a..#cb#b#cb#b..................",
-"#a..#bc#b#bc#b..................",
-"#a..#cb#b#cb#b####..............",
-"#a..#bc#b#bc#b#bc#..............",
-"#a..#cb#b#cb#b#cb#b.............",
-"#a..#bc#b#bc#b#bc#b.............",
-"#a..#cb#b#cb#b#cb#b####.........",
-"#a..#bc#b#bc#b#bc#b#bc#b........",
-"#a..#cb#b#cb#b#cb#b#cb#b........",
-"#a..#bc#b#bc#b#bc#b#bc#b........",
-"#a..#cb#b#cb#b#cb#b#cb#b####....",
-"#a..#bc#b#bc#b#bc#b#bc#b#bc#....",
-"#a..#cb#b#cb#b#cb#b#cb#b#cb#b...",
-"#a..#bc#b#bc#b#bc#b#bc#b#bc#b...",
-"#a..#cb#b#cb#b#cb#b#cb#b#cb#b...",
-"#a..#bc#b#bc#b#bc#b#bc#b#bc#b...",
-"#a..####b####b####b####b####b...",
-"#a..............................",
-"#a..............................",
-"###############################.",
-"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.",
-"................................",
-"................................"};
-
-static const char* const loanIconImage[] = {
-"32 32 8 1",
-". c None",
-"# c #000000",
-"b c #0000ff",
-"a c #008183",
-"c c #838183",
-"e c #c5c2c5",
-"f c #ffff00",
-"d c #ffffff",
-".....##########################.",
-".....#abababababababababababab#c",
-".....#bababdddddddddddddddbaba#c",
-".....#ababaaaaaaaaaaaaaaaaabab#c",
-".....#babababababababababababa#c",
-".....#dddddddddddddddddddddddd#c",
-".....#dddddddddddddddccdccdccd#c",
-".....#dddddddddddddddccdccdccd#c",
-".....#dddddddddddddddddddddddd#c",
-".....#dddcc###dccdccdccdccdccd#c",
-".....#ddd##efe##cdccdccdccdccd#c",
-".....#d##dededed##dddddddddddd#c",
-".....##efef###eefe##dccdccdccd#c",
-"...##dede##dee##eded##cdccdccd#c",
-".##efef##efefefe##eefe##dddddd#c",
-"#eede##eededededee##eded##dccd#c",
-".##efef##efefefefefe##eefe##cd#c",
-"#cc##dede##eededededee###ded#d#c",
-".##cc##efee##efefefee##eee##cd#c",
-"#cc##cc##dede##eeee##eed##cc#d#c",
-".##cc##cc##efee####efe##cc##cd#c",
-"#cc##cc##cc##ded#eed##cc##cc#d#c",
-".##cc##cc##cc##efe##cc##cc##c##c",
-".cc##cc##cc##cc###cc##cc##cc#ccc",
-"...cc##cc##cc##c#c##cc##cc##c...",
-".....cc##cc##cc###cc##cc##ccc...",
-".......cc##cc##c#c##cc##ccc.....",
-".........cc##cc###cc##ccc.......",
-"...........cc##c#c##ccc.........",
-".............cc###ccc...........",
-"...............c#cc.............",
-"................cc.............."};
-
-
-KAccountsView::KAccountsView(QWidget *parent, const char *name)
+KAccountsView::KAccountsView(QWidget *parent, const char *name, bool bInstitutionView)
  : KBankViewDecl(parent,name),
-   m_suspendUpdate(false)
+   m_suspendUpdate(false),
+   m_bViewNormalAccountsView(bInstitutionView)
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", false);
+  titleLabel->setText(bInstitutionView ? i18n("Institutions") : i18n("Accounts"));
 
   accountListView->setRootIsDecorated(true);
   accountListView->setAllColumnsShowFocus(true);
 
   accountListView->addColumn(i18n("Account"));
-  accountListView->addColumn(i18n("Entries"));
+  accountListView->addColumn(i18n("Transactions"));
   accountListView->addColumn(i18n("Balance"));
-  // accountListView->addColumn(i18n("Type"));
+  accountListView->addColumn(i18n("Value"));
 
   accountListView->setMultiSelection(false);
 
   accountListView->setColumnWidthMode(0, QListView::Maximum);
   accountListView->setColumnWidthMode(1, QListView::Maximum);
   accountListView->setColumnWidthMode(2, QListView::Maximum);
-  // accountListView->setColumnWidthMode(3, QListView::Maximum);
+  accountListView->setColumnWidthMode(3, QListView::Maximum);
 
-  accountListView->setColumnAlignment(2, Qt::AlignRight);
   accountListView->setColumnAlignment(1, Qt::AlignRight);
+  accountListView->setColumnAlignment(2, Qt::AlignRight);
+  accountListView->setColumnAlignment(3, Qt::AlignRight);
 
   accountListView->setResizeMode(QListView::AllColumns);
 
@@ -325,9 +79,14 @@ KAccountsView::KAccountsView(QWidget *parent, const char *name)
 
   accountListView->header()->setFont(KMyMoneyUtils::headerFont());
 
-  // select the type the user viewed last
-  config->setGroup("Last Use Settings");
-  accountTabWidget->setCurrentPage(config->readNumEntry("KAccountsView_LastType", 0));
+  if(!m_bViewNormalAccountsView) {
+    // select the type the user viewed last (list or icon) but only
+    // for the accounts view
+    KConfig *config = KGlobal::config();
+    config->setGroup("Last Use Settings");
+    accountTabWidget->setCurrentPage(config->readNumEntry("KAccountsView_LastType", 0));
+  } else
+    accountTabWidget->setCurrentPage(0);
 
   connect(accountListView, SIGNAL(selectionChanged(QListViewItem*)),
     this, SLOT(slotSelectionChanged(QListViewItem*)));
@@ -357,12 +116,20 @@ KAccountsView::KAccountsView(QWidget *parent, const char *name)
   accountIconView->setSorting(-1);
   accountListView->setSorting(0);
 
+  // don't enable the icon tab in institutions mode
+  if(m_bViewNormalAccountsView) {
+    accountTabWidget->removePage(accountTabWidget->page(1));
+  }
+
   // never show a horizontal scroll bar
   //accountListView->setHScrollBarMode(QScrollView::AlwaysOff);
 
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAccountHierarchy, this);
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAccount, this);
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassInstitution, this);
+  MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassSecurity, this);
+  MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassCurrency, this);
+  MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassPrice, this);
 
   refresh(QCString());
 }
@@ -372,6 +139,9 @@ KAccountsView::~KAccountsView()
   MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassAccountHierarchy, this);
   MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassAccount, this);
   MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassInstitution, this);
+  MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassSecurity, this);
+  MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassCurrency, this);
+  MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassPrice, this);
 }
 
 void KAccountsView::slotListDoubleClicked(QListViewItem* item, const QPoint& /* pos */, int /* c */)
@@ -379,34 +149,45 @@ void KAccountsView::slotListDoubleClicked(QListViewItem* item, const QPoint& /*
   KAccountListItem* accountItem = static_cast<KAccountListItem*> (item);
   if(accountItem)
   {
-
-    // Only emit the signal if its an account
-    MyMoneyFile *file = MyMoneyFile::instance();
-
-    try
+    if ( item->childCount() )
+      item->setOpen( ! item->isOpen() );
+    else
     {
-      MyMoneyAccount account = file->account(accountItem->accountID());
 
-      // this signal will only be emitted for asset and liability accounts
-      if(!file->isStandardAccount(account.id())) {
-        switch(file->accountGroup(account.accountType())) {
-          case MyMoneyAccount::Asset:
-          case MyMoneyAccount::Liability:
-            m_bSelectedAccount=true;
-            m_bSelectedInstitution=false;
-            m_selectedAccount = accountItem->accountID();
-            emit accountDoubleClick();
-            break;
+      MyMoneyFile *file = MyMoneyFile::instance();
+      try
+      {
+        MyMoneyAccount account = file->account(accountItem->accountID());
 
-          default:
-            break;
+        if(!file->isStandardAccount(account.id())) {
+          switch(file->accountGroup(account.accountType())) {
+            // the account signal will only be emitted for asset and liability accounts
+            case MyMoneyAccount::Asset:
+            case MyMoneyAccount::Liability:
+              m_bSelectedAccount=true;
+              m_bSelectedInstitution=false;
+              m_selectedAccount = accountItem->accountID();
+              emit accountDoubleClick();
+              break;
+            // the category signal will only be emitted for income & expense accounts
+            case MyMoneyAccount::Income:
+            case MyMoneyAccount::Expense:
+              m_bSelectedAccount=true;
+              m_bSelectedInstitution=false;
+              m_selectedAccount = accountItem->accountID();
+              emit categoryDoubleClick();
+              break;
+
+            default:
+              break;
+          }
         }
       }
-    }
-    catch (MyMoneyException *e)
-    {
-      // Probably clicked on the institution in normal view
-      delete e;
+      catch (MyMoneyException *e)
+      {
+        // Probably clicked on the institution in normal view
+        delete e;
+      }
     }
   }
 }
@@ -439,7 +220,7 @@ void KAccountsView::slotIconRightMouse(QIconViewItem* item, const QPoint&)
       m_bSelectedAccount=true;
       m_bSelectedInstitution=false;
       m_selectedAccount = accountItem->accountID();
-      
+
       emit accountRightMouseClick();
 
     } catch (MyMoneyException *e) {
@@ -463,7 +244,7 @@ void KAccountsView::slotListRightMouse(QListViewItem* item, const QPoint& , int
         MyMoneyFile *file = MyMoneyFile::instance();
         MyMoneyAccount account = file->account(accountItem->accountID());
 
-        
+
         m_bSelectedAccount=true;
         m_bSelectedInstitution=false;
         m_selectedAccount = accountItem->accountID();
@@ -475,7 +256,7 @@ void KAccountsView::slotListRightMouse(QListViewItem* item, const QPoint& , int
         m_bSelectedInstitution=true;
         m_selectedInstitution = accountItem->accountID();
         delete e;
-        
+
         emit bankRightMouseClick();
       }
     }
@@ -502,10 +283,22 @@ void KAccountsView::update(const QCString& id)
   // updates in this phase. The switch is controlled with suspendUpdate().
   if(m_suspendUpdate == false) {
     if(id == MyMoneyFile::NotifyClassAccountHierarchy
-    || (id == MyMoneyFile::NotifyClassInstitution && m_bViewNormalAccountsView == true)) {
+    || id == MyMoneyFile::NotifyClassCurrency
+    || id == MyMoneyFile::NotifyClassSecurity
+    || id == MyMoneyFile::NotifyClassPrice) {
       refresh(id);
+
+    } else if(m_bViewNormalAccountsView == true) {
+      if(id == MyMoneyFile::NotifyClassInstitution
+      || id == MyMoneyFile::NotifyClassAccount) {
+        refresh(id);
+      }
     }
-    if(id == MyMoneyFile::NotifyClassAccount)
+
+    if(id == MyMoneyFile::NotifyClassAccount
+    || id == MyMoneyFile::NotifyClassCurrency
+    || id == MyMoneyFile::NotifyClassPrice
+    || id == MyMoneyFile::NotifyClassSecurity)
       refreshNetWorth();
   }
 }
@@ -528,7 +321,7 @@ void KAccountsView::suspendUpdate(const bool suspend)
       refresh(MyMoneyFile::NotifyClassInstitution);
     refreshNetWorth();
   }
-  
+
   m_suspendUpdate = suspend;
 }
 
@@ -547,12 +340,21 @@ void KAccountsView::refreshNetWorth(void)
   MyMoneyAccount liabilityAccount = file->liability();
   MyMoneyAccount assetAccount = file->asset();
 
-  netWorth = file->totalBalance(assetAccount.id()) +
-             file->totalBalance(liabilityAccount.id());
+  netWorth = file->totalValue(assetAccount.id()) +
+             file->totalValue(liabilityAccount.id());
 
   QString s(i18n("Net Worth: "));
-  s += netWorth.formatMoney();
-
+  if(!(file->totalValueValid(assetAccount.id()) & file->totalValueValid(liabilityAccount.id())))
+    s += "~ ";
+  s.replace(QString(" "), QString("&nbsp;"));
+  if(netWorth.isNegative()) {
+    s += "<b><font color=\"red\">";
+  }
+  QString v(netWorth.formatMoney(file->baseCurrency().tradingSymbol()));
+  s += v.replace(QString(" "), QString("&nbsp;"));
+  if(netWorth.isNegative()) {
+    s += "</font></b>";
+  }
 
   totalProfitsLabel->setFont(KMyMoneyUtils::cellFont());
   totalProfitsLabel->setText(s);
@@ -567,31 +369,37 @@ const QPixmap KAccountsView::accountImage(const MyMoneyAccount::accountTypeE typ
   switch(type) {
     default:
       if(MyMoneyFile::instance()->accountGroup(type) == MyMoneyAccount::Asset)
-        rc = QPixmap(assetIconImage);
+        rc = DesktopIcon("account-types_asset");
       else
-        rc = QPixmap(liabilityIconImage);
-      break;
-      
-    case MyMoneyAccount::Checkings:
-    case MyMoneyAccount::Savings:
-      rc = QPixmap(checkingsIconImage);
+        rc = DesktopIcon("account-types_liability");
       break;
 
-    case MyMoneyAccount::Loan:
-      rc = QPixmap(loanIconImage);
+    case MyMoneyAccount::Investment:
+      rc = DesktopIcon("account-types_investments");
       break;
-      
+
+    case MyMoneyAccount::Checkings:
+      rc = DesktopIcon("account-types_checking");
+      break;
+    case MyMoneyAccount::Savings:
+      rc = DesktopIcon("account-types_savings");
+      break;
+
+    case MyMoneyAccount::AssetLoan:
+    case MyMoneyAccount::Loan:
+      rc = DesktopIcon("account-types_loan");
+      break;
+
     case MyMoneyAccount::CreditCard:
-      rc = QPixmap(creditCardIconImage);
+      rc = DesktopIcon("account-types_credit-card");
       break;
 
     case MyMoneyAccount::Asset:
-    case MyMoneyAccount::AssetLoan:
-      rc = QPixmap(assetIconImage);
+      rc = DesktopIcon("account-types_asset");
       break;
 
     case MyMoneyAccount::Cash:
-      rc = QPixmap(cashIconImage);
+      rc = DesktopIcon("account-types_cash");
       break;
   }
 
@@ -639,10 +447,12 @@ void KAccountsView::refresh(const QCString& selectAccount)
 
   KConfig *config = KGlobal::config();
   config->setGroup("List Options");
-  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", false);
   m_hideCategory = config->readBoolEntry("HideUnusedCategory", false);
   bool accountUsed;
 
+  // Disable the note about hidden categories
+  m_hiddenCategories->hide();
+
   clear();
 
   m_selectedAccount = selectAccount;
@@ -659,35 +469,55 @@ void KAccountsView::refresh(const QCString& selectAccount)
       // scan for all asset/liability accounts w/o an institution
       KAccountListItem *topLevelInstitution = new KAccountListItem(accountListView,
                        i18n("Accounts with no institution assigned"));
+      topLevelInstitution->setPixmap(0, QPixmap(KGlobal::dirs()->findResource("appdata",QString( "icons/hicolor/22x22/actions/%1.png").arg("bank"))));
       QValueList<MyMoneyAccount> acclist = file->accountList();
       QValueList<MyMoneyAccount>::ConstIterator accountIterator;
+      MyMoneyMoney balance;
+      bool balanceValid = true;
+
       for(accountIterator = acclist.begin();
           accountIterator != acclist.end();
           ++accountIterator) {
 
+        // Don't show stock accounts, they will be handled with the
+        // investment accounts further down
+        if((*accountIterator).accountType() == MyMoneyAccount::Stock) {
+          continue;
+        }
+
         switch((*accountIterator).accountGroup()) {
           case MyMoneyAccount::Asset:
-
           case MyMoneyAccount::Liability:
             if((*accountIterator).institutionId().isEmpty()) {
               KAccountListItem *accountItem;
               accountItem = new KAccountListItem(topLevelInstitution, (*accountIterator));
               accountItem->setText(1, QString("%1").arg(m_transactionCountMap[(*accountIterator).id()]));
-              new KAccountIconItem(accountIconView, (*accountIterator),
-                accountImage((*accountIterator).accountType()));
+              // new KAccountIconItem(accountIconView, (*accountIterator),
+              //   accountImage((*accountIterator).accountType()));
+              if((*accountIterator).accountType() == MyMoneyAccount::Investment) {
+                balance += file->totalValue((*accountIterator).id());
+                if(!file->totalValueValid((*accountIterator).id()))
+                  balanceValid = false;
+              } else {
+                balance += file->accountValue((*accountIterator).id());
+                if(!file->accountValueValid((*accountIterator).id()))
+                  balanceValid = false;
+              }
             }
             break;
           default:
             break;
         }
       }
-      
+
+      topLevelInstitution->setValue(balance, balanceValid);
+
       // in case there is no account w/o reference to an institution
       // we can safely remove this entry to avoid user's confusion
       if(topLevelInstitution->childCount() == 0) {
         delete topLevelInstitution;
       }
-      
+
       QValueList<MyMoneyInstitution> list = file->institutionList();
       QValueList<MyMoneyInstitution>::ConstIterator institutionIterator;
       for (institutionIterator = list.begin();
@@ -695,27 +525,35 @@ void KAccountsView::refresh(const QCString& selectAccount)
             institutionIterator != list.end();
             ++institutionIterator) {
 
+        MyMoneyMoney balance;
+        bool balanceValid = true;
         KAccountListItem *topLevelInstitution = new KAccountListItem(accountListView,
                       *institutionIterator);
 
         QCStringList accountList = (*institutionIterator).accountList();
+        balance = MyMoneyMoney(0, 1);
         for ( QCStringList::ConstIterator it = accountList.begin();
               it != accountList.end();
               ++it ) {
+          // don't show stock accounts
+          if(m_accountMap[*it].accountType() == MyMoneyAccount::Stock)
+            continue;
+
           KAccountListItem *accountItem = new KAccountListItem(topLevelInstitution,
               m_accountMap[*it]);
           accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
-          new KAccountIconItem(accountIconView,
-              m_accountMap[*it],
-              accountImage(m_accountMap[*it].accountType()));
-
-          QCStringList subAccounts = m_accountMap[*it].accountList();
-          if (subAccounts.count() >= 1) {
-
-            showSubAccounts(subAccounts, accountItem, "");
+          if(m_accountMap[*it].accountType() == MyMoneyAccount::Investment) {
+            balance += file->totalValue(*it);
+            if(!file->totalValueValid(*it))
+              balanceValid = false;
+          } else {
+            balance += file->accountValue(*it);
+            if(!file->accountValueValid(*it))
+              balanceValid = false;
           }
         }
+        topLevelInstitution->setValue(balance, balanceValid);
       }
 
     } catch (MyMoneyException *e) {
@@ -725,7 +563,7 @@ void KAccountsView::refresh(const QCString& selectAccount)
 
   } else {       // Show new 'advanced' view
     accountListView->header()->setLabel(0, i18n("Account"));
-    // Do all 4 account roots
+    // Do all 5 account roots
     try {
       // Asset
       MyMoneyAccount assetAccount = file->asset();
@@ -735,6 +573,12 @@ void KAccountsView::refresh(const QCString& selectAccount)
       for ( QCStringList::ConstIterator it = assetAccount.accountList().begin();
             it != assetAccount.accountList().end();
             ++it ) {
+
+        // don't show stock accounts in non-expert mode
+        if((m_accountMap[*it].accountType() == MyMoneyAccount::Stock)
+        && !KMyMoneyUtils::isExpertMode())
+          continue;
+
         KAccountListItem *accountItem = new KAccountListItem(assetTopLevelAccount,
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
@@ -745,7 +589,7 @@ void KAccountsView::refresh(const QCString& selectAccount)
 
         QCStringList subAccounts = m_accountMap[*it].accountList();
         if (subAccounts.count() >= 1) {
-          showSubAccounts(subAccounts, accountItem, i18n("Asset"));
+          showSubAccounts(subAccounts, accountItem, true);
         }
       }
 
@@ -757,6 +601,12 @@ void KAccountsView::refresh(const QCString& selectAccount)
       for ( QCStringList::ConstIterator it = liabilityAccount.accountList().begin();
             it != liabilityAccount.accountList().end();
             ++it ) {
+
+        // don't show stock accounts in non-expert mode
+        if((m_accountMap[*it].accountType() == MyMoneyAccount::Stock)
+        && !KMyMoneyUtils::isExpertMode())
+          continue;
+
         KAccountListItem *accountItem = new KAccountListItem(liabilityTopLevelAccount,
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
@@ -768,7 +618,7 @@ void KAccountsView::refresh(const QCString& selectAccount)
 
         QCStringList subAccounts = m_accountMap[*it].accountList();
         if (subAccounts.count() >= 1) {
-          showSubAccounts(subAccounts, accountItem, i18n("Liability"));
+          showSubAccounts(subAccounts, accountItem, true);
         }
       }
 
@@ -789,13 +639,14 @@ void KAccountsView::refresh(const QCString& selectAccount)
 
         QCStringList subAccounts = m_accountMap[*it].accountList();
         if (subAccounts.count() >= 1) {
-          accountUsed |= showSubAccounts(subAccounts, accountItem, i18n("Income"));
+          accountUsed |= showSubAccounts(subAccounts, accountItem, false);
         }
         if(accountUsed == false && m_hideCategory == true) {
           // in case hide category is on and the account or any of it's
           // subaccounts has no split, we can safely remove it and all
           // it's sub-ordinate accounts from the list
           delete accountItem;
+          m_hiddenCategories->show();
         }
       }
 
@@ -816,13 +667,37 @@ void KAccountsView::refresh(const QCString& selectAccount)
         QCStringList subAccounts = m_accountMap[*it].accountList();
 
         if (subAccounts.count() >= 1) {
-          accountUsed |= showSubAccounts(subAccounts, accountItem, i18n("Expense"));
+          accountUsed |= showSubAccounts(subAccounts, accountItem, false);
         }
         if(accountUsed == false && m_hideCategory == true) {
           // in case hide category is on and the account or any of it's
           // subaccounts has no split, we can safely remove it and all
           // it's sub-ordinate accounts from the list
           delete accountItem;
+          m_hiddenCategories->show();
+        }
+      }
+
+      // Equity
+      if(KMyMoneyUtils::isExpertMode()) {
+        MyMoneyAccount equityAccount = file->equity();
+        KAccountListItem *equityTopLevelAccount = new KAccountListItem(accountListView,
+              equityAccount);
+
+        for ( QCStringList::ConstIterator it = equityAccount.accountList().begin();
+              it != equityAccount.accountList().end();
+              ++it ) {
+          KAccountListItem *accountItem = new KAccountListItem(equityTopLevelAccount,
+              m_accountMap[*it]);
+          accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
+
+          accountUsed = m_transactionCountMap[*it] > 0;
+
+          QCStringList subAccounts = m_accountMap[*it].accountList();
+
+          if (subAccounts.count() >= 1) {
+            accountUsed |= showSubAccounts(subAccounts, accountItem, false);
+          }
         }
       }
 
@@ -844,20 +719,27 @@ void KAccountsView::refresh(const QCString& selectAccount)
 */
 }
 
-const bool KAccountsView::showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const QString& group)
+const bool KAccountsView::showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const bool used)
 {
-  bool  accountUsed = false;
+  bool  accountUsed = used;
 
   for ( QCStringList::ConstIterator it = accounts.begin(); it != accounts.end(); ++it ) {
+
+    // don't show stock accounts in non-expert mode
+    if((m_accountMap[*it].accountType() == MyMoneyAccount::Stock)
+    && !KMyMoneyUtils::isExpertMode())
+      continue;
+
     KAccountListItem *accountItem  = new KAccountListItem(parentItem,
           m_accountMap[*it]);
+
     accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
-    accountUsed = m_transactionCountMap[*it] > 0;
+    accountUsed |= (m_transactionCountMap[*it] > 0);
 
     QCStringList subAccounts = m_accountMap[*it].accountList();
     if (subAccounts.count() >= 1) {
-      accountUsed |= showSubAccounts(subAccounts, accountItem, group) || m_transactionCountMap[*it] > 0;
+      accountUsed |= showSubAccounts(subAccounts, accountItem, used) || m_transactionCountMap[*it] > 0;
     }
 
     if(accountUsed == false && m_hideCategory == true) {
@@ -934,13 +816,13 @@ void KAccountsView::slotEditClicked(void)
 
 {
   QCString  accountId;
-  
+
   if(accountTabWidget->currentPageIndex() == 0) {   // ListView?
     KAccountListItem *item = (KAccountListItem *)accountListView->currentItem();
     if(!item)
       return;
     accountId = item->accountID();
-    
+
   } else {
     KAccountIconItem *item = (KAccountIconItem *)accountIconView->currentItem();
     if(!item)
@@ -1012,3 +894,6 @@ void KAccountsView::slotDeleteClicked(void)
     delete e;
   }
 }
+
+
+#include "kbanksview.moc"
