@@ -31,6 +31,7 @@
 
 #include <klocale.h>
 #include <klistview.h>
+#include <kpopupmenu.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -41,7 +42,7 @@
 #include "../views/kbanklistitem.h"
 
 /**
-  *@author Michael Edwardes, Thomas Baumgart
+  * @author Michael Edwardes, Thomas Baumgart
   */
 
 /**
@@ -49,8 +50,9 @@
   * It handles the resize event, the totals widgets
   * and the KAccountListView itself
   */
-class KAccountsView : public KBankViewDecl, MyMoneyObserver  {
-   Q_OBJECT
+class KAccountsView : public KBankViewDecl, MyMoneyObserver
+{
+  Q_OBJECT
 private:
   bool m_bSelectedAccount;
   QCString m_selectedAccount;
@@ -60,17 +62,16 @@ private:
   bool m_hideCategory;
   QCString m_selectedInstitution;
   bool m_bSelectedInstitution;
-
   void refresh(const QCString& selectAccount);
   void refreshNetWorth(void);
-  const bool showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const QString& group);
-
+  const bool showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const bool used);
+  KPopupMenu* m_bankMenu;
   QMap<QCString, MyMoneyAccount> m_accountMap;
   QMap<QCString, unsigned long> m_transactionCountMap;
 
-public: 
-  KAccountsView(QWidget *parent=0, const char *name=0);
-  ~KAccountsView();
+public:
+  KAccountsView(QWidget *parent=0, const char *name=0, bool bInstitutionView=false);
+  virtual ~KAccountsView();
   QCString currentAccount(bool&);
   QCString currentInstitution(bool&);
   void clear(void);
@@ -100,13 +101,13 @@ public:
     * the view will be refreshed once automatically.
     */
   void suspendUpdate(const bool suspend);
-  
+
 public slots:
   void slotEditClicked(void);
   void slotDeleteClicked(void);
   void slotRefreshView(void);
   void slotReloadView(void) { slotRefreshView(); };
-      
+
 protected:
   void resizeEvent(QResizeEvent*);
 
@@ -114,7 +115,7 @@ protected:
   void fillAccountMap(void);
 
 protected slots:
-  
+
   /**
     * This slot receives the signal from the listview control that an item was double-clicked,
     * if this item was a bank account, try to show the list of transactions for that account.
@@ -161,6 +162,8 @@ signals:
     */
   void accountDoubleClick();
 
+  void categoryDoubleClick();
+
   void signalViewActivated();
 
   void accountRightMouseClick();
