@@ -34,12 +34,7 @@
 #include <klistview.h>
 #include <klocale.h>
 #include <kglobal.h>
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-
 #include <kmessagebox.h>
 #include <kconfig.h>
 
@@ -49,6 +44,7 @@
 #include "../mymoney/mymoneyfile.h"
 #include "../views/kmymoneyfile.h"
 #include "../dialogs/knewaccountdlg.h"
+#include "../widgets/kmymoneyaccounttree.h"
 #include "kcategoriesview.h"
 #include "kbanklistitem.h"
 #include "../kmymoneyutils.h"
@@ -58,20 +54,25 @@ KCategoriesView::KCategoriesView(QWidget *parent, const char *name )
 {
   categoryListView->setRootIsDecorated(true);
   categoryListView->addColumn(i18n("Category"));
-  categoryListView->addColumn(i18n("Type"));
+  categoryListView->addColumn(i18n("Transactions"));
 
   // FIXME: We could make this configurable
   categoryListView->addColumn(i18n("Balance"));
+  // categoryListView->addColumn(i18n("Value"));
 
   categoryListView->setMultiSelection(false);
   categoryListView->setColumnWidthMode(0, QListView::Maximum);
   categoryListView->setColumnWidthMode(1, QListView::Maximum);
   categoryListView->setColumnWidthMode(2, QListView::Maximum);
+  // categoryListView->setColumnWidthMode(3, QListView::Maximum);
   categoryListView->header()->setResizeEnabled(true);
 
   categoryListView->setAllColumnsShowFocus(true);
 
+  categoryListView->setColumnAlignment(1, Qt::AlignRight);
   categoryListView->setColumnAlignment(2, Qt::AlignRight);
+  // categoryListView->setColumnAlignment(3, Qt::AlignRight);
+
   categoryListView->setResizeMode(QListView::AllColumns);
 
   // never show a horizontal scroll bar
@@ -84,6 +85,9 @@ KCategoriesView::KCategoriesView(QWidget *parent, const char *name )
   connect(categoryListView, SIGNAL(rightButtonPressed(QListViewItem* , const QPoint&, int)),
     this, SLOT(slotListRightMouse(QListViewItem*, const QPoint&, int)));
 
+  connect(categoryListView, SIGNAL(doubleClicked(QListViewItem* , const QPoint&, int)),
+    this, SLOT(slotListDoubleClicked(QListViewItem*, const QPoint&, int)));
+
   m_suspendUpdate = false;
 
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAccount, this);
@@ -99,12 +103,17 @@ KCategoriesView::~KCategoriesView()
 
 void KCategoriesView::slotRefreshView(void)
 {
+  MyMoneyTracer tracer("KCategoriesView", "slotRefreshView");
+
   KConfig *config = KGlobal::config();
   config->setGroup("List Options");
   m_hideCategory = config->readBoolEntry("HideUnusedCategory", false);
   categoryListView->header()->setFont(KMyMoneyUtils::headerFont());
   bool accountUsed;
 
+  // Disable the note about hidden categories
+  m_hiddenCategories->hide();
+
   categoryListView->clear();
   m_accountMap.clear();
   m_transactionCountMap = MyMoneyFile::instance()->transactionCountMap();
@@ -117,8 +126,12 @@ void KCategoriesView::slotRefreshView(void)
     MyMoneyAccount incomeAccount = file->income();
 
     // Income
-    KAccountListItem *incomeTopLevelAccount = new KAccountListItem(categoryListView,
+    KCategoryListItem *incomeTopLevelAccount = new KCategoryListItem(categoryListView,
               incomeAccount);
+    // Expense
+    KCategoryListItem *expenseTopLevelAccount = new KCategoryListItem(categoryListView,
+              expenseAccount);
+
 
     QValueList<MyMoneyAccount> list = file->accountList();
     QValueList<MyMoneyAccount>::ConstIterator it_a;
@@ -130,7 +143,8 @@ void KCategoriesView::slotRefreshView(void)
           it != incomeAccount.accountList().end();
           ++it )
     {
-      KAccountListItem *accountItem = new KAccountListItem(incomeTopLevelAccount,
+      tracer.printf("Adding category '%s' to income tree", m_accountMap[*it].name().ascii());
+      KCategoryListItem *accountItem = new KCategoryListItem(incomeTopLevelAccount,
             m_accountMap[*it]);
 
       accountUsed = m_transactionCountMap[*it] > 0;
@@ -145,18 +159,16 @@ void KCategoriesView::slotRefreshView(void)
         // subaccounts has no split, we can safely remove it and all
         // it's sub-ordinate accounts from the list
         delete accountItem;
+        m_hiddenCategories->show();
       }
     }
 
-    // Expense
-    KAccountListItem *expenseTopLevelAccount = new KAccountListItem(categoryListView,
-              expenseAccount);
-
     for ( QCStringList::ConstIterator it = expenseAccount.accountList().begin();
           it != expenseAccount.accountList().end();
           ++it )
     {
-      KAccountListItem *accountItem = new KAccountListItem(expenseTopLevelAccount,
+      tracer.printf("Adding category '%s' to expense tree", m_accountMap[*it].name().ascii());
+      KCategoryListItem *accountItem = new KCategoryListItem(expenseTopLevelAccount,
             m_accountMap[*it]);
 
       accountUsed = m_transactionCountMap[*it] > 0;
@@ -171,11 +183,12 @@ void KCategoriesView::slotRefreshView(void)
         // subaccounts has no split, we can safely remove it and all
         // it's sub-ordinate accounts from the list
         delete accountItem;
+        m_hiddenCategories->show();
       }
     }
 
-    categoryListView->setOpen(incomeTopLevelAccount, true);
-    categoryListView->setOpen(expenseTopLevelAccount, true);
+    incomeTopLevelAccount->setOpen(true);
+    expenseTopLevelAccount->setOpen(true);
   }
   catch (MyMoneyException *e)
   {
@@ -222,7 +235,6 @@ void KCategoriesView::resizeEvent(QResizeEvent* e)
   // categoryListView->setColumnWidth(2, 70);
 
   // call base class resizeEvent()
-
   kCategoriesViewDecl::resizeEvent(e);
 }
 
@@ -243,7 +255,7 @@ void KCategoriesView::slotDeleteClicked(MyMoneyAccount& account)
       KMessageBox::error(this, message);
       delete e;
     }
-  }  
+  }
 }
 
 void KCategoriesView::slotDeleteClicked(void)
@@ -278,7 +290,7 @@ void KCategoriesView::slotEditClicked(MyMoneyAccount& account)
       MyMoneyAccount parent = dlg.parentAccount();
 
       MyMoneyFile* file = MyMoneyFile::instance();
-      
+
       // we need to reparent first, as modify will check for same type
       if(account.parentAccountId() != parent.id()) {
         file->reparentAccount(account, parent);
@@ -297,7 +309,7 @@ void KCategoriesView::slotEditClicked(MyMoneyAccount& account)
 
 void KCategoriesView::slotEditClicked(void)
 {
-  KAccountListItem *item = (KAccountListItem *)categoryListView->selectedItem();
+  KAccountListItem *item = dynamic_cast<KAccountListItem*>(categoryListView->selectedItem());
   if (!item)
     return;
 
@@ -339,6 +351,9 @@ void KCategoriesView::writeConfig(void)
 
 void KCategoriesView::update(const QCString& id)
 {
+  MyMoneyTracer tracer("KCategoriesView", "update");
+  tracer.printf("id = '%s'", id.data());
+
   // to avoid constant update when a lot of accounts are added
   // (e.g. during creation of a new MyMoneyFile object when the
   // default accounts are loaded) a switch is supported to suppress
@@ -359,7 +374,7 @@ void KCategoriesView::suspendUpdate(const bool suspend)
     item->suspendUpdate(suspend);
     item = static_cast<KAccountListItem *>(item->itemBelow());
   }
-  
+
   // force a refresh, if update was off
   if(m_suspendUpdate == true
   && suspend == false)
@@ -388,6 +403,17 @@ void KCategoriesView::slotListRightMouse(QListViewItem* item, const QPoint& , in
   }
 }
 
+void KCategoriesView::slotListDoubleClicked(QListViewItem* item, const QPoint& , int)
+{
+  if (item)
+  {
+    if ( item->childCount() )
+      item->setOpen( ! item->isOpen() );
+    else
+      slotEditClicked();
+  }
+}
+
 void KCategoriesView::refreshProfits(void)
 {
   MyMoneyMoney profit;
@@ -400,9 +426,11 @@ void KCategoriesView::refreshProfits(void)
              file->totalBalance(expenseAccount.id()));
 
   totalProfitsLabel->setFont(KMyMoneyUtils::cellFont());
-  if(profit < 0)
-    totalProfitsLabel->setText(i18n("Loss: %1").arg((-profit).formatMoney()));
+  if(profit.isNegative())
+    totalProfitsLabel->setText(i18n("Loss: %1").arg((-profit).formatMoney(file->baseCurrency().tradingSymbol())));
   else
-    totalProfitsLabel->setText(i18n("Profit: %1").arg(profit.formatMoney()));
+    totalProfitsLabel->setText(i18n("Profit: %1").arg(profit.formatMoney(file->baseCurrency().tradingSymbol())));
 
 }
+
+#include "kcategoriesview.moc"
