@@ -33,7 +33,7 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <kdeversion.h>
+#include "kdecompat.h"
 #include <klocale.h>
 #include <kcombobox.h>
 #include <kpushbutton.h>
@@ -50,14 +50,16 @@
 #include "kledgerviewcash.h"
 #include "kledgerviewloan.h"
 #include "kledgerviewliability.h"
+#include "kledgerviewinvestments.h"
 
-#include "../mymoney/mymoneyaccount.h"
-#include "../mymoney/mymoneyfile.h"
-#include "../widgets/kmymoneycombo.h"
-#include "../kmymoneyutils.h"
+#include <kmymoney/mymoneyexception.h>
+#include <kmymoney/mymoneyaccount.h>
+#include <kmymoney/mymoneyfile.h>
+#include "../widgets/kmymoneyaccountcombo.h"
+#include "../widgets/kmymoneytitlelabel.h"
 
 KGlobalLedgerView::KGlobalLedgerView(QWidget *parent, const char *name )
-  : QWidget(parent,name)
+  : KMyMoneyViewBase(parent, name, i18n("Ledgers"))
 {
   m_currentView = 0;
   KLedgerView* view;
@@ -69,118 +71,126 @@ KGlobalLedgerView::KGlobalLedgerView(QWidget *parent, const char *name )
     setName( "Account register" );
 
   setCaption( i18n( "Account register" ) );
-  m_formLayout = new QVBoxLayout( this, 0, 6, "Form1Layout");
 
   QHBoxLayout* Layout2 = new QHBoxLayout( 0, 0, 6, "Layout2");
 
-  m_accountComboBox = new kMyMoneyCombo(false, this, "accountComboBox" );
-  m_accountComboBox->setMinimumSize( QSize( 240, 0 ) );
+  m_accountComboBox = new kMyMoneyAccountCombo(this, "AccountCombo");
+  // m_accountComboBox->setMinimumSize( QSize( 240, 0 ) );
   Layout2->addWidget( m_accountComboBox );
   QSpacerItem* spacer = new QSpacerItem( 20, 20,
                                          QSizePolicy::Expanding,
                                          QSizePolicy::Minimum );
   Layout2->addItem( spacer );
-  m_formLayout->addLayout( Layout2 );
+  m_viewLayout->addLayout( Layout2 );
 
-  m_accountStack = new QWidgetStack(this, "Stack");
+  m_accountStack = new QWidgetStack(this, "AccountStack");
 
   // Checkings account
-  view = m_specificView[MyMoneyAccount::Checkings] = new KLedgerViewCheckings(this);
+  view = m_specificView[MyMoneyAccount::Checkings] = new KLedgerViewCheckings(this, "CheckingsView");
   m_accountStack->addWidget(view, MyMoneyAccount::Checkings);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
-  
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+
   // Savings account
-  view = m_specificView[MyMoneyAccount::Savings] = new KLedgerViewSavings(this);
+  view = m_specificView[MyMoneyAccount::Savings] = new KLedgerViewSavings(this, "SavingsView");
   m_accountStack->addWidget(view, MyMoneyAccount::Savings);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
 
   // Credit card account
-  view = m_specificView[MyMoneyAccount::CreditCard] = new KLedgerViewCreditCard(this);
+  view = m_specificView[MyMoneyAccount::CreditCard] = new KLedgerViewCreditCard(this, "CreditCardView");
   m_accountStack->addWidget(view, MyMoneyAccount::CreditCard);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
 
   // Cash account
-  view = m_specificView[MyMoneyAccount::Cash] = new KLedgerViewCash(this);
+  view = m_specificView[MyMoneyAccount::Cash] = new KLedgerViewCash(this, "CashView");
   m_accountStack->addWidget(view, MyMoneyAccount::Cash);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
 
   // Asset account
-  view = m_specificView[MyMoneyAccount::Asset] = new KLedgerViewAsset(this);
+  view = m_specificView[MyMoneyAccount::Asset] = new KLedgerViewAsset(this, "AssetView");
   m_accountStack->addWidget(view, MyMoneyAccount::Asset);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
 
   // Loan account
-  view = m_specificView[MyMoneyAccount::Loan] = new KLedgerViewLoan(this);
+  view = m_specificView[MyMoneyAccount::Loan] = m_specificView[MyMoneyAccount::AssetLoan] = new KLedgerViewLoan(this, "LoanView");
   m_accountStack->addWidget(view, MyMoneyAccount::Loan);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
-
-  // AssetLoan account
-  view = m_specificView[MyMoneyAccount::AssetLoan] = new KLedgerViewLoan(this);
-  m_accountStack->addWidget(view, MyMoneyAccount::AssetLoan);
-  connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
-  connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
-    SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
 
   // Liability account
-  view = m_specificView[MyMoneyAccount::Liability] = new KLedgerViewLiability(this);
+  view = m_specificView[MyMoneyAccount::Liability] = new KLedgerViewLiability(this, "LiabilityView");
   m_accountStack->addWidget(view, MyMoneyAccount::Liability);
   connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
-    this, SLOT(slotSelectAccountAndTransaction(const QCString&, const QCString&)));
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
   connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
     SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
-  connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
-  
-  m_formLayout->addWidget(m_accountStack);
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+
+#if 0
+  // FIXME: I removed the below code to enable switching to the investment view
+  // when an investment account is selected via the combo box.  People reported,
+  // that the comboboxes on the globalledgerview and the investmentview are not
+  // in sync.  Not providing an investment ledger view within the global ledger view
+  // solves this problem.
+
+  // Investment account
+  view = m_specificView[MyMoneyAccount::Investment] = new KLedgerViewInvestments(this, "InvestmentsView");
+  m_accountStack->addWidget(view, MyMoneyAccount::Investment);
+  connect(view, SIGNAL(accountAndTransactionSelected(const QCString&, const QCString&)),
+    this, SLOT(slotSelectAccount(const QCString&, const QCString&)));
+  connect(view, SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)),
+    SIGNAL(payeeSelected(const QCString&, const QCString&, const QCString&)));
+  // connect(this, SIGNAL(cancelEdit()), view, SLOT(slotCancelEdit()));
+#endif
+
+  m_viewLayout->addWidget(m_accountStack);
   setMinimumHeight(m_accountComboBox->minimumHeight() + m_accountStack->sizeHint().height());
-  
+
   m_accountId = QCString();
 
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAccount, this);
-  
+
   // setup connections
-  connect(m_accountComboBox, SIGNAL(activated(const QString&)),
-          this, SLOT(slotSelectAccount(const QString&)));
+  connect(m_accountComboBox, SIGNAL(accountSelected(const QCString&)),
+          this, SLOT(slotSelectAccount(const QCString&)));
+
 }
 
 KGlobalLedgerView::~KGlobalLedgerView()
 {
   MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassAccount, this);
-  delete m_formLayout;
+  delete m_viewLayout;
 }
 
 void KGlobalLedgerView::slotReloadView(void)
 {
   // qDebug("KGlobalLedgerView::slotReloadView()");
-  
+
   // make sure to determine the current account from scratch
   m_accountId = QCString();
-  
+
   slotRefreshView();
 }
 
@@ -189,8 +199,8 @@ void KGlobalLedgerView::slotRefreshView(void)
   QCString id = m_accountId;
 
   // qDebug("KGlobalLedgerView::slotRefreshView()");
-  
-  // load the combobox from scratch and determine the current account  
+
+  // load the combobox from scratch and determine the current account
   loadAccounts();
 
   // if the current account differs from the previous selection
@@ -207,96 +217,103 @@ void KGlobalLedgerView::slotRefreshView(void)
     m_currentView->refreshView();
   } else
     qFatal("Houston: we have a problem in KGlobalLedgerView::slotRefreshView()");
-  
+
+  // Enable rest of view only, if we have at least one account out of this group
+  QValueList<MyMoneyAccount::accountTypeE> typeList;
+  typeList << MyMoneyAccount::Checkings;
+  typeList << MyMoneyAccount::Savings;
+  typeList << MyMoneyAccount::Cash;
+  typeList << MyMoneyAccount::CreditCard;
+  typeList << MyMoneyAccount::Loan;
+  typeList << MyMoneyAccount::Asset;
+  typeList << MyMoneyAccount::Liability;
+  typeList << MyMoneyAccount::Currency;
+  typeList << MyMoneyAccount::AssetLoan;
+  m_accountStack->setEnabled(m_accountComboBox->accountList(typeList).count() > 0);
+
   m_accountComboBox->setEnabled(m_accountComboBox->count() > 0);
 }
 
 void KGlobalLedgerView::loadAccounts(void)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
-  QString currentName;
-  
-  // qDebug("KGlobalLedgerView::loadAccounts()");
-  m_accountComboBox->clear();
-
-  MyMoneyAccount acc, subAcc;
+  MyMoneyAccount acc;
 
   // check if the current account still exists and make it the
   // current account
   if(!m_accountId.isEmpty()) {
     try {
       acc = file->account(m_accountId);
-      currentName = acc.name();
+      m_accountId = acc.id();
     } catch(MyMoneyException *e) {
       delete e;
       m_accountId = QCString();
+      acc = MyMoneyAccount();
     }
   }
 
-  // load all asset and liability accounts into the combobox  
-  QCStringList::ConstIterator it_s;
-  acc = file->asset();
-  for(it_s = acc.accountList().begin(); it_s != acc.accountList().end(); ++it_s) {
-    subAcc = file->account(*it_s);
-    m_accountComboBox->insertItem(subAcc.name());
-    if(m_accountId.isEmpty()) {
-      m_accountId = *it_s;
-      currentName = subAcc.name();
+  m_accountComboBox->loadList((KMyMoneyUtils::categoryTypeE)(KMyMoneyUtils::asset | KMyMoneyUtils::liability));
+
+  if(acc.id().isEmpty()) {
+    QCStringList list = m_accountComboBox->accountList();
+    if(list.count()) {
+      QCStringList::Iterator it;
+      for(it = list.begin(); it != list.end(); ++it) {
+        MyMoneyAccount a = file->account(*it);
+        if(a.accountType() != MyMoneyAccount::Investment) {
+          if(a.value("PreferredAccount") == "Yes") {
+            acc = a;
+            break;
+          } else if(acc.id().isEmpty()) {
+            acc = a;
+          }
+        }
+      }
     }
   }
 
-  acc = file->liability();
-  for(it_s = acc.accountList().begin(); it_s != acc.accountList().end(); ++it_s) {
-    subAcc = file->account(*it_s);
-    m_accountComboBox->insertItem(subAcc.name());
-    if(m_accountId.isEmpty()) {
-      m_accountId = *it_s;
-      currentName = subAcc.name();
-    }
-  }
-
-  // sort list by name of accounts
-  m_accountComboBox->listBox()->sort();
-  if(!currentName.isEmpty())
-    m_accountComboBox->setCurrentItem(currentName);
+  slotSelectAccount(acc.id());
 }
 
-const bool KGlobalLedgerView::slotSelectAccount(const QString& accountName)
-{
-  // qDebug("KGlobalLedgerView::slotSelectAccount(const QString& accountName)");
-  
-  QCString id = MyMoneyFile::instance()->nameToAccount(accountName);
-  bool     rc = false;
-  if(!id.isEmpty()) {
-    rc = slotSelectAccount(id);
-  }
-  return rc;
-}
-
-const bool KGlobalLedgerView::slotSelectAccount(const QCString& id, const bool reconciliation)
+const bool KGlobalLedgerView::slotSelectAccount(const QCString& id, const QCString& transactionId, const bool reconciliation)
 {
   bool    rc = false;
 
-  // qDebug("KGlobalLedgerView::slotSelectAccount(const QCString& id, const bool reconciliation)");
-  
-  // cancel any pending edit operation in the ledger views
-  emit cancelEdit();
-
   if(!id.isEmpty()) {
     // if the account id differs, then we have to do something
     MyMoneyAccount acc = MyMoneyFile::instance()->account(id);
     if(m_accountId != id) {
+      // cancel any pending edit operation in the ledger views
+      // when switching to a different account
+      slotCancelEdit();
+
       if(m_specificView[acc.accountType()] != 0) {
-        m_accountStack->raiseWidget(acc.accountType());
+        // loan and asset-loan share a view. So there's
+        // only one widget on the widget stack. Make sure
+        // we pick the right one
+        int viewType = acc.accountType();
+        if(viewType == MyMoneyAccount::AssetLoan)
+          viewType = MyMoneyAccount::Loan;
+        m_accountStack->raiseWidget(viewType);
         m_currentView = m_specificView[acc.accountType()];
         m_currentView->slotSelectAccount(id);
-        m_accountComboBox->setCurrentItem(acc.name());
-        rc = true;
+        if(!transactionId.isEmpty())
+          m_currentView->selectTransaction(transactionId);
+        m_accountComboBox->setSelected(acc);
 
+        // keep this as the current account
+        m_accountId = id;
+
+        rc = true;
       } else {
-        QString msg = "Specific ledger view for account type '" +
-          KMyMoneyUtils::accountTypeToString(acc.accountType()) + "' not yet implemented";
-        KMessageBox::sorry(0, msg, "Implementation problem");
+        // keep the current selection ...
+        if(!m_accountId.isEmpty()) {
+          acc = MyMoneyFile::instance()->account(m_accountId);
+          m_accountComboBox->setSelected(acc);
+        } else
+          m_accountComboBox->setSelected(QCString());
+        // ... and let's see, if someone else can handle this request
+        emit accountSelected(id, transactionId);
       }
     } else {
 #if KDE_VERSION < 310
@@ -304,38 +321,44 @@ const bool KGlobalLedgerView::slotSelectAccount(const QCString& id, const bool r
       m_accountStack->raiseWidget(acc.accountType());
 #endif
       rc = true;
+
+      // keep this as the current account
+      m_accountId = id;
+
+      if(reconciliation == true && m_currentView)
+        m_currentView->slotReconciliation();
+      if(!transactionId.isEmpty()) {
+        // cancel any pending edit operation in the ledger views
+        // when switching to a specific transaction
+        slotCancelEdit();
+        m_currentView->selectTransaction(transactionId);
+      }
     }
-    
+
   } else {
     if(m_specificView[MyMoneyAccount::Checkings] != 0) {
+      // cancel any pending edit operation in the ledger views
+      // when switching to a non existing account
+      slotCancelEdit();
+
       m_accountStack->raiseWidget(MyMoneyAccount::Checkings);
       m_currentView = m_specificView[MyMoneyAccount::Checkings];
       m_currentView->slotSelectAccount(id);
-      
+      m_accountComboBox->setSelected(QCString());
+
+      // keep this as the current account
+      m_accountId = QCString();
+
+      if(reconciliation == true && m_currentView)
+        m_currentView->slotReconciliation();
+
     } else {
       qFatal("Houston: we have a serious problem in KGlobalLedgerView");
     }
   }
-
-  // keep this as the current account
-  m_accountId = id;
-
-  if(reconciliation == true && m_currentView)
-    m_currentView->slotReconciliation();
-
   return rc;
 }
 
-
-void KGlobalLedgerView::slotSelectAccountAndTransaction(const QCString& accountId,
-                                                const QCString& transactionId)
-{
-  // if the selection of the account succeeded then select the desired transaction
-  if(slotSelectAccount(accountId)) {
-    m_currentView->selectTransaction(transactionId);
-  }
-}
-
 void KGlobalLedgerView::suspendUpdate(const bool suspend)
 {
   for(int i = 0; i < MyMoneyAccount::MaxAccountTypes; ++i) {
@@ -348,33 +371,34 @@ void KGlobalLedgerView::suspendUpdate(const bool suspend)
 void KGlobalLedgerView::slotCancelEdit(void)
 {
   // cancel any pending edit operation in the ledger views
-  emit cancelEdit();
+  if(m_accountStack->isVisible()) {
+    KLedgerView* view = dynamic_cast<KLedgerView*>(m_accountStack->visibleWidget());
+    Q_CHECK_PTR(view);
+    view->slotCancelEdit();
+  }
 }
 
 void KGlobalLedgerView::show()
 {
-  QWidget::show();
-  
   // only show selection box if filled with at least one account
   m_accountComboBox->setEnabled(m_accountComboBox->count() > 0);
 
+  KMyMoneyViewBase::show();
   emit signalViewActivated();
 }
 
-void KGlobalLedgerView::hide()
-{
-  for(int i = 0; i < MyMoneyAccount::MaxAccountTypes; ++i) {
-    if(m_specificView[i] != 0)
-      m_specificView[i]->hide();
-  }
-}
-
 void KGlobalLedgerView::update(const QCString& /* id */)
 {
-  QCString lastUsed = m_accountId;
-  loadAccounts();
-  if(m_accountId != lastUsed) {
-    m_accountId = lastUsed;
+  if(m_accountStack->isEnabled()) {
+    QCString lastUsed = m_accountId;
+    loadAccounts();
+    if(m_accountId != lastUsed) {
+      m_accountId = lastUsed;
+      slotRefreshView();
+    }
+  } else {
     slotRefreshView();
   }
 }
+
+#include "kgloballedgerview.moc"
