@@ -42,28 +42,27 @@
 #include "../mymoney/mymoneyutils.h"
 #include "../mymoney/mymoneyaccount.h"
 #include "../mymoney/mymoneyobserver.h"
+#include "../views/kmymoneyview.h"
 
 class QVBoxLayout;
 class QHBoxLayout;
 class QGridLayout;
 class QPopupMenu;
 
-class kMyMoneyCombo;
+class kMyMoneyAccountCombo;
 class KLedgerView;
 
 /**
-  *@author Thomas Baumgart
+  * @author Thomas Baumgart
   */
 
-class KGlobalLedgerView : public QWidget, MyMoneyObserver
+class KGlobalLedgerView : public KMyMoneyViewBase, MyMoneyObserver
 {
    Q_OBJECT
 public:
   KGlobalLedgerView(QWidget *parent=0, const char *name=0);
   ~KGlobalLedgerView();
 
-  // void refreshView(void);
-
   /**
     * utility method to suspend/activate updates of the MyMoney engine on
     * all views. This is used to speed up operations with lot's of updates
@@ -80,57 +79,21 @@ public:
   /**
     */
   void update(const QCString& id);
-  
+
+  /**
+    * This method returns the id of the currently selected account
+    * or QCString() if none is selected.
+    */
+  const QCString accountId(void) const { return m_accountId; }
+
 public slots:
-  // void reloadView(void);
-  
-  /**
-    * This slot calls the hide() slot of all known specific ledger views
-    */
-  void hide(void);
-
   void show(void);
 
-  /**
-    * Called when the user changes the visibility
-    * setting of the transaction form
-    *
-    * @param show if true, the transaction form will be shown
-    */
-  // void slotShowTransactionForm(bool show);
-
   /**
     * This slot cancels any edit session in the ledger views when called.
     */
   void slotCancelEdit(void);
 
-  /**
-    * This slot can be used to popup a specific transaction for a
-    * specific account. Both entities are defined by the corresponding Id's.
-    *
-    * @param accountId const QCString reference to the account id
-    * @param transactionId const QCString reference to the transaction id
-    */
-  void slotSelectAccountAndTransaction(const QCString& accountId, const QCString& transactionId);
-
-  /**
-    * This slot is used to select an account by it's @p id.
-    *
-    * @param id const QCString reference to the account's id
-    *
-    * @note The account will be selected in standard mode (not in reconciliation mode)
-    */
-  // void slotAccountSelected(const QString& id);
-
-  /**
-    * This slot is used to select an account by it's @p id.
-    *
-    * @param id const QCString reference to the account's id
-    * @param reconciliation if false (default), the standard ledger is
-    *                       opened, if true, the reconciliation mode is entered
-    */
-  // void slotAccountSelected(const QString& id, const bool reconciliation);
-
   /**
     * This slot is used to reload all data from the MyMoneyFile engine.
     * All existing data in the view will be discarded.
@@ -148,9 +111,10 @@ public slots:
 
   /**
     * This slot is used to select the correct ledger view type for
-    * the account specified by @p id.
+    * the account specified by @p id in a specific mode.
     *
-    * @param id Internal id used for the account to show
+    * @param accountId Internal id used for the account to show
+    * @param transactionId Internal id used for the transaction to select
     * @param reconciliation if true, the account will be selected in
     *                       reconciliation mode. If false, it will
     *                       be selected in regular ledger mode.
@@ -158,21 +122,8 @@ public slots:
     * @retval true selection of account referenced by @p id succeeded
     * @retval false selection of account failed
     */
-  const bool slotSelectAccount(const QCString& id, const bool reconciliation = false);
+  const bool slotSelectAccount(const QCString& accountId, const QCString& transactionId = QCString(), const bool reconciliation = false);
 
-  /**
-    * This is an overloaded version of the above method.
-    *
-    * Using this method one can select an account by it's name. The name
-    * must match an asset or liability account name.
-    *
-    * @param accountName name of an existing account
-    *
-    * @retval true selection of account referenced by @p id succeeded
-    * @retval false selection of account failed
-    */
-  const bool slotSelectAccount(const QString& accountName);
-      
 protected:
   /**
     * This method reloads the account selection combo box of the
@@ -183,31 +134,8 @@ protected:
     */
   void loadAccounts(void);
 
-  /**
-    * This method is used to open the account with the specified id
-    * in the ledger view. The respective view for this account type
-    * will be selected and the account data loaded.
-    * If a transaction ID is given in @p transaction, the specified
-    * transaction will be selected and shown. If transaction is empty,
-    * then the last transaction will be selected.
-    * The parameter @p reconciliation determines, if the reconciliation
-    * mode is started or not.
-    *
-    * @param id id of the account in the MyMoneyFile object
-    * @param transaction ID of the transaction to be selected
-    * @param reconciliation if false (default), the standard ledger is
-    *                       opened, if true, the reconciliation mode is entered
-    * @param forceLoad if set to true, the account is reloaded into the view in any case
-    */
-  // void selectAccount(const QCString& id, const QCString& transaction = "", const bool reconciliation = false, const bool forceLoad = false);
-  
-protected slots:
-
-protected:
-
-
 private:
-  kMyMoneyCombo* m_accountComboBox;
+  kMyMoneyAccountCombo* m_accountComboBox;
 
   /**
     * This member holds the id of the currently selected account
@@ -223,21 +151,29 @@ private:
   KLedgerView* m_specificView[MyMoneyAccount::MaxAccountTypes];
   QWidgetStack* m_accountStack;
   KLedgerView* m_currentView;
-  QVBoxLayout* m_formLayout;
-  
+
 signals:
-  void signalViewActivated();
+  /**
+    * This signal is emitted whenever this view is activated.
+    */
+  void signalViewActivated(void);
 
   /**
     * This signal is emitted whenever the ledger views are required to
     * cancel any pending edit operation.
     */
-  void cancelEdit();
-  
+  // void cancelEdit();
+
   /**
     */
   void payeeSelected(const QCString& payeeId, const QCString& accountId, const QCString& transactionId);
 
+  /**
+    * This signal is emitted, if an account has been selected
+    * which cannot handled by this view.
+    */
+  void accountSelected(const QCString& accountId, const QCString& transactionId);
+
 };
 
 #endif
