@@ -22,56 +22,50 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+
 #include <qlayout.h>
 #include <qdatetime.h>
+#include <qapplication.h>
+#include <dom/dom_element.h>
+#include <dom/dom_doc.h>
+#include <dom/dom_text.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
+
 #include <kglobal.h>
 #include <klocale.h>
-#if QT_VERSION > 300
 #include <kstandarddirs.h>
-#else
-#include <kstddirs.h>
-#endif
-
 #include <khtmlview.h>
 #include <kconfig.h>
+#include <kstdaction.h>
+#include <kmainwindow.h>
+#include <kactioncollection.h>
+#include <kapplication.h>
+#include <kmessagebox.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "khomeview.h"
-#include "../mymoney/mymoneyfile.h"
 #include "../kmymoneyutils.h"
+#include "../mymoney/mymoneyfile.h"
 
 #define VIEW_LEDGER         "ledger"
 #define VIEW_SCHEDULE       "schedule"
 #define VIEW_WELCOME        "welcome"
 #define VIEW_HOME           "home"
+#define VIEW_REPORTS        "reports"
 
-KHomeView::KHomeView(QWidget *parent, const char *name )
- : QWidget(parent,name)
+KHomeView::KHomeView(QWidget *parent, const char *name ) :
+  KMyMoneyViewBase(parent, name, i18n("Home")),
+  m_showAllSchedules(false)
 {
-  m_qvboxlayoutPage = new QVBoxLayout(this);
-  m_qvboxlayoutPage->setSpacing( 6 );
-  m_qvboxlayoutPage->setMargin( 11 );
-
   m_part = new KHTMLPart(this, "htmlpart_km2");
-  m_qvboxlayoutPage->addWidget(m_part->view());
-  QString language = KGlobal::locale()->language();
-  QString country = KGlobal::locale()->country();
+  m_viewLayout->addWidget(m_part->view());
 
-  m_filename = KGlobal::dirs()->findResource("appdata", QString("html/home_%1.%2.html").arg(country).arg(language));
-  if(m_filename.isEmpty()) {
-    // qDebug(QString("html/home_%1.%2.html not found").arg(country).arg(language).latin1());
-    m_filename = KGlobal::dirs()->findResource("appdata", QString("html/home_%1.html").arg(country));
-  }
-  if(m_filename.isEmpty()) {
-    // qDebug(QString("html/home_%1.html not found").arg(country).latin1());
-    m_filename = KGlobal::dirs()->findResource("appdata", "html/home.html");
-  }
+  m_filename = KMyMoneyUtils::findResource("appdata", QString("html/home%1.html"));
 
-  m_part->openURL(m_filename);
+//   m_part->openURL(m_filename);
   connect(m_part->browserExtension(), SIGNAL(openURLRequest(const KURL&, const KParts::URLArgs&)),
           this, SLOT(slotOpenURL(const KURL&, const KParts::URLArgs&)));
 }
@@ -91,10 +85,52 @@ void KHomeView::slotRefreshView(void)
 {
   if(MyMoneyFile::instance()->accountList().count() == 0) {
     m_part->openURL(m_filename);
+
+#if 0
+    // (ace) I am experimenting with replacing links in the
+    // html depending on the state of the engine.  It's not
+    // working.  That's why it's #if0'd out.
+
+    DOM::Element e = m_part->document().getElementById("test");
+    if ( e.isNull() )
+    {
+      qDebug("Element id=test not found");
+    }
+    else
+    {
+      qDebug("Element id=test found!");
+      QString tagname = e.tagName().string();
+      qDebug("%s",tagname.latin1());
+      qDebug("%s id=%s",e.tagName().string().latin1(),e.getAttribute("id").string().latin1());
+
+      // Find the character data node
+      DOM::Node n = e.firstChild();
+      while (!n.isNull())
+      {
+        qDebug("Child type %u",static_cast<unsigned>(n.nodeType()));
+        if ( n.nodeType() == DOM::Node::TEXT_NODE )
+        {
+          DOM::Text t = n;
+          t.setData("Success!!");
+          e.replaceChild(n,t);
+          m_part->document().setDesignMode(true);
+          m_part->document().importNode(e,true);
+          m_part->document().updateRendering();
+
+          qDebug("Data is now %s",t.data().string().latin1());
+        }
+        n = n.nextSibling();
+      }
+    }
+#endif
   } else {
     QString filename = KGlobal::dirs()->findResource("appdata", "html/kmymoney2.css");
-    QString header = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n") +
-      QString("<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"%1\"></head>\n").arg(filename);
+    QString header = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"%1\">\n").arg(filename);
+
+    header += KMyMoneyUtils::variableCSS();
+
+    header += "</head><body>\n";
+
     QString footer = "</body></html>\n";
 
     m_part->begin();
@@ -132,6 +168,9 @@ void KHomeView::slotRefreshView(void)
               showAccounts(Payment, i18n("Payment Accounts"));
             }
             break;
+          case 4:         // favorite reports
+            showFavoriteReports();
+            break;
         }
         m_part->write("<div class=\"gap\">&nbsp;</div>\n");
       }
@@ -200,10 +239,10 @@ void KHomeView::showPayments(void)
     qBubbleSort(overdues);
     QValueList<MyMoneySchedule>::Iterator it;
     QValueList<MyMoneySchedule>::Iterator it_f;
-    tmp = "<div class=\"warning\">" + i18n("Overdue payments") + "</div>\n";
-    m_part->write(tmp);
 
-    m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"1\">");
+    m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\">");
+    tmp = "<tr><th class=\"warning\" colspan=\"3\">" + i18n("Overdue payments") + "</th></tr>\n";
+    m_part->write(tmp);
     for(it = overdues.begin(); it != overdues.end(); ++it) {
       m_part->write(QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd"));
       showPaymentEntry(*it);
@@ -231,8 +270,7 @@ void KHomeView::showPayments(void)
       if ((*t_it).nextPayment((*t_it).lastPayment()) == QDate::currentDate())
       {
         todays.append(*t_it);
-        t_it = schedule.remove(t_it);
-        continue;
+        (*t_it).setLastPayment((*t_it).nextPayment((*t_it).lastPayment()));
       }
       ++t_it;
     }
@@ -240,9 +278,9 @@ void KHomeView::showPayments(void)
     if (todays.count() > 0)
     {
       m_part->write("<div class=\"gap\">&nbsp;</div>\n");
-      tmp = "<div class=\"item\">" + i18n("Todays payments") + "</div>\n";
+      m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\">");
+      tmp = "<tr class=\"item\"><th class=\"left\" colspan=\"3\">" + i18n("Todays payments") + "</th></tr>\n";
       m_part->write(tmp);
-      m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"1\">");
 
       for(t_it = todays.begin(); t_it != todays.end(); ++t_it) {
         m_part->write(QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd"));
@@ -257,16 +295,56 @@ void KHomeView::showPayments(void)
       m_part->write("<div class=\"gap\">&nbsp;</div>\n");
 
       QValueList<MyMoneySchedule>::Iterator it;
-      tmp = "<div class=\"item\">" + i18n("Future payments") + "</div>\n";
+
+      m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\">");
+      tmp = "<tr class=\"item\"><th class=\"left\" colspan=\"3\">" + i18n("Future payments") + "</th></tr>\n";
       m_part->write(tmp);
 
-      m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"1\">");
-      for(it = schedule.begin(); it != schedule.end(); ++it) {
+      // show all or the first 6 entries
+      int cnt;
+      cnt = (m_showAllSchedules) ? -1 : 6;
+      bool needMoreLess = m_showAllSchedules;
+
+      QDate lastDate = QDate::currentDate().addMonths(1);
+      qBubbleSort(schedule);
+      do {
+        it = schedule.begin();
+        if(it == schedule.end())
+          break;
+
+        QDate nextDate = (*it).nextPayment((*it).lastPayment());
+        if(!nextDate.isValid()) {
+          schedule.remove(it);
+          continue;
+        }
+
+        if (nextDate > lastDate)
+          break;
+
+        if(cnt == 0) {
+          needMoreLess = true;
+          break;
+        }
+        if(cnt > 0)
+          --cnt;
+
         m_part->write(QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd"));
         showPaymentEntry(*it);
         m_part->write("</tr>");
+
+        (*it).setLastPayment((*it).nextPayment((*it).lastPayment()));
+        qBubbleSort(schedule);
       }
+      while(1);
+
       m_part->write("</table>");
+      if (needMoreLess) {
+        if(m_showAllSchedules) {
+          m_part->write(link(VIEW_SCHEDULE,  QString("?mode=%1").arg("reduced")) + i18n("Less ...") + linkend());
+        } else {
+          m_part->write(link(VIEW_SCHEDULE,  QString("?mode=%1").arg("full")) + i18n("More ...") + linkend());
+        }
+      }
     }
   }
 }
@@ -274,24 +352,33 @@ void KHomeView::showPayments(void)
 void KHomeView::showPaymentEntry(const MyMoneySchedule& sched)
 {
   QString tmp;
-  MyMoneyAccount acc = sched.account();
-  if(acc.id()) {
-    MyMoneyTransaction t = sched.transaction();
-    // only show the entry, if it is still active
-    if(!sched.isFinished() && sched.nextPayment(sched.lastPayment()) != QDate()) {
-      MyMoneySplit sp = t.splitByAccount(acc.id(), true);
+  try {
+    MyMoneyAccount acc = sched.account();
+    if(acc.id()) {
+      MyMoneyTransaction t = sched.transaction();
+      // only show the entry, if it is still active
+      if(!sched.isFinished() && sched.nextPayment(sched.lastPayment()) != QDate()) {
+        MyMoneySplit sp = t.splitByAccount(acc.id(), true);
 
-      tmp = QString("<td width=\"20%\">") +
-        KGlobal::locale()->formatDate(sched.nextPayment(sched.lastPayment()), true) +
-        "</td><td width=\"70%\">" +
-        link(VIEW_SCHEDULE, QString("?id=%1").arg(sched.id())) + sched.name() + linkend() +
-        "</td>" +
-        "<td width=\"10%\" align=\"right\">";
-      tmp += sp.value().formatMoney();
-      tmp += "</td>";
-      // qDebug("paymentEntry = '%s'", tmp.latin1());
-      m_part->write(tmp);
+        tmp = QString("<td width=\"20%\">") +
+          KGlobal::locale()->formatDate(sched.nextPayment(sched.lastPayment()), true) +
+          "</td><td width=\"70%\">" +
+          link(VIEW_SCHEDULE, QString("?id=%1").arg(sched.id())) + sched.name() + linkend() +
+          "</td>" +
+          "<td width=\"10%\" align=\"right\">";
+
+        MyMoneySecurity currency = MyMoneyFile::instance()->currency(acc.currencyId());
+        QString amount = sp.value().formatMoney(currency.tradingSymbol());
+        amount.replace(" ","&nbsp;");
+        tmp += amount;
+        tmp += "</td>";
+        // qDebug("paymentEntry = '%s'", tmp.latin1());
+        m_part->write(tmp);
+      }
     }
+  } catch(MyMoneyException* e) {
+    qDebug("Unable to display schedule entry: %s", e->what().data());
+    delete e;
   }
 }
 
@@ -319,6 +406,7 @@ void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header
       // have the preferred flag set
       case MyMoneyAccount::Asset:
       case MyMoneyAccount::Liability:
+      case MyMoneyAccount::Investment:
         // if preferred accounts are requested, then keep in list
         if((*it).value("PreferredAccount") != "Yes"
         || (type & Preferred) == 0) {
@@ -370,12 +458,12 @@ void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header
           "</div>\n<div class=\"gap\">&nbsp;</div>\n";
 
     m_part->write(tmp);
-    m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\" width=\"60%\">");
-    m_part->write("<tr class=\"item\"><td width=\"70%\">");
+    m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\">");
+    m_part->write("<tr class=\"item\"><th class=\"left\" width=\"70%\">");
     m_part->write(i18n("Account"));
-    m_part->write("</td><td width=\"30%\" align=\"right\">");
+    m_part->write("</th><th width=\"30%\" class=\"right\">");
     m_part->write(i18n("Balance"));
-    m_part->write("</td></tr>");
+    m_part->write("</th></tr>");
 
     for(it = accounts.begin(); it != accounts.end(); ++it) {
       m_part->write(QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd"));
@@ -389,16 +477,55 @@ void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header
 void KHomeView::showAccountEntry(const MyMoneyAccount& acc)
 {
   QString tmp;
+  MyMoneySecurity currency = MyMoneyFile::instance()->currency(acc.currencyId());
+
+  QString amount;
+  if(acc.accountType() == MyMoneyAccount::Investment)
+    amount = MyMoneyFile::instance()->totalValue(acc.id()).formatMoney(currency.tradingSymbol());
+  else
+    amount = MyMoneyFile::instance()->balance(acc.id()).formatMoney(currency.tradingSymbol());
+  amount.replace(" ","&nbsp;");
 
   tmp = QString("<td width=\"70%\">") +
       link(VIEW_LEDGER, QString("?id=%1").arg(acc.id())) + acc.name() + linkend() + "</td>";
-  tmp += QString("<td width=\"30%\" align=\"right\">") +
-      MyMoneyFile::instance()->balance(acc.id()).formatMoney() +
-      "</td>";
+  tmp += QString("<td width=\"30%\" align=\"right\">%1</td>").arg(amount);
   // qDebug("accountEntry = '%s'", tmp.latin1());
   m_part->write(tmp);
 }
 
+void KHomeView::showFavoriteReports(void)
+{
+  QValueList<MyMoneyReport> reports = MyMoneyFile::instance()->reportList();
+
+  if ( reports.count() > 0 )
+  {
+    m_part->write(QString("<div class=\"itemheader\">%1</div>\n<div class=\"gap\">&nbsp;</div>\n").arg(i18n("Favorite Reports")));
+    m_part->write("<table width=\"75%\" cellspacing=\"0\" cellpadding=\"2\">");
+    m_part->write("<tr class=\"item\"><th class=\"left\" width=\"50%\">");
+    m_part->write(i18n("Report"));
+    m_part->write("</th><th width=\"50%\" class=\"right\">");
+    m_part->write(i18n("Comment"));
+    m_part->write("</th></tr>");
+
+    int row = 0;
+    QValueList<MyMoneyReport>::const_iterator it_report = reports.begin();
+    while( it_report != reports.end() )
+    {
+      if ( (*it_report).isFavorite() )
+        m_part->write(QString("<tr class=\"row-%1\"><td>%2%3%4</td><td align=\"right\">%5</td></tr>")
+          .arg(row++ & 0x01 ? "even" : "odd")
+          .arg(link(VIEW_REPORTS, QString("?id=%1").arg((*it_report).id())))
+          .arg((*it_report).name())
+          .arg(linkend())
+          .arg((*it_report).comment())
+        );
+
+      ++it_report;
+    }
+    m_part->write("</table>");
+  }
+}
+
 const QString KHomeView::link(const QString& view, const QString& query) const
 {
   return QString("<a href=\"/") + view + query + "\">";
@@ -411,26 +538,83 @@ const QString KHomeView::linkend(void) const
 
 void KHomeView::slotOpenURL(const KURL &url, const KParts::URLArgs& /* args */)
 {
+  QString protocol = url.protocol();
   QString view = url.fileName(false);
   QCString id = url.queryItem("id").data();
+  QCString mode = url.queryItem("mode").data();
 
-  // qDebug("view = '%s'", url.fileName(false).latin1());
-  // qDebug("id = '%s'", url.queryItem("id").latin1());
+  if ( protocol == "http" )
+  {
+    KApplication::kApplication()->invokeBrowser(url.prettyURL());
+  }
+  else if ( protocol == "mailto" )
+  {
+    KApplication::kApplication()->invokeMailer(url);
+  }
+  else
+  {
+    if(view == VIEW_LEDGER) {
+      emit ledgerSelected(id, QCString());
 
-  if(view == VIEW_LEDGER) {
-    emit ledgerSelected(id, QCString());
+    } else if(view == VIEW_SCHEDULE) {
+      if(!id.isEmpty())
+        emit scheduleSelected(id);
+      if(!mode.isEmpty()) {
+        m_showAllSchedules = (mode == QCString("full"));
+        slotRefreshView();
+      }
 
-  } else if(view == VIEW_SCHEDULE) {
-    emit scheduleSelected(id);
+    } else if(view == VIEW_REPORTS) {
+      emit reportSelected(id);
 
-  } else if(view == VIEW_WELCOME) {
-    m_part->openURL(m_filename);
+    } else if(view == VIEW_WELCOME) {
+      KMainWindow* mw = dynamic_cast<KMainWindow*>(qApp->mainWidget());
+      Q_CHECK_PTR(mw);
+      if ( mode == "whatsnew" )
+      {
+        QString fname = KMyMoneyUtils::findResource("appdata",QString("html/whats_new%1.html"));
+        if(!fname.isEmpty())
+          m_part->openURL(fname);
+      }
+      else if ( mode == "manual" )
+      {
+        KMessageBox::sorry(qApp->mainWidget(),i18n("There is no user manual yet"),i18n("No manual"));
+      }
+      else
+        m_part->openURL(m_filename);
 
-  } else if(view == VIEW_HOME) {
-    slotRefreshView();
+    } else if(view == "action") {
+        KMainWindow* mw = dynamic_cast<KMainWindow*>(qApp->mainWidget());
+        Q_CHECK_PTR(mw);
+        mw->actionCollection()->action( id )->activate();
 
-  } else {
-    qDebug("Unknown view '%s' in KHomeView::slotOpenURL()", view.latin1());
+#if 0
+        // Enable this to get a dump of all action names
+        unsigned idx = mw->actionCollection()->count();
+        while( idx-- )
+        {
+          qDebug("%u: %s\n",idx,mw->actionCollection()->action(idx)->name());
+        }
+#endif
+
+    } else if(view == VIEW_HOME) {
+      slotRefreshView();
+
+    } else {
+      qDebug("Unknown view '%s' in KHomeView::slotOpenURL()", view.latin1());
+    }
   }
 }
 
+// Make sure, that these definitions are only used within this file
+// this does not seem to be necessary, but when building RPMs the
+// build option 'final' is used and all CPP files are concatenated.
+// So it could well be, that in another CPP file these definitions
+// are also used.
+#undef VIEW_LEDGER
+#undef VIEW_SCHEDULE
+#undef VIEW_WELCOME
+#undef VIEW_HOME
+#undef VIEW_REPORTS
+
+#include "khomeview.moc"
