@@ -23,6 +23,7 @@
 #ifndef KLEDGERVIEW_H
 #define KLEDGERVIEW_H
 
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -32,11 +33,13 @@
 #include <qvaluevector.h>
 #include <qtimer.h>
 #include <qwidgetlist.h>
+#include <qguardedptr.h>
 class QWidgetStack;
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include "kdecompat.h"
 #include <klocale.h>
 
 // ----------------------------------------------------------------------------
@@ -149,6 +152,7 @@ private:
   TransactionSortE  m_sortType;
 };
 
+
 /**
   * This class is the abstract base class for all ledger views. Common functionality
   * for all ledger views. The specifics of the ledger view depend on the account type
@@ -160,11 +164,14 @@ private:
   * - KLedgerViewCreditCard
   * - KLedgerViewSavings
   * - KLedgerViewCash
+  * - KLedgerViewLoan
+  * - KLedgerViewInvestment
   *
   * Each KLedgerView is devided into three parts:
   *
   * - a @b register specific to the account type
-  * - a set of @b buttons to control the dialog
+  * - a @b summary line shown beneath the register
+  * - a @b reconciliation widget
   * - a @b transaction-form specific to the account type
   *
   * If the specific account provides multiple transaction types, the
@@ -179,10 +186,11 @@ private:
   * transaction:
   *
   * - enter new transaction (New)
-  * - start editing new transaction (Edit)
+  * - start editing selected transaction (Edit)
   * - cancel editing (Cancel)
   * - end editing (Enter)
   * - selecting additional functions (More)
+  * - accessing special account options (Account)
   *
   * The buttons are provided by the kMyMoneyTransactionForm class.
   *
@@ -201,12 +209,36 @@ class KLedgerView : public QWidget, MyMoneyObserver, public IMyMoneyRegisterPare
   friend class kMyMoneyTransactionFormTable;
 
 public:
-  enum transactionTypeE {
-    Check = 0,
-    Deposit,
-    Transfer,
-    Withdrawal,
-    ATM
+  enum transactionDirectionE {
+    /**
+      * In case the value of the split is 0, the direction is unknown.
+      */
+    UnknownDirection,
+
+    /**
+      * A credit denotes a transaction consisting of two or more splits.
+      * With respect to a given account, the flow of money is into this
+      * account.
+      */
+    Credit,
+
+    /**
+      * A debit denotes a transaction consisting of two or more splits.
+      * With respect to a given account, the flow of money is out of this
+      * account.
+      */
+    Debit
+  };
+
+  enum investTransactionTypeE {
+    UnknownTransactionType = -1,
+    BuyShares,
+    SellShares,
+    Dividend,
+    ReinvestDividend,
+    Yield,
+    AddShares,
+    RemoveShares
   };
 
   enum editModeE {
@@ -322,6 +354,30 @@ public:
     */
   const bool isEditMode(void) const;
 
+  /**
+    * Called when the action has been changed by the user.
+    * m_transaction and m_split will be updated accordingly.
+    *
+    * @param type const QCString reference to the new action value
+    */
+  void actionChanged(const QCString& type);
+
+  /**
+    * This method returns the direction of the cash flow for
+    * a given split @p split. If the value is positive (> 0), @p
+    * Deposit is returned, @p Withdrawal otherwise.
+    *
+    * @param split const reference to a MyMoneySplit object
+    * @retval UnknownDirection returned if value of the split is zero
+    * @retval Deposit returned if value of the split is positive
+    * @retval Withdrawal returned if value is negative
+    */
+  static int transactionDirection(const MyMoneySplit& split);
+
+  virtual bool eventFilter( QObject *o, QEvent *e );
+
+  virtual int currentActionTab(void) const = 0;
+
 public slots:
   /**
     * This method is used to select a specific transaction. If @p id is equal
@@ -376,6 +432,11 @@ public slots:
     */
   virtual void slotReconciliation(void) = 0;
 
+  /**
+    * Calling this slot enters the account edit mode.
+    */
+  virtual void slotAccountDetail(void) = 0;
+
   /**
     * This method selects the next transaction if not
     * at the end of the ledger.
@@ -414,9 +475,9 @@ public slots:
     * Called when the category field has been changed.
     * m_transaction and m_split will be updated accordingly.
     *
-    * @param name const reference to the name of the category
+    * @param categoryId const reference to the id of the category
     */
-  virtual void slotCategoryChanged(const QString& category);
+  virtual void slotCategoryChanged(const QCString& categoryId);
 
   /**
     * Called when the amount field has been changed by the user.
@@ -458,38 +519,6 @@ public slots:
     */
   virtual void slotDateChanged(const QDate& date);
 
-  /**
-    * Called when the from account field has been changed by the user.
-    * m_transaction and m_split will be updated accordingly.
-    *
-    * @param from const reference to the from account name
-    */
-  virtual void slotFromChanged(const QString& from);
-
-  /**
-    * Called when the to field has been changed by the user.
-    * m_transaction and m_split will be updated accordingly.
-    *
-    * @param date const reference to the to account name
-    */
-  virtual void slotToChanged(const QString& to);
-
-  /**
-    * Called when the type field has been changed by the user.
-    * m_transaction and m_split will be updated accordingly.
-    *
-    * @param type index of the selected item in the combo box
-    */
-  virtual void slotTypeChanged(int type);
-
-  /**
-    * Called when the type field has been changed by the user.
-    * m_transaction and m_split will be updated accordingly.
-    *
-    * @param action const QCString reference to the new action value
-    */
-  virtual void slotTypeChanged(const QCString& type);
-
   /**
     * Called when a new payee entry has been edited
     * This routine will call the payee dialog and possibly add
@@ -548,6 +577,8 @@ public slots:
     */
   virtual void hide(void);
 
+  virtual void slotActionSelected(int transactionType) = 0;
+
 protected slots:
   /**
     * This method marks the split referencing the account in the current
@@ -604,17 +635,8 @@ protected slots:
     * transaction data and creates a scheduled transaction this way.
     */
   void slotCreateSchedule(void);
-  
-protected:
-  /**
-    * This method is called to create the widget stack for the
-    * specific ledger view. It is the responsability of the
-    * derived object to fill the stack with widgets and panes.
-    * This method simply creates the object and attaches it
-    * to m_infoStack.
-    */
-  void createInfoStack(void);
 
+protected:
   /**
     * This method is called to fill the transaction form
     * with the data of the currently selected transaction
@@ -623,6 +645,14 @@ protected:
     */
   virtual void fillForm(void) = 0;
 
+  /**
+    * This method is called to fill the transaction form
+    * with the descriptions for the currently selected transaction
+    * in m_register. It must be overridden by any derived
+    * class.
+    */
+  virtual void fillFormStatics(void) = 0;
+
   /**
     * This method is called to fill the summary line with
     * the necessary data. It must be overridden by any derived
@@ -646,24 +676,6 @@ protected:
 
   void setupPointerAndBalanceArrays(void);
 
-  /**
-    * This method converts the actions strings contained in a split
-    * (e.g. MyMoneySplit::ActionATM) into the internal used numeric values.
-    *
-    * @param split const reference to the split
-    * @return KLedgerView::transactionTypeE value of the action
-    */
-  int transactionType(const MyMoneySplit& split) const;
-
-  /**
-    * This method converts the internal used numeric value for actions
-    * into the strings contained in a split (e.g. MyMoneySplit::ActionATM)
-    *
-    * @param type const int representing KLedgerView::transactionTypeE value of the action
-    * @return const action string
-    */
-  const QCString transactionType(const int type) const;
-
   /**
     * This method handles the focus of the keyboard. When in edit mode
     * (m_editDate widget is visible) the keyboard focus is handled
@@ -676,23 +688,39 @@ protected:
   virtual bool focusNextPrevChild(bool next);
 
   /**
-    * This method is called when the edit widgets for the transaction
-    * should be shown and editing a transaction starts. Since it's pure
-    * virtual, it must be provided by the derived classes.
+    * This method creates, loads, arranges and shows the widgets required
+    * to edit a transaction. See createEditWidgets(), loadEditWidgets(),
+    * arrangeEditWidgetsInForm() and arrangeEditWidgetsInRegister() for details.
     */
-  virtual void showWidgets(void) = 0;
+  virtual void showWidgets(void);
+
+  virtual void createEditWidgets(void) = 0;
+  virtual void loadEditWidgets(void) = 0;
+  virtual QWidget* arrangeEditWidgetsInForm(void) = 0;
+  virtual QWidget* arrangeEditWidgetsInRegister(void) = 0;
+
+  void setFormCellWidget(const int r, const int c, QWidget* w);
+  void setRegisterCellWidget(const int r, const int c, QWidget* w);
+  void setCellWidget(QTable* t, const int r, const int c, QWidget* w);
 
   /**
     * This method is called when the edit widgets for the transaction
-    * should be hidden and editing a transaction ends. Since it's pure
-    * virtual, it must be provided by the derived classes.
+    * should be destroyed and editing of a transaction ends.
     */
-  virtual void hideWidgets(void) = 0;
+  virtual void destroyWidgets(void);
 
   virtual void createContextMenu(void);
 
+  virtual void createAccountMenu(void);
+
   virtual void createMoreMenu(void);
 
+  /**
+    * This method creates a frame with three buttons that
+    * can be used for enter/cancel/more during register editing.
+    */
+  virtual void createRegisterButtons(void);
+
   virtual void reloadEditWidgets(const MyMoneyTransaction& t) = 0;
 
   /**
@@ -703,13 +731,42 @@ protected:
     * @retval false if transfer transactions are not (yet) possible
     */
   const bool transfersPossible(void) const;
-  
-protected:
-  /**
-    * This member keeps a pointer to the specific info stack for the account
-    */
-  QWidgetStack     *m_infoStack;
 
+  virtual void updateTabBar(const MyMoneyTransaction& t, const MyMoneySplit& s, const bool enableAll = false) = 0;
+
+  /**
+    * Resets the tab order for input mode to contain no widgets. New
+    * widgets can be added using addToTabOrder().
+    */
+  void clearTabOrder(void);
+
+  /**
+    * Adds the widget pointed to by @p w to the tab order during edit mode.
+    * The tab order can be cleared using clearTabOrder().
+    *
+    * @param w pointer to input widget
+    */
+  void addToTabOrder(QWidget* w);
+
+  void enableOkButton(const bool enabled);
+  void enableCancelButton(const bool enabled);
+  void enableMoreButton(const bool enabled);
+
+  /**
+    * This method is used by the derived classes to setup the necessary connections
+    * for the register of the view.
+    */
+  void createRegister(kMyMoneyRegister* reg);
+
+  virtual void vatCheck(MyMoneyTransaction& t, MyMoneySplit& s);
+
+  /**
+    * Depending on the setting of [General Options]:FocusChangeIsEnter this
+    * method calls either slotEndEdit() or slotCancelEdit().
+    */
+  void cancelOrEndEdit(void);
+
+protected:
   /**
     * This member keeps a pointer to the specific register for the account
     */
@@ -730,7 +787,7 @@ protected:
     * If it is true, the user is in reconciliation mode, otherwise
     * he's in edit mode
     */
-  bool            m_inReconciliation;
+  bool m_inReconciliation;
 
   /**
     * This member holds the date from which on transactions should be shown
@@ -786,23 +843,26 @@ protected:
     * and is used to preset the posting date when new transactions are set.
     * This member is initialised to the current date when the program is started.
     */
-  static  QDate         m_lastPostDate;
-  
+  static QDate         m_lastPostDate;
+
   /*
    * The following members are pointers to the edit widgets
    */
-  kMyMoneyPayee*        m_editPayee;      ///< pointer to payee edit widget
-  kMyMoneyCategory*     m_editCategory;   ///< pointer to category edit widget
-  kMyMoneyLineEdit*     m_editMemo;       ///< pointer to memo edit widget
-  kMyMoneyEdit*         m_editAmount;     ///< pointer to amount edit widget
-  kMyMoneyEdit*         m_editPayment;    ///< pointer to payment edit widget
-  kMyMoneyEdit*         m_editDeposit;    ///< pointer to deposit edit widget
-  kMyMoneyLineEdit*     m_editNr;         ///< pointer to number edit widget
-  kMyMoneyDateInput*    m_editDate;       ///< pointer to date edit widget
-  kMyMoneyCategory*     m_editFrom;       ///< pointer to 'from account' edit widget
-  kMyMoneyCategory*     m_editTo;         ///< pointer to 'to account' edit widget
-  KPushButton*          m_editSplit;      ///< pointer to split button
-  kMyMoneyCombo*        m_editType;       ///< pointer to transaction type
+  QGuardedPtr<kMyMoneyPayee>        m_editPayee;      ///< pointer to payee edit widget
+  QGuardedPtr<kMyMoneyCategory>     m_editCategory;   ///< pointer to category edit widget
+  QGuardedPtr<kMyMoneyLineEdit>     m_editMemo;       ///< pointer to memo edit widget
+  QGuardedPtr<kMyMoneyEdit>         m_editAmount;     ///< pointer to amount edit widget
+  QGuardedPtr<kMyMoneyEdit>         m_editPayment;    ///< pointer to payment edit widget
+  QGuardedPtr<kMyMoneyEdit>         m_editDeposit;    ///< pointer to deposit edit widget
+  QGuardedPtr<kMyMoneyLineEdit>     m_editNr;         ///< pointer to number edit widget
+  QGuardedPtr<kMyMoneyDateInput>    m_editDate;       ///< pointer to date edit widget
+  QGuardedPtr<KPushButton>          m_editSplit;      ///< pointer to split button
+  QGuardedPtr<kMyMoneyCombo>        m_editType;       ///< pointer to transaction type
+
+  /**
+    * This member identifies, if we are editing a transaction (@p true) or not (@p false).
+    */
+  bool          m_editMode;
 
   /**
     * This member keeps the tab order for the above widgets
@@ -815,10 +875,15 @@ protected:
   bool          m_ledgerLens;
 
   /**
-    * This member keeps a pointer to the popup-menu
+    * This member keeps a pointer to the context popup-menu
     */
   KPopupMenu*   m_contextMenu;
 
+  /**
+    * This member keeps a pointer to the account popup menu
+    */
+  KPopupMenu*   m_accountMenu;
+
   /**
     * This member keeps a pointer to the sort-menu
     */
@@ -829,29 +894,36 @@ protected:
     */
   KPopupMenu*   m_moreMenu;
 
-private:
-  void fromToChanged(const bool fromChanged, const QString& accountName);
+  QGuardedPtr<QFrame>       m_registerButtonFrame;
+  QGuardedPtr<KPushButton>  m_registerEnterButton;
+  QGuardedPtr<KPushButton>  m_registerCancelButton;
+  QGuardedPtr<KPushButton>  m_registerMoreButton;
 
+private:
   /**
     * This method creates a second split if the current @p m_transaction
     * only contains a single split and adds it to @p m_transaction.
     */
   void createSecondSplit(void);
 
-private:
-   /**
-    * This is the blink timer used to toggle the color for
-    * erronous transactions in the register. The current state (on/off)
-    * is kept in m_blinkState.
+  /**
+    * This method is used to update the value of the transaction.
+    * It is provided for internal use by slotPaymentChanged() and
+    * slotDepositChanged() which need an ability to override the
+    * direction of m_split.
     */
-  QTimer        m_blinkTimer;
+  void amountChanged(const QString& value, const int dir = UnknownDirection);
 
   /**
-    * This member holds the state of the toggle switch used
-    * to colorize erronous transactions in the register of the ledger view.
+    * Enables the widgets in the list to the state passed by
+    * the parameter @p enabled.
+    *
+    * @param list QPtrList of widgets
+    * @param enabled enable (true) or disable (false) the widgets in the list
     */
-  bool          m_blinkState;
+  void enableWidgets(QPtrList<QWidget> list, const bool enabled);
 
+private:
   /**
     * This member holds the state of the toggle switch used
     * to suppress updates due to MyMoney engine data changes
@@ -859,15 +931,12 @@ private:
   bool m_suspendUpdate;
 
 private slots:
-  void timerDone(void);
-
   /**
-    * This method is called by m_blinkTimer upon timeout and toggles
-    * m_blinkState between true and false.
-    * If a register is available, it informs the register about
-    * the state of m_blinkState.
+    * This method finishes the initialization of the object
     */
-  void slotBlinkTimeout(void);
+  void finishInitialization(void);
+
+  void timerDone(void);
 
   /**
     * This method sets the reconcileFlag of the selected transaction
@@ -887,7 +956,7 @@ signals:
   void transactionSelected(void);
 
   /**
-    * The signal accountAndTransactionSelected() is emitted, when a 
+    * The signal accountAndTransactionSelected() is emitted, when a
     * transaction in a different account should be shown. It should be
     * handled by KGlobalLedgerView which shows the correct view and loads
     * the appropriate account.
@@ -900,7 +969,7 @@ signals:
   /**
     * The signal payeeSelected() is emitted, when a the user selects the
     * 'Goto payee/receiver' option. It will be routed by the KGlobalLedgerView()
-    * to the KMyMoneyView() for further processing. The parameters @payeeId,
+    * to the KMyMoneyView() for further processing. The parameters @p payeeId,
     * @p accountId and @p transactionId specify some options for the initial
     * display of the payee view.
     *
