@@ -42,11 +42,14 @@
 
 class KLedgerViewCreditCard : public KLedgerViewCheckings
 {
-   Q_OBJECT
-public: 
+  Q_OBJECT
+public:
   KLedgerViewCreditCard(QWidget *parent=0, const char *name=0);
   ~KLedgerViewCreditCard();
 
+  /// This has to be included for internal reasons, no API change
+  bool eventFilter(QObject* o, QEvent* e);
+
 protected:
   /* documented in base class */
   void fillSummary(void);
@@ -59,7 +62,6 @@ protected:
 
   /* documented in base class */
   virtual void slotReconciliation(void);
-
 };
 
 #endif
