@@ -27,6 +27,7 @@
 #include <qmessagebox.h>
 #include <qvbox.h>
 class QFile;
+class QVBoxLayout;
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -38,19 +39,24 @@ class QFile;
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "../mymoney/mymoneyaccount.h"
-#include "../dialogs/kreconciledlg.h"
-#include "../dialogs/kfindtransactiondlg.h"
-#include "../dialogs/kcsvprogressdlg.h"
+#include <kmymoney/mymoneyaccount.h>
+#include <kmymoney/mymoneyinstitution.h>
+#include <kmymoney/mymoneytransaction.h>
+#include <kmymoney/mymoneyscheduled.h>
 
 class KHomeView;
 class KAccountsView;
 class KCategoriesView;
 class KPayeesView;
 class KScheduledView;
+class KNewAccountWizard;
 class KGlobalLedgerView;
 class IMyMoneyStorageFormat;
 class MyMoneyTransaction;
+class KInvestmentView;
+class KReportsView;
+class KMyMoneyViewBase;
+class KFindTransactionDlg;
 
 /**
   * This class represents the view of the MyMoneyFile which contains
@@ -59,17 +65,25 @@ class MyMoneyTransaction;
   * is represented by a tab within the view.
   *
   * @author Michael Edwardes 2001 Copyright 2000-2001
-  * $Id: kmymoneyview.h,v 1.45.2.5 2004/09/13 18:34:13 ipwizard Exp $
   *
   * @short Handles the view of the MyMoneyFile.
-**/
+  */
 class KMyMoneyView : public KJanusWidget {
    Q_OBJECT
 
 
 public:
-  enum viewType { None=0, BankList=1, TransactionList=2, InvestmentList=3 };
-  enum viewShowing { AccountsView, HomeView, PayeeView, CategoryView, ScheduledView, AccountView };
+  enum viewID {
+    HomeView = 0,
+    AccountsView,
+    InstitutionsView,
+    SchedulesView,
+    CategoriesView,
+    PayeesView,
+    LedgersView,
+    InvestmentsView,
+    ReportsView
+  };
 
 private:
   enum menuID {
@@ -78,59 +92,70 @@ private:
     AccountReconcile,
     AccountEdit,
     AccountDelete,
+    AccountOnlineMap,
+    AccountOnlineUpdate,
+    AccountOfxConnect,
     CategoryNew
   };
 
   KHomeView *m_homeView;
   KAccountsView *m_accountsView;
+  KAccountsView *m_institutionsView;
   KCategoriesView *m_categoriesView;
   KPayeesView *m_payeesView;
   KScheduledView *m_scheduledView;
   KGlobalLedgerView *m_ledgerView;
+  KInvestmentView *m_investmentView;
+  KReportsView* m_reportsView;
   KFindTransactionDlg* m_searchDlg;
-  
+
   QVBox* m_homeViewFrame;
   QVBox* m_accountsViewFrame;
+  QVBox* m_institutionsViewFrame;
   QVBox* m_categoriesViewFrame;
   QVBox* m_payeesViewFrame;
   QVBox* m_scheduleViewFrame;
   QVBox* m_ledgerViewFrame;
+  QVBox* m_investmentViewFrame;
+  QVBox* m_reportsViewFrame;
 
   bool m_fileOpen;
 
-/*
-  bool m_inReconciliation;  // True if the reconciliaton dialog needs updating when the user adds/deletes transactions
-  bool m_reconcileInited;  // True if a reconciliation has already been completed this execution
-  KReconcileDlg *reconcileDlg;  // These exists during app run time ?
-*/
-//  KFindTransactionDlg *transactionFindDlg;
-//  KImportDlg       *importDlg;
-
   KPopupMenu* m_accountMenu;
   KPopupMenu* m_bankMenu;
   KPopupMenu* m_rightMenu;
-  
-  
-  // The schedule view
-  // KScheduleView *m_scheduledView;
+
   bool m_bankRightClick;
   MyMoneyInstitution m_accountsInstitution;
+  // Keep a note of the file type
+  typedef enum _fileTypeE {
+    KmmBinary = 0, // native, binary
+    KmmXML,        // native, XML
+    /* insert new native file types above this line */
+    MaxNativeFileType,
+    /* and non-native types below */
+    GncXML
+  }fileTypeE;
+  fileTypeE m_fileType;
 private:
-  /**
-    * This method gets a filename from the user for the template
-    * of accounts to be used when the file is created. The directory
-    * where the dialog is positioned first is $KDEDIR/share/apps/kmymoney2
-    * It uses readDefaultCategories() to actually process the file.
-    */
-  void loadDefaultCategories(void);
+  void ungetString(QIODevice *qfile, char * buf, int len);
 
   /**
-    * This method loads the accounts specified in the file @p filename
-    * into the KMyMoney engine.
-    *
-    * @param filename absolute filename of the file to be loaded
+    * This method preloads all known currencies into the engine.
     */
-  void readDefaultCategories(const QString& filename);
+  void loadDefaultCurrencies(void);
+
+  /**
+    *
+    */
+  void loadAncientCurrency(const QCString& id, const QString& name, const QString& sym, const QDate& date, const MyMoneyMoney& rate, const QCString& newId, const int partsPerUnit = 100, const int smallestCashFraction = 100, const int smallestAccountFraction = 0);
+
+  void loadAncientCurrencies(void);
+
+  /**
+    * if no base currency is defined, start the dialog and force it to be set
+    */
+  void selectBaseCurrency(void);
 
   /**
     * This method attaches an empty storage object to the MyMoneyFile
@@ -145,8 +170,6 @@ private:
     */
   void removeStorage(void);
 
-  // Parses a line in the default categories file
-  bool parseDefaultCategory(QString& line, bool& income, QString& name, QStringList& minors);
   void viewAccountList(const QCString& selectAccount); // Show the accounts view
 
   static void progressCallback(int current, int total, const QString&);
@@ -154,11 +177,22 @@ private:
   /**
     */
   void fixFile(void);
-  
+
   /**
     */
   void fixLoanAccount(MyMoneyAccount acc);
 
+  /**
+    * This method converts a possible opening balance held with @p
+    * account into a transaction between the account in question and
+    * the opening balances account for the currency of @p account.
+    * @p account is updated. If the opening balance is 0, then nothing
+    * is changed.
+    *
+    * @param account reference to the account object to be checked
+    */
+  void fixOpeningBalance(MyMoneyAccount& account);
+
   /**
     */
   void fixTransactions(void);
@@ -166,29 +200,29 @@ private:
   void fixDuplicateAccounts(MyMoneyTransaction& t);
 
   void createSchedule(MyMoneySchedule s, MyMoneyAccount& a);
-    
+
 public:
   /**
     * The constructor for KMyMoneyView. Just creates all the tabs for the
     * different aspects of the MyMoneyFile.
-  **/
+    */
   KMyMoneyView(QWidget *parent=0, const char *name=0);
-  
+
   /**
     * Destructor
-  **/
+    */
   ~KMyMoneyView();
 
   /**
     * Makes sure that a MyMoneyFile is open and has been created succesfully.
     *
     * @return Whether the file is open and initialised
-  **/
+    */
   bool fileOpen(void);
 
   /**
     * Closes the open MyMoneyFile and frees all the allocated memory, I hope !
-  **/
+    */
   void closeFile(void);
 
 
@@ -201,7 +235,7 @@ public:
     *            If no protocol is specified, file:// is assumed.
     *
     * @return Whether the read was successfull.
-  **/
+    */
   bool readFile(const KURL& url);
 
   /**
@@ -214,6 +248,13 @@ public:
     * @retval true save operation was successful
     */
   const bool saveFile(const KURL& url);
+  /**
+    * Call this to find out if the currently open file is native KMM
+    *
+    * @retval true file is native
+    * @retval false file is foreign
+    */
+  const bool isNativeFile() { return (m_fileType < MaxNativeFileType);};
 
   /**
     * Call this to see if the MyMoneyFile contains any unsaved data.
@@ -231,10 +272,12 @@ public:
     *                        information and will be offered to load default income
     *                        and expense accounts. If true, a completely empty file
     *                        will be created without user interaction.
+    * @retval false User pressed Cancel button
+    * @retval true User pressed Ok
     *
     * @see MyMoneyFile
     */
-  void newFile(const bool createEmtpyFile = false);
+  bool newFile(const bool createEmtpyFile = false);
 
   /**
     * Brings up a dialog that displays information about the user who created
@@ -246,21 +289,14 @@ public:
 
   /**
     * Moves the view up from transaction to Bank/Account view.
-  **/
+    */
   void viewUp(void);
 
-  /**
-    * Utility method to retrieve the currently selected bank name.
-    *
-    * @return The currently selected bank name.
-  **/
-  //QString currentBankName(void);
-
   /**
     * Utility method to retrieve the currently selected account name.
     *
     * @return The currently selected account name.
-  **/
+    */
   QString currentAccountName(void);
 
   /**
@@ -275,7 +311,7 @@ public:
     * @li false updates will be performed immediately
     */
   void suspendUpdate(const bool suspend);
-  
+
   void memoryDump();
 
   /**
@@ -288,6 +324,12 @@ public:
     */
   void enableViews(int state = -1);
 
+  KPopupMenu* accountContextMenu(void) const { return m_accountMenu; };
+
+  KMyMoneyViewBase* addPage(const QString& title, const QPixmap& pixmap = QPixmap());
+
+  void addWidget(QWidget* w);
+
 public slots:
   /**
     * This slot writes information about the page passed as argument @p widget
@@ -317,9 +359,7 @@ public slots:
   /**
     * Called whenever the user 'executes' an account. This operation opens the account
     * and shows the register view.
-    *
-    * @param account The account which was 'executed'.
-  **/
+    **/
   void slotAccountDoubleClick(void);
 
   /**
@@ -334,11 +374,11 @@ public slots:
 
   /**
     * Called, whenever the payees view should pop up and a specific
-    * transaction in an account should be shown. 
+    * transaction in an account should be shown.
     *
-    * @param payee The ID of the payee to be shown
-    * @param account The ID of the account to be shown
-    * @param transaction The ID of the transaction to be selected
+    * @param payeeId The ID of the payee to be shown
+    * @param accountId The ID of the account to be shown
+    * @param transactionId The ID of the transaction to be selected
     */
   void slotPayeeSelected(const QCString& payeeId, const QCString& accountId, const QCString& transactionId);
 
@@ -350,6 +390,14 @@ public slots:
     */
   void slotScheduleSelected(const QCString& schedule);
 
+  /**
+    * Called, whenever the report view should pop up and a specific
+    * report should be shown.
+    *
+    * @param reportid The ID of the report to be shown
+    */
+  void slotReportSelected(const QCString& reportid);
+
   /**
     * Called whenever the user wishes to create a new bank.  Brings up the input
     * dialog and saves the information.  It then enables the banks view.
@@ -357,8 +405,8 @@ public slots:
     * @see KBanksView
     * @see KNewBankDlg
     * @see MyMoneyFile
-    * @see MyMoneyBank
-  **/
+    * @see MyMoneyInstitution
+    */
   void slotBankNew(void);
 
   /**
@@ -367,11 +415,9 @@ public slots:
     *
     * @see KBanksView
     * @see KNewAccountDlg
-
-
     * @see MyMoneyFile
     * @see MyMoneyAccount
-  **/
+    */
   void slotAccountNew(void);
 
   /**
@@ -382,11 +428,9 @@ public slots:
     *
     * @see KBanksView
     * @see KNewAccountDlg
-
-
     * @see MyMoneyFile
     * @see MyMoneyAccount
-  **/
+    */
   void slotBankAccountNew(void);
 
   /**
@@ -395,11 +439,9 @@ public slots:
     *
     * @see KBanksView
     * @see KNewAccountDlg
-
     * @see MyMoneyFile
-
     * @see MyMoneyAccount
-  **/
+    */
   void slotCategoryNew(void);
 
   /**
@@ -412,7 +454,7 @@ public slots:
     * @see KReconcileDlg
     * @see KMyMoneyFile
     * @see MyMoneyAccount
-  **/
+    */
   void slotAccountReconcile(void);
 
   // Not implemented, not documented!
@@ -420,106 +462,68 @@ public slots:
   void slotAccountImportAscii(void);
   void slotAccountExportAscii(void);
 
-//  void slotAccountImportQIF(void);
-//  void slotAccountExportQIF(void);
-
   /**
     * This slot cancels any edit activity in any view. It will
     * be called e.g. before entering the settings dialog.
-
     */
   void slotCancelEdit(void) const;
 
-protected slots:
   /**
-    * This slot is called whenever the transaction list is changed and is used
-    * in the reconciliation process to update the view.
-    *
-    * @see slotAccountReconcile.
+    * This slot prints the current view.
+    */
+  void slotPrintView(void);
 
-  **/
-//  void slotTransactionListChanged();
+  /**
+    * This slot switches the view to present the home page
+    */
+  void slotShowHomePage(void) { showPage(0); }
 
+protected slots:
   /**
     * Called whenever the user right clicks on an account.  It brings up
     * a context menu.  TODO: move the context menu into kmymoney2ui.rc, move
     * this method into KBanksView, remove the param inList.
-    *
-
-    * @param account The account which has been clicked on.
-    * @param inList Whether the click was on an account (REDUNDANT).
-  **/
+    */
   void slotAccountRightMouse();
   void slotBankRightMouse();
   void slotRightMouse();
 
-
-
   /**
     * Called by the context menu created in slotAccountRightMouse.  Brings up
     * a dialog which allows the user to edit the account details.  TODO: move this
     * method into KBanksView.
-  **/
-
+    */
   void slotAccountEdit();
 
   /**
-
     * Called by the context menu created in slotAccountRightMouse.  Deletes the currently
     * selected account. TODO: move this method into KBanksView.
-  **/
+    */
   void slotAccountDelete();
 
+  /**
+    * Called by the context menu created in slotAccountRightMouse.  Connects to the users
+    * bank via OFX.  Only valid if ofxConnectionSettings have been set in the institution.
+    */
+  void slotAccountOfxConnect();
+
   /**
     * Called by the context menu created in slotBankRightMouse.  Brings up
     * a dialog which allows the user to edit the bank details.  TODO: move this
     * method into KBanksView.
-  **/
+    */
   void slotBankEdit();
 
   /**
     * Called by the context menu created in slotBankRightMouse.  Deletes the currently
     * selected bank. TODO: move this method into KBanksView.
-  **/
+    */
   void slotBankDelete();
 
   /**
     * Destroys the search dialog
-    **/
-  void slotCloseSearchDialog(void);
-
-  /**
-
-    * Called when the user clicks on the homepage button.
-    *
-    * @see KHomeView
-  **/
-  void slotActivatedHomePage();
-
-
-  /**
-    * Called when the user clicks on the accounts button
-    *
-    * @see KBanksView
-  **/
-  void slotActivatedAccountsView();
-
-  void slotActivatedScheduledView();
-
-  void slotActivatedCategoriesView();
-
-  void slotActivatedPayeeView();
-
-  void slotActivatedAccountView();
-
-  /**
-    * Called when the user changes the visibility
-    * setting of the transaction form
-    *
-    * @param show if true, the transaction form is shown
     */
-  // void slotShowTransactionForm(bool show);
-
+  void slotCloseSearchDialog(void);
 
   /**
     * Called when the user changes the detail
@@ -535,29 +539,52 @@ private:
     * either directly in the destination file if it is on
     * the local file system or in a temporary file when
     * the final destination is reached over a network
-
     * protocol (e.g. FTP)
     *
     * @param qf pointer to QFile representing the opened file
     * @param writer pointer to the formatter
+    * @param plaintext whether to override any compression & encryption settings
     *
-
     * @note This method will close the file when it is written.
     */
-  void saveToLocalFile(QFile* qf, IMyMoneyStorageFormat* writer);
+  void saveToLocalFile(QFile* qf, IMyMoneyStorageFormat* writer, bool plaintext=false);
 
   /**
     * Internal method used by slotAccountNew() and slotAccountCategory().
     */
   void accountNew(const bool createCategory);
-  
+
 signals:
-  void signalHomeView();
-  void signalAccountsView();
-  void signalScheduledView();
-  void signalCategoryView();
-  void signalPayeeView();
-  void signalAccountView();
+  /**
+    * This signal is emitted whenever a view is selected.
+    * The parameter @p view is identified as one of KMyMoneyView::viewID.
+    */
+  void viewActivated(int view);
+
+  void accountSelectedForContextMenu(const MyMoneyAccount& acc);
+
+  void viewStateChanged(bool enabled);
 };
 
+class kMyMoneyTitleLabel;
+
+/**
+  * This class is an abstract base class that all specific views
+  * should be based on.
+  */
+class KMyMoneyViewBase : public QWidget
+{
+  Q_OBJECT
+public:
+  KMyMoneyViewBase(QWidget* parent, const char *name, const QString& title);
+  virtual ~KMyMoneyViewBase() {};
+
+  void setTitle(const QString& title);
+  QVBoxLayout* layout(void) const { return m_viewLayout; }
+  void addWidget(QWidget* w);
+
+protected:
+  kMyMoneyTitleLabel*    m_titleLabel;
+  QVBoxLayout*           m_viewLayout;
+};
 #endif
