@@ -34,10 +34,11 @@
 
 // ----------------------------------------------------------------------------
 // KDE Includes
-#include <kdeversion.h>
+#include "kdecompat.h"
 #include <kglobal.h>
 #include <klocale.h>
-#include <kstandarddirs.h>
+#include <kiconloader.h>
+#include <kpushbutton.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -82,13 +83,13 @@ kMyMoneyDateInput::kMyMoneyDateInput(QWidget *parent, const char *name, Qt::Alig
   dateEdit->setSeparator(separator);
 
   m_datePicker = new KDatePicker(m_dateFrame, m_date);
-#if KDE_VERSION >= 310
+#if KDE_IS_VERSION(3,1,0)
   // Let the date picker have a close button (Added in 3.1)
   m_datePicker->setCloseButton(true);
 #endif
 
   // the next line is a try to add an icon to the button
-  m_dateButton = new QPushButton(QIconSet(QPixmap( locate("icon","hicolor/16x16/apps/korganizer.png"))), QString(""), this);
+  m_dateButton = new KPushButton(QIconSet(QPixmap(KGlobal::iconLoader()->iconPath("date", -KIcon::SizeSmall))), QString(""), this);
   m_dateButton->setMinimumWidth(30);
 
   connect(m_dateButton,SIGNAL(clicked()),SLOT(toggleDatePicker()));
@@ -266,3 +267,5 @@ QWidget* kMyMoneyDateInput::focusWidget(void) const
     w = w->focusProxy();
   return w;
 }
+
+#include "kmymoneydateinput.moc"
